#include "stdafx.h"
#include "mchat.h"

// channel info

Chan::Chan()
{
	m_fAutoJoin = FALSE;
	m_pMode = 0;
}

Chan::Chan(LPCTSTR pszChan, BOOL fAutoJoin)
{
	m_sChan = pszChan;
	m_fAutoJoin = fAutoJoin;
	m_pMode = 0;
}

Chan::Chan(const Chan& chan)
{
	m_sChan = chan.m_sChan;
	m_fAutoJoin = chan.m_fAutoJoin;
	if (chan.m_pMode)
		m_pMode = new ChanMode(*chan.m_pMode);
	else
		m_pMode = 0;
	m_sTopic = chan.m_sTopic;
	m_lsOpPlus.Copy(chan.m_lsOpPlus);
}

BOOL Chan::Write(CStdioFile& file)
{
	CString s;
	s.Format("chan %s\n", m_sChan);
	TRY {
		file.WriteString(s);
		if (m_fAutoJoin)
			file.WriteString("join auto\n");
		if (m_pMode) {
			s.Format("mode %s\n", (LPCTSTR)m_pMode->ToString());
			file.WriteString(s);
		}
		if (!m_sTopic.IsEmpty()) {
			s.Format("topic :%s\n", (LPCTSTR)m_sTopic);
			file.WriteString(s);
		}
		POSITION pos = m_lsOpPlus.GetHeadPosition();
		while (pos) {
			NickMatch& match = m_lsOpPlus.GetNext(pos);
			s.Format("op+ %s\n", (LPCTSTR)match.GetString());
			file.WriteString(s);
		}
	} CATCH(CFileException, e) {
		return FALSE;
	} END_CATCH
	return TRUE;
}

void Chan::Parse(const CString& s, LPCTSTR pParm)
{
	if (s == "join") {
		if (Token(pParm) == "auto")
			m_fAutoJoin = TRUE;
	} else if (s == "mode") {
		if (*pParm)
			SetMode(pParm);
	} else if (s == "topic") {
		if (*pParm == ':')
			pParm++;
		CString s(pParm);
		if (!s.IsEmpty())
			SetTopic(s);
	} else if (s == "op+") {
		CString s = Token(pParm);
		if (!s.IsEmpty()) {
			NickMatch nm(s);
			m_lsOpPlus.AddTail(nm);
		}
	} else
		TRACE("MapChan.Load Skip <%s %s>\n", (LPCTSTR)s, pParm);
}

void Chan::Clear()
{
	m_sChan.Empty();
	m_fAutoJoin = FALSE;
	if (m_pMode) {
		delete m_pMode;
		m_pMode = 0;
	}
	m_sTopic.Empty();
	m_lsOpPlus.RemoveAll();
}

void Chan::SetMode(ChanMode* pMode)
{
	if (m_pMode) {
		if (pMode) {
			*m_pMode = *pMode;
		} else {
			delete m_pMode;
			m_pMode = 0;
		}
	} else if (pMode)
		m_pMode = new ChanMode(*pMode);
}

void Chan::SetMode(LPCTSTR pszMode)
{
	if (!m_pMode)
		m_pMode = new ChanMode;
	m_pMode->FromString(pszMode);
}

void AFXAPI DestructElements(Chan** ppChan, int nCount )
{
	while (nCount-- > 0) {
		Chan* pChan = *ppChan++;
		if (pChan->m_pMode)
			delete pChan->m_pMode;
		delete pChan;
	}
}

// channel map

MapChan g_mapChan;

BOOL MapChan::Save()
{
	CString sFile(GetDataPathname(IDS_FILE_CHANNELS));
	CStdioFile file;
	if (!file.Open(sFile,
			CFile::modeCreate | CFile::modeWrite | CFile::typeText)) {
		TRACE("MapChan.Save file open error\n");
		return FALSE;
	}

	CString s;
	POSITION pos = GetStartPosition();
	while (pos) {
		CString sChan;
		Chan* pChan;
		GetNextAssoc(pos, sChan, pChan);
		if (!pChan->Write(file))
			return FALSE;
	}
	return TRUE;
}

int MapChan::Load()
{
	RemoveAll();
	CString sFile(GetDataPathname(IDS_FILE_CHANNELS));
	CStdioFile file;
	if (!file.Open(sFile,
			CFile::modeRead | CFile::shareDenyWrite | CFile::typeText))
		return 0;

	int nChan = 0;
	CString sBuf;
	CString s;

	Chan chan;
	for (;;) {
		BOOL fRead;
		TRY {
			fRead = file.ReadString(sBuf);
		} CATCH(CFileException, e) {
			fRead = FALSE;
		} END_CATCH
		if (!fRead)
			break;

		LPCTSTR pBuf = sBuf;
		s = Token(pBuf);
		if (s == "chan") {
			if (!chan.m_sChan.IsEmpty()) {
				Chan* pChan = new Chan(chan);
				SetAt(chan.m_sChan, pChan);
			}

			chan.Clear();
			chan.m_sChan = Token(pBuf);
			nChan++;
			continue;
		}
		if (chan.m_sChan.IsEmpty())
			continue;
		chan.Parse(s, pBuf);
	}
	if (!chan.m_sChan.IsEmpty()) {
		Chan* pChan = new Chan(chan);
		SetAt(chan.m_sChan, pChan);
	}

	return nChan;
}

Chan* MapChan::Append(LPCTSTR pszChan)
{
	Chan* pChan;
	if (Lookup(pszChan, pChan))
		return pChan;
	pChan = new Chan(pszChan);
	SetAt(pszChan, pChan);
	return pChan;
}

void MapChan::Delete(LPCTSTR pszChan)
{
	Chan* pChan;
	if (!Lookup(pszChan, pChan))
		return;
	RemoveKey(pszChan);
}

void MapChan::SetAutoJoin(LPCTSTR pszChan, BOOL fAutoJoin)
{
	Chan* pChan;
	if (Lookup(pszChan, pChan))
		pChan->m_fAutoJoin = fAutoJoin;
	else if (fAutoJoin) {
		Chan* pChan = new Chan(pszChan, fAutoJoin);
		SetAt(pszChan, pChan);
	}
}

void MapChan::SetMode(LPCTSTR pszChan, ChanMode* pMode)
{
	Chan* pChan;
	if (Lookup(pszChan, pChan))
		pChan->SetMode(pMode);
	else if (pMode) {
		Chan* pChan = new Chan(pszChan);
		pChan->SetMode(pMode);
		SetAt(pszChan, pChan);
	}
}

void MapChan::SetTopic(LPCTSTR pszChan, LPCTSTR pszTopic)
{
	TRACE("MapChan.SetTopic(%s, %s)\n", pszChan, pszTopic);

	Chan* pChan;
	if (Lookup(pszChan, pChan))
		pChan->SetTopic(pszTopic);
	else if (pszTopic) {
		pChan = new Chan(pszChan);
		pChan->SetTopic(pszTopic);
		SetAt(pszChan, pChan);
	}
}

void MapChan::AutoJoin()
{
	POSITION pos = GetStartPosition();
	while (pos) {
		CString sChan;
		Chan* pChan;
		GetNextAssoc(pos, sChan, pChan);
		if (pChan->m_fAutoJoin)
			CmdJoin(sChan);
	}
}

// channel mode

ChanMode::ChanMode()
{
	Clear();
}

ChanMode::ChanMode(const ChanMode& mode)
{
	*this = mode;
}

void ChanMode::Clear()
{
	m_fPrivate = FALSE;
	m_fSecret = FALSE;
	m_fInviteOnly = FALSE;
	m_fTopicOp = FALSE;
	m_fNoOutside = FALSE;
	m_fModerated = FALSE;
	m_fAnonymous = FALSE;
	m_fQuiet = FALSE;
	m_fReop = FALSE;
	m_nLimit = 0;
	m_sKey.Empty();
}

CString ChanMode::ToString()
{
	CString s( '+');
	if (m_fPrivate)
		s += 'p';
	if (m_fSecret)
		s += 's';
	if (m_fInviteOnly)
		s += 'i';
	if (m_fTopicOp)
		s += 't';
	if (m_fNoOutside)
		s += 'n';
	if (m_fModerated)
		s += 'm';
	if (m_fAnonymous)
		s += 'a';
	if (m_fQuiet)
		s += 'q';
	if (m_fReop)
		s += 'r';
	if (m_nLimit != 0)
		s += 'l';
	if (!m_sKey.IsEmpty())
		s += 'k';
	if (s == "+")
		s.Empty();

	if (m_nLimit != 0) {
		s += ' ';
		char buf[20];
		itoa(m_nLimit, buf, 10);
		s += buf;
	}
	if (!m_sKey.IsEmpty()) {
		s += ' ';
		s += m_sKey;
	}
	return s;
}

void ChanMode::FromString(LPCTSTR p, FrameChan* pFrame)
{
	CString sOP;
	CString sParm;
	CString s;
	SkipSpace(p);
	while (*p) {
		s = Token(p);
		if (s[0] == '+' || s[0] == '-')
			sOP += s;
		else
			sParm += (s + " ");
	}

	if (sOP.GetLength() < 2 && sParm.IsEmpty())
		return;

	LPCTSTR pParm = sParm;
	BOOL fPlus = TRUE;
	LPCTSTR pOP;
	for (pOP = sOP; *pOP; pOP++) {
		switch (*pOP) {
		case '+':
			fPlus = TRUE;
			break;
		case '-':
			fPlus = FALSE;
			break;
		case 'o':
			s = Token(pParm);
			if (s.IsEmpty())
				break;
			if (pFrame)
				pFrame->RplModeOP(fPlus, s);
			break;
		case 'p':
			m_fPrivate = fPlus;
			break;
		case 's':
			m_fSecret = fPlus;
			break;
		case 'i':
			m_fInviteOnly = fPlus;
			break;
		case 't'://t - topic settable by channel operator only flag;
			m_fTopicOp = fPlus;
			break;
		case 'n'://n - no messages to channel from clients on the outside;
			m_fNoOutside = fPlus;
			break;
		case 'm'://m - moderated channel;
			m_fModerated = fPlus;
			break;
		case 'a'://a - anonymous channel
			m_fAnonymous = fPlus;
			break;
		case 'q'://q - quiet channel
			m_fQuiet = fPlus;
			break;
		case 'r'://r - reop channel
			m_fReop = fPlus;
			break;
		case 'l'://l - set the user limit to channel;
			if (!fPlus) {
				m_nLimit = 0;
				break;
			}
			s = Token(pParm);
			if (s.IsEmpty())
				break;
			m_nLimit = atoi(s);
			break;
		case 'b'://b - set a ban mask to keep users out;
			s = Token(pParm);
			if (s.IsEmpty())
				break;
			if (pFrame)
				pFrame->RplModeBan(fPlus, s);
			break;
		case 'e'://e - set exception mask to allow users matches to ban list
			s = Token(pParm);
			if (s.IsEmpty())
				break;
			if (pFrame)
				pFrame->RplModeException(fPlus, s);
			break;
		case 'I'://I - set invite mask to allow users to join to invite-only channel
			s = Token(pParm);
			if (s.IsEmpty())
				break;
			if (pFrame)
				pFrame->RplModeInvite(fPlus, s);
			break;
		case 'v'://v - give/take the ability to speak on a moderated channel;
			s = Token(pParm);
			break;
		case 'k'://k - set a channel key (password).
			s = Token(pParm);
			if (s.IsEmpty())
				break;
			if (fPlus)
				m_sKey = s;
			else
				m_sKey.Empty();
			break;
		}
	}
}

void ChanMode::Apply(LPCTSTR sTarget, const ChanMode& mode)
{
	ModeCmd cmd(sTarget);
	if (m_fPrivate != mode.m_fPrivate)
		cmd.Add(mode.m_fPrivate, 'p');
	if (m_fSecret != mode.m_fSecret)
		cmd.Add(mode.m_fSecret, 's');
	if (m_fInviteOnly != mode.m_fInviteOnly)
		cmd.Add(mode.m_fInviteOnly, 'i');
	if (m_fTopicOp != mode.m_fTopicOp)
		cmd.Add(mode.m_fTopicOp, 't');
	if (m_fNoOutside != mode.m_fNoOutside)
		cmd.Add(mode.m_fNoOutside, 'n');
	if (m_fModerated != mode.m_fModerated)
		cmd.Add(mode.m_fModerated, 'm');
	if (m_fAnonymous != mode.m_fAnonymous)
		cmd.Add(mode.m_fAnonymous, 'a');
	if (m_fReop != mode.m_fReop)
		cmd.Add(mode.m_fReop, 'r');
	if (m_nLimit != mode.m_nLimit) {
		if (mode.m_nLimit > 0) {
			CString s;
			s.Format("%u", mode.m_nLimit);
			cmd.Add(TRUE, 'l', s);
		} else
			cmd.Add(FALSE, 'l');
	}
	if (m_sKey != mode.m_sKey) {
		if (!m_sKey.IsEmpty()) {
			cmd.Add(FALSE, 'k', m_sKey);
			cmd.Flush();
		}
		if (!mode.m_sKey.IsEmpty())
			cmd.Add(TRUE, 'k', mode.m_sKey);
	}
	cmd.Flush();
}

//
// DlgOPList _CAO
//

DlgOPList::DlgOPList(FrameChan* pFrame, LPCTSTR pText)
	: CDialog(DlgOPList::IDD, pFrame)
{
	//{{AFX_DATA_INIT(DlgOPList)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂͍폜܂B
	//}}AFX_DATA_INIT

	m_pFrame = pFrame;
	if (pText)
		m_sInitialText = pText;
}

void DlgOPList::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DlgOPList)
	DDX_Control(pDX, IDC_OPLIST_DEL, m_btnDel);
	DDX_Control(pDX, IDC_OPLIST_ADD, m_btnAdd);
	DDX_Control(pDX, IDC_OPLIST_EDIT, m_edOpList);
	DDX_Control(pDX, IDC_OPLIST_LIST, m_lbOpList);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(DlgOPList, CDialog)
	//{{AFX_MSG_MAP(DlgOPList)
	ON_BN_CLICKED(IDC_OPLIST_ADD, OnOplistAdd)
	ON_BN_CLICKED(IDC_OPLIST_DEL, OnOplistDel)
	ON_EN_CHANGE(IDC_OPLIST_EDIT, OnChangeOplistEdit)
	ON_LBN_SELCHANGE(IDC_OPLIST_LIST, OnSelchangeOplistList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// DlgOPList bZ[W nh

BOOL DlgOPList::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CString s;
	s.Format(IDS_TITLE_OPLIST, m_pFrame->m_sTarget);
	SetWindowText(s);

	LsNickMatch& ls = m_pFrame->m_lsOpPlus;
	POSITION pos = ls.GetHeadPosition();
	while (pos) {
		NickMatch& nm = ls.GetNext(pos);
		m_lbOpList.AddString(nm.GetString());
	}

	m_btnAdd.EnableWindow(FALSE);
	m_edOpList.SetWindowText(m_sInitialText);

	return TRUE;
}

void DlgOPList::OnChangeOplistEdit() 
{
	m_btnAdd.EnableWindow(m_edOpList.LineLength() > 0);
}

void DlgOPList::OnOplistAdd() 
{
	CString s;
	m_edOpList.GetWindowText(s);
	if (s.IsEmpty())
		return;

	NickMatch nm(s);
	s = nm.GetString();
	m_edOpList.SetWindowText("");

	POSITION pos = m_lsOpAdd.Find(nm);
	if (pos) {
		int idx = m_lbOpList.FindStringExact(-1, s);
		if (idx >= 0) {
			m_lbOpList.SetSel(-1, FALSE);
			m_lbOpList.SetSel(idx, TRUE);
		}
		return;
	}

	pos = m_lsOpDel.Find(nm);
	if (pos)
		m_lsOpDel.RemoveAt(pos);
	else
		m_lsOpAdd.AddTail(nm);

	int idx = m_lbOpList.AddString(s);
	if (idx >= 0) {
		m_lbOpList.SetSel(-1, FALSE);
		m_lbOpList.SetSel(idx, TRUE);
	}
}

void DlgOPList::OnSelchangeOplistList() 
{
	m_btnDel.EnableWindow(m_lbOpList.GetSelCount() > 0);
}

void DlgOPList::OnOplistDel() 
{
	int nCount = m_lbOpList.GetSelCount();
	if (nCount <= 0)
		return;
	int* pIdx = new int[nCount];
	if (m_lbOpList.GetSelItems(nCount, pIdx) > 0) {
		int i;
		for (i = nCount - 1; i >= 0; i--) {
			CString s;
			int idx = pIdx[i];
			m_lbOpList.GetText(idx, s);
			NickMatch nm(s);
			POSITION pos = m_lsOpAdd.Find(nm);
			if (pos)
				m_lsOpAdd.RemoveAt(pos);
			else
				m_lsOpDel.AddTail(nm);

			m_lbOpList.DeleteString(idx);
			m_edOpList.SetWindowText(s);
		}
	}
	delete [] pIdx;
}

void DlgOPList::OnOK() 
{
	LsNickMatch* plsMatch = &m_pFrame->m_lsOpPlus;

	POSITION pos = m_lsOpDel.GetHeadPosition();
	while (pos) {
		NickMatch& nm = m_lsOpDel.GetNext(pos);
		POSITION posNM = plsMatch->Find(nm);
		if (posNM)
			plsMatch->RemoveAt(posNM);
	}

	int nAdd = 0;
	pos = m_lsOpAdd.GetHeadPosition();
	while (pos) {
		NickMatch& nm = m_lsOpAdd.GetNext(pos);
		plsMatch->AddTail(nm);
		nAdd++;
	}

	Chan* pChan;
	if (!g_mapChan.Lookup(m_pFrame->m_sTarget, pChan))
		pChan = g_mapChan.Append(m_pFrame->m_sTarget);
	pChan->m_lsOpPlus.Copy(*plsMatch);

	if (nAdd)
		if (!DeferPostMessage(m_pFrame, WMU_DEFER_CHANMODE))
			m_pFrame->ModeAutoOP();

	CDialog::OnOK();
}
