// DlgDcc.cpp : implementation file

#include "stdafx.h"
#include "mchat.h"
#include "DlgDcc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TIME_WAIT	(10 * 1000)	// mSec

// CDlgDcc dialog

CDlgDcc::CDlgDcc(int nType, CWnd* pParent /*=NULL*/)
	: CDialog(CDlgDcc::IDD, pParent), m_sockData(FALSE)
{
	//{{AFX_DATA_INIT(CDlgDcc)
	//}}AFX_DATA_INIT
	m_nType = nType;
	m_sockListen.m_parent = m_sockData.m_parent = this;
	m_sockListen.m_pData = &m_sockData;
	m_fConnect = FALSE;
	m_nBytes = m_nBytesComplete = 0;
	m_nSecond = 0;
	m_nTMIDFinished = m_nTMIDSec = 0;
	m_nPrecedePackets = g_opt.nDccSendPackets;
	if (m_nPrecedePackets < 1 || m_nPrecedePackets > 32)
		m_nPrecedePackets = 5;
	Create(IDD, CWnd::GetDesktopWindow());
}

void CDlgDcc::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgDcc)
	DDX_Control(pDX, IDC_DCC_PROGRESS, m_pcSize);
	DDX_Control(pDX, IDC_FILENAME, m_stFilename);
	DDX_Control(pDX, IDC_STATUS, m_stStatus);
	DDX_Control(pDX, IDC_FILESIZE, m_stFileSize);
	//}}AFX_DATA_MAP
}

void CDlgDcc::Msg(LPCTSTR pStr)
{
	m_stStatus.SetWindowText(pStr);
}

void CDlgDcc::ShowSize()
{
	CString sRate;
	if (m_nBytesPerSec < 1024)
		sRate.Format("%d bytes/sec", m_nBytesPerSec);
	else {
		int nKB = m_nBytesPerSec / 1024;
		int nKB1 = (m_nBytesPerSec - (nKB * 1024)) / 103;
		sRate.Format("%d.%d KBytes/sec", nKB, nKB1);
	}
	CString s;
	s.Format("%d/%d bytes (%s)", m_nBytesComplete, m_nSize, (LPCTSTR)sRate);
	m_stFileSize.SetWindowText(s);
	if (m_nSize)
		m_pcSize.SetPos(m_nBytesComplete * 100 / m_nSize);
}

BOOL CDlgDcc::OpenFile(LPCTSTR psFile, u_long ulSize /*=0*/)
{
	if (psFile && *psFile)
		m_sFile = psFile;

	UINT nMode = (m_nType == TypeRcv)
		? (CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite)
		: (CFile::modeRead | CFile::shareDenyWrite);

	if (m_sFile.IsEmpty()) {
		CFileDialog dlg(m_nType == TypeSnd,
			NULL, m_sFile,
			OFN_HIDEREADONLY | (m_nType == TypeSnd ? OFN_FILEMUSTEXIST : 0),
			"All Files(*.*)|*.*||", this);

		if (dlg.DoModal() != IDOK) {
			DestroyWindow();
			return FALSE;
		}
		m_sFile = dlg.GetPathName();
	}

	if (!m_file.Open(m_sFile, nMode)) {
		CString s;
		s.Format(IDS_MSG_OPENFILE_ERR, (LPCTSTR)m_sFile);
		Msg(s);
		SlowClose();
		return FALSE;
	}

	m_stFilename.SetWindowText(m_sFile);
	m_nSize = ulSize ? ulSize : m_file.GetLength();
	return TRUE;
}

BOOL CDlgDcc::DoListen(u_short& usPort)
{
	if (!m_sockListen.DoListen(usPort))
		return FALSE;
	Msg("Waiting for connection");
	return TRUE;
}

BOOL CDlgDcc::DoConnect(u_long ulHost, u_short usPort)
{
	if (!m_sockData.DoConnect(ulHost, usPort)) {
		Msg("Connection failed");
		return FALSE;
	}
	return TRUE;
}

void CDlgDcc::DoSend()
{
	UINT nAck = m_nBytesComplete + (m_nPrecedePackets * sizeof m_aBuf);
	while (m_nBytes < m_nSize && nAck >= m_nBytes) {
		UINT nLen = m_nSize - m_nBytes;
		if (nLen > sizeof m_aBuf)
			nLen = sizeof m_aBuf;
		UINT nRead;
		try {
			nRead = m_file.Read(m_aBuf, nLen);
		} catch (CFileException *e) {
			e->Delete();
			Msg("Error on file-read");
			return;
		}
		if (nRead == 0) {
			Msg("File end");
			break;
		}
		m_sockData.DoSend(m_aBuf, nRead);
		m_nBytes += nRead;
		//TRACE("Sent: %d\n", m_nBytes);
	}
}

void CDlgDcc::SendAck()
{
	u_long n = htonl(m_nBytes);
	m_sockData.DoSend((char*)&n, sizeof (u_long));
}

void CDlgDcc::SlowClose()
{
	GetDlgItem(IDCANCEL)->SetWindowText("OK");
	m_nTMIDFinished = SetTimer(TMID_DCCEND, TIME_WAIT, 0);
}

void CDlgDcc::Finished()
{
	m_fConnect = FALSE;
	Msg("Finished");
	ShowSize();

	m_sockData.Close();
	if (m_file.m_hFile != CFile::hFileNull)
		m_file.Close();

	SlowClose();
}

BEGIN_MESSAGE_MAP(CDlgDcc, CDialog)
	//{{AFX_MSG_MAP(CDlgDcc)
	ON_MESSAGE(WMU_SOCK, OnSock)
	ON_MESSAGE(WMU_SOCK_DATA, OnSockData)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CDlgDcc message handlers

LONG CDlgDcc::OnSock(UINT wParam, LONG lParam)
{
	switch (wParam) {
	case WMU_SOCK_MSG:
		Msg((LPCTSTR)lParam);
		break;
	case WMU_SOCK_CONNECT:
		Msg("Connection established");
		m_fConnect = TRUE;
		m_nTMIDSec = SetTimer(TMID_DCC, 1000, 0); //1sec

#ifdef _DEBUG
		{
			int n1;
			int nOptSize = sizeof n1;
			m_sockData.GetSockOpt(SO_SNDBUF, &n1, &nOptSize);
			int n2;
			nOptSize = sizeof n2;
			m_sockData.GetSockOpt(SO_RCVBUF, &n2, &nOptSize);
			TRACE("bufsize snd=%d rcv=%d\n", n1, n2);
		}
#endif

		if (m_nType == TypeSnd) {
			UINT i;
			for (i = 0; m_nBytes < m_nSize && i < m_nPrecedePackets; i++)
				DoSend();
		} else {
			if (m_nBytes >= m_nSize) {
				SendAck();
				Finished();
			}
		}
		break;
	case WMU_SOCK_CLOSE:
		Msg("Connection closed");
		m_fConnect = FALSE;
		break;
	default:
		TRACE("OnSock: unknown wParam %d\n", wParam);
	}
	return 0;
}

LONG CDlgDcc::OnSockData(UINT wParam, LONG lParam)
{
	if (m_nType == TypeSnd) {
		LPCTSTR pStr = (LPCTSTR)lParam;
		int nLen = (int)wParam;
		if (nLen < sizeof (u_long))
			return 0;
		u_long* pSize = (u_long*)&pStr[nLen - sizeof (u_long)];
		m_nBytesComplete = (UINT)ntohl(*pSize);
		//TRACE("Ack=%ld\n", m_nBytesComplete);
		if (m_nBytesComplete >= m_nSize) {
			Finished();
			return 0;
		}
		if (m_nBytesComplete + (m_nPrecedePackets * sizeof m_aBuf) >= m_nBytes)
			DoSend();
	} else if (m_nType == TypeRcv) {
		try {
			m_file.Write((LPCTSTR)lParam, wParam);
		} catch (CFileException *e) {
			Msg("Error on file-write");
			e->Delete();
			return 0;
		}
		m_nBytes += wParam;
		m_nBytesComplete = m_nBytes;
		SendAck();
		if (m_nBytesComplete >= m_nSize)
			Finished();
	}
	return 0;
}

void CDlgDcc::OnCancel() 
{
	DestroyWindow();
}

void CDlgDcc::OnOK() 
{
	DestroyWindow();
}

BOOL CDlgDcc::OnInitDialog() 
{
	CDialog::OnInitDialog();
	SetWindowText(m_nType == TypeSnd ? "DCC Send" : "DCC Get");
	m_pcSize.SetRange(0, 100);
	return TRUE;
}

void CDlgDcc::PostNcDestroy() 
{
	delete this;
}

void CDlgDcc::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == m_nTMIDSec) {
		if (!m_fConnect)
			KillTimer(m_nTMIDSec);
		else {
			m_nBytesPerSec = m_nBytesComplete / ++m_nSecond;
			ShowSize();
		}
		return;
	} else if (nIDEvent == m_nTMIDFinished) {
		if (m_hWnd)
			DestroyWindow();
		return;
	}
	CDialog::OnTimer(nIDEvent);
}

void CDlgDcc::OnDestroy() 
{
	if (m_nTMIDFinished) {
		KillTimer(m_nTMIDFinished);
		m_nTMIDFinished = 0;
	}
	if (m_nTMIDSec) {
		KillTimer(m_nTMIDSec);
		m_nTMIDSec = 0;
	}
	CDialog::OnDestroy();
}
