// DlgIgn.cpp : implementation file
//

#include "stdafx.h"
#include "mchat.h"
#include "DlgIgn.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

class CIgnore
{
public:
	CIgnore() {};
	CIgnore(LPCTSTR pStr)
	{
		operator=(pStr);
	}

	CString m_sNick;
	CString m_sUser;
	CString m_sAddr;
	operator CString() const;
	CIgnore& operator=(LPCTSTR pStr);
};

CArray<CIgnore, CIgnore&> arIgnore;

CIgnore::operator CString() const
{
	CString s;
	s = m_sNick + '!' + m_sUser + '@' + m_sAddr;
	return s;
}

CIgnore& CIgnore::operator=(LPCTSTR pStr)
{
	LPCTSTR p, q;
	for (p = q = pStr; *q; q++)
		if (*q == '!' || *q <= ' ')
			break;
	m_sNick = CString(p, q - p);
	if (*q == '!')
		q++;
	for (p = q; *q > ' '; q++) {
		if (*q == '@') {
			m_sUser = CString(p, q - p);
			for (p = ++q; *q > ' '; q++)
				;
			break;
		}
	}
	m_sAddr = CString(p, q - p);
	return *this;
}

static CString sFileIgnore;

void IgnoreInit()
{
	sFileIgnore = GetDataPathname(IDS_FILE_IGNORE);

	CStdioFile file;
	if (!file.Open(sFileIgnore,
			CFile::modeRead | CFile::shareDenyWrite | CFile::typeText))
		return;
	CString s;
	TRY {
		while (file.ReadString(s)) {
			CIgnore ign(s);
			arIgnore.Add(ign);
		}
	} CATCH(CFileException, e) {
	} END_CATCH
}

void IgnoreSave()
{
	CStdioFile file;
	if (!file.Open(sFileIgnore,
			CFile::modeCreate | CFile::modeWrite | CFile::typeText))
		return;
	int i;
	for (i = 0; i < arIgnore.GetSize(); i++) {
		CString s(CString(arIgnore[i]) + '\n');
		file.WriteString(s);
	}
}

BOOL IgnoreMatch(const CString sNick, const CString sAddr)
{
	CString sUser;
	LPCTSTR pAddr = sAddr;
	LPCTSTR p = strchr(pAddr, '@');
	if (p) {
		sUser = CString(pAddr, p - pAddr);
		pAddr = p + 1;
	}

	int i;
	for (i = 0; i < arIgnore.GetSize(); i++) {
		const CIgnore& ign = arIgnore[i];

		if (!ign.m_sNick.IsEmpty() && ign.m_sNick != "*" &&
				ign.m_sNick.CompareNoCase(sNick) != 0)
			continue;

		if (!ign.m_sUser.IsEmpty() && ign.m_sUser != "*" &&
				ign.m_sUser.CompareNoCase(sUser) != 0)
			continue;

		const CString& s = ign.m_sAddr;
		if (s.IsEmpty() || s == "*")
			return TRUE;

		int nAddr = strlen(pAddr);
		int ns = s.GetLength();
		if (s[0] == '*') {
			ns--;
			if (nAddr < ns)
				continue;
			if (_stricmp(pAddr + (nAddr - ns), (LPCTSTR)s + 1) == 0)
				return TRUE;
		} else if (s[ns - 1] == '*') {
			if (_strnicmp(pAddr, (LPCTSTR)s, ns - 1) == 0)
				return TRUE;
		} else if (s == pAddr)
			return TRUE;
	}
	return FALSE;
}

CDlgIgnore* pDlgIgnore = 0;

void IgnorePopup(CWnd* pWnd, LPCTSTR pStr)
{
	if (pDlgIgnore) {
		pDlgIgnore->SetWindowPos(&CWnd::wndTop,
			0, 0, 0, 0,
			SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
	} else {
		CDlgIgnore* p = new CDlgIgnore(pWnd);
		if (!p->Create(CDlgIgnore::IDD, pWnd))
			return;
		if (!pDlgIgnore)
			return;
	}
	if (pStr)
		pDlgIgnore->m_edIgnore.SetWindowText(pStr);
}

// CDlgIgnore dialog

CDlgIgnore::CDlgIgnore(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgIgnore::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgIgnore)
	//}}AFX_DATA_INIT
}


void CDlgIgnore::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgIgnore)
	DDX_Control(pDX, IDC_IGNORE, m_edIgnore);
	DDX_Control(pDX, IDC_DELETE, m_btnDelete);
	DDX_Control(pDX, IDC_APPEND, m_btnAppend);
	DDX_Control(pDX, IDC_IGNORELIST, m_lbIgnore);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDlgIgnore, CDialog)
	//{{AFX_MSG_MAP(CDlgIgnore)
	ON_EN_CHANGE(IDC_IGNORE, OnChangeIgnore)
	ON_LBN_SELCHANGE(IDC_IGNORELIST, OnSelchangeIgnorelist)
	ON_BN_CLICKED(IDC_APPEND, OnAppend)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CDlgIgnore message handlers

BOOL CDlgIgnore::OnInitDialog() 
{
	CDialog::OnInitDialog();

	pDlgIgnore = this;
	m_fModify = FALSE;

	int i;
	for (i = 0; i < arIgnore.GetSize(); i++) {
		CIgnore& ign = arIgnore[i];
		CString s(ign);
		m_lbIgnore.AddString(s);
	}

	GotoDlgCtrl(GetDlgItem(IDC_IGNORE));
	return FALSE;
}

void CDlgIgnore::OnChangeIgnore() 
{
	BOOL f = m_edIgnore.LineLength() > 0;
	m_btnAppend.EnableWindow(f);
	m_btnDelete.EnableWindow(FALSE);
}

void CDlgIgnore::OnSelchangeIgnorelist() 
{
	int n = m_lbIgnore.GetCurSel();
	if (n < 0) {
		m_btnDelete.EnableWindow(FALSE);
		return;
	}
	CString s;
	m_lbIgnore.GetText(n, s);
	m_edIgnore.SetWindowText(s);
	m_btnAppend.EnableWindow(FALSE);
	m_btnDelete.EnableWindow(TRUE);
}

static BOOL PtnCheck(LPCTSTR p, BOOL f = FALSE)
{
	if (!p || !*p)
		return FALSE;

	if (!f) {
		if (strchr(p, '*') != NULL) {
			if (p[1] != 0)
				return FALSE;
		}
		return TRUE;
	}

	if (*p++ == '*') {
		while (*p) {
			if (*p++ == '*')
				return FALSE;
		}
		return TRUE;
	}
	while (*p) {
		if (*p++ == '*') {
			if (*p)
				return FALSE;
		}
	}
	return TRUE;
}

void CDlgIgnore::OnAppend() 
{
	CString s;
	m_edIgnore.GetWindowText(s);

	BOOL f = TRUE;
	LPCTSTR p = s;
	LPCTSTR q = strchr(p, '!');
	if (q) {
		CString s1(p, q - p);
		f = PtnCheck(s1);
		p = q + 1;
	} else if (strchr(p, '@')) {
		s.Insert(0, "*!");
		p = (LPCTSTR)s + 2;
	} else {
		s += '!';
		p = (LPCTSTR)s + s.GetLength();
	}

	q = strchr(p, '@');
	if (q) {
		CString s1(p, q - p);
		f &= PtnCheck(s1);
		p = q + 1;
	} else {
		int n = p - (LPCTSTR)s;
		s.Insert(n, "*@");
		p = (LPCTSTR)s + n + 2;
	}

	if (*p)
		f &= PtnCheck(p, TRUE);
	else
		s += '*';

	if (!f) {
		AfxMessageBox(IDS_MSG_IGNORE_FORMAT,
			MB_OK|MB_ICONEXCLAMATION);
		return;
	}

	int n = m_lbIgnore.AddString(s);
	if (n >= 0) {
		m_lbIgnore.SetCurSel(n);
		m_fModify = TRUE;
	}
}

void CDlgIgnore::OnDelete() 
{
	int n = m_lbIgnore.GetCurSel();
	if (n >= 0) {
		CString s;
		m_lbIgnore.GetText(n, s);
		m_lbIgnore.DeleteString(n);
		m_edIgnore.SetWindowText(s);
		m_fModify = TRUE;
	}
}

void CDlgIgnore::PostNcDestroy() 
{
	pDlgIgnore = 0;
	delete this;
}

void CDlgIgnore::OnCancel() 
{
	DestroyWindow();
}

void CDlgIgnore::OnOK() 
{
	if (m_fModify) {
		arIgnore.RemoveAll();
		int i;
		CString s;
		for (i = 0; i < m_lbIgnore.GetCount(); i++) {
			m_lbIgnore.GetText(i, s);
			CIgnore ign(s);
			arIgnore.Add(ign);
		}
		IgnoreSave();
	}
	DestroyWindow();
}
