// DlgWhois.cpp : implementation file

#include "stdafx.h"
#include "mchat.h"
#include "DlgWhois.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CMapStringToDlgWhois mapWhois;

BOOL WhoisCreate(LPCTSTR pNick, LPCTSTR pUser,
				 LPCTSTR pHost, LPCTSTR pReal)
{
	CDlgWhois* pDlg;
	if (!mapWhois.Lookup(pNick, pDlg)) {
		pDlg = new CDlgWhois();
		if (!pDlg->Create(pDlg->IDD, CWnd::GetDesktopWindow())) {
			delete pDlg;
			return FALSE;
		}
		mapWhois.SetAt(pNick, pDlg);
	}
	pDlg->m_sNick = pNick;
	pDlg->m_sAddr = CString(pUser) + '@' + pHost;
	pDlg->m_sRealName = pReal;
	pDlg->UpdateData(FALSE);
	return TRUE;
}

BOOL WhoisServer(LPCTSTR pNick, LPCTSTR pServer, LPCTSTR pServerInfo)
{
	CDlgWhois* pDlg;
	if (!mapWhois.Lookup(pNick, pDlg))
		return FALSE;
	pDlg->m_sServer = pServer;
	pDlg->m_sServInfo = pServerInfo;
	pDlg->UpdateData(FALSE);
	((CButton*)pDlg->GetDlgItem(IDC_USE_SERVER))->EnableWindow(TRUE);
	return TRUE;
}

BOOL WhoisInfo(LPCTSTR pNick, LPCTSTR pInfo)
{
	CDlgWhois* pDlg;
	if (!mapWhois.Lookup(pNick, pDlg))
		return FALSE;
	pDlg->m_sInfo = pInfo;
	pDlg->UpdateData(FALSE);
	return TRUE;
}

static CString Sec2Str(int nSec)
{
	CString s;
	char buf[50];
	int nHour = 0;
	int nMin = 0;
	if (nSec >= 60) {
		nMin = nSec / 60;
		nSec %= 60;
		if (nMin >= 60) {
			nHour = nMin / 60;
			nMin %= 60;
			if (nHour >= 24) {
				int nDay = nHour / 24;
				nHour %= 24;
				sprintf(buf, "%d day ", nDay);
				s += buf;
			}
		}
	}
	sprintf(buf, "%02d:%02d:%02d", nHour, nMin, nSec);
	s += buf;
	return s;
}

BOOL WhoisIdle(LPCTSTR pNick, LPCTSTR pNum, LPCTSTR pText)
{
	CDlgWhois* pDlg;
	if (!mapWhois.Lookup(pNick, pDlg))
		return FALSE;

	CString sTime(Sec2Str(atoi(pNum)));
	CString s;
	s.Format("%s / %s %s", sTime, pNum, pText);

	pDlg->m_sIdle = s;
	pDlg->UpdateData(FALSE);
	return TRUE;
}

BOOL WhoisAway(LPCTSTR pNick, LPCTSTR pMsg)
{
	CDlgWhois* pDlg;
	if (!mapWhois.Lookup(pNick, pDlg))
		return FALSE;
	pDlg->m_sAway = pMsg;
	pDlg->UpdateData(FALSE);
	return TRUE;
}

BOOL WhoisChannels(LPCTSTR pNick, LPCTSTR pChannels)
{
	CDlgWhois* pDlg;
	if (!mapWhois.Lookup(pNick, pDlg))
		return FALSE;
	CComboBox& cb = pDlg->m_cbChannels;
	cb.ResetContent();
	BOOL fNew = (cb.GetCount() == 0);
	do {
		CString sChan(Token(pChannels));
		if (sChan.IsEmpty())
			break;
		int nIdx = cb.AddString(sChan);
		if (fNew && nIdx >= 0) {
			cb.SetCurSel(nIdx);
			((CButton*)pDlg->GetDlgItem(IDC_JOIN))->EnableWindow(TRUE);
			fNew = FALSE;
		}
	} while (*pChannels);
	return TRUE;
}

// CDlgWhois dialog

CDlgWhois::CDlgWhois(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgWhois::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgWhois)
	m_sAddr = _T("");
	m_sAway = _T("");
	m_sIdle = _T("");
	m_sInfo = _T("");
	m_sNick = _T("");
	m_sServer = _T("");
	m_sServInfo = _T("");
	m_sRealName = _T("");
	//}}AFX_DATA_INIT
	m_pApp = (CApp*)AfxGetApp();
}


void CDlgWhois::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgWhois)
	DDX_Control(pDX, IDOK, m_btOK);
	DDX_Control(pDX, IDC_CHANNELS, m_cbChannels);
	DDX_Text(pDX, IDC_NICK, m_sNick);
	DDX_Text(pDX, IDC_ADDR, m_sAddr);
	DDX_Text(pDX, IDC_AWAY, m_sAway);
	DDX_Text(pDX, IDC_IDLE, m_sIdle);
	DDX_Text(pDX, IDC_INFO, m_sInfo);
	DDX_Text(pDX, IDC_SERVER, m_sServer);
	DDX_Text(pDX, IDC_SERVINFO, m_sServInfo);
	DDX_Text(pDX, IDC_REAL, m_sRealName);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDlgWhois, CDialog)
	//{{AFX_MSG_MAP(CDlgWhois)
	ON_BN_CLICKED(IDC_JOIN, OnJoin)
	ON_BN_CLICKED(IDC_COPY, OnCopy)
	ON_BN_CLICKED(IDC_TALK, OnTalk)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_WHOIS_IGNORE, OnWhoisIgnore)
	ON_BN_CLICKED(IDC_USE_SERVER, OnUseServer)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CDlgWhois message handlers

void CDlgWhois::OnOK() 
{
	DestroyWindow();
}

void CDlgWhois::OnCancel() 
{
	DestroyWindow();
}

void CDlgWhois::PostNcDestroy() 
{
	mapWhois.RemoveKey(m_sNick);
	delete this;
}

void CDlgWhois::OnJoin() 
{
	GotoDlgCtrl(&m_btOK);
	int n = m_cbChannels.GetCurSel();
	if (n < 0)
		return;
	CString sChan;
	m_cbChannels.GetLBText(n, sChan);
	LPCTSTR pChan = sChan;
	if (!pChan)
		return;
	while (*pChan == '@' || *pChan == '+')
		pChan++;
	if (*pChan)
		CmdJoin(pChan);
}

void CDlgWhois::OnTalk() 
{
	GotoDlgCtrl(&m_btOK);
	CmdTalk(m_sNick);
}

void CDlgWhois::OnCopy() 
{
	GotoDlgCtrl(&m_btOK);
	if (!OpenClipboard())
		return;
	if (EmptyClipboard()) {
		CString sData, s;
		sData.Format("%s (%s) %s\r\n"
			"Server: %s (%s)\r\n",
			m_sNick, m_sRealName, m_sAddr, m_sServer, m_sServInfo);
		if (!m_sAway.IsEmpty()) {
			s.Format("Away: %s\r\n", m_sAway);
			sData += s;
		}
		if (!m_sIdle.IsEmpty()) {
			s.Format("Idle: %s\r\n", m_sIdle);
			sData += s;
		}
		if (!m_sInfo.IsEmpty()) {
			s.Format("Info: %s\r\n", m_sInfo);
			sData += s;
		}
		HGLOBAL hnd = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE,
			sData.GetLength() + 1);
		if (hnd) {
			BYTE* pBytes = (BYTE*)GlobalLock(hnd);
			if (pBytes) {
				memcpy(pBytes, (LPCTSTR)sData, sData.GetLength() + 1);
				GlobalUnlock(hnd);
				if (!SetClipboardData(CF_TEXT, hnd))
					GlobalFree(hnd);
			}
		}
	}
	CloseClipboard();
}

BOOL CDlgWhois::OnInitDialog() 
{
	CDialog::OnInitDialog();

	GotoDlgCtrl(&m_btOK);
	return FALSE;
}

void CDlgWhois::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	if (IsIconic()) {
		// Erase the icon background when placed over other app window
		DefWindowProc(WM_ICONERASEBKGND, (WORD)dc.m_hDC, 0L);
		// Center the icon
		CRect rc;
		GetClientRect(&rc);
		rc.left = (rc.right  - ::GetSystemMetrics(SM_CXICON)) >> 1;
		rc.top  = (rc.bottom - ::GetSystemMetrics(SM_CYICON)) >> 1;
		// Draw the icon
		HICON hIcon = m_pApp->LoadIcon(IDI_WHOIS);
		dc.DrawIcon(rc.left, rc.top, hIcon);
     }
}

BOOL CDlgWhois::OnEraseBkgnd(CDC* pDC)
{
	if (IsIconic())
		return TRUE;
	else
		return CDialog::OnEraseBkgnd(pDC);
}

HCURSOR CDlgWhois::OnQueryDragIcon()
{
	return m_pApp->LoadIcon(IDI_WHOIS);
}

void CDlgWhois::OnWhoisIgnore() 
{
	GotoDlgCtrl(&m_btOK);
	CString s;
	if (m_sAddr.IsEmpty())
		s.Format("%s!*", m_sNick);
	else
		s.Format("*!%s", m_sAddr);
	IgnorePopup(this, s);
}

void CDlgWhois::OnUseServer() 
{
	GotoDlgCtrl(&m_btOK);
	ChangeServer(m_sServer, 6667, "");
}
