// FontBtn.cpp : Cve[V t@C

#include "stdafx.h"
#include "mchat.h"
#include "FontBtn.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// FontBtn

FontBtn::FontBtn()
{
	m_pFont = 0;
}

FontBtn::~FontBtn()
{
	if (m_pFont)
		delete m_pFont;
}

void FontBtn::Init(int idc, CWnd* pParent, CFont* pFont,
				   COLORREF* pColor, COLORREF crBack)
{
	m_fChanged = FALSE;
	m_pFontOrg = pFont;
	m_pColorOrg = pColor;
	m_color = *pColor;
	m_crBack = crBack;
	SubclassDlgItem(idc, pParent);
}

class myFontDlg : public CFontDialog 
{
public:
	myFontDlg(LPLOGFONT lplfInitial = NULL,
		DWORD dwFlags = CF_EFFECTS | CF_SCREENFONTS,
		CDC* pdcPrinter = NULL, CWnd* pParentWnd = NULL)
		: CFontDialog(lplfInitial, dwFlags, pdcPrinter, pParentWnd) {}

	COLORREF m_color;
	ColorBtn m_colorBtn;
};

static UINT CALLBACK HookProc(
  HWND hWnd,      // handle to the dialog box window
  UINT uiMsg,     // message identifier
  WPARAM wParam,  // message parameter
  LPARAM lParam   // message parameter
)
{
	if (uiMsg == WM_INITDIALOG) {
		myFontDlg* pDlg = (myFontDlg*)CWnd::FromHandlePermanent(hWnd);
		if (!pDlg) {
			TRACE("Can't get font dialog\n");
			return 1;
		}
		pDlg->m_colorBtn.Init(IDC_COLOR, pDlg, &pDlg->m_color);
	}
	if (uiMsg != WM_COMMAND)
		return 0;
	if (LOWORD(wParam) == IDC_COLOR) {
		myFontDlg* pDlg = (myFontDlg*)CWnd::FromHandlePermanent(hWnd);
		if (!pDlg) {
			TRACE("Can't get font dialog\n");
			return 1;
		}
		COLORREF clr = pDlg->GetColor();
		CColorDialog dlg(clr);
		if (dlg.DoModal() == IDOK) {
			clr = dlg.GetColor();
			pDlg->m_cf.rgbColors = pDlg->m_color = clr;
			SendMessage(hWnd, WM_CHOOSEFONT_SETFLAGS, 0, (LPARAM)&pDlg->m_cf);
		}
		return 1;
	} else if (LOWORD(wParam) == IDOK) {
		myFontDlg* pDlg = (myFontDlg*)CWnd::FromHandlePermanent(hWnd);
		if (pDlg && pDlg->m_colorBtn.IsChanged())
			pDlg->m_colorBtn.Apply();
	}
	return 0;
}

void FontBtn::ChooseFont()
{
	CFont* pFont = m_pFont ? m_pFont : m_pFontOrg;
	LOGFONT lf;
	pFont->GetLogFont(&lf);
	myFontDlg dlg(&lf,
		//CF_EFFECTS|
		CF_FORCEFONTEXIST|CF_INITTOLOGFONTSTRUCT|
		CF_NOVERTFONTS|CF_SCREENFONTS|CF_ENABLETEMPLATE|CF_ENABLEHOOK,
		0, this);
	dlg.m_cf.rgbColors = dlg.m_color = m_color;
	dlg.m_cf.lpTemplateName = (LPTSTR)MAKEINTRESOURCE(IDD_FONT_DLG);
	dlg.m_cf.lpfnHook = (LPCFHOOKPROC)HookProc;
	if (dlg.DoModal() != IDOK)
		return;
	if (m_pFont)
		delete m_pFont;
	m_pFont = new CFont;
	m_pFont->CreateFontIndirect(&lf);
	//m_color = dlg.GetColor();
	m_color = dlg.m_color;
	m_fChanged = TRUE;
	Invalidate();
}

void FontBtn::SetBkColor(COLORREF cr)
{
	m_fChanged = TRUE;
	m_crBack = cr;
	if (m_color == m_crBack)
		m_color = (m_crBack ^ RGB(255, 255, 255));
	Invalidate();
}

void FontBtn::Apply()
{
	*m_pColorOrg = m_color;
	if (!m_pFont)
		return;
	LOGFONT lf;
	m_pFont->GetLogFont(&lf);
	m_pFontOrg->DeleteObject();
	m_pFontOrg->CreateFontIndirect(&lf);
}

BEGIN_MESSAGE_MAP(FontBtn, CButton)
	//{{AFX_MSG_MAP(FontBtn)
	ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// FontBtn bZ[W nh

void FontBtn::DrawItem(LPDRAWITEMSTRUCT lpDis)
{
	UINT itemState = lpDis->itemState;
	UINT nDfcs = (itemState & ODS_SELECTED) ? DFCS_PUSHED : 0;
	CDC* pDC = CDC::FromHandle(lpDis->hDC);
	pDC->DrawFrameControl(&lpDis->rcItem, DFC_BUTTON,
		DFCS_BUTTONPUSH | nDfcs);

	COLORREF crTextSave = pDC->SetTextColor(m_color);
	COLORREF crBackSave = pDC->SetBkColor(m_crBack);

	CRect rc(lpDis->rcItem);
	rc.DeflateRect(3, 3);
	pDC->FillSolidRect(&rc, m_crBack);
	if (itemState & ODS_FOCUS)
		pDC->DrawFocusRect(&rc);

	CString sText;
	GetWindowText(sText);
	CFont* pFont = m_pFont ? m_pFont : m_pFontOrg;
	CFont* pFontSave = (CFont*)pDC->SelectObject(pFont);
	pDC->DrawText(sText, &lpDis->rcItem,
			DT_CENTER|DT_NOPREFIX|DT_VCENTER|DT_SINGLELINE);
	pDC->SetTextColor(crTextSave);
	pDC->SetBkColor(crBackSave);
	pDC->SelectObject(pFontSave);
}

void FontBtn::OnClicked() 
{
	ChooseFont();
}

// ColorBtn

ColorBtn::ColorBtn()
{
}

ColorBtn::~ColorBtn()
{
}

void ColorBtn::Init(int idc, CWnd* pParent, COLORREF* pColor)
{
	m_fChanged = FALSE;
	m_pColorOrg = pColor;
	m_color = *pColor;
	SubclassDlgItem(idc, m_pParent = pParent);
}

void ColorBtn::Apply()
{
	*m_pColorOrg = m_color;
}

void ColorBtn::DoSelect(void)
{
	CColorDialog dlg(m_color, 0, this);
	if (dlg.DoModal() != IDOK)
		return;
	COLORREF cr = dlg.GetColor();
	if (cr != m_color) {
		m_color = cr;
		m_fChanged = TRUE;
		m_pParent->PostMessage(WMU_BKCOLOR_CHANGE, (WPARAM)cr);
		Invalidate();
	}
}

BEGIN_MESSAGE_MAP(ColorBtn, CButton)
	//{{AFX_MSG_MAP(ColorBtn)
	ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// ColorBtn bZ[W nh

void ColorBtn::DrawItem(LPDRAWITEMSTRUCT lpDis) 
{
	UINT itemState = lpDis->itemState;
	UINT nDfcs = (itemState & ODS_SELECTED) ? DFCS_PUSHED : 0;
	CDC* pDC = CDC::FromHandle(lpDis->hDC);
	pDC->DrawFrameControl(&lpDis->rcItem, DFC_BUTTON,
		DFCS_BUTTONPUSH | nDfcs);

	COLORREF crBackSave = pDC->SetBkColor(m_color);
	COLORREF crTextSave = pDC->SetTextColor(m_color ^ RGB(255,255,255));

	CRect rc(lpDis->rcItem);
	rc.DeflateRect(3, 3);
	pDC->FillSolidRect(&rc, m_color);
	if (itemState & ODS_FOCUS)
		pDC->DrawFocusRect(&rc);

	CString sText;
	GetWindowText(sText);
	pDC->DrawText(sText, &lpDis->rcItem,
			DT_CENTER|DT_VCENTER|DT_SINGLELINE);
	pDC->SetTextColor(crTextSave);
	pDC->SetBkColor(crBackSave);
}

void ColorBtn::OnClicked() 
{
	DoSelect();
}
