// MainFrm.cpp : implementation of the CFrame class

#include "stdafx.h"
#include "mchat.h"
#include "text.h"
#include "input.h"
#include "dlgjoin.h"
#include "dlgtalk.h"
#include "dlgmisc.h"
#include "wndlist.h"
#include "dlgnick.h"
#include "DlgQuit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

inline int GetCurrentDay() { return CTime::GetCurrentTime().GetDay(); }

CFrame* g_pFrame;	// main frame

BOOL g_fAvoidPopup = FALSE;	// tempolary avoid restoring windows

// defer message process

UINT g_tmidDefer = 0;

class WndMsg
{
public:
	WndMsg() {
		m_pWnd = 0;
		m_nMsg = 0;
	}
	WndMsg(CWnd* pWnd, UINT nMsg) {
		m_pWnd = pWnd;
		m_nMsg = nMsg;
	}

	BOOL operator==(WndMsg& wm) {
		return m_pWnd == wm.m_pWnd && m_nMsg == m_nMsg;
	}

	CWnd* m_pWnd;
	UINT m_nMsg;
};

CList<WndMsg, WndMsg&> g_lsWndMsg;

BOOL DeferPostMessage(CWnd* pWnd, UINT nMsg)
{
	WndMsg wmNew(pWnd, nMsg);

	POSITION pos = g_lsWndMsg.GetHeadPosition();
	while (pos) {
		WndMsg& wm = g_lsWndMsg.GetNext(pos);
		if (wm == wmNew)
			return TRUE;
	}
	g_lsWndMsg.AddTail(wmNew);
	if (!g_tmidDefer)
		g_tmidDefer = g_pFrame->SetTimer(TMID_DEFERMSG, 2000, NULL);
	return g_tmidDefer != 0;
}

void DeferDeliver()
{
	if (g_lsWndMsg.IsEmpty()) {
		g_pFrame->KillTimer(g_tmidDefer);
		g_tmidDefer = 0;
		return;
	}
	WndMsg& wm = g_lsWndMsg.GetHead();
	if (IsWindow(wm.m_pWnd->GetSafeHwnd()))
		wm.m_pWnd->PostMessage(wm.m_nMsg);
	g_lsWndMsg.RemoveHead();
}

void DeferRemove(CWnd* pWnd)
{
	POSITION pos = g_lsWndMsg.GetHeadPosition();
	while (pos) {
		POSITION pos1 = pos;
		WndMsg& wm = g_lsWndMsg.GetNext(pos);
		if (wm.m_pWnd == pWnd) {
			g_lsWndMsg.RemoveAt(pos1);
			return;
		}
	}
}

// Manipurate Frames

Frames g_frames;

CFrame* GetFrame(LPCTSTR pszTarget)
{
	CString sTarget(pszTarget);
	sTarget.MakeLower();

	CFrame* pFrame;
	if (sTarget.IsEmpty() ||
			!g_frames.Lookup(sTarget, pFrame)) {
		return g_pFrame;
	}
	return pFrame;
}

void Frames::AddFrame(CFrame* pFrame)
{
	CString sKey = pFrame->m_sTarget;
	sKey.MakeLower();
	if (pFrame->GetType() == CFrame::TypeChat) {
		int n = 0;
		CFrame* p;
		do {
			sKey.Format("%s-%d", pFrame->m_sTarget, n++);
		} while (Lookup(sKey, p));
	}
	pFrame->m_sKey = sKey;
	SetAt(sKey, pFrame);
}

void Frames::DelFrame(CFrame* pFrame)
{
	CFrame* pFrame1;
	if (!Lookup(pFrame->m_sKey, pFrame1))
		TRACE("DelFrame: not found %s\n", pFrame->m_sKey);
	else if (pFrame != pFrame1)
		TRACE("DelFrame: mismatch %s\n", pFrame->m_sKey);
	RemoveKey(pFrame->m_sKey);
	DeferRemove(pFrame);

	if (GetCount() <= 0 && IsWindow(g_pFrame->GetSafeHwnd()) && g_pFrame->IsIconic())
		g_pFrame->ShowWindow(SW_RESTORE);
}

void Frames::ChangeKey(CFrame* pFrame)
{
	DelFrame(pFrame);
	AddFrame(pFrame);
}

void Frames::ClearFrames()
{
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		pFrame->DestroyWindow();
	}
	RemoveAll();
}

// user message
void Frames::PostMsgChan(UINT msg, WPARAM wParam, LPARAM lParam)
{
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->GetType() == CFrame::TypeChan)
			pFrame->PostMessage(msg, wParam, lParam);
	}
}

// irc messages
void Frames::MsgNick(LPCTSTR pNick, LPCTSTR pNickTo)
{
	CString sKey;
	CFrame* pFrame;
	BOOL f = FALSE;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->HasNick(pNick)) {
			pFrame->MsgNick(pNick, pNickTo);
			f = TRUE;
		}
	}
	if (!f)
		g_pFrame->MsgNick(pNick, pNickTo);
}

void Frames::MsgQuit(LPCTSTR pNick, LPCTSTR pTrail)
{
	CString sKey;
	CFrame* pFrame;
	BOOL f = FALSE;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->HasNick(pNick)) {
			pFrame->MsgQuit(pNick, pTrail);
			f = TRUE;
		}
	}
	if (!f)
		g_pFrame->MsgQuit(pNick, pTrail);
}

void Frames::MsgDisconnect()
{
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->GetType() == CFrame::TypeChat)
			continue;
		pFrame->MsgDisconnect();
	}
	g_pFrame->MsgDisconnect();
}

void Frames::MsgReply(LPCTSTR pNick, LPCTSTR pMsg)
{
	if (g_opt.fMsgReplyToMain) {
		g_pFrame->LineNotice(pNick, pMsg);
		return;
	}
	BOOL f = FALSE;
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->HasNick(pNick)) {
			pFrame->LineNotice(pNick, pMsg);
			f = TRUE;
		}
	}
	if (!f)
		g_pFrame->LineNotice(pNick, pMsg);
}

void Frames::BroadCast(LPCTSTR pStr)
{
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		pFrame->SendData(pStr);
	}
}

void Frames::BroadCastMe(LPCTSTR pStr)
{
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (!pFrame->m_sTarget.IsEmpty()) {
			CmdMe(pFrame->m_sTarget, pStr);
			pFrame->LineNotice(g_con.m_sNick, pStr);
		}
	}
}

void Frames::RplWho(LPCTSTR pChan, CUser& user)
{
	BOOL f = FALSE;
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->GetType() == CFrame::TypeChan) {
			if (pFrame->m_sTarget.CompareNoCase(pChan) == 0) {
				pFrame->RplWho(pChan, user);
				f = TRUE;
			}
		} else if (pFrame->HasNick(user.m_sNick)) {
			pFrame->RplWho(pChan, user);
			f = TRUE;
		}
	}
	if (!f)
		g_pFrame->RplWho(pChan, user);
}

void Frames::Rejoin()
{
	POSITION pos = GetStartPosition();
	while (pos) {
		CString sKey;
		CFrame* pFrame;
		GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->GetType() == CFrame::TypeChan)
			CmdJoin(pFrame->m_sTarget);
	}
}

void Frames::MakeAllTitle()
{
	// update all frames' title
	g_pFrame->MakeTitle();
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		pFrame->MakeTitle();
	}
}

void Frames::FontChange()
{
	g_pFrame->FontChange();
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		pFrame->FontChange();
	}
}

void Frames::SelectFrame()
{
	if (WndList::g_pWndList) {
		WndList::g_pWndList->SetFocus();
		return;
	}
	CFrame* pFrame = (CFrame*)CWnd::GetFocus()->GetParentFrame();
	new WndList(pFrame);
}

void Frames::MinimizeAll()
{
	if (!g_pFrame->IsIconic())
		g_pFrame->ShowWindow(SW_MINIMIZE);
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		if (!pFrame->IsIconic())
			pFrame->ShowWindow(SW_MINIMIZE);
	}
}

void Frames::CloseAll()
{
	CString sKey;
	CFrame* pFrame;
	POSITION pos = GetStartPosition();
	while (pos) {
		GetNextAssoc(pos, sKey, pFrame);
		pFrame->PostMessage(WM_CLOSE);
	}
}

#if 0
void Frames::Tick()
{
	CPoint pt;
	if (!GetCursorPos(&pt))
		return;
	HWND hWndCursor =
		CWnd::WindowFromPoint(pt)->GetParentOwner()->GetSafeHwnd();
	HWND hWndActive =
		CWnd::GetActiveWindow()->GetParentOwner()->GetSafeHwnd();

	CString sKey;
	POSITION pos = GetStartPosition();
	for (;;) {
		CFrame* pFrame;
		if (!pos)
			pFrame = g_pFrame;
		else
			GetNextAssoc(pos, sKey, pFrame);
		if (pFrame->m_fAutoSimple &&
				!pFrame->m_fSimple &&
				!pFrame->IsIconic() &&
				pFrame->m_hWnd != hWndActive &&
				pFrame->m_hWnd != hWndCursor) {
			TRACE("SimpleCount %d: %s\n", pFrame->m_nSimpleCount,
				pFrame->GetFrameTitle());
			if (++pFrame->m_nSimpleCount >= 10) {
				pFrame->Simple(TRUE);
				pFrame->m_nSimpleCount = 0;
			}
		} else
			pFrame->m_nSimpleCount = 0;
		if (pFrame == g_pFrame)
			break;
	}
}
#endif

#if 0
// map for flashing frame

UINT g_tmidFlash = 0;

MapFrameFlash mapFrameFlash;

void MapFrameFlash::Start(CFrame* pFrame)
{
	TRACE("Flash start: %s\n", pFrame->GetFrameName());

	SetAt(pFrame, 1);
	pFrame->m_fFlashing = TRUE;
	if (!g_opt.fContinuousFlash) {
		pFrame->FlashWindow(FALSE);
		pFrame->FlashWindow(TRUE);
		return;
	}
	if (!g_tmidFlash) {
		g_tmidFlash = g_pFrame->SetTimer(TMID_FLASH, 1000, NULL);
		ASSERT(g_tmidFlash);
	}
}

void MapFrameFlash::Stop(CFrame* pFrame)
{
	TRACE("Flash remove: %s\n", pFrame->GetFrameName());

	RemoveKey(pFrame);
	pFrame->m_fFlashing = FALSE;
	pFrame->FlashWindow(FALSE);
	if (IsEmpty() && g_tmidFlash) {
		if (IsWindow(g_pFrame->GetSafeHwnd()))
			g_pFrame->KillTimer(g_tmidFlash);
		g_tmidFlash = 0;
	}
}

void MapFrameFlash::Flash()
{
	POSITION pos = GetStartPosition();
	while (pos) {
		CFrame* pFrame;
		int n;
		GetNextAssoc(pos, pFrame, n);
		pFrame->FlashWindow(TRUE);

		TRACE("Flash: %s\n", pFrame->GetFrameName());
	}
}
#endif

// Registry
TCHAR BASED_CODE szWindow[] = _T("Window");
// Window\
//    main: left,top,right,bottom
//    talk: (same)
//    chat: (same)
//    chan: (same)
//    chan-#channel: left,top,right,bottom,text-percentage
static char szFmtWindowRect[] = "%d,%d,%d,%d";

// CFrame

BEGIN_MESSAGE_MAP(CFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CFrame)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_WM_DESTROY()
	ON_COMMAND(ID_JOIN, OnJoin)
	ON_COMMAND(ID_VK_NEXT, OnVkNext)
	ON_COMMAND(ID_VK_PRIOR, OnVkPrior)
	ON_COMMAND(ID_TALKWITH, OnTalkwith)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_VERSION, OnVersion)
	ON_COMMAND(ID_PING, OnPing)
	ON_COMMAND(ID_USERINFO, OnUserinfo)
	ON_COMMAND(ID_FINGER, OnFinger)
	ON_COMMAND(ID_CLIENTINFO, OnClientinfo)
	ON_COMMAND(ID_RESTOREMINIMIZED, OnRestoreMinimized)
	ON_UPDATE_COMMAND_UI(ID_RESTOREMINIMIZED, OnUpdateRestoreMinimized)
	ON_UPDATE_COMMAND_UI(ID_LOGGING, OnUpdateLogging)
	ON_COMMAND(ID_LOGGING, OnLogging)
	ON_MESSAGE(WMU_SOCK_HOST, OnSockHost)
	ON_MESSAGE(WMU_SOCK_ADDR, OnSockAddr)
	ON_COMMAND(ID_CLOSE, OnCmdClose)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_IGNORE, OnIgnore)
	ON_COMMAND(ID_WHOIS, OnWhois)
	ON_WM_TIMER()
	ON_COMMAND(ID_CTCP_TIME, OnCtcpTime)
	ON_WM_ACTIVATE()
	ON_COMMAND(ID_DCC_CHAT, OnDccChat)
	ON_COMMAND(ID_DCC_SEND, OnDccSend)
	ON_COMMAND(ID_VK_LINEUP, OnVkLineup)
	ON_COMMAND(ID_VK_LINEDOWN, OnVkLinedown)
	ON_COMMAND(ID_VK_TOP, OnVkTop)
	ON_COMMAND(ID_VK_BOTTOM, OnVkBottom)
	ON_COMMAND(ID_CMD_AWAY, OnCmdAway)
	ON_UPDATE_COMMAND_UI(ID_CMD_AWAY, OnUpdateCmdAway)
	ON_WM_DROPFILES()
	ON_COMMAND(ID_NICK, OnNick)
	ON_COMMAND(ID_QUIT_MSG, OnQuitMsg)
	ON_UPDATE_COMMAND_UI(ID_QUIT_MSG, OnUpdateQuitMsg)
	ON_COMMAND(ID_HELP_HOME, OnHelpHome)
	ON_COMMAND(ID_HELP_BBS, OnHelpBbs)
	ON_COMMAND(ID_HELP_TUTORIAL, OnHelpTutorial)
	ON_WM_MOUSEWHEEL()
	ON_COMMAND(ID_ME, OnMe)
	ON_COMMAND(ID_AVOID_POPUP, OnAvoidPopup)
	ON_UPDATE_COMMAND_UI(ID_AVOID_POPUP, OnUpdateAvoidPopup)
	ON_COMMAND(ID_ALWAYS_ON_TOP, OnAlwaysOnTop)
	ON_UPDATE_COMMAND_UI(ID_ALWAYS_ON_TOP, OnUpdateAlwaysOnTop)
	ON_COMMAND(ID_VIEW_LOG, OnViewLog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LOG, OnUpdateViewLog)
	ON_MESSAGE(WMU_KEYWORD, OnKeyword)
	ON_WM_PAINT()
	ON_COMMAND(ID_VIEW_ISON, OnViewIson)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ISON, OnUpdateViewIson)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_HELP_FINDER, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpFinder)
	//ON_MESSAGE(WM_COMMANDHELP, OnCommandHelp)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	//ID_INDICATOR_SCRL,
};

// CFrame construction/destruction

IMPLEMENT_DYNCREATE(CFrame, CFrameWnd)

CFrame::CFrame(int nType, int nResourceID, LPCTSTR pszFrameName, LPCTSTR pszTarget)
{
	m_nType = nType;
	m_nResourceID = nResourceID;
	m_sFrameName = pszFrameName;
	m_sTarget = pszTarget;

	m_fRestoreMinimized = g_opt.fRestoreMinimized;
	//m_fFlashing = FALSE;
	m_fRectCalc = FALSE;
	m_fAlwaysOnTop = FALSE;
	m_hndLogFile = INVALID_HANDLE_VALUE;
	m_nLastDate = GetCurrentDay();

	m_pText = 0;
	m_pInput = 0;
}

CFrame::~CFrame()
{
	if (m_pText)
		delete m_pText;
	if (m_pInput)
		delete m_pInput;
}

int CFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (!m_wndToolBar.Create(this) ||
			!m_wndToolBar.LoadToolBar(GetResourceID())) {
		TRACE0("Failed to create toolbar\n");
		return -1;// fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
			!m_wndStatusBar.SetIndicators(indicators,
				sizeof(indicators)/sizeof(UINT))) {
		TRACE0("Failed to create status bar\n");
		return -1;// fail to create
	}

	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_wndToolBar.SetWindowText(m_sFrameName);
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);

	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	CString s;
	s.Format("%s\\%s", szWindow, m_sFrameName);
	CDockState state;
	state.LoadState(s);
	SetDockState(state);

	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	MakeTitle();
	if (m_nType != TypeMain)
		g_frames.AddFrame(this);
	LogOpen();

	if (m_nType == TypeTalk || m_nType == TypeChat)
		DragAcceptFiles(TRUE);

	if (g_opt.fMinimizeMain && m_nType != TypeMain && !g_pFrame->IsIconic())
		g_pFrame->PostMessage(WM_SYSCOMMAND, SC_MINIMIZE);

	InputSetFocus();

	return 0;
}

BOOL CFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	if (!m_pText) {
		m_pText = new CText;
		if (!m_pText->DoCreate(this, AFX_IDW_PANE_FIRST))
			return FALSE;
	}
	m_pText->SetWindowContextHelpId(IDC_TEXTAREA);
	m_pText->FontChange();

	ASSERT(!m_pInput);
	m_pInput = new CInput;
	if (!m_pInput->DoCreate(this, IDW_INPUT)) {
		TRACE("can't create input window\n");
		return FALSE;
	}
	m_pInput->SetWindowContextHelpId(IDW_INPUT);
	m_pInput->FontChange();

	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

void CFrame::FontChange()
{
	m_pText->FontChange();
	m_pInput->FontChange();
	RecalcLayout();
}

// log

static CString MakeProperFilename(LPCTSTR pStr)
{
	CString sFile;
	char* pBuf = sFile.GetBuffer(_MAX_PATH);
	int iBuf = 0;
	char c;
	while ((c = *pStr++) != 0 && iBuf < 128) { // 128͓K
		if (_ismbblead(c)) {
			if (*pStr) {
				pBuf[iBuf++] = c;
				pBuf[iBuf++] = *pStr++;
				continue;
			}
		}
		if (strchr("<>:\"/\\|.*", c)) {
			if (c != ':')
				continue;
			// #chan:*.jp -> #chan-jp
			while (*pStr && strchr("<>:\"/\\|.*", *pStr))
				pStr++;
			c = '-';
		}
		pBuf[iBuf++] = c;
	}
	sFile.ReleaseBuffer(iBuf);
	if (g_opt.fLogDate) {
		CTime time = CTime::GetCurrentTime();
		CString sDate = time.Format(g_opt.sLogDateFormat);
		sFile += sDate;
	}
	return sFile + g_opt.sLogExt;
}

static CString NextFilename(LPCTSTR pPath, int nCount)
{
	CString s(pPath);
	int n = s.ReverseFind('.');
	if (n < 0)
		n = s.GetLength();

	CString s1;
	s1.Format("-%d", nCount);

	s.Insert(n, s1);
	return s;
}

void CFrame::LogOpen(BOOL fForce /*=FALSE*/)
{
	switch (m_nType) {
	case TypeChan:
		if (!fForce && !g_opt.fLogChan)
			return;
		m_sLogFile = g_opt.sDirChan;
		DirExtend(m_sLogFile, MakeProperFilename(m_sTarget));
		break;
	case TypeTalk:
	case TypeChat:
		if (!fForce && !g_opt.fLogTalk)
			return;
		m_sLogFile = g_opt.sDirTalk;
		DirExtend(m_sLogFile, MakeProperFilename(m_sKey));
		// use m_sKey to avoid filename conflict
		break;
	default:
		if (!fForce && !g_opt.fLogMain)
			return;
		m_sLogFile = g_opt.sFileMain;
	}

	DWORD dwFlag = g_opt.fAppend ? OPEN_ALWAYS : CREATE_ALWAYS;

	CString sFile(m_sLogFile);
	int nRetry = 0;

	while ((m_hndLogFile = CreateFile(m_sLogFile,
			GENERIC_WRITE, FILE_SHARE_READ, 0, dwFlag,
			FILE_ATTRIBUTE_NORMAL, 0)) == INVALID_HANDLE_VALUE) {
		CString s;
		s.Format(IDS_MSG_ERR_LOGOPEN, m_sLogFile);
		m_pText->ShowLine(NULL, s, Fonts::TypeError);

		if (++nRetry > 9)
			return;

		m_sLogFile = NextFilename(sFile, nRetry);
	}

	if (g_opt.fAppend)
		SetFilePointer(m_hndLogFile, 0, 0, FILE_END);

	LogStamp(IDS_FMT_LOGSTART);

	CString s;
	s.Format(IDS_MSG_LOG_OPENNED, m_sLogFile);
	m_pText->ShowLine(NULL, s, Fonts::TypeStat);
}

void CFrame::LogClose()
{
	if (m_hndLogFile != INVALID_HANDLE_VALUE) {
		LogStamp(IDS_FMT_LOGSTOP);
		CloseHandle(m_hndLogFile);
		m_hndLogFile = INVALID_HANDLE_VALUE;
	}
}

void CFrame::LogWrite(const CString& sStr)
{
	DWORD dwWrote;
	CString s(
		CTime::GetCurrentTime().Format(g_opt.sMsgTime)
		+ " " + sStr + "\r\n");
	if (!WriteFile(m_hndLogFile, s, s.GetLength(), &dwWrote, NULL)) {
		CString s;
		s.Format(IDS_MSG_ERR_LOGWRITE, m_sLogFile);
		AfxMessageBox(s, MB_OK | MB_ICONEXCLAMATION);
		LogClose();
		return;
	}
	FlushFileBuffers(m_hndLogFile);
}

void CFrame::LogStamp(int id)
{
	CString s(CTime::GetCurrentTime().Format(id) + "\r\n");
	DWORD dwWrote;
	WriteFile(m_hndLogFile, s, s.GetLength(), &dwWrote, NULL);
}

// title

void CFrame::MakeTitle()
{
	LPCTSTR p;
	switch (m_nType) {
	case TypeChan:
		p = g_opt.sChan;
		break;
	case TypeTalk:
		p = g_opt.sTalk;
		break;
	case TypeChat:
		p = g_opt.sChat;
		break;
	default:
		p = g_opt.sMain;
	}

	CString s;
	char c;
	while ((c = *p++) != 0) {
		if (c != '%') {
			s += c;
			continue;
		}
		if ((c = *p++) == 0)
			break;
		switch (c) {
		case 'S':
			s += g_sd.m_sHostname;
			break;
		case 'P':
			{
				char buf[20];
				itoa(g_sd.m_nPort, buf, 10);
				s += buf;
			}
			break;
		case 'T':
			s += m_sTarget;
			break;
		case 'N':
			s += g_con.m_sNick;
			break;
		case 't':
			s += GetTopic();
			break;
		case '@':
			if (IsOpMe())
				s += '@';
			break;
		case 'M':
			s += GetModeString();
			break;
		default:
			s += c;
		}
	}
	SetWindowText(m_sTitle = s);

#if 0
	if (m_pText)
		m_pText->NewTitle(m_sTitle);
#endif
}

// CFrame diagnostics

#ifdef _DEBUG
void CFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
	dc << "idRes=" << m_nType
		<< "frame=" << m_sFrameName
		<< "target=" << m_sTarget;
}

#endif //_DEBUG

void CFrame::ShowStatus(int id)
{
	CString s;
	s.LoadString(id);
	ShowStatus(s);
}

void CFrame::ShowLine(LPCTSTR pNick, LPCTSTR pStr, Fonts::FType type, BOOL fInClient)
{
	CString s;
	if (pNick)
		s = pNick;
	s += pStr;

	if (fInClient)
		m_pText->ShowLine(pNick, pStr, type);
	else
		SetMessageText(s);
	if (m_hndLogFile != INVALID_HANDLE_VALUE) {
		if (g_opt.fLogNewDate) {
			int nDay = GetCurrentDay();
			if (m_nLastDate != nDay) {
				LogClose();
				m_nLastDate = nDay;
				LogOpen();
			}
		}
		LogWrite(s);
	}
}

// restore frame by events
void CFrame::RestoreByEvent()
{
	CWnd* pWnd = GetFocus();
	ShowWindow(SW_RESTORE);
	if (pWnd)
		pWnd->SetFocus();
}

// notify somebody's message to minimized/in-active frame
void CFrame::NotifyMessage()
{
#if 0
	if (g_opt.fForegroundMsg && !m_fActivate) {
		// bring to top without activate
		TempForeground(m_fSimple ? m_pText->GetParent() : this);
	}
#endif

	if (!g_fAvoidPopup && m_fRestoreMinimized && IsIconic())
		RestoreByEvent();

	//if (!m_fFlashing &&
	if (g_opt.fFlashMinimized && IsIconic() ||
			g_opt.fFlashInactive && (CFrame*)CWnd::GetActiveWindow() != this) {
		FlashWindow(FALSE);
		FlashWindow(TRUE);
		//@ mapFrameFlash.Start(this);
	}
}

// show normal message text, always notify by option
void CFrame::LineText(LPCTSTR pNick, LPCTSTR pStr)
{
	CString s;
	if (pNick)
		s.Format(g_opt.sMsgNick, pNick);
	ShowLine(s, pStr, Fonts::TypeText);
	NotifyMessage();
}

// show notice messages, do not notify
void CFrame::LineNotice(LPCTSTR pNick, LPCTSTR pStr,
						BOOL fInClient /*=TRUE*/)
{
	if (pNick) {
		CString s;
		s.Format("*%s %s", pNick, pStr);
		ShowLine(NULL, s, Fonts::TypeNotice, fInClient);
	} else
		ShowLine(NULL, pStr, Fonts::TypeNotice, fInClient);
}

void CFrame::LineError(int id)
{
	CString s;
	s.LoadString(id);
	LineError(s);
}

void CFrame::LineStat(int id, BOOL fInClient)
{
	CString s;
	s.LoadString(id);
	LineStat(s, fInClient);
}

void CFrame::LineRaw()
{
	ShowLine(NULL, sock.m_aLine, Fonts::TypeText);
}

// Input area

void CFrame::InputSetFocus()
{
	if (IsWindow(m_pInput->GetSafeHwnd())) {
		//if (m_fSimple)
		//	Simple(FALSE);
		m_pInput->SetFocus();
	}
}

BOOL CFrame::InputText(CString& s)
{
	if (s.IsEmpty()) {
		if (g_opt.fMinimizeByReturnKey)
			ShowWindow(SW_MINIMIZE);
		else
			MessageBeep(MB_ICONHAND);
		return TRUE;
	}
	if (!s.IsEmpty() && s[0] == '/') {
		if ((sock.GetStatus() & SOCK_CONNECT) == 0)
			return FALSE;
		IrcCommand((LPCTSTR)s + 1);
	} else
		return SendData(s);
	return TRUE;
}

void CFrame::KeyTab()
{
	if (!IsKeyCtrl())
		return;
	g_pFrame->PostMessage(WM_COMMAND, ID_WINDOWS);
}

BOOL CFrame::SendData(LPCTSTR pStr)
{
	IrcCommand(pStr);
	return TRUE;
}

void CFrame::IrcCommand(LPCTSTR pLine)
{
	LPCTSTR p = pLine;
	CString sCmd(Token(p));
	sCmd.MakeUpper();
	if (sCmd == "CTCP") {
		CString sTo(Token(p));
		if (sTo.IsEmpty()) {
			LineError("$ /ctcp <target> <parameters..>");
			return;
		}
		CString s;
		s.Format("PRIVMSG %s :\001%s\001", sTo, p);
		Cmd(s);
		return;
	}
	if (sCmd == "MSG" || sCmd == "PRIV") {
		CString sTo(Token(p));
		CString s;
		if (sTo.IsEmpty()) {
			s.Format("$ %s <target> <message..>", sCmd);
			LineError(s);
			return;
		}
		s.Format("PRIVMSG %s :%s", sTo, p);
		Cmd(s);
		return;
	}
	if (sCmd == "NOTICE" || sCmd == "N") {
		CString sTo(Token(p));
		CString s;
		if (sTo.IsEmpty()) {
			s.Format("$ %s <target> <message..>", sCmd);
			LineError(s);
			return;
		}
		s.Format("NOTICE %s :%s", sTo, p);
		Cmd(s);
		s.Format("notice to %s: %s", sTo, p);
		LineNotice(g_con.m_sNick, (LPCTSTR)s);
		return;
	}
	if (sCmd == "ME") {
		if (m_sTarget.IsEmpty()) {
			LineError("$ use /me on channel or talk window");
			return;
		}
		CmdMe(m_sTarget, p);
		LineNotice(g_con.m_sNick, p);
		return;
	}
	if (sCmd == "TOPIC") {
		CmdTopic(p);
		return;
	}
	if (sCmd == "WHO") {
		if (!*p && m_nType != TypeMain)
			Cmd("WHO", m_sTarget);
		else
			Cmd(pLine);
		return;
	}
	if (sCmd == "CHAT" || sCmd == "C") {
		CString sTo(Token(p));
		if (sTo.IsEmpty()) {
			LineError("$ /chat <nick>");
			return;
		}
		CmdDccChat(sTo);
		return;
	}
	if (sCmd == "TALK" || sCmd == "T") {
		CString sTo(Token(p));
		if (sTo.IsEmpty()) {
			LineError("$ /talk <nick>");
			return;
		}
		CmdTalk(sTo);
		return;
	}
	if (sCmd == "SEND" || sCmd == "S") {
		CString sTo(Token(p));
		if (sTo.IsEmpty()) {
			LineError("$ /send <nick> [file]");
			return;
		}
		CString sFile(Token(p));
		if (sFile.IsEmpty())
			CmdDccSend(sTo);
		else if (!IsFileReadable(sFile)) {
			CString s;
			s.Format(IDS_FMT_OPENFILE_ERR, sFile);
			LineError(s);
		} else
			CmdDccSend(sTo, sFile);
		return;
	}
	if (sCmd == "KICK" || sCmd == "K") {
		if (*p == '#' || *p == '&') {
			CString sChan(Token(p));
			CString sNick(Token(p));
			CmdTrail(sCmd, sChan, sNick, p);
		} else if (m_nType != TypeChan) {
			CString s;
			s.Format(IDS_FMT_CMD_CHAN, sCmd);
			LineError(s);
		} else if (*p == 0) {
			CString s((LPCSTR)IDS_MSG_USAGE_KICK);
			LineError(s);
		} else {
			CString sNick(Token(p));
			CmdTrail(sCmd, m_sTarget, sNick, p);
		}
		return;
	}
	if (sCmd == "INVITE" || sCmd == "I") {
		CString sNick(Token(p));
		CString sChan(Token(p));
		if (!sChan.IsEmpty())
			CmdInvite(sNick, sChan);
		else if (sNick.IsEmpty()) {
			CString s((LPCSTR)IDS_MSG_USAGE_INVITE);
			LineError(s);
		} else if (m_nType != TypeChan) {
			CString s;
			s.Format(IDS_FMT_CMD_CHAN, sCmd);
			LineError(s);
		} else
			CmdInvite(sNick, m_sTarget);
		return;
	}
	if (sCmd == "AWAY" || sCmd == "A") {
		CmdAway(p);
		return;
	}
	if (sCmd == "QUIT" || sCmd == "Q") {
		CmdQuit(*p ? p : 0);
		return;
	}
	if (sCmd == "BROADCAST" || sCmd == "BROAD" || sCmd == "B") {
		g_frames.BroadCast(p);
		return;
	}
	if (sCmd == "BM" || sCmd == "BROADCASTME") {
		g_frames.BroadCastMe(p);
		return;
	}
	if (sCmd == "JOIN" || sCmd == "J") {
		Cmd("JOIN", p);
		return;
	}
	Cmd(pLine);
}

LONG CFrame::OnSockHost(UINT wParam, LONG lParam)
{
	sock.ContinueConnect(wParam, lParam);
	return 0;
}

LONG CFrame::OnSockAddr(UINT wParam, LONG lParam)
{
	if (sock.GotMyHostname(wParam, lParam))
		g_opt.sHostName = sock.m_sHostname;
	return 0;
}

void CFrame::MenuPopup(const CPoint pt)
{
	CMenu* pMenu = GetMenu();
	int nMenu = 0;
	CMenu menuPop;
	menuPop.CreatePopupMenu();
	UINT uStat;
	while ((uStat = pMenu->GetMenuState(nMenu, MF_BYPOSITION)) != -1) {
		if (uStat != -1) {
			// Get the menu string.
			CString sItem;
			pMenu->GetMenuString(nMenu, sItem, MF_BYPOSITION);
			if (LOBYTE(uStat) & MF_POPUP) {
				CMenu* pSubMenu = pMenu->GetSubMenu(nMenu);
				menuPop.AppendMenu(LOBYTE(uStat),
					(UINT)pSubMenu->m_hMenu, sItem);
			} else {
				int nID = pMenu->GetMenuItemID(nMenu);
				menuPop.AppendMenu(LOBYTE(uStat), nID, sItem);
			}
			nMenu++;
		}
	}
	menuPop.TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		pt.x, pt.y, this);
	// Because we are using parts of the main menu in our
	// pop-up menu, we can't just delete the pop-up menu, because
	// that would also delete the main menu. So we must
	// go through the pop-up menu and remove all the items.
	while (menuPop.RemoveMenu(0 ,MF_BYPOSITION))
		;
	// Destroy the pop-up menu.
	menuPop.DestroyMenu();
}

// command

void CFrame::CmdTopic(LPCTSTR pStr)
{
	CString sChan(Token(pStr));
	if (sChan.IsEmpty()) {
		LineError("$ use /topic on channel-frame");
		return;
	}
	if (*pStr == 0)
		Cmd("TOPIC", sChan);
	else
		CmdTrail("TOPIC", sChan, pStr);
}

// server msg handling

void CFrame::MsgTopic(const Packet& pac)
{
	LineRaw();
}

void CFrame::MsgJoin(const Packet& pac, LPCTSTR pChan, LPCTSTR pMode)
{
	CString s;
	s.Format("(%s) join %s", pac.sAddr, pChan);
	LineNotice(pac.sNick, s, g_opt.fMsgJoin);
}

void CFrame::MsgPart(const Packet& pac, LPCTSTR pChan)
{
	//LineRaw();
}

void CFrame::MsgKick(LPCTSTR pNick, LPCTSTR pChan, LPCTSTR pParm, LPCTSTR pTrail)
{
	LineRaw();
}

void CFrame::MsgNick(LPCTSTR pNick, LPCTSTR pNickTo)
{
	CString s;
	s.Format("nick %s", pNickTo);
	LineNotice(pNick, s, g_opt.fMsgNick);
}

void CFrame::MsgQuit(LPCTSTR pNick, LPCTSTR pTrail)
{
	CString s;
	s.Format("quit (%s)", pTrail);
	LineNotice(pNick, s, g_opt.fMsgQuit);
}

void CFrame::MsgUserMode(LPCTSTR pNick, LPCTSTR pParm)
{
	CString s;
	s.Format("*%s usermode %s", pNick, pParm);
	LineStat(s, g_opt.fMsgMode);
}

void CFrame::MsgMode(const Packet& pac, LPCTSTR pChan, LPCTSTR pParm)
{
	CString s;
	s.Format("*%s mode %s %s", pac.sNick, pChan, pParm);
	LineStat(s, g_opt.fMsgMode);
}

void CFrame::MsgDisconnect()
{
	CString s;
	s.LoadString(IDS_MSG_DISCONNECT);
	LineStat(s);
	ShowStatus(s);
}

// server reply handling

void CFrame::RplNames(LPCTSTR pChan, LPCTSTR pNames)
{
	CString s;
	s.Format("*Names %s: %s", pChan, pNames);
	LineNotice(NULL, s);
}

void CFrame::RplBanList(LPCTSTR pChan, LPCTSTR pBan)
{
	CString s;
	s.Format("Ban: %s %s", pChan, pBan);
	LineNotice(NULL, s);
}

void CFrame::RplInviteList(LPCTSTR pChan, LPCTSTR pBan)
{
	CString s;
	s.Format("Invite: %s %s", pChan, pBan);
	LineNotice(NULL, s);
}

void CFrame::RplExceptionList(LPCTSTR pChan, LPCTSTR pBan)
{
	CString s;
	s.Format("Exception: %s %s", pChan, pBan);
	LineNotice(NULL, s);
}

void CFrame::RplWho(LPCTSTR pChan, const CUser& user)
{
}

void CFrame::RplWhoEnd()
{
}

void CFrame::RplNoTopic(LPCTSTR pChan)
{
}

void CFrame::RplTopic(LPCTSTR pChan, LPCTSTR pTopic)
{
}

void CFrame::RplChanMode(const Packet& pac, LPCTSTR pChan, LPCTSTR pParm)
{
	CString s;
	s.Format("*%s mode %s %s", pac.sNick, pChan, pParm);
	LineStat(s, g_opt.fMsgMode);
}

// CFrame message handlers

BOOL CFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if (!m_fRectCalc) {
		CString s = theApp.GetProfileString(szWindow, m_sFrameName);
		if (!s.IsEmpty()) {
			CRect rcWork(0, 0, 0, 0);
			::SystemParametersInfo(SPI_GETWORKAREA, 0, &rcWork, 0);
			CRect rc;
			if (sscanf(s, szFmtWindowRect,
					&rc.left, &rc.top,
					&rc.right, &rc.bottom) == 4) {
				if (m_nType == TypeMain) {
					cs.x = rcWork.left + rc.left;
					cs.y = rcWork.top + rc.top;
				}
				cs.cx = rc.Width();
				cs.cy = rc.Height();
			}
		} else {
			cs.cx = 400;
			cs.cy = 250;
		}
		MoveIntoDesktop(cs.x, cs.y, cs.cx, cs.cy);
	}

	return CFrameWnd::PreCreateWindow(cs);
}

void CFrame::OnSetFocus(CWnd* pOldWnd) 
{
	CFrameWnd::OnSetFocus(pOldWnd);

#if 0
	if (m_fFlashing)
		mapFrameFlash.Stop(this);
#endif

	InputSetFocus();
}

void CFrame::OnDestroy() 
{
	if (m_nType == TypeTalk || m_nType == TypeChat)
		DragAcceptFiles(FALSE);

	//mapFrameFlash.Stop(this);

	WINDOWPLACEMENT wp;
	if (GetWindowPlacement(&wp)) {
		RECT& rc = wp.rcNormalPosition;
		CString s;
		s.Format(szFmtWindowRect, rc.left, rc.top, rc.right, rc.bottom);
		theApp.WriteProfileString(szWindow, m_sFrameName, s);
	}

	CString s;
	s.Format("%s\\%s", szWindow, m_sFrameName);
	CDockState state;
	GetDockState(state);
	state.SaveState(s);

	LogClose();
	if (!m_sKey.IsEmpty())
		g_frames.DelFrame(this);

	CFrameWnd::OnDestroy();
}

void CFrame::OnCmdClose() 
{
	OnClose();
}

void CFrame::OnJoin() 
{
	new CDlgJoin(g_con.m_sChannelDefault, this);
}

void CFrame::OnVkNext() 
{
	m_pText->MovePageDown();
}

void CFrame::OnVkPrior() 
{
	m_pText->MovePageUp();
}

void CFrame::OnVkLineup() 
{
	m_pText->MoveLineUp();
}

void CFrame::OnVkLinedown() 
{
	m_pText->MoveLineDown();
}

void CFrame::OnVkTop() 
{
	m_pText->MoveTop();
}

void CFrame::OnVkBottom() 
{
	m_pText->MoveBottom();
}

void CFrame::OnTalkwith() 
{
	CDlgTalk dlg(this, &g_aKnownNicks);
	if (dlg.DoModal() != IDOK)
		return;
	CmdTalk(dlg.m_sNick);
}

void CFrame::OnPing() 
{
	CmdCtcpPing(m_sTarget);
}

void CFrame::OnCtcpTime() 
{
	CmdCtcpTime(m_sTarget);
}

void CFrame::OnUserinfo() 
{
	CmdCtcpUserinfo(m_sTarget);
}

void CFrame::OnFinger() 
{
	CmdCtcpFinger(m_sTarget);
}

void CFrame::OnVersion() 
{
	CmdCtcpVersion(m_sTarget);
}

void CFrame::OnClientinfo() 
{
	CmdCtcpClientinfo(m_sTarget);
}

void CFrame::OnDccChat() 
{
	CmdDccChat(m_sTarget);
}

void CFrame::OnDccSend() 
{
	CmdDccSend(m_sTarget);
}

// right button click on names bar

void CFrame::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (IsWindow(m_pText->m_hWnd)) {
		CRect rc;
		m_pText->GetWindowRect(&rc);
		if (rc.PtInRect(point)) {
			MenuPopup(point);
			return;
		}
	}
}

void CFrame::OnRestoreMinimized() 
{
	m_fRestoreMinimized = !m_fRestoreMinimized;
}

void CFrame::OnUpdateRestoreMinimized(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fRestoreMinimized);
}

void CFrame::OnUpdateLogging(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_hndLogFile != INVALID_HANDLE_VALUE);
}

void CFrame::OnUpdateViewLog(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_hndLogFile != INVALID_HANDLE_VALUE);
}

void CFrame::OnLogging() 
{
	BOOL f = (m_hndLogFile != INVALID_HANDLE_VALUE);// now logging
	if (f)
		LogClose();
	else
		LogOpen(TRUE);
}

void CFrame::OnViewLog() 
{
	if (m_hndLogFile == INVALID_HANDLE_VALUE)
		return;

	CString sCmd;
	sCmd.Format("%s", m_sLogFile);
	HINSTANCE hInst = ShellExecute(::GetDesktopWindow(),
		NULL, sCmd, NULL, NULL, SW_SHOW);
	if ((DWORD)hInst <= 32) {
		TRACE("ShellExecute: %lu\n", (DWORD)hInst);
		CString s;
		s.Format(IDS_FMT_OPENTEXT_ERR, sCmd);
		AfxMessageBox(s, MB_OK|MB_ICONEXCLAMATION);
	}
}

void CFrame::MemPaste(LPCTSTR pStr)
{
	BOOL fSent = FALSE;
	while (*pStr) {
		LPCTSTR p;
		for (p = pStr; *p && *p != '\r' && *p != '\n'; p++)
			;
		CString s(pStr, p - pStr);
		if (*p == 0 && !fSent) {
			// First line is terminated without newline,
			//  then set it to input-line and return
			m_pInput->InsertText(s);
			return;
		}
		// Any of line is terminated with newline,
		//  then all lines are sent directory
		if (!fSent && g_opt.fConfirmPaste)
			if (AfxMessageBox(IDS_ASK_PASTE, MB_YESNO) != IDYES)
				return;
		SendData(s);
		fSent = TRUE;
		if (*p == '\r')
			p++;
		if (*p == '\n')
			p++;
		pStr = p;
	}
}

void CFrame::OnEditPaste() 
{
	if (!OpenClipboard())
		return;
	HANDLE hnd = GetClipboardData(CF_TEXT);
	if (hnd) {
		LPCTSTR pData = (LPCTSTR)GlobalLock(hnd);
		if (pData) {
			MemPaste(pData);
			GlobalUnlock(hnd);
		}
	}
	CloseClipboard();
}

void CFrame::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(IsClipboardFormatAvailable(CF_TEXT));
}

void CFrame::OnIgnore() 
{
	IgnorePopup(this);
}

void CFrame::OnWhois() 
{
	TRACE("OnWhois\n");
}

void CFrame::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == g_tmidIsOn)
		TimerEventIsOn();
//	else if (nIDEvent == g_tmidFlash)
//		mapFrameFlash.Flash();
	else if (nIDEvent == g_tmidQuit)
		TimerEventQuit();
	else if (nIDEvent == g_tmidDefer)
		DeferDeliver();
	else {
		CFrameWnd::OnTimer(nIDEvent);
		TRACE("nIDEvent=%d\n", nIDEvent);
	}
}

void CFrame::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	//TRACE("CFrame OnActivate %d %s {\n", nState, m_sFrameName);

	CFrameWnd::OnActivate(nState, pWndOther, bMinimized);

	switch (nState) {
	case WA_INACTIVE:
		//m_fActivate = FALSE;
		break;
	default:
		//m_fActivate = TRUE;
		//if (m_fFlashing)
		//	mapFrameFlash.Stop(this);
		break;
	}

	//TRACE("} CFrame OnActivate %d\n", nState);
}

void CFrame::OnCmdAway() 
{
	if (g_con.m_fAway) {
		CmdAway();
		return;
	}
	DlgAwaySet dlg(this);
	if (dlg.DoModal() != IDOK)
		return;
	CmdAway(dlg.m_sMsg);
}

void CFrame::OnUpdateCmdAway(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((sock.GetStatus() & SOCK_CONNECT) != 0);
	pCmdUI->SetCheck(g_con.m_fAway);
}

void CFrame::OnDropFiles(HDROP hDropInfo) 
{
	CmdDccSend(m_sTarget, hDropInfo);
}

void CFrame::OnNick() 
{
	CDlgNick dlg(this);
	dlg.m_sOldNick = dlg.m_sNick = g_con.m_sNick;
	if (dlg.DoModal() != IDOK)
		return;
	CmdNick(dlg.m_sNick);
}

void CFrame::OnQuitMsg() 
{
	DlgQuit dlg(this);
	dlg.DoModal();
}

void CFrame::OnUpdateQuitMsg(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_con.m_fInConversation);
}

void CFrame::OnHelpHome() 
{
	CString sUrl;
	sUrl.LoadString(IDS_HOMEPAGE);
	OpenURL(sUrl);
}

void CFrame::OnHelpBbs() 
{
	CString sUrl;
	sUrl.LoadString(IDS_BBS);
	OpenURL(sUrl);
}

void CFrame::OnHelpTutorial() 
{
	CString sUrl;
	sUrl.LoadString(IDS_TUTORIAL);
	OpenURL(sUrl);
}

BOOL CFrame::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	if (m_pText)
		return m_pText->OnMouseWheel(nFlags, zDelta, pt);
	return CFrameWnd::OnMouseWheel(nFlags, zDelta, pt);
}

void CFrame::OnMe() 
{
	if (m_sTarget.IsEmpty())
		return;
	DlgMe dlg(this);
	if (dlg.DoModal() != IDOK)
		return;
	CString& sMe = dlg.m_sMe;
	if (sMe.IsEmpty())
		return;
	CmdMe(m_sTarget, sMe);
	LineNotice(g_con.m_sNick, sMe);
}

void CFrame::OnAvoidPopup() 
{
	g_fAvoidPopup = !g_fAvoidPopup;
}

void CFrame::OnUpdateAvoidPopup(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(g_fAvoidPopup);
}

void CFrame::OnAlwaysOnTop() 
{
	m_fAlwaysOnTop = !m_fAlwaysOnTop;
	SetWindowPos(
		m_fAlwaysOnTop ? &wndTopMost : &wndNoTopMost,
		0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
}

void CFrame::OnUpdateAlwaysOnTop(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_fAlwaysOnTop);
}

#if 0
void CFrame::Simple(BOOL f)
{
	if (m_pText == NULL || m_fSimple == f) {
		TRACE("Simple: do nothing\n");
		return;
	}

	TRACE("CFrame Simple %d {\n", f);
	if (f) {
		if (!IsIconic()) {
			m_fSimple = TRUE;
			ShowWindow(SW_HIDE);
			m_pText->Isolate(TRUE);
			m_fActivate = FALSE;
		}
	} else {
		m_fSimple = FALSE;
		m_pText->Isolate(FALSE);
		RecalcLayout();
		ShowWindow(SW_SHOW);
	}
	TRACE("} CFrame Simple %d\n", f);
}
#endif

LONG CFrame::OnKeyword(UINT wParam, LONG lParam)
{
	if (!g_fAvoidPopup && g_opt.fRestoreByKeyword && IsIconic())
		RestoreByEvent();

	optBeep.Play(BEEP_Keyword);

	return 0;
}

void CFrame::OnPaint() 
{
	CPaintDC dc(this); // `p̃foCX ReLXg
	if (m_pInput && m_pInput->m_cyBorder) {
		int n = m_pInput->m_cyBorder;
		CRect rc;
		m_pInput->GetWindowRect(&rc);
		ScreenToClient(&rc);
		int nTop = rc.top;
		GetClientRect(&rc);
		rc.bottom = nTop;
		rc.top = rc.bottom - n;

		dc.FillSolidRect(rc, GetSysColor(COLOR_BTNFACE));
		//COLORREF clr1 = GetSysColor(COLOR_BTNHILIGHT);
		//COLORREF clr2 = GetSysColor(COLOR_BTNSHADOW);
		//dc.Draw3dRect(rc, clr2, clr1);
	}
}

void CFrame::OnViewIson() 
{
	CIsonPop::Show(!CIsonPop::IsShow());
}

void CFrame::OnUpdateViewIson(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(CIsonPop::IsShow());
}

BOOL CFrame::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
	//return CFrameWnd::OnEraseBkgnd(pDC);
}
