// MainFrm.h : interface of the CFrame class

#include "plug.h"

inline BOOL IsKeyCtrl()
{
	return (GetKeyState(VK_CONTROL) & 0x8000) != 0;
}

class CNickWnd;
class CApp;
class CText;
class CUser;
class CInput;

class CFrame : public CFrameWnd
{
	DECLARE_DYNCREATE(CFrame)
public:
	CFrame(int nType = TypeMain,
		int nResourceID = IDR_MAIN,
		LPCTSTR pszFrameName = "Main",
		LPCTSTR pszTarget = "");

	enum {
		TypeMain = 0,
		TypeChan,
		TypeTalk,
		TypeChat,
	};

	int GetType() { return m_nType; };
	int GetResourceID() { return m_nResourceID; };
	const CString& GetFrameName() const { return m_sFrameName; };
	const CString& GetFrameTitle() const { return m_sTitle; };

protected:
	int m_nType;
	int m_nResourceID;
	CString m_sFrameName;
	CString m_sTitle;	// title bar text

public:
	CString m_sTarget;
	CString m_sKey;		// frame map key
	BOOL m_fRestoreMinimized;	// restore window by new message
	//BOOL m_fFlashing;	// now flashing
	BOOL m_fRectCalc;	// initial window rect is already calculated
	BOOL m_fAlwaysOnTop;

// Operations
public:
	// to make window caption
	virtual CString GetTopic() { return ""; }
	virtual BOOL IsOpMe() { return FALSE; }
	virtual CString GetModeString() { return ""; }

	// send data for each frame
	virtual BOOL SendData(LPCTSTR pStr);

	// key handling
	virtual void KeyTab();

	virtual void StatChange(BOOL f) {};

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CFrame();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	void MakeTitle();

	// input bar
	void InputSetFocus();
	BOOL InputText(CString& s);
	void InputSetText(LPCTSTR p = 0);
	void InputGetText(CString& s);
	void MenuPopupDestNick(const CPoint pt);

	void MemPaste(LPCTSTR pStr);
	void IrcCommand(LPCTSTR pLine);

	virtual void FontChange();

	void MenuPopup(const CPoint pt);

	// command
	virtual void CmdTopic(LPCTSTR pStr);

	// message handling
	virtual BOOL HasNick(LPCTSTR pNick) { return TRUE; }

	virtual void ParseMyAddress(LPCTSTR pHost) {};
	//virtual void MsgWelcome(LPCTSTR pTrail) {};
	virtual void MsgTopic(const Packet& pac);
	virtual void MsgJoin(const Packet& pac, LPCTSTR pChan, LPCTSTR pMode = 0);
	virtual void MsgPart(const Packet& pac, LPCTSTR pChan);
	virtual void MsgKick(LPCTSTR pNick, LPCTSTR pChan, LPCTSTR pParm, LPCTSTR pTrail);
	virtual void MsgNick(LPCTSTR pNick, LPCTSTR pNickTo);
	virtual void MsgQuit(LPCTSTR pNick, LPCTSTR pTrail);
	void MsgUserMode(LPCTSTR pNick, LPCTSTR pParm);
	virtual void MsgMode(const Packet& pac, LPCTSTR pChan, LPCTSTR pParm);
	virtual void MsgDisconnect();

	void RplNames(LPCTSTR pChan, LPCTSTR pNames);
	virtual void RplBanList(LPCTSTR pChan, LPCTSTR pBan);
	virtual void RplExceptionList(LPCTSTR pChan, LPCTSTR pBan);
	virtual void RplInviteList(LPCTSTR pChan, LPCTSTR pBan);
	virtual void RplWho(LPCTSTR pChan, const CUser& user);
	virtual void RplWhoEnd();
	virtual void RplNoTopic(LPCTSTR pChan);
	virtual void RplTopic(LPCTSTR pChan, LPCTSTR pTopic);
	virtual void RplChanMode(const Packet& pac, LPCTSTR pChan, LPCTSTR pParm);

	void RestoreByEvent();
	void NotifyMessage();
	void ShowLine(LPCTSTR pNick, LPCTSTR pStr,
		Fonts::FType type, BOOL fInClient = TRUE);
	void LineText(LPCTSTR pNick, LPCTSTR pStr);
	void LineNotice(LPCTSTR pNick, LPCTSTR pStr, BOOL fInClient = TRUE);
	void LineError(LPCTSTR pStr)
	{
		ShowLine(NULL, pStr, Fonts::TypeError);
	};
	void LineError(int id);
	void LineStat(LPCTSTR pStr, BOOL fInClient = TRUE)
	{
		ShowLine(NULL, pStr, Fonts::TypeStat, fInClient);
	};
	void LineStat(int id, BOOL fInClient = TRUE);
	void LineRaw();

	void ShowStatus(int id);
	void ShowStatus(LPCTSTR pStr)
	{
		SetMessageText(pStr);
	}

protected:
	HANDLE m_hndLogFile;	// log file handle
	CString m_sLogFile;		// log file name
	int m_nLastDate;

	void LogOpen(BOOL fForce = FALSE);
	void LogClose();
	void LogWrite(const CString& sStr);
	void LogStamp(int id);

protected:  // control bar embedded members
	CStatusBar m_wndStatusBar;
	CToolBar m_wndToolBar;
	CText* m_pText;
	CInput* m_pInput;

	//void SaveBarStateBetter(LPCTSTR lpszProfileName) const;

// Generated message map functions
protected:
	//{{AFX_MSG(CFrame)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnDestroy();
	afx_msg void OnJoin();
	afx_msg void OnVkNext();
	afx_msg void OnVkPrior();
	afx_msg void OnTalkwith();
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnVersion();
	afx_msg void OnPing();
	afx_msg void OnUserinfo();
	afx_msg void OnFinger();
	afx_msg void OnClientinfo();
	afx_msg void OnRestoreMinimized();
	afx_msg void OnUpdateRestoreMinimized(CCmdUI* pCmdUI);
	afx_msg void OnUpdateLogging(CCmdUI* pCmdUI);
	afx_msg void OnLogging();
	afx_msg LONG OnSockHost(UINT wParam, LONG lParam);
	afx_msg LONG OnSockAddr(UINT wParam, LONG lParam);
	afx_msg void OnCmdClose();
	afx_msg void OnEditPaste();
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnIgnore();
	afx_msg void OnWhois();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnCtcpTime();
	afx_msg void OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized);
	afx_msg void OnDccChat();
	afx_msg void OnDccSend();
	afx_msg void OnVkLineup();
	afx_msg void OnVkLinedown();
	afx_msg void OnVkTop();
	afx_msg void OnVkBottom();
	afx_msg void OnCmdAway();
	afx_msg void OnUpdateCmdAway(CCmdUI* pCmdUI);
	afx_msg void OnDropFiles(HDROP hDropInfo);
	afx_msg void OnNick();
	afx_msg void OnQuitMsg();
	afx_msg void OnUpdateQuitMsg(CCmdUI* pCmdUI);
	afx_msg void OnHelpHome();
	afx_msg void OnHelpBbs();
	afx_msg void OnHelpTutorial();
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnMe();
	afx_msg void OnAvoidPopup();
	afx_msg void OnUpdateAvoidPopup(CCmdUI* pCmdUI);
	afx_msg void OnAlwaysOnTop();
	afx_msg void OnUpdateAlwaysOnTop(CCmdUI* pCmdUI);
	afx_msg void OnViewLog();
	afx_msg void OnUpdateViewLog(CCmdUI* pCmdUI);
	afx_msg LONG OnKeyword(UINT wParam, LONG lParam);
	afx_msg void OnPaint();
	afx_msg void OnViewIson();
	afx_msg void OnUpdateViewIson(CCmdUI* pCmdUI);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

class FrameMain : public CFrame
{
	DECLARE_DYNCREATE(FrameMain)
public:
	FrameMain();

	bool m_fUseTray;

	virtual void ParseMyAddress(LPCTSTR pHost);

	//virtual void MsgWelcome(LPCTSTR pTrail);
	virtual void RplWho(LPCTSTR pChan, const CUser& user);

protected:
	void TrayMenu(const CPoint& pt);
	void StatChange(BOOL f);

	CImageList m_ilTray;
	HICON m_hIconTray;

	HANDLE m_hAsyncReq; // for WSAAsyncGetHostByName()
	char m_bufHostEnt[MAXGETHOSTSTRUCT];	// work for async req


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(FrameMain)
	//}}AFX_VIRTUAL

protected:
	//{{AFX_MSG(FrameMain)
	afx_msg void OnClose();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnOpenMain();
	//}}AFX_MSG
	LONG OnNotifyIcon(UINT wParam, LONG lParam);
	LONG OnSockMyHost(UINT wParam, LONG lParam);
	DECLARE_MESSAGE_MAP()
};

class CSplit : public CSplitterWnd
{
public:
	DECLARE_DYNCREATE(CSplit)

// I[o[Ch
	void StopTracking(BOOL bAccept);
	// ClassWizard ͉z֐𐶐I[o[Ch܂B
	//{{AFX_VIRTUAL(CSplit)
	//}}AFX_VIRTUAL

	// ꂽbZ[W }bv֐
protected:
	//{{AFX_MSG(CSplit)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

class CDlgChanSwitch;
class CDlgBan;
class DlgTopic;

class FrameChan;//forward

class ChanMode
{
public:
	ChanMode();
	ChanMode(const ChanMode& mode);

	void Clear();
	CString ToString();
	void FromString(LPCTSTR p, FrameChan* pFrame = 0);
	void Apply(LPCTSTR sTarget, const ChanMode& mode);

	BOOL m_fPrivate;
	BOOL m_fSecret;
	BOOL m_fInviteOnly;
	BOOL m_fTopicOp;
	BOOL m_fNoOutside;
	BOOL m_fModerated;
	BOOL m_fAnonymous;
	BOOL m_fQuiet;
	BOOL m_fReop;
	int m_nLimit;
	CString m_sKey;
};

#include "chan.h"

class FrameChan : public CFrame
{
public:
	FrameChan(LPCTSTR pszTarget);
	~FrameChan();

	// overrides
	void FontChange();
	CString GetTopic();
	BOOL IsOpMe() { return m_fOpMe; }
	CString GetModeString() { return m_mode.ToString(); }

	void CmdTopic(LPCTSTR pStr);
	BOOL SendData(LPCTSTR pStr);

	void KeyTab();

	BOOL HasNick(LPCTSTR pNick);

	void MsgTopic(const Packet& pac);
	void MsgJoin(const Packet& pac, LPCTSTR pChan, LPCTSTR pMode = 0);
	void MsgPart(const Packet& pac, LPCTSTR pChan);
	void MsgKick(LPCTSTR pNick, LPCTSTR pChan, LPCTSTR pParm, LPCTSTR pTrail);
	void MsgNick(LPCTSTR pNick, LPCTSTR pNickTo);
	void MsgQuit(LPCTSTR pNick, LPCTSTR pTrail);
	void MsgMode(const Packet& pac, LPCTSTR pChan, LPCTSTR pParm);
	void MsgDisconnect();

	void RplBanList(LPCTSTR pChan, LPCTSTR pBan);
	void RplExceptionList(LPCTSTR pChan, LPCTSTR pBan);
	void RplInviteList(LPCTSTR pChan, LPCTSTR pBan);
	void RplWho(LPCTSTR pChan, const CUser& user);
	void RplWhoEnd();
	void RplNoTopic(LPCTSTR pChan);
	void RplTopic(LPCTSTR pChan, LPCTSTR pTopic);
	void RplChanMode(const Packet& pac, LPCTSTR pChan, LPCTSTR pParm);

	// not override
	void RplModeOP(BOOL fPlus, LPCTSTR pParm);
	void RplModeBan(BOOL fPlus, LPCTSTR pParm);
	void RplModeException(BOOL fPlus, LPCTSTR pParm);
	void RplModeInvite(BOOL fPlus, LPCTSTR pParm);

protected:
	CString m_sTopic;

	BOOL m_fOpMe;
	BOOL m_fChanJoin;		// now joining

	// channel modes
	void ModeClear();
	void ModeChange(const ChanMode* pMode, BOOL fKeep = FALSE);
	void ModeAutoOP();
	void TopicChange(LPCTSTR pszTopic, BOOL fKeep = FALSE, BOOL fSet = TRUE);
	void TopicKeep();

	ChanMode m_mode;
	ChanMode* m_pModeKeep;
	CString m_sTopicKeep;
	LsNickMatch m_lsOpPlus;

	CDlgChanSwitch* m_pDlgChanSwitch;
	friend class CDlgChanSwitch;

	CDlgBan* m_pDlgBanList;
	CDlgBan* m_pDlgExceptionList;
	CDlgBan* m_pDlgInviteList;

	friend class CDlgBan;

	DlgTopic* m_pDlgTopic;
	friend class DlgTopic;

	friend class DlgOPList;
	friend class CNickList;

	void ClearNames();			// refresh nicklist
	void NamesRedraw(BOOL fOn);	// on/off nicklist update
	void MenuOnNames(const CPoint pt);

	CSplit m_split;
	CNickWnd* m_pNicks;
	CSize m_sizeText;	// size of text area
	CSize m_sizeNicks;	// size of nicklist window

	void OpHelper(char chPlus);

	//virtual void Simple(BOOL f);

protected:
	CStringArray arNames;	// work for name operation

	HANDLE m_hAsyncDnslookup;
	char m_bufHostEntDnslookup[MAXGETHOSTSTRUCT];	// work for async req

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(FrameChan)
	protected:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(FrameChan)
	afx_msg void OnClose();
	afx_msg void OnPart();
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnNamesWhois();
	afx_msg void OnNamesTalk();
	afx_msg void OnNamesIgnore();
	afx_msg void OnNamesOpKick();
	afx_msg void OnNamesOpGiveop();
	afx_msg void OnNamesOpStripop();
	afx_msg void OnNamesOpAutoop();
	afx_msg void OnNamesOpBan();
	afx_msg void OnUpdateNamesOp(CCmdUI* pCmdUI);
	afx_msg void OnUpdateNamesHasSel(CCmdUI* pCmdUI);
	afx_msg void OnNamesPing();
	afx_msg void OnNamesTime();
	afx_msg void OnNamesUserinfo();
	afx_msg void OnNamesFinger();
	afx_msg void OnNamesVersion();
	afx_msg void OnNamesClientinfo();
	afx_msg void OnNamesHelp();
	afx_msg void OnNamesDccChat();
	afx_msg void OnNamesDccSend();
	afx_msg void OnUpdateNamesOnlyOne(CCmdUI* pCmdUI);
	afx_msg void OnNamesRefresh();
	afx_msg void OnChanMode();
	afx_msg void OnChanBan();
	afx_msg void OnViewSplit();
	afx_msg void OnDestroy();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnTopic();
	afx_msg void OnOplist();
	afx_msg void OnInvite();
	afx_msg void OnChanException();
	afx_msg void OnChanInvite();
	afx_msg void OnNamesDnslookup();
	//}}AFX_MSG
	LRESULT OnNicklistRecreate(WPARAM, LPARAM);
	LRESULT OnDeferChanMode(WPARAM, LPARAM);
	LONG OnSockDnslookup(UINT wParam, LONG lParam);
	DECLARE_MESSAGE_MAP()
};

class FrameTalk : public CFrame
{
public:
	FrameTalk(LPCTSTR pszTarget);

	CString m_sAddr; // peer address

	// overrides
	BOOL SendData(LPCTSTR pStr);
	BOOL HasNick(LPCTSTR pNick);
	void MsgNick(LPCTSTR pNick, LPCTSTR pNickTo);
	void RplWho(LPCTSTR pChan, const CUser& user);

protected:
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(FrameTalk)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(FrameTalk)
	afx_msg void OnIgnore();
	afx_msg void OnWhois();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

class CSockListen;
class CSockData;

class FrameChat : public CFrame
{
public:
	FrameChat(LPCTSTR pszTarget);
	~FrameChat();

	CString m_sAddr; // peer address

	// overrides
	BOOL SendData(LPCTSTR pStr);
	BOOL HasNick(LPCTSTR pNick);
	void MsgNick(LPCTSTR pNick, LPCTSTR pNickTo);
	void MsgQuit(LPCTSTR pNick, LPCTSTR pTrail);
	void RplWho(LPCTSTR pChan, const CUser& user);

	// dcc
	BOOL ChatListen();
	BOOL ChatConnect(u_long ulHost, u_short usPort);

	CSockListen* m_pSockListen;
	CSockData* m_pSockData;
	BOOL m_fConnect;

protected:
	void GotMsg(LPCTSTR pStr, int nLen);

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(FrameChat)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(FrameChat)
	afx_msg LONG OnSock(UINT wParam, LONG lParam);
	afx_msg LONG OnSockData(UINT wParam, LONG lParam);
	afx_msg void OnIgnore();
	afx_msg void OnWhois();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

// map target to frame
class Frames : public CMap<CString, LPCTSTR, CFrame*, CFrame*&>
{
public:
	//void CloseFrame(LPCTSTR ps);
	//void ActivateFrame(LPCTSTR ps);
	void ClearFrames();
	void AddFrame(CFrame* pFrame);
	void DelFrame(CFrame* pFrame);
	void ChangeKey(CFrame* pFrame);

	void PostMsgChan(UINT msg, WPARAM wParam = 0, LPARAM lParam = 0);

	void MsgNick(LPCTSTR pNick, LPCTSTR pNickTo);
	void MsgQuit(LPCTSTR pNick, LPCTSTR pTrail);
	void MsgDisconnect();
	void MsgReply(LPCTSTR pNick, LPCTSTR pMsg);
	void BroadCast(LPCTSTR pStr);
	void BroadCastMe(LPCTSTR pStr);
	void RplWho(LPCTSTR pChan, CUser& user);
	void Rejoin();
	void MakeAllTitle();
	void FontChange();

	void SelectFrame();
	void MinimizeAll();
	void CloseAll();

	//void Tick();//global timer
};

extern Frames g_frames;
extern CFrame* g_pFrame;

inline BOOL IsMainFrame(CFrame* pFrame)
{
	return pFrame == g_pFrame;
}

CFrame* GetFrame(LPCTSTR pszTarget);

#if 0
class MapFrameFlash : public CMap<CFrame*, CFrame*&, int, int>
{
public:
	void Start(CFrame* pFrame);
	void Stop(CFrame* pFrame);
	void Flash();
};

extern MapFrameFlash mapFrameFlash;
extern UINT g_tmidFlash;
#endif

BOOL DeferPostMessage(CWnd* pWnd, UINT nMsg);
