#include "stdafx.h"
#include "mchat.h"

#include "dcc.h"
#include "conv.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(FrameChat, CFrame)
	//{{AFX_MSG_MAP(FrameChat)
	ON_MESSAGE(WMU_SOCK, OnSock)
	ON_MESSAGE(WMU_SOCK_DATA, OnSockData)
	ON_COMMAND(ID_IGNORE, OnIgnore)
	ON_COMMAND(ID_WHOIS, OnWhois)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

FrameChat::FrameChat(LPCTSTR pszTarget)
	: CFrame(TypeChat, IDR_CHAT, "Chat", pszTarget)
{
	m_pSockListen = 0;
	m_pSockData = 0;
	m_fConnect = FALSE;

	LoadFrame(IDR_CHAT, WS_OVERLAPPEDWINDOW);
	ShowWindow(SW_SHOW);
	UpdateWindow();
}

FrameChat::~FrameChat()
{
	if (m_pSockListen)
		delete m_pSockListen;
	if (m_pSockData)
		delete m_pSockData;
}

#define MAXLINE_SEND	512

BOOL FrameChat::SendData(LPCTSTR pStr)
{
	if (!m_fConnect) {
		LineError("*no connection");
		return FALSE;
	}
	int nStr = lstrlen(pStr);
	while (nStr > 0) {
		int nLineLen = nStr;
		if (nLineLen > MAXLINE_SEND) {
			nLineLen = MAXLINE_SEND;
			if (_mbsbtype((unsigned char*)pStr, nLineLen) == _MBC_TRAIL)
				nLineLen--;
		}

		CString s(pStr, nLineLen);
		LineText(g_con.m_sNick, s);

		s += (g_opt.nChatEOL == CHATEOL_LF ? "\n" : "\r\n");

		if (g_opt.nChatCode == CHATCODE_JIS) {
			BYTE buf[MAXLINE_SEND * 4];
			int n1 = s.GetLength();
			int n2 = sizeof buf;
			sjis2jis((const BYTE*)(LPCTSTR)s, n1, buf, n2);
			m_pSockData->DoSend((LPCTSTR)&buf[0], n2);
		} else
			m_pSockData->DoSend(s, nLineLen);

		pStr += nLineLen;
		nStr -= nLineLen;
	}
	return TRUE;
}

BOOL FrameChat::ChatListen()
{
	ASSERT(m_pSockListen == 0);
	ASSERT(m_pSockData == 0);
	m_pSockListen = new CSockListen;
	m_pSockData = new CSockData(TRUE);

	m_pSockListen->m_parent = m_pSockData->m_parent = this;
	m_pSockListen->m_pData = m_pSockData;

	u_short usPort = g_opt.nDccPort;
	if (!m_pSockListen->DoListen(usPort)) {
		LineError("*Listen failed");
		return FALSE;
	}

	u_long ulHost = htonl(sock.sockaddr.sin_addr.s_addr);
	if (g_opt.fUseDccFakeAddr)
		ulHost = htonl(g_opt.addrDccFakeAddr);
	usPort = htons(usPort);

	CString s;
	s.Format("PRIVMSG %s :\001DCC CHAT chat %lu %u\001",
		m_sTarget, ulHost, usPort);
	Cmd(s);

	s.Format("*Waiting connection with %s", m_sTarget);
	LineStat(s);

	return TRUE;
}

BOOL FrameChat::ChatConnect(u_long ulHost, u_short usPort)
{
	ASSERT(m_pSockData == 0);
	m_pSockData = new CSockData(TRUE);
	m_pSockData->m_parent = this;
	if (!m_pSockData->DoConnect(ulHost, usPort)) {
		LineError("*Connect failed");
		return FALSE;
	}
	return TRUE;
}

LONG FrameChat::OnSock(UINT wParam, LONG lParam)
{
	switch (wParam) {
	case WMU_SOCK_MSG:
		LineStat((LPCTSTR)lParam);
		break;
	case WMU_SOCK_CONNECT:
		LineStat("*Connection established");
		m_fConnect = TRUE;
		break;
	case WMU_SOCK_CLOSE:
		LineStat("*Connection closed");
		m_fConnect = FALSE;
		break;
	default:
		TRACE("OnSock: unknown wParam %d\n", wParam);
	}
	return 0;
}

void FrameChat::GotMsg(LPCTSTR pStr, int nLen)
{
	TCHAR buf[1025];
	int nBuf = sizeof buf - 1;
	jis2sjis((const BYTE*)pStr, nLen, (BYTE*)buf, nBuf);
	buf[nBuf] = 0;
	LineText(m_sTarget, buf);

	if (!IsIconic() || !optBeep.Play(BEEP_PrivMsgIcon))
		optBeep.Play(BEEP_PrivMsg);
}

LONG FrameChat::OnSockData(UINT wParam, LONG lParam)
{
	LPTSTR pStr = (LPTSTR)lParam;
	LPTSTR p = pStr;
	LPTSTR pEnd = pStr + wParam;
	while (p < pEnd) {
		TCHAR c = *p;
		if (c == '\r') {
			GotMsg(pStr, p - pStr);
			if (++p < pEnd && *p == '\n')
				p++;
			pStr = p;
			continue;
		}
		if (c == '\n') {
			GotMsg(pStr, p - pStr);
			pStr = ++p;
			continue;
		}
		p++;
	}
	if (pStr < pEnd)
		GotMsg(pStr, pEnd - pStr);
	return 0;
}

BOOL FrameChat::HasNick(LPCTSTR pNick)
{
	return m_sTarget.CompareNoCase(pNick) == 0;
}

void FrameChat::MsgNick(LPCTSTR pNick, LPCTSTR pNickTo)
{
	m_sTarget = pNickTo;
	MakeTitle();
	CFrame::MsgNick(pNick, pNickTo);
}

void FrameChat::MsgQuit(LPCTSTR pNick, LPCTSTR pTrail)
{
	// quit from server, but it's unrelated to dcc connection
}

void FrameChat::RplWho(LPCTSTR pChan, const CUser& user)
{
	if (m_sTarget.CompareNoCase(user.m_sNick) == 0)
		m_sAddr = user.m_sAddr;
}

void FrameChat::OnIgnore() 
{
	if (m_sAddr.IsEmpty()) {
		IgnorePopup(this);
		return;
	}
	CString s;
	s.Format("*!%s", m_sAddr);
	IgnorePopup(this, s);
}

void FrameChat::OnWhois() 
{
	CmdWhois(m_sTarget);
}
