#include "stdafx.h"
#include "mchat.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ID_NOTIFYICON	1

#define TRAY_NORMAL		0
#define TRAY_CONNECT	1

IMPLEMENT_DYNCREATE(FrameMain, CFrame)

void FrameMain::TrayMenu(const CPoint& pt)
{
	CMenu menu;
	if (!menu.LoadMenu(IDR_TRAY))
		return;
	CMenu* pMenu = menu.GetSubMenu(0);
	::SetMenuDefaultItem(pMenu->m_hMenu, 0, TRUE);
	SetForegroundWindow();
	pMenu->TrackPopupMenu(TPM_LEFTALIGN|TPM_RIGHTBUTTON,
		pt.x, pt.y, this);
	PostMessage(WM_NULL, 0, 0);
}

void FrameMain::StatChange(BOOL f)
{
	if (m_fUseTray) {
		int i = f ? TRAY_CONNECT : TRAY_NORMAL;
		m_hIconTray = m_ilTray.ExtractIcon(i);
		NOTIFYICONDATA nid = {
			sizeof (NOTIFYICONDATA),
			m_hWnd,
			ID_NOTIFYICON,
			NIF_MESSAGE | NIF_ICON | NIF_TIP,
			WMU_NOTIFYICON,
			m_hIconTray
		};
		lstrcpyn(nid.szTip, m_sTitle, sizeof nid.szTip);

		Shell_NotifyIcon(NIM_MODIFY, &nid);
	}
}

BEGIN_MESSAGE_MAP(FrameMain, CFrame)
	//{{AFX_MSG_MAP(FrameMain)
	ON_WM_CLOSE()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(ID_OPEN_MAIN, OnOpenMain)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_NOTIFYICON, OnNotifyIcon)
	ON_MESSAGE(WMU_SOCK_MYHOST, OnSockMyHost)
END_MESSAGE_MAP()

FrameMain::FrameMain() : CFrame()
{
	m_hAsyncReq = NULL;
	m_fUseTray = false;
}

void FrameMain::RplWho(LPCTSTR pChan, const CUser& user)
{
	CString s;
	s.Format("%s: %s (%s) <%s>", pChan, user.m_sNick, user.m_sRealName, user.m_sAddr);
	LineNotice(NULL, s);
}

void FrameMain::ParseMyAddress(LPCTSTR pszHost)
{
	struct in_addr in;
	in.s_addr = inet_addr(pszHost);
	if (in.s_addr != INADDR_NONE) {
		CString s;
		s.Format("DCC Local address: %s", inet_ntoa(in));
		LineStat(s);
		g_opt.addrDccFakeAddr = in.s_addr;
		return;
	}

	CString s;
	s.Format("Looking up local host ip: %s", pszHost);
	LineStat(s);

	if (m_hAsyncReq)
		WSACancelAsyncRequest(m_hAsyncReq);

	m_hAsyncReq = WSAAsyncGetHostByName(
		m_hWnd,
		WMU_SOCK_MYHOST, pszHost,
		m_bufHostEnt, sizeof (m_bufHostEnt));
	if (!m_hAsyncReq) {
		int err = WSAGetLastError();
		TRACE("WSAAsyncGetHostByName() return %d\n", err);
		CString s;
		s.Format("Cannot lookup my address: code=%d", err);
		LineError(s);
	}
}

void FrameMain::OnClose() 
{
	if (m_hAsyncReq) {
		WSACancelAsyncRequest(m_hAsyncReq);
		m_hAsyncReq = NULL;
	}

	WORD stat = sock.GetStatus();
	if (stat & SOCK_CONNECT) {
		CmdQuit();
		g_fExitApp = TRUE;
		return;
	}
	ShowWindow(SW_HIDE);
	if (stat & SOCK_CONNWAIT)
		sock.DoClose();

	g_pFrame = 0;

	if (m_fUseTray) {
		NOTIFYICONDATA nid = { sizeof(NOTIFYICONDATA), m_hWnd, ID_NOTIFYICON };
		Shell_NotifyIcon(NIM_DELETE, &nid); 
		m_fUseTray = false;
	}

	CFrame::OnClose();
}

int FrameMain::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrame::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_ilTray.Create(IDB_TRAY, 16, 2, RGB(255, 255, 255))) {
		AfxMessageBox("Can't load tray image");
		return -1;
	}
	if (g_opt.fHideMain) {
		m_hIconTray = m_ilTray.ExtractIcon(TRAY_NORMAL);
		NOTIFYICONDATA nid = {
			sizeof (NOTIFYICONDATA),
			m_hWnd,
			ID_NOTIFYICON,
			NIF_MESSAGE | NIF_ICON | NIF_TIP,
			WMU_NOTIFYICON,
			m_hIconTray
		};
		lstrcpyn(nid.szTip, "MuriChat", sizeof nid.szTip);

		Shell_NotifyIcon(NIM_ADD, &nid);
		m_fUseTray = true;
	}
	return 0;
}

LONG FrameMain::OnNotifyIcon(UINT wParam, LONG lParam)
{
	switch (lParam) {
	case WM_RBUTTONUP:
		{
			CPoint pt;
			if (GetCursorPos(&pt))
				TrayMenu(pt);
		}
		break;
	case WM_LBUTTONDBLCLK:
	case WM_LBUTTONDOWN:
		SetForegroundWindow();
		ActivateFrame();
		PostMessage(WM_NULL, 0, 0);
		break;
	}
	return 0;
}

void FrameMain::OnSize(UINT nType, int cx, int cy) 
{
	CFrame::OnSize(nType, cx, cy);

	switch (nType) {
	case SIZE_MINIMIZED:
		if (g_opt.fHideMain && m_fUseTray)
			ShowWindow(SW_HIDE);
		break;
	case SIZE_RESTORED:
		ShowWindow(SW_SHOW);
		break;
	}
}

void FrameMain::OnOpenMain() 
{
	ActivateFrame();
}

LONG FrameMain::OnSockMyHost(UINT wParam, LONG lParam)
{
	HANDLE hHandle = (HANDLE)wParam;
	if (hHandle != m_hAsyncReq) {
		TRACE("asyncreq handle mismatch %d, %d\n", hHandle, m_hAsyncReq);
	}
	m_hAsyncReq = 0;

	WORD wErr = WSAGETASYNCERROR(lParam);
	if (wErr) {
		CString s;
		s.Format("Cannot resolve local host: code=%d", wErr);
		LineError(s);
		return 0;
	}

	HOSTENT* pEnt = (HOSTENT*)m_bufHostEnt;
	struct in_addr in;
	in.s_addr = *(unsigned long*)pEnt->h_addr_list[0];

	CString s;
	s.Format("DCC Local address: %s", inet_ntoa(in));
	LineStat(s);

	g_opt.addrDccFakeAddr = in.s_addr;

	return 0;
}
