// Input.cpp : Cve[V t@C
//

#include "stdafx.h"
#include <afxpriv.h>
#include "resource.h"
#include "profile.h"
#include "frame.h"
#include "input.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CInput

CInput::CInput()
{
	m_nSelBgn = m_nSelEnd = 0;
	m_nHeight = 20;
	m_cyBorder = 0;
	m_posHist = 0;

	m_fFocus = FALSE;

	m_hIMC = m_hIMCSave = NULL;

	m_hBrushBack = NULL;

	m_dwStyle = ES_AUTOHSCROLL|WS_VISIBLE|WS_CHILD|
		WS_CLIPSIBLINGS|WS_TABSTOP;
}

CInput::~CInput()
{
}

BOOL CInput::DoCreate(CFrame* pFrame, int nID)
{
	m_pFrame = pFrame;
	CRect rc(0, 0, 10, 10);

	m_dwStyle |= WS_BORDER;
	if (pFrame->GetType() != CFrame::TypeChan)
		m_cyBorder = GetSystemMetrics(SM_CYBORDER) * 2;

	return Create(m_dwStyle, rc, pFrame, nID);
}

void CInput::FontChange()
{
	// see KB: "How to Calculate the Height of Edit Control to Resize It"
	HDC hdc = ::GetDC(m_hWnd);

	HFONT hFontSys = (HFONT)GetStockObject(SYSTEM_FONT);
	HFONT hFont = (HFONT)SelectObject(hdc, hFontSys);
	TEXTMETRIC tm;
	GetTextMetrics(hdc, &tm);
	int nHeightSys = tm.tmHeight;

	SelectObject(hdc, (HFONT)g_fonts.m_fontInput);
	GetTextMetrics(hdc, &tm);
	int nHeight = tm.tmHeight;

	SelectObject(hdc, hFont);
	//DeleteObject(hFontSys);
	::ReleaseDC(m_hWnd, hdc);

	m_nHeight = nHeight + min(nHeight, nHeightSys) / 2;

	CRect rc(0, 0, 0, m_nHeight);
	CalcWindowRect(&rc);
	m_nHeight = rc.Height();

	m_hBrushBack = CreateSolidBrush(g_fonts.m_crBack);

	SetFont(&g_fonts.m_fontInput);
}

void CInput::SetText(LPCTSTR p)
{
	SetWindowText(p ? p : "");
	int n = LineLength();
	m_nSelBgn = m_nSelEnd = n;
	SetSel(n, n);
}

void CInput::ClearText()
{
	SetText("");
	m_posHist = 0;
}

#define HISTORY_MAX		30

void CInput::KeyEnter()
{
	GetWindowText(m_sInput);

	LPCTSTR pInput = m_sInput;
	CString sText;
	LPTSTR pText = sText.GetBuffer(m_sInput.GetLength() * 2);
	int nText = 0;

	TCHAR c;
	while ((c = *pInput++) != 0) {
		if (c == g_opt.cCtrlChar) {
			switch (*pInput) {
			case 'B':
				c = CHAR_BOLD;
				pInput++;
				break;
			case 'L':
				c = CHAR_UNDERLINE;
				pInput++;
				break;
			case 'R':
				c = CHAR_REVERSE;
				pInput++;
				break;
			}
		}
		*pText++ = c;
		nText++;
	}
	sText.ReleaseBuffer(nText);

	if (!m_pFrame->InputText(sText))
		return;

	m_lsHist.AddTail(m_sInput);
	if (m_lsHist.GetCount() > HISTORY_MAX)
		m_lsHist.RemoveHead();
	m_posHist = 0;

	if (InputParseNick(m_sInput) && g_opt.fDestNick)
		SetText(m_sDestNick + ": ");
	else
		SetWindowText("");
}

void CInput::KeyUp()
{
	if (!m_posHist) {
		GetText(m_sInput);
		m_posHist = m_lsHist.GetTailPosition();
	} else
		m_lsHist.GetPrev(m_posHist);
	if (!m_posHist) {
		SetText(m_sInput);
		return;
	}
	SetText(m_lsHist.GetAt(m_posHist));
}

void CInput::KeyDown()
{
	if (!m_posHist) {
		GetText(m_sInput);
		m_posHist = m_lsHist.GetHeadPosition();
	} else
		m_lsHist.GetNext(m_posHist);
	if (!m_posHist) {
		SetText(m_sInput);
		return;
	}
	SetText(m_lsHist.GetAt(m_posHist));
}

// Dest Nick

#define NUM_DESTNICK	(ID_DESTNICK_LAST - ID_DESTNICK + 1)

static int InputGetDestNickLen(LPCTSTR pStr)
{
	LPCTSTR p;
	for (p = pStr;; p++) {
		char c = *p;
		if (c == '\0' || c == ' ')
			return 0;
		if (c == ':') {
			if (p[1] == '/')
				return 0;	// avoid url like "http://"
			break;
		}
	}
	return p - pStr;
}

BOOL CInput::InputParseNick(LPCTSTR pStr)
{
	int nLen = InputGetDestNickLen(pStr);
	if (nLen < 1)
		return FALSE;
	m_sDestNick = CString(pStr, nLen);

	int i;
	for (i = 0; i < m_arDestNick.GetSize(); i++) {
		CString& s = m_arDestNick.ElementAt(i);
		if (m_sDestNick.CompareNoCase(s) == 0) {
			// if same with most recent nick, do nothing
			// if match to not recent nick, or case is changed, then replace it
			if (i == 0 && m_sDestNick == s)
				return TRUE;
			m_arDestNick.RemoveAt(i);
			break;
		}
	}
	// lower element (index 0) is most recent nick
	m_arDestNick.InsertAt(0, m_sDestNick);
	if (m_arDestNick.GetSize() > NUM_DESTNICK)
		m_arDestNick.RemoveAt(m_arDestNick.GetUpperBound());
	return TRUE;
}

void CInput::InsertNick(LPCTSTR pNick)
{
	m_sDestNick = pNick;
	CString s;
	GetWindowText(s);
	int nLen = InputGetDestNickLen(s);
	if (nLen < 1)
		m_sInput = m_sDestNick + ": " + s;
	else
		m_sInput = m_sDestNick + ((LPCTSTR)s + nLen);
	SetText(m_sInput);
}

void CInput::SelectNick(int nIdx)
{
	if (nIdx < 0 || nIdx >= m_arDestNick.GetSize())
		return;

	CString sNick = m_arDestNick.GetAt(nIdx);
	if (nIdx != 0) {
		// move element to most recent
		m_arDestNick.RemoveAt(nIdx);
		m_arDestNick.InsertAt(0, sNick);
	}

	InsertNick(sNick);
}

void CInput::MenuPopup(POINT* pPoint)
{
	CMenu menu;
	menu.LoadMenu(IDR_INPUT);
	if (!pPoint) {
		CRect rc;
		GetWindowRect(&rc);
		pPoint = &rc.TopLeft();
	}
	menu.EnableMenuItem(ID_EDIT_PASTE,
		IsClipboardFormatAvailable(CF_TEXT) ? MF_ENABLED : MF_GRAYED);
	menu.EnableMenuItem(ID_DESTNICK_POPUP,
		 m_arDestNick.GetSize() > 0 ? MF_ENABLED : MF_GRAYED);
	CMenu* pMenu = menu.GetSubMenu(0);
	pMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pPoint->x, pPoint->y, this);
	menu.DestroyMenu();
}

void CInput::PopupNicks()
{
	int nSize = m_arDestNick.GetSize();
	if (nSize <= 0 )
		return;
	CMenu menu;
	if (!menu.CreatePopupMenu())
		return;
	int i;
	for (i = 0; i < nSize; i++)
		menu.AppendMenu(MF_STRING, ID_DESTNICK + i, m_arDestNick.GetAt(i));
	CRect rc;
	GetWindowRect(&rc);
	menu.TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, rc.left, rc.top, this);
	menu.DestroyMenu();
}

void CInput::SetAttribute(BYTE bAt)
{
	TCHAR c;
	switch (bAt) {
	case F_BOLD:
		c = _T('B');
		break;
	case F_REVERSE:
		c = _T('R');
		break;
	//case F_UNDERLINE;
	default:
		c = _T('L');
		break;
	}
	CString sAt(CString(g_opt.cCtrlChar) + c);
	GetSel(m_nSelBgn, m_nSelEnd);
	if (m_nSelBgn < m_nSelEnd) {
		GetWindowText(m_sInput);
		ReplaceSel(sAt +
			CString((LPCTSTR)m_sInput + m_nSelBgn, m_nSelEnd - m_nSelBgn) + sAt);
	} else
		ReplaceSel(sAt);
}

BEGIN_MESSAGE_MAP(CInput, CEdit)
	//{{AFX_MSG_MAP(CInput)
	ON_WM_KEYDOWN()
	ON_WM_KILLFOCUS()
	ON_WM_SETFOCUS()
	ON_WM_CONTEXTMENU()
	ON_WM_CHAR()
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_DESTNICK_POPUP, OnDestnickPopup)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_CTLCOLOR_REFLECT()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_SIZEPARENT, OnSizeParent)
	ON_COMMAND_RANGE(ID_DESTNICK, ID_DESTNICK_LAST, OnDestNickSel)
	ON_COMMAND(ID_INPUT_HELP, OnInputHelp)
END_MESSAGE_MAP()

// CInput bZ[W nh

LRESULT CInput::OnSizeParent(WPARAM wParam, LPARAM lParam)
{
	AFX_SIZEPARENTPARAMS* pSize = (AFX_SIZEPARENTPARAMS*)lParam;
	if (pSize->hDWP != NULL) {
		CRect rc(&pSize->rect);
		rc.top = rc.bottom - m_nHeight;
		MoveWindow(&rc);
	}
	pSize->rect.bottom -= (m_nHeight + m_cyBorder);
	return 0;
}

void CInput::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar) {
	case VK_UP:
		KeyUp();
		break;
	case VK_DOWN:
		KeyDown();
		break;
	case VK_TAB:
		m_pFrame->KeyTab();
		break;
	default:
		//TRACE("CInput::OnKeyDown %02X %c\n", nChar, nChar);
		CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
	}
}

#define CTRL(c)		(c - '@')

void CInput::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar) {
	case CTRL('P'):
		KeyUp();
		break;
	case CTRL('N'):
		KeyDown();
		break;
	case CTRL('K'):
		PopupNicks();
		break;
	case CTRL('U'):
	case CTRL('[')://ESC
		ClearText();
		break;
	case CTRL('M')://ENTER
		KeyEnter();
		break;
	case CTRL('B'):
		SetAttribute(F_BOLD);
		break;
	case CTRL('R'):
		SetAttribute(F_REVERSE);
		break;
	case CTRL('L'):
		SetAttribute(F_UNDERLINE);
		break;
	case '\t':
		break;
	default:
		//TRACE("CInput::OnChar %02X %c\n", nChar, nChar);
		CEdit::OnChar(nChar, nRepCnt, nFlags);
	}
}

void CInput::OnKillFocus(CWnd* pNewWnd) 
{
	GetSel(m_nSelBgn, m_nSelEnd);
	CEdit::OnKillFocus(pNewWnd);
	m_fFocus = FALSE;
}

void CInput::OnSetFocus(CWnd* pOldWnd) 
{
	CEdit::OnSetFocus(pOldWnd);
	SetSel(m_nSelBgn, m_nSelEnd);
	m_fFocus = TRUE;
}

void CInput::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	MenuPopup(&point);
}

void CInput::OnDestNickSel(UINT nID)
{
	SelectNick(nID - ID_DESTNICK);
}

void CInput::OnInputHelp() 
{
	AfxGetApp()->WinHelp(0x20000UL + IDR_INPUT, HELP_CONTEXT);
}

void CInput::OnEditPaste() 
{
	m_pFrame->SendMessage(WM_COMMAND, ID_EDIT_PASTE);
}

void CInput::OnDestnickPopup()
{
	PopupNicks();
}

int CInput::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CEdit::OnCreate(lpCreateStruct) == -1)
		return -1;

	SetFont(&g_fonts.m_fontInput);

	m_hIMC = ImmCreateContext();
	if (m_hIMC != NULL) {
		m_hIMCSave = ImmAssociateContext(m_hWnd, m_hIMC);
		ImmSetOpenStatus(m_hIMC, g_opt.fImeOn);
	}

	return 0;
}

void CInput::OnDestroy() 
{
	if (m_hIMCSave)
		ImmAssociateContext(m_hWnd, m_hIMCSave);
	if (m_hIMC)
		ImmDestroyContext(m_hIMC);

	CEdit::OnDestroy();
}

HBRUSH CInput::CtlColor(CDC* pDC, UINT nCtlColor) 
{
	FontColor& fc = g_fonts.GetFontColor(Fonts::TypeText);
	pDC->SetTextColor(fc.m_color);
	pDC->SetBkColor(g_fonts.m_crBack);

	return m_hBrushBack;
}
