// NickList.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "mchat.h"
#include "nicklist.h"
#include "input.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IMAGE_SIZE		16
#define COLUMN_MARGIN	2

#define NICK_IMAGEMAX		2

#define DRAG_NONE	0
#define DRAG_KICK	1
#define DRAG_INVITE	2
#define DRAG_BAN	3

#define DRAG_IMAGE_KICK		2
#define DRAG_IMAGE_INVITE	3
#define DRAG_IMAGE_BAN		4

static CImageList g_ilNick;
static CImageList g_ilDrag;

void NickListInit()
{
	g_ilNick.Create(IMAGE_SIZE, IMAGE_SIZE, TRUE, NICK_IMAGEMAX, 2);
	g_ilNick.Add(theApp.LoadIcon(IDI_NICKNORMAL));
	g_ilNick.Add(theApp.LoadIcon(IDI_NICKOP));

	g_ilDrag.Create(IDB_NICKDRAG, IMAGE_SIZE, 2, COLORREF(0x00ffffff));
}

static const CString& GetUserCol(const CUser* pUser, int nCol)
{
	static CString sNull("?");

	if (nCol >= g_opt.arNLItem.GetSize())
		return sNull;
	switch (g_opt.arNLItem[nCol]) {
	case NL_NICK:
		return pUser->m_sNick;
	case NL_ADDR:
		return pUser->m_sAddr;
	case NL_NAME:
		return pUser->m_sRealName;
	case NL_SERV:
		return pUser->m_sServer;
	default:
		ASSERT(FALSE);
		return sNull;
	}
}

// CNickList

IMPLEMENT_DYNCREATE(CNickList, CListCtrl)

CNickList::CNickList()
{
	m_fDragging = FALSE;
}

CNickList::~CNickList()
{
}

void CNickList::Reset()
{
	DeleteAllItems();

	CString sNick;
	CUser* pUser;
	POSITION pos = mapNick.GetStartPosition();
	while (pos) {
		mapNick.GetNextAssoc(pos, sNick, pUser);
		delete pUser;
	}
	mapNick.RemoveAll();
}

BOOL CNickList::Exists(LPCTSTR pNick)
{
	CUser* pUser;
	return mapNick.Lookup(pNick, pUser);
}

BOOL CNickList::GetFocusedNick(CString& sNick)
{
	int iItem = GetNextItem(-1, LVNI_FOCUSED);
	if (iItem < 0)
		return FALSE;
	CUser* pUser = (CUser*)GetItemData(iItem);
	sNick = pUser->m_sNick;
	return TRUE;
}

BOOL CNickList::GetSelNickAddr(CString& sNick, CString& sAddr)
{
	int iItem = GetNextItem(-1, LVNI_SELECTED);
	if (iItem < 0)
		return FALSE;
	CUser* pUser = (CUser*)GetItemData(iItem);
	sNick = pUser->m_sNick;
	sAddr = pUser->m_sAddr;
	return TRUE;
}

BOOL CNickList::GetSelPrefix(CString& sPrefix)
{
	int iItem = GetNextItem(-1, LVNI_SELECTED);
	if (iItem < 0)
		return FALSE;
	CUser* pUser = (CUser*)GetItemData(iItem);
	sPrefix = pUser->m_sPrefix;
	return TRUE;
}

int CNickList::GetSelectedNames(CStringArray& ar)
{
	ar.RemoveAll();
	int iItem = -1;
	while ((iItem = GetNextItem(iItem, LVNI_SELECTED)) >= 0) {
		CUser* pUser = (CUser*)GetItemData(iItem);
		if (!pUser->m_sNick.IsEmpty())
			ar.Add(pUser->m_sNick);
	}
	return ar.GetSize();
}

void CNickList::Invite(LPCTSTR psTarget)
{
	CStringArray ar;
	GetSelectedNames(ar);
	int i;
	for (i = 0; i < ar.GetSize(); i++) {
		CString& sNick = ar[i];
		Cmd("INVITE", sNick, psTarget);
	}
}

void CNickList::Kick()
{
	int iItem = -1;
	while ((iItem = GetNextItem(iItem, LVNI_SELECTED)) >= 0) {
		CUser* pUser = (CUser*)GetItemData(iItem);
		if (!pUser->m_sNick.IsEmpty())
			CmdKick(m_pFrame->m_sTarget, pUser->m_sNick);
	}
}

void CNickList::KickBan()
{
	CStringArray ar;

	int i;
	int iItem = -1;
	while ((iItem = GetNextItem(iItem, LVNI_SELECTED)) >= 0) {
		CUser* pUser = (CUser*)GetItemData(iItem);
		CString sAddr(pUser->Hostname());
		if (sAddr.IsEmpty())
			continue;
		bool bDup = false;
		for (i = 0; i < ar.GetSize(); i++) {
			if (ar[i] == sAddr) {
				bDup = true;
				break;
			}
		}
		if (!bDup)
			ar.Add(sAddr);
	}

	for (i = 0; i < ar.GetSize(); i++)
		CmdBan(m_pFrame->m_sTarget, (LPCTSTR)(CString("*!*@") + ar[i]));

	Kick();
}

int CNickList::GetIndex(LPCTSTR pNick, CUser*& pUser)
{
	if (!mapNick.Lookup(pNick, pUser)) {
		pUser = 0;
		return -1;
	}
	LV_FINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)pUser;
	return FindItem(&find);
}

void CNickList::Add(const CUser& user)
{
	//TRACE("nick add (%s) (%s)\n", pNick, pAddr);
	CUser* pUser = new CUser(user);
	mapNick.SetAt(pUser->m_sNick, pUser);

	LV_ITEM item;
	item.mask = LVIF_TEXT | LVIF_PARAM | LVIF_IMAGE;
	item.iItem = 0;
	item.iSubItem = 0;
	item.state = 0;
	item.stateMask = 0;
	item.pszText = LPSTR_TEXTCALLBACK;
	item.iImage = I_IMAGECALLBACK;
	item.lParam = (LPARAM)pUser;
	int iItem = InsertItem(&item);
	if (iItem < 0) {
		ASSERT(FALSE);
		return;
	}

	int i;
	for (i = 0; i < m_arWidth.GetSize(); i++) {
		int cx = GetStringWidth(GetUserCol(pUser, i));
		if (i == 0)
			cx += (IMAGE_SIZE + COLUMN_MARGIN);
		if (cx > m_arWidth[i])
			m_arWidth[i] = cx;
	}
}

void CNickList::Add(LPCTSTR pNick, LPCTSTR pAddr)
{
	CUser user(pNick, pAddr);
	Add(user);
}

BOOL CNickList::Del(LPCTSTR pNick)
{
	CUser* pUser;
	int iItem = GetIndex(pNick, pUser);
	if (iItem >= 0)
		DeleteItem(iItem);
	if (pUser) {
		mapNick.RemoveKey(pNick);
		delete pUser;
		return TRUE;
	}
	return FALSE;
}

BOOL CNickList::Rename(LPCTSTR pNick, LPCTSTR pNickTo)
{
	BOOL r;
	CUser* pUser;
	int iItem = GetIndex(pNick, pUser);
	if (pUser) {
		pUser->m_sNick = pNickTo;
		r = TRUE;
	} else
		r = FALSE;
	mapNick.RemoveKey(pNick);
	mapNick.SetAt(pNickTo, pUser);
	if (iItem >= 0)
		Update(iItem);
	return r;
}

BOOL CNickList::Modify(const CUser& user)
{
	CUser* pUser;
	int iItem = GetIndex(user.m_sNick, pUser);
	if (pUser) {
		*pUser = user;
		if (iItem >= 0)
			Update(iItem);
	} else
		Add(user);
	return TRUE;
}

void CNickList::ModeOP(LPCTSTR pNick, BOOL f)
{
	CUser* pUser;
	int iItem = GetIndex(pNick, pUser);
	if (!pUser)
		return;
	if (pUser->m_fOp != f) {
		pUser->m_fOp = f;
		if (iItem >= 0)
			Update(iItem);
	}
}

void CNickList::ModeAutoOP()
{
	ModeCmd cmd(m_pFrame->m_sTarget);

	CString sNick;
	CUser* pUser;
	POSITION pos = mapNick.GetStartPosition();
	while (pos) {
		mapNick.GetNextAssoc(pos, sNick, pUser);
		if (pUser->m_fOp)
			continue;
		if (m_pFrame->m_lsOpPlus.IsMatch(pUser->m_sPrefix))
			cmd.Add(TRUE, 'o', sNick);
	}
	cmd.Flush();
}

void CNickList::Redraw(BOOL fOn)
{
	m_fRedraw = fOn;
	if (!m_fRedraw) {
		SetRedraw(FALSE);
		return;
	}
	SetItemState(0, LVIS_FOCUSED, LVIS_FOCUSED);
	int i;
	for (i = 0; i < m_arWidth.GetSize(); i++) {
		int cx = m_arWidth[i];
		if (cx > GetColumnWidth(i))
			SetColumnWidth(i, cx);
	}
	SetRedraw(TRUE);
	UpdateWindow();
}

void CNickList::ShowMenu()
{
	int idx = GetTopIndex() - 1;
	int iFocus = GetNextItem(idx, LVNI_FOCUSED);
	if (iFocus > 0)
		idx = iFocus;
	CPoint pt;
	if (idx < 0 || !GetItemPosition(idx, &pt))
		pt = CPoint(0, 0);
	ClientToScreen(&pt);
	//TRACE("idx=%d pt=%d,%d\n", idx, pt.x, pt.y);
	m_pFrame->MenuOnNames(pt);
}

CUser* CNickList::ClientPtToUser(const CPoint& pt)
{
	UINT flag;
	int iItem = HitTest(pt, &flag);
	if (iItem == -1)
		return 0;
	return (CUser*)GetItemData(iItem);
}

BEGIN_MESSAGE_MAP(CNickList, CListCtrl)
	//{{AFX_MSG_MAP(CNickList)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT(LVN_BEGINDRAG, OnBegindrag)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetdispinfo)
	ON_WM_KILLFOCUS()
	ON_NOTIFY_REFLECT(NM_CLICK, OnClick)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_WM_MEASUREITEM()
	ON_WM_CHAR()
	ON_WM_KEYDOWN()
	ON_WM_DROPFILES()
	ON_NOTIFY_REFLECT(NM_RETURN, OnReturn)
	ON_WM_DRAWITEM()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CNickList message handlers

BOOL CNickList::PreCreateWindow(CREATESTRUCT& cs) 
{
	static char psClass[] = WC_LISTVIEW;
	cs.lpszClass = psClass;
	cs.style = WS_CHILD | WS_VISIBLE |
		LVS_ALIGNLEFT | LVS_REPORT |
		LVS_SHAREIMAGELISTS | LVS_OWNERDRAWFIXED;

	return CListCtrl::PreCreateWindow(cs);
}

int CNickList::OnCreate(LPCREATESTRUCT lpCS) 
{
	if (CListCtrl::OnCreate(lpCS) == -1)
		return -1;

	Redraw(FALSE);

	m_arWidth.RemoveAll();
	m_arWidth.SetSize(g_opt.arNLItem.GetSize());

	SetImageList(&g_ilNick, LVSIL_SMALL);

	int i;
	for (i = 0; i < g_opt.arNLItem.GetSize(); i++) {
		int nNLItem = g_opt.arNLItem[i];
		int nWidth;
		LPCTSTR pLabel = GetNLString(nNLItem, &nWidth);
		InsertColumn(i, pLabel, LVCFMT_LEFT, nWidth, i);
		m_arWidth[i] = nWidth;
	}

	m_pFrame = (FrameChan*)GetParentFrame();

	DragAcceptFiles(TRUE);

	return 0;
}

void CNickList::OnDestroy() 
{
	DragAcceptFiles(FALSE);
	Reset();

	CListCtrl::OnDestroy();
}

void CNickList::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	ASSERT(!m_fDragging);

	int nSel = GetSelectedCount();
	if (nSel < 1)
		return;

	m_fDragging = TRUE;
	m_nDragOperation = DRAG_NONE;
	m_sInviteTarget.Empty();
	m_iDragImage = nSel > 1 ? 1 : 0;
	CPoint ptItem;
	GetItemPosition(pNMListView->iItem, &ptItem);
	g_ilDrag.DragShowNolock(TRUE);
	g_ilDrag.SetDragCursorImage(m_iDragImage, CPoint(0, 0));
	ShowCursor(FALSE);
	g_ilDrag.BeginDrag(m_iDragImage, CPoint(0, 0));
	CPoint pt = pNMListView->ptAction;
	ClientToScreen(&pt);
	g_ilDrag.DragEnter(NULL, pt);

	SetCapture();
	*pResult = 0;
}

void CNickList::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_fDragging)
	{
		CPoint pt(point);
		ClientToScreen(&pt);

		CRect rc;
		m_pFrame->GetWindowRect(&rc);
		int iImage;
		int nOperation;
		if (rc.PtInRect(pt)) {
			nOperation = DRAG_NONE;
			iImage = m_iDragImage;
		} else {
			CFrame* pFrame = (CFrame*)WindowFromPoint(pt);
			if (!pFrame->IsKindOf(RUNTIME_CLASS(CFrame)))
				pFrame = (CFrame*)pFrame->GetParentFrame();
			if (pFrame && pFrame->IsKindOf(RUNTIME_CLASS(CFrame))
					&& pFrame->GetType() == CFrame::TypeChan) {
				nOperation = DRAG_INVITE;
				iImage = DRAG_IMAGE_INVITE;
				m_sInviteTarget = pFrame->m_sTarget;
			} else if ((GetKeyState(VK_CONTROL) & 0x8000) != 0) {
				nOperation = DRAG_BAN;
				iImage = DRAG_IMAGE_BAN;
			} else {
				nOperation = DRAG_KICK;
				iImage = DRAG_IMAGE_KICK;
			}
		}
		if (nOperation != m_nDragOperation) {
			m_nDragOperation = nOperation;
			g_ilDrag.DragLeave(NULL);
			g_ilDrag.EndDrag();
			g_ilDrag.SetDragCursorImage(iImage, CPoint(0, 0));
			g_ilDrag.BeginDrag(iImage, CPoint(0, 0));
			g_ilDrag.DragEnter(NULL, pt);
		}
		g_ilDrag.DragMove(pt);
	}

	CListCtrl::OnMouseMove(nFlags, point);
}

void CNickList::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_fDragging) {
		m_fDragging = FALSE;
		g_ilDrag.DragLeave(NULL);
		g_ilDrag.EndDrag();
		::ReleaseCapture();
		ShowCursor(TRUE);

		if (m_nDragOperation != DRAG_NONE) {
			switch (m_nDragOperation) {
			case DRAG_KICK:
				Kick();
				break;
			case DRAG_BAN:
				KickBan();
				break;
			case DRAG_INVITE:
				Invite(m_sInviteTarget);
				break;
			}
		}
	}
	CListCtrl::OnLButtonUp(nFlags, point);
}

void CNickList::OnKillFocus(CWnd* pNewWnd) 
{
	if (m_fDragging) {
		m_fDragging = FALSE;
		g_ilDrag.DragLeave(NULL);
		g_ilDrag.EndDrag();
		::ReleaseCapture();
		ShowCursor(TRUE);
	}
	CListCtrl::OnKillFocus(pNewWnd);
}

#define DT_FLAGS	(DT_LEFT|DT_NOPREFIX|DT_SINGLELINE|DT_VCENTER)

void CNickList::DrawItem(LPDRAWITEMSTRUCT lpDIS)
{
	CDC *pDC = CDC::FromHandle(lpDIS->hDC);
	COLORREF crText, crBack;
	if ((lpDIS->itemState & ODS_SELECTED) != 0) {
		crText = GetSysColor(COLOR_HIGHLIGHTTEXT);
		crBack = GetSysColor(COLOR_HIGHLIGHT);
	} else {
		crText = g_fonts.GetFontColor(Fonts::TypeNickList).m_color;
		crBack = g_fonts.m_crBack;
	}
	COLORREF crTextOld = pDC->SetTextColor(crText);
	COLORREF crBackOld = pDC->SetBkColor(crBack);
	pDC->SetBkMode(OPAQUE);

	CRect rc = lpDIS->rcItem;
	pDC->FillSolidRect(&rc, crBack);
	int iItem = lpDIS->itemID;

	CUser* pUser = (CUser*)GetItemData(iItem);
	int nImage = pUser->m_fOp ? 1 : 0;
	CPoint pt(rc.left, rc.top);
	g_ilNick.Draw(pDC, nImage, pt, ILD_NORMAL);
	rc.left += IMAGE_SIZE;

	int i;
	for (i = 0; i < g_opt.arNLItem.GetSize(); i++) {
		int nWidth = GetColumnWidth(i);
		if (i == 0)
			nWidth -= IMAGE_SIZE;
		rc.right = rc.left + nWidth;
		pDC->DrawText(GetUserCol(pUser, i), &rc, DT_FLAGS);
		rc.left = rc.right;
	}

	if (lpDIS->itemState & ODS_FOCUS)
		pDC->DrawFocusRect(&lpDIS->rcItem);

	pDC->SetTextColor(crTextOld);
	pDC->SetBkColor(crBackOld);
}

int CALLBACK UserSortFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CUser* p1 = (CUser*)lParam1;
	CUser* p2 = (CUser*)lParam2;

	int r = lstrcmp(GetUserCol(p1, lParamSort), GetUserCol(p2, lParamSort));
	if (r == 0 && lParamSort != 0)
		r = lstrcmp(GetUserCol(p1, 0), GetUserCol(p2, 0));
	return r;
}

void CNickList::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMLV = (NM_LISTVIEW*)pNMHDR;
	SortItems(UserSortFunc, (LPARAM)(pNMLV->iSubItem));
	*pResult = 0;
	m_pFrame->InputSetFocus();
}

void CNickList::OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
	LV_ITEM& item = pDispInfo->item;
	CUser* pUser = (CUser*)item.lParam;
	if (!pUser)
		return;
	if ((item.mask & LVIF_TEXT) != 0 && item.pszText)
		lstrcpyn(item.pszText, GetUserCol(pUser, item.iSubItem), item.cchTextMax);
	if (item.mask & LVIF_IMAGE)
		item.iImage = pUser->m_fOp ? 1 : 0;
	*pResult = 0;
}

void CNickList::OnClick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint pt(GetMessagePos());
	ScreenToClient(&pt);
	CUser* pUser = ClientPtToUser(pt);
	if (pUser != 0 && pUser->m_sServer.IsEmpty())
		CmdWho(m_pFrame->m_sTarget);
	*pResult = 0;
}

void CNickList::OnDblclk(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint pt(GetMessagePos());
	ScreenToClient(&pt);
	CUser* pUser = ClientPtToUser(pt);
	if (pUser != 0) {
		CString& sNick = pUser->m_sNick;
		if (!sNick.IsEmpty())
			CmdWhois(sNick);
	}
	*pResult = 0;
}

void CNickList::OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMIS) 
{
	CClientDC dc(this);
	FontColor& fc = g_fonts.GetFontColor(Fonts::TypeNickList);
	CFont* pFont = dc.SelectObject(&fc.m_font);
	TEXTMETRIC tm;
	if (dc.GetTextMetrics(&tm))
		lpMIS->itemHeight = tm.tmHeight + tm.tmExternalLeading;
	if (pFont)
		dc.SelectObject(pFont);
	CListCtrl::OnMeasureItem(nIDCtl, lpMIS);
}

void CNickList::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_TAB) {
		m_pFrame->KeyTab();
		return;
	}
	CListCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CNickList::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == 0x1b) { //esc
		m_pFrame->KeyTab();
		return;
	}
	if (nChar == ' ' && !IsKeyCtrl()) {
		ShowMenu();
		return;
	}
	CListCtrl::OnChar(nChar, nRepCnt, nFlags);
}

void CNickList::OnDropFiles(HDROP hDropInfo) 
{
	CPoint pt;
	if (DragQueryPoint(hDropInfo, &pt)) {
		CUser* pUser = ClientPtToUser(pt);
		if (pUser)
			CmdDccSend(pUser->m_sNick, hDropInfo);
	}
	DragFinish(hDropInfo);
}

void CNickList::OnReturn(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CString sNick;
	if (GetFocusedNick(sNick))
		m_pFrame->m_pInput->InsertNick(sNick);
	m_pFrame->m_pInput->SetFocus();
	*pResult = 0;
}

// CNickWnd

IMPLEMENT_DYNCREATE(CNickWnd, CWnd)

CNickWnd::CNickWnd()
{
}

CNickWnd::~CNickWnd()
{
}

BEGIN_MESSAGE_MAP(CNickWnd, CWnd)
	//{{AFX_MSG_MAP(CNickWnd)
	ON_WM_CREATE()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CNickWnd message handlers

#define STYLE	\
	(WS_CHILD|WS_VISIBLE|LVS_ALIGNLEFT|LVS_REPORT|LVS_SHAREIMAGELISTS|LVS_SORTASCENDING)

int CNickWnd::OnCreate(LPCREATESTRUCT lpCS) 
{
	if (CWnd::OnCreate(lpCS) == -1)
		return -1;

	CRect rc(0, 0, lpCS->cx, lpCS->cy);
	if (!m_list.Create(STYLE, rc, this, IDW_NICKLIST))
		return -1;
	return 0;
}

void CNickWnd::ReCreate()
{
	CRect rc;
	GetClientRect(&rc);
	m_list.DestroyWindow();
	m_list.Create(STYLE, rc, this, IDW_NICKLIST);
	FontChange();
	CmdWho(m_list.m_pFrame->m_sTarget);
}

void CNickWnd::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);

	m_list.MoveWindow(0, 0, cx, cy);
}

void CNickWnd::FontChange()
{
	FontColor& fc = g_fonts.GetFontColor(Fonts::TypeNickList);
	m_list.SetBkColor(g_fonts.m_crBack);
	m_list.SetFont(&fc.m_font);
}
