// Text.cpp : implementation file

#include "stdafx.h"
#include "mchat.h"
#include "text.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

inline BOOL IsUrlChar(LPCTSTR pStr, int nCol)
{
	return _mbsbtype((const BYTE*)pStr, nCol) == _MBC_SINGLE
		&& pStr[nCol] > ' ';
}

BOOL IsUrl(LPCTSTR pStr, int nLen)
{
	const BYTE* pByte = (const BYTE*)pStr;
	int nTag = 0;
	while (nLen > 0 && isascii(*pByte) && isalpha(*pByte)) {
		pByte++;
		nTag++;
		nLen--;
	}
	int nColon = 0;
	while (nLen > 0 && isascii(*pByte) && *pByte == ':') {
		pByte++;
		nColon++;
		nLen--;
	}
	int nSlash = 0;
	while (nLen > 0 && *pByte == '/') {
		pByte++;
		nSlash++;
		nLen--;
	}
	return nTag > 0 && nColon == 1 && nSlash == 2 && nLen > 0;
}

inline int NextUrl(LPCTSTR pStr, int nCol, int& nStart)
{
	for (;;) {
		while (pStr[nCol] != 0 && !IsUrlChar(pStr, nCol))
			nCol++;
		if (pStr[nCol] == 0)
			break;
		nStart = nCol;
		while (pStr[++nCol] != 0) {
			if (!IsUrlChar(pStr, nCol))
				break;
		}
		int nLen = nCol - nStart;
		if (IsUrl(pStr + nStart, nLen))
			return nLen;
	}
	return 0;
}

BOOL ExtractURL(LPCTSTR pStr, int nCol, CString& sResult)
{
	int nStart = nCol;
	while (nStart > 0 && IsUrlChar(pStr, nStart - 1))
		nStart--;
	int nEnd = nCol;
	while (pStr[nEnd] != 0 && IsUrlChar(pStr, nEnd))
		nEnd++;
	int nLen = nEnd - nStart;
	if (nLen <= 0)
		return FALSE;
	if (!IsUrl(pStr + nStart, nEnd - nStart))
		return FALSE;
	sResult = CString(pStr + nStart, nEnd - nStart);
	return TRUE;
}

#define MAX_SCROLLRANGE		10000
#define MARGIN				2
#define MIN_CLIENT_WIDTH	20
#define EMPTY_LINE_HEIGHT	10

inline void SetAttBytes(BYTE* pAt, int nStart, int nEnd, BYTE bAt)
{
	int i;
	for (i = nStart; i < nEnd; i++)
		pAt[i] |= bAt;
}

inline void SetAttBytes(BYTE* pAt, int nLen, BYTE bAt)
{
	while (nLen-- > 0)
		*pAt++ = bAt;
}

// CLine

CLine::CLine(LPCTSTR pszNick, LPCTSTR pszSrc, Fonts::FType type)
{
	m_fKeyword = FALSE;
	m_type = type;
	m_nLines = 1;
	CString sLine;

	int nTime;
	int nPreText;
	BOOL fAt = FALSE;
	if (g_opt.fMsgTime) {
		fAt = TRUE;
		CString sTime;
		sTime = CTime::GetCurrentTime().Format(g_opt.sMsgTime);
		nTime = sTime.GetLength();
		nPreText = nTime + 1;
		sLine = sTime + ' ';
	} else {
		nPreText = nTime = 0;
	}

	int nNickFrom = nPreText;
	int nNick = 0;
	if (pszNick) {
		fAt = TRUE;
		sLine += pszNick;
		nNick = lstrlen(pszNick);
		nPreText += nNick;
	}
	sLine += pszSrc;

	int nLen = sLine.GetLength();
	LPCTSTR pLine = sLine;
	m_pbAt = new BYTE[nLen];
	LPTSTR pDst = m_sStr.GetBuffer(nLen);

	if (nPreText) {
		memcpy(pDst, pLine, nPreText * sizeof (pDst[0]));
		memset(m_pbAt, F_TIME, nTime);
		m_pbAt[nTime] = 0;
		memset(m_pbAt + nNickFrom, F_NICK, nNick);
	}

	BYTE bAt = 0;
	LPCTSTR pSrc = pLine + nPreText;
	int iDst = nPreText;
	char c;
	while ((c = *pSrc++) != 0) {
		switch(c) {
		case CHAR_BOLD:
			fAt = TRUE;
			if (bAt & F_BOLD)
				bAt &= ~F_BOLD;
			else
				bAt |= F_BOLD;
			continue;
		case CHAR_UNDERLINE:
			fAt = TRUE;
			if (bAt & F_UNDERLINE)
				bAt &= ~F_UNDERLINE;
			else
				bAt |= F_UNDERLINE;
			continue;
		case CHAR_REVERSE:
			fAt = TRUE;
			if (bAt & F_REVERSE)
				bAt &= ~F_REVERSE;
			else
				bAt |= F_REVERSE;
			continue;
		}
		ASSERT(iDst < nLen);
		pDst[iDst] = c;
		m_pbAt[iDst++] = bAt;
	}

	int nStart;
	int nWord;
	int nCol = nPreText;
	while ((nWord = NextUrl(pDst, nCol, nStart)) > 0) {
		SetAttBytes(m_pbAt, nStart, nStart + nWord, F_UNDERLINE);
		nCol = nStart + nWord;
		fAt = TRUE;
	}
	if (nNick) {
		nCol = nPreText;
		while ((nWord = g_arKeyword.Search(pDst, nCol, nStart)) > 0) {
			SetAttBytes(m_pbAt, nStart, nStart + nWord, F_KEYWORD);
			nCol = nStart + nWord;
			fAt = TRUE;
			m_fKeyword = TRUE;
		}
		//if (m_fKeyword)
		//	optBeep.Play(BEEP_Keyword);
	}

	m_sStr.ReleaseBuffer(iDst);
	if (!fAt) {
		delete [] m_pbAt;
		m_pbAt = 0;
	}
}

CLine::~CLine()
{
	if (m_pbAt)
		delete [] m_pbAt;
}

// count chars of same attributes
int CLine::SameChars(CDC& dc, int nCol, int nLen, BYTE* pAt)
{
	ASSERT(nCol >= 0 && nCol + nLen <= m_sStr.GetLength());

	if (!m_pbAt) {
		g_fonts.Select(dc, m_type);
		if (pAt)
			*pAt = 0;
		return nLen;
	}

	int nSameChars = 1;
	BYTE* pbAt = m_pbAt + nCol;
	BYTE bAt;
	for (bAt = *pbAt++; nSameChars < nLen && bAt == *pbAt; pbAt++)
		nSameChars++;

	if (bAt & F_TIME)
		g_fonts.Select(dc, Fonts::TypeTime);
	else if (bAt & F_NICK)
		g_fonts.Select(dc, Fonts::TypeNick);
	else if (bAt & F_KEYWORD)
		g_fonts.Select(dc, Fonts::TypeKeyword);
	else if ((bAt & (F_BOLD|F_UNDERLINE)) == (F_BOLD|F_UNDERLINE)
			&& (HFONT)g_fonts.m_fontBoldUL)
		g_fonts.Select(dc, m_type, &g_fonts.m_fontBoldUL);
	else if ((bAt & F_BOLD) && (HFONT)g_fonts.m_fontBold)
		g_fonts.Select(dc, m_type, &g_fonts.m_fontBold);
	else if ((bAt & F_UNDERLINE) && (HFONT)g_fonts.m_fontUL)
		g_fonts.Select(dc, m_type, &g_fonts.m_fontUL);
	else if (g_opt.fKeywordWholeLine && HasKeyword())
		g_fonts.Select(dc, Fonts::TypeKeyword);
	else
		g_fonts.Select(dc, m_type);

	if (pAt)
		*pAt = bAt;
	return nSameChars;
}

// get chars and height to fit in width
int CLine::FitInWidth(CDC& dc, int nWidth, int nCol, int nLen, int& nHeight)
{
	int nCharsFit = 0;
	LPCTSTR pStr = (LPCTSTR)m_sStr + nCol;
	nHeight = 0;
	SIZE size;

	while (nLen > 0) {
		int nSameChars = SameChars(dc, nCol, nLen);
		int nChars;
		if (!GetTextExtentExPoint(dc.m_hDC,
				pStr, nSameChars, nWidth,
				&nChars, NULL, &size)) {
			TRACE("GetTextExtentExPoint: 0x%X\n", GetLastError());
			nChars = nLen;
		}
		if (size.cy > nHeight)
			nHeight = size.cy;

		if (nChars > nSameChars)
			nChars = nSameChars;
		else if (nChars > 0 && _mbsbtype((BYTE*)pStr, nChars - 1) == _MBC_LEAD)
			nChars--;
		if (nChars <= 0)
			break;

		nCharsFit += nChars;
		if (nChars < nSameChars || (nWidth -= size.cx) <= MARGIN)
			break;

		nCol += nChars;
		pStr += nChars;
		nLen -= nChars;
	}

	return nCharsFit;
}

// calc #of-lines and height
// update ordinate for next line
void CLine::SetDim(CDC& dc, int nWidth, int &nOrd)
{
	m_nOrdinate = nOrd;
	m_nLines = 1;
	int nLen = m_sStr.GetLength();
	if (nLen < 1) {
		nOrd += (m_nHeight = EMPTY_LINE_HEIGHT);
		return;
	}
	int nCol = 0;
	m_nHeight = 0;
	nWidth -= MARGIN;
	while (nLen > 0) {
		int nHeight;
		int nChars = FitInWidth(dc, nWidth, nCol, nLen, nHeight);
		m_nHeight += nHeight;
		if (!nChars || nChars >= nLen)
			break;
		m_nLines++;
		nCol += nChars;
		nLen -= nChars;
	}
	nOrd += m_nHeight;
}

void CLine::Draw(CDC& dc, CRect& rc, int nCol, int nLen)
{
	//TRACE("CLine::Draw (%d,%d)-(%d,%d)\n", rc.left, rc.top, rc.right, rc.bottom);

	dc.FillSolidRect(&rc, g_fonts.m_crBack);
	rc.left += MARGIN;
	int nMaxRight = rc.right;
	LPCTSTR pStr = (LPCTSTR)m_sStr + nCol;
	for (;;) {
		BYTE bAt;
		int nSameChars = SameChars(dc, nCol, nLen, &bAt);
		int cx = dc.GetOutputTextExtent(pStr, nSameChars).cx;
		dc.ExtTextOut(rc.left, rc.bottom, ETO_OPAQUE,
			&rc, pStr, nSameChars, NULL);
		if (bAt & F_REVERSE) {
			CRect rcRev(rc.left, rc.top, rc.left + cx, rc.bottom);
			dc.InvertRect(&rcRev);
		}
		rc.left += cx;
		if ((nLen -= nSameChars) <= 0)
			break;
		nCol += nSameChars;
		pStr += nSameChars;
	}
}

int CLine::Extent(CDC& dc, int nCol, int nLen)
{
	int nWidth = 0;
	while (nLen > 0) {
		int nChars = SameChars(dc, nCol, nLen);
		SIZE sz = dc.GetOutputTextExtent((LPCTSTR)m_sStr + nCol, nChars);
		nWidth += sz.cx;
		nCol += nChars;
		nLen -= nChars;
	}
	return nWidth;
}

int CLine::GetCol(CDC& dc, int nWidth, int cx, int cy)
{
	LPCTSTR pStr = m_sStr;
	int nLen = m_sStr.GetLength();
	int nCol = 0;
	int nHeightSum = 0;
	int nHeight;
	while (nLen > 0) {
		int nChars = FitInWidth(dc, nWidth, nCol, nLen, nHeight);
		if (cy <= (nHeightSum += nHeight))
			break;
		if (nChars <= 0)
			break;
		nLen -= nChars;
		pStr += nChars;
		nCol += nChars;
	}
	if (cx < 0)
		cx = 0;
	return nCol + FitInWidth(dc, cx, nCol, nLen, nHeight);
}

IMPLEMENT_DYNCREATE(CText, CWnd)

// CText

CText::CText()
{
	m_iViewFrom = m_iViewTo = 0;
	m_nClientWidth = m_nClientHeight = 
		m_nCharHeight = m_nCharWidth = m_nOrdMax = 0;
	m_nScrollRange = 1;
	m_fCapture = FALSE;

	m_aLine.SetSize(0, 100);

	m_nWheelDelta = 0;

	m_pFrame = NULL;
}

CText::~CText()
{
}

BOOL CText::Create(LPCTSTR lpszClass,
	LPCTSTR lpszWindowName, DWORD dwStyle,
	const RECT& rect,
	CWnd* pParentWnd, UINT nID,
	CCreateContext* pContext)
{
	m_pParent = pParentWnd;

	if (!CWnd::Create(lpszClass, lpszWindowName, dwStyle,
			rect, pParentWnd, nID, pContext))
		return FALSE;

	SetParentFrame((CFrame*)GetParentFrame());

	return TRUE;
}

BOOL CText::DoCreate(CFrame* parent, UINT nID)
{
	LPCTSTR lpszClass =
		AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS,
			LoadCursor(NULL, IDC_IBEAM));

	if (!Create(lpszClass, 0,
		WS_CHILD|WS_VISIBLE|WS_VSCROLL|WS_BORDER,
		CFrameWnd::rectDefault, parent, nID))
		return FALSE;

	return TRUE;
}

void CText::SetParentFrame(CFrame* pFrame)
{
	//TRACE("SetParentFrame type=%s\n", pFrame->GetFrameName());
	m_pFrame = pFrame;
};

void CText::NewSize()
{
	if (m_nCharHeight == 0)
		return;

	RECT rc;
	GetClientRect(&rc);

	m_nClientWidth = rc.right;
	if (m_nClientWidth < MIN_CLIENT_WIDTH)
		m_nClientWidth = MIN_CLIENT_WIDTH;
	m_nClientHeight = rc.bottom;

	CClientDC dc(this);
	FSave fsave(dc, Fonts::TypeText);

	m_nOrdMax = 0;
	int i;
	for (i = 0; i < m_aLine.GetSize(); i++)
		m_aLine[i]->SetDim(dc, m_nClientWidth, m_nOrdMax);

	if (m_iViewTo >= m_aLine.GetSize())
		MoveBottom();
	else
		CalcViewTo();

	Invalidate();
}

void CText::FontChange()
{
	CClientDC dc(this);
	FSave fsave(dc, Fonts::TypeText);

	TEXTMETRIC tm;
	dc.GetTextMetrics(&tm);
	m_nCharHeight = tm.tmHeight +
		(tm.tmExternalLeading > 0 ? tm.tmExternalLeading : 1);
	m_nCharWidth = tm.tmMaxCharWidth;

	NewSize();
}

void CText::ShowLine(LPCTSTR pNick, LPCTSTR pStr, Fonts::FType type)
{
	CClientDC dc(this);
	FSave save(dc, type);
	CLine* pLine = new CLine(pNick, pStr, type);
	pLine->SetDim(dc, m_nClientWidth, m_nOrdMax);
	int iLine = m_aLine.Add(pLine);
	if (iLine <= m_iViewTo)
		MoveBottom();
	if (pLine->HasKeyword() && m_pFrame)
		m_pFrame->PostMessage(WMU_KEYWORD);
}

void CText::CalcViewTo()
{
	int nHeight = 0;
	int i;
	for (i = m_iViewFrom; i < m_aLine.GetSize(); ) {
		nHeight += m_aLine[i++]->m_nHeight;
		if (nHeight >= m_nClientHeight)
			break;
	}
	m_iViewTo = i;
}

void CText::CalcViewFrom()
{
	int nHeight = 0;
	int i = m_iViewTo;
	for (;;) {
		if (i < 1) {
			m_iViewFrom = 0;
			CalcViewTo();
			return;
		}
		nHeight += m_aLine[--i]->m_nHeight;
		if (nHeight > m_nClientHeight) {
			i++;
			break;
		}
	}
	m_iViewFrom = i;
}

void CText::MoveTop()
{
	m_iViewFrom = 0;
	CalcViewTo();
	Invalidate();
}

void CText::MoveBottom()
{
	m_iViewTo = m_aLine.GetSize();
	CalcViewFrom();
	Invalidate();
}

void CText::MovePageUp()
{
	if (m_iViewFrom < 1)
		return;
	m_iViewTo = m_iViewFrom;
	CalcViewFrom();
	Invalidate();
}

void CText::MovePageDown()
{
	if (m_iViewTo >= m_aLine.GetSize())
		return;
	m_iViewFrom = m_iViewTo;
	CalcViewTo();
	Invalidate();
}

void CText::MoveLineUp()
{
	if (m_iViewFrom > 0) {
		m_iViewFrom--;
		CalcViewTo();
		Invalidate();
	}
}

void CText::MoveLineDown()
{
	if (m_iViewFrom + 1 < m_aLine.GetSize()) {
		m_iViewFrom++;
		CalcViewTo();
		Invalidate();
	}
}

BEGIN_MESSAGE_MAP(CText, CWnd)
	//{{AFX_MSG_MAP(CText)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_VSCROLL()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_ERASEBKGND()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CText message handlers

void CText::DrawLine(CDC& dc, int nY, CLine* pLine, int iLine)
{
	//TRACE("DrawLine i=%d,Y=%d,s=%s\n", iLine, nY, pLine->m_sStr);

	int nLen = pLine->m_sStr.GetLength();
	int nLines = 1;
	int nChars;
	CRect rc;
	int nCol = 0;
	int nHeight;

	for (;;) {
		if (nLen > 0) {
			nChars = pLine->FitInWidth(dc, m_nClientWidth - MARGIN, nCol, nLen, nHeight);
			if (nChars <= 0)
				nChars = nLen;
		} else {
			nHeight = EMPTY_LINE_HEIGHT;
			nChars = 0;
		}
		rc.SetRect(0, nY, m_nClientWidth, nY + nHeight);
		pLine->Draw(dc, rc, nCol, nChars);

		if (m_fCapture && iLine >= m_iSelLineBgn && iLine <= m_iSelLineEnd) {
			int nColBgn = (iLine == m_iSelLineBgn)
				? max(nCol, m_nSelColBgn) : nCol;
			int nColEnd = (iLine == m_iSelLineEnd)
				? min(nCol + nChars, m_nSelColEnd) : nCol + nChars;

			if (nColBgn < nColEnd) {
				int cxBgn = pLine->Extent(dc, nCol, nColBgn - nCol);
				int cxEnd = pLine->Extent(dc, nCol, nColEnd - nCol);
				CRect rc(cxBgn + MARGIN, nY, cxEnd + MARGIN, nY + nHeight);
				dc.InvertRect(&rc);
			}
		}

		nY += nHeight;
		if ((nLen -= nChars) <= 0)
			break;
		nLines++;
		nCol += nChars;
	}
#ifdef _DEBUG
	if (pLine->m_nLines != nLines)
		TRACE("lines changed: %d->%d %s\n",
			pLine->m_nLines, nLines,
			(LPCTSTR)pLine->m_sStr);
#endif
	pLine->m_nLines = nLines;
}

BOOL CText::GetSelBgnEnd()
{
	if (m_iSelLine < m_iSelLine1 ||
			m_iSelLine == m_iSelLine1 &&
				m_nSelCol <= m_nSelCol1) {
		m_iSelLineBgn = m_iSelLine;
		m_nSelColBgn = m_nSelCol;
		m_iSelLineEnd = m_iSelLine1;
		m_nSelColEnd = m_nSelCol1;
	} else {
		m_iSelLineBgn = m_iSelLine1;
		m_nSelColBgn = m_nSelCol1;
		m_iSelLineEnd = m_iSelLine;
		m_nSelColEnd = m_nSelCol;
	}
	return m_iSelLine != m_iSelLine1 || m_nSelCol != m_nSelCol1;
}

void CText::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	FSave fsave(dc, Fonts::TypeText);
	COLORREF crBack = g_fonts.m_crBack;
	dc.SetBkColor(crBack);
	dc.SetTextAlign(TA_BOTTOM);
	CRect rcFill;

	if (m_fCapture)
		GetSelBgnEnd();

	RECT &rc = dc.m_ps.rcPaint;
	//dc.FillSolidRect(&rc, crBack);
	int nHeight = 0;
	int i;
	int nMaxLine = m_aLine.GetSize();
	for (i = m_iViewFrom; i < m_iViewTo; i++) {
		if (i >= nMaxLine)
			break;
		CLine* pLine = m_aLine[i];
		if (nHeight <= rc.bottom && nHeight + pLine->m_nHeight >= rc.top)
			DrawLine(dc, nHeight, pLine, i);
		nHeight += pLine->m_nHeight;
	}
	if (nHeight < rc.bottom) {
		rcFill.SetRect(rc.left, nHeight, rc.right, rc.bottom);
		dc.FillSolidRect(&rcFill, crBack);
	}

	if (nMaxLine) {
		int nNewRange = (nMaxLine <= MAX_SCROLLRANGE) ? nMaxLine : MAX_SCROLLRANGE;
		if (nNewRange != m_nScrollRange) {
			m_nScrollRange = nNewRange;
			SetScrollRange(SB_VERT, 0, m_nScrollRange, FALSE);
		}
		int nPos = (int)((long)m_iViewFrom * m_nScrollRange / nMaxLine);
		SetScrollPos(SB_VERT, nPos, TRUE);
	}
}

void CText::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	if (nType != SIZE_MINIMIZED && nType != SIZE_MAXHIDE)
		NewSize();
}

void CText::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch(nSBCode) {
	case SB_TOP:
		MoveTop();
		break;
	case SB_BOTTOM:
		MoveBottom();
		break;
	case SB_LINEUP:
		MoveLineUp();
		break;
	case SB_LINEDOWN:
		MoveLineDown();
		break;
	case SB_PAGEUP:
		MovePageUp();
		break;
	case SB_PAGEDOWN:
		MovePageDown();
		break;
	case SB_THUMBTRACK:
		m_iViewFrom = (int)((long)nPos *
			m_aLine.GetSize() / m_nScrollRange);
		CalcViewTo();
		Invalidate();
		break;
	default:
		CWnd::OnVScroll(nSBCode, nPos, pScrollBar);
	}
}

BOOL CText::GetLineCol(CPoint& pt, int &iLine, int &nCol)
{
	if (pt.y < 0)
		pt.y = 0;
	int nHeight = 0;
	int nMaxLine = m_aLine.GetSize();
	if (nMaxLine <= 0) {
		iLine = nCol = 0;
		return FALSE;
	}

	int i;
	for (i = m_iViewFrom; i < nMaxLine; i++) {
		CLine* pLine = m_aLine[i];
		int nHeightNext = nHeight + pLine->m_nHeight;
		if (pt.y < nHeightNext) {
			CClientDC dc(this);
			FSave save(dc, pLine->m_type);
			iLine = i;
			nCol = pLine->GetCol(dc,
				m_nClientWidth - MARGIN,
				pt.x - MARGIN, pt.y - nHeight);
			return TRUE;
		}
		nHeight = nHeightNext;
	}
	iLine = nMaxLine - 1;
	nCol = m_aLine[iLine]->m_sStr.GetLength();
	return TRUE;
}

void CText::OnLButtonDown(UINT nFlags, CPoint point) 
{
	//if (m_pFrame->m_fSimple)
	//	m_pFrame->PostMessage(WM_COMMAND, ID_VIEW_SIMPLE);

	if (!m_fCapture && m_aLine.GetSize() > 0) {
		SetCapture();
		m_fCapture = TRUE;
		GetLineCol(point, m_iSelLine, m_nSelCol);
		m_iSelLine1 = m_iSelLine;
		m_nSelCol1 = m_nSelCol;
	} else
		CWnd::OnLButtonDown(nFlags, point);
}

void CText::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_fCapture) {
		ReleaseCapture();
		m_fCapture = FALSE;
		GetLineCol(point, m_iSelLine1, m_nSelCol1);
		if (!GetSelBgnEnd())
			return;
		SelCopy();
		Invalidate(FALSE);
	} else
		CWnd::OnLButtonUp(nFlags, point);
}

void CText::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (!m_fCapture) {
		CWnd::OnMouseMove(nFlags, point);
		return;
	}
	if (point.y < 0)
		MoveLineUp();
	else if (point.y > m_nClientHeight)
		MoveLineDown();
	int iSelLine, nSelCol;
	GetLineCol(point, iSelLine, nSelCol);
	if (iSelLine != m_iSelLine1 || nSelCol != m_nSelCol1) {
		m_iSelLine1 = iSelLine;
		m_nSelCol1 = nSelCol;
		Invalidate(FALSE);
	}
}

void CText::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	int nLine;
	int nCol;
	if (!GetLineCol(point, nLine, nCol))
		return;

	CString& s = m_aLine[nLine]->m_sStr;
	CString sURL;
	if (ExtractURL(s, nCol, sURL))
		OpenURL(sURL);

	//CWnd::OnLButtonDblClk(nFlags, point);
}

void CText::SelCopy()
{
	int nSize = 0;
	int nCol, nColEnd;
	int iLine;
	for (iLine = m_iSelLineBgn; iLine <= m_iSelLineEnd; iLine++) {
		nCol = (iLine == m_iSelLineBgn) ? m_nSelColBgn : 0;
		nColEnd = (iLine == m_iSelLineEnd)
			? m_nSelColEnd : m_aLine[iLine]->m_sStr.GetLength();
		nSize += nColEnd - nCol;
		if (iLine < m_iSelLineEnd)
			nSize += 2; //crlf
	}

	HGLOBAL hnd = GlobalAlloc(GHND | GMEM_DDESHARE, nSize + 1);
	if (hnd == 0) {
		AfxMessageBox(IDS_ER_MEM, MB_OK|MB_APPLMODAL);
		return;
	}
	LPTSTR pBuf = (LPTSTR)GlobalLock(hnd);
	if (pBuf == NULL) {
		AfxMessageBox(IDS_ER_MEM, MB_OK|MB_APPLMODAL);
		GlobalFree(hnd);
		return;
	}

	int nLen;
	LPTSTR p = pBuf;
	for (iLine = m_iSelLineBgn; iLine <= m_iSelLineEnd; iLine++) {
		CString& s = m_aLine[iLine]->m_sStr;
		nCol = (iLine == m_iSelLineBgn) ? m_nSelColBgn : 0;
		nLen = (iLine == m_iSelLineEnd)
			? (m_nSelColEnd - nCol) : s.GetLength() - nCol;
		memcpy(p, (LPCTSTR)s + nCol, nLen);
		p += nLen;
		if (iLine < m_iSelLineEnd) {
			*p++ = '\r';
			*p++ = '\n';
		}
	}
	*p++ = 0;

	GlobalUnlock(hnd);

	if (!OpenClipboard()) {
		AfxMessageBox(IDS_ER_OPENCLIPBOARD, MB_OK|MB_APPLMODAL);
		GlobalFree(hnd);
		return;
	}
	::EmptyClipboard();
	::SetClipboardData(CF_TEXT, hnd);
	::CloseClipboard();
}

BOOL CText::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

BOOL CText::PreCreateWindow(CREATESTRUCT& cs) 
{
	LPCTSTR lpszClass =
		AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS,
			LoadCursor(NULL, IDC_IBEAM));

	cs.lpszClass = lpszClass;
	cs.style |= (WS_CHILD|WS_VISIBLE|WS_VSCROLL);

	return CWnd::PreCreateWindow(cs);
}

void CText::OnDestroy() 
{
	int i;
	for (i = 0; i < m_aLine.GetSize(); i++)
		delete m_aLine[i];
	m_aLine.RemoveAll();

#if 0
	if (m_pTextFrame) {
		m_pTextFrame->DestroyWindow();
		m_pTextFrame = NULL;
	}
#endif

	CWnd::OnDestroy();
}

BOOL CText::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	m_nWheelDelta -= zDelta;
	if (abs(m_nWheelDelta) >= WHEEL_DELTA && g_nScrollLines > 0) {
		int nLinesInWindow = m_iViewTo - m_iViewFrom;
		if (nLinesInWindow < 1)
			nLinesInWindow = 1;
		int nLines = min(nLinesInWindow, (int)g_nScrollLines);
        nLines *= (m_nWheelDelta / WHEEL_DELTA);
        ASSERT(nLines != 0);
        m_nWheelDelta %= WHEEL_DELTA;

		int i = 0;
		if (nLines < 0) {
			while (i-- > nLines)
				MoveLineUp();
		} else {
			while (i++ < nLines)
				MoveLineDown();
		}
    }
	return TRUE;
	//return CWnd::OnMouseWheel(nFlags, zDelta, pt);
}

#if 0
void CText::Isolate(BOOL f)
{
	TRACE("Isolate %d {\n", f);
	if (f) {
		if (!m_pTextFrame)
			m_pTextFrame = new TextFrame(this, m_pFrame);
		m_pTextFrame->Show(m_sTitle);
		SetParent(m_pTextFrame);
	} else {
		ASSERT(m_pParent);
		SetParent(m_pParent);
		m_pTextFrame->Hide();
	}
	TRACE("} Isolate %d\n", f);
}

void CText::NewTitle(LPCTSTR pszTitle)
{
	m_sTitle = pszTitle;
	if (m_pTextFrame)
		m_pTextFrame->SetWindowText(pszTitle);
}
#endif

#if 0
// TextFrame

TextFrame::TextFrame(CText* pText, CFrame* pFrame)
{
	m_pText = pText;
	m_pFrame = pFrame;

	m_fCreated = FALSE;
}

TextFrame::~TextFrame()
{
}

static const DWORD dwStyle = WS_POPUP|WS_SYSMENU|
	WS_BORDER|WS_THICKFRAME|WS_CLIPSIBLINGS;
static const DWORD dwExStyle = 0;

void TextFrame::CalcRect(CRect& rc)
{
	m_pText->GetWindowRect(&rc);
	if (!AdjustWindowRectEx(&rc, dwStyle, FALSE, dwExStyle))
		ASSERT(FALSE);
}

BOOL TextFrame::Show(LPCTSTR pszTitle)
{
	CRect rc;
	CalcRect(rc);

	if (!m_fCreated) {
		if (!DoCreate(pszTitle, rc))
			return FALSE;
	} else {
		SetWindowText(pszTitle);
		MoveWindow(&rc);
	}

	NewSize();
	ShowWindow(SW_SHOWNOACTIVATE);
	return TRUE;
}

BOOL TextFrame::DoCreate(LPCTSTR pszTitle, CRect& rc)
{
	TRACE("TextFrame DoCreate %s {\n", pszTitle);

	LPCTSTR lpszClass =
		AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS,
			LoadCursor(NULL, IDC_IBEAM),
			(HBRUSH)(COLOR_WINDOW+1),
			(HICON)GetClassLong(m_pFrame->m_hWnd, GCL_HICON));

	if (!CreateEx(dwExStyle, lpszClass, pszTitle,
			dwStyle, rc.left, rc.top, rc.Width(), rc.Height(),
			NULL, NULL)) {
		ASSERT(FALSE);
		return FALSE;
	}

	m_fCreated = TRUE;

	TRACE("} TextFrame DoCreate %s\n", pszTitle);
	return TRUE;
}

void TextFrame::NewSize(void)
{
	CRect rc;
	GetClientRect(&rc);
	m_pText->MoveWindow(&rc);
}

BEGIN_MESSAGE_MAP(TextFrame, CWnd)
	//{{AFX_MSG_MAP(TextFrame)
	ON_WM_SIZE()
	ON_WM_ACTIVATE()
	ON_WM_NCMOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// TextFrame bZ[W nh

void TextFrame::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);

	if (nType == SIZE_MAXIMIZED || nType == SIZE_RESTORED)
		NewSize();
}

void TextFrame::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CWnd::OnActivate(nState, pWndOther, bMinimized);

	if (!m_fCreated)
		return;
	TRACE("TextFrame::OnActive %d, %d\n", nState, bMinimized);
	//if (nState != WA_INACTIVE)
	//	m_pFrame->Simple(FALSE);
}

void TextFrame::OnNcMouseMove(UINT nHitTest, CPoint point) 
{
	CWnd::OnNcMouseMove(nHitTest, point);

	//m_pFrame->Simple(FALSE);
}
#endif
