// WndList.cpp : Cve[V t@C
//

#include "stdafx.h"
#include "mchat.h"
#include "WndList.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// IconList

IconList::IconList()
{
}

IconList::~IconList()
{
}

void IconList::AddIcon(CFrame* pFrame, BOOL fSel)
{
	int nType = pFrame->GetType();
	CString sLabel = (nType == CFrame::TypeMain)
		? "Main" : pFrame->m_sTarget;
	LPTSTR pLabel = sLabel.GetBuffer(0);

	LV_ITEM item = {
		LVIF_TEXT|LVIF_IMAGE|LVIF_PARAM|LVIF_STATE,
		GetItemCount(),
		0,
		fSel ? (LVIS_FOCUSED|LVIS_SELECTED) : 0,
		0,
		pLabel,
		0,
		nType,
		(LPARAM)pFrame
	};
	InsertItem(&item);

	sLabel.ReleaseBuffer();
}

void IconList::NextFocus(BOOL fBackward)
{
	int nItem = GetNextItem(-1, LVNI_ALL|LVNI_FOCUSED);
	if (fBackward) {
		if (--nItem < 0)
			nItem = GetItemCount() - 1;
		if (nItem < 0)
			nItem = 0;
	} else if (++nItem >= GetItemCount())
		nItem = 0;
	SetItemState(nItem,
		(LVIS_FOCUSED|LVIS_SELECTED),
		(LVIS_FOCUSED|LVIS_SELECTED));
	EnsureVisible(nItem, FALSE);
}

CFrame* IconList::GetCurFrame()
{
	int nItem = GetNextItem(-1, LVNI_ALL|LVNI_FOCUSED);
	if (nItem >= 0) {
		CFrame* pFrame = (CFrame*)GetItemData(nItem);
		if (IsWindow(pFrame->GetSafeHwnd()))
			return pFrame;
	}
	return 0;
}

BEGIN_MESSAGE_MAP(IconList, CListCtrl)
	//{{AFX_MSG_MAP(IconList)
	ON_WM_KILLFOCUS()
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// IconList bZ[W nh

BOOL IconList::PreTranslateMessage(MSG* pMsg) 
{
	if (pMsg->message == WM_KEYDOWN) {
		if (pMsg->wParam == VK_TAB &&
				(GetKeyState(VK_CONTROL) & 0x80000000) != 0) {
			BOOL fBack = ((GetKeyState(VK_SHIFT) & 0x80000000) != 0);
			NextFocus(fBack);
			return TRUE;
		} else if (pMsg->wParam == VK_ESCAPE) {
			GetParent()->PostMessage(WM_COMMAND, IDCANCEL);
			return TRUE;
		}
	} else if (pMsg->message == WM_KEYUP &&
			pMsg->wParam == VK_CONTROL) {
		GetParent()->PostMessage(WM_COMMAND, IDOK);
		return TRUE;
	}

	return CListCtrl::PreTranslateMessage(pMsg);
}

void IconList::OnKillFocus(CWnd* pNewWnd) 
{
	CListCtrl::OnKillFocus(pNewWnd);

	if (!GetParent()->IsChild(pNewWnd))
		GetParent()->PostMessage(WM_COMMAND, IDCANCEL);
}

void IconList::OnDblclk(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint pt(GetMessagePos());
	ScreenToClient(&pt);
	UINT nFlag;
	int nItem = HitTest(pt, &nFlag);
	if (nItem >= 0 && (nFlag & LVHT_ONITEM)) {
		EnsureVisible(nItem, FALSE);
		GetParent()->PostMessage(WM_COMMAND, IDOK);
	}
	*pResult = 0;
}

// WndList _CAO

WndList* WndList::g_pWndList = 0;

WndList::WndList(CFrame* pCurFrame /*=NULL*/)
	: CDialog(WndList::IDD, pCurFrame)
{
	//{{AFX_DATA_INIT(WndList)
	//}}AFX_DATA_INIT
	m_fActive = FALSE;
	m_pCurFrame = pCurFrame;
	if (Create(IDD, pCurFrame))
		g_pWndList = this;
}

void WndList::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(WndList)
	DDX_Control(pDX, IDC_WNDLIST_LIST, m_list);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(WndList, CDialog)
	//{{AFX_MSG_MAP(WndList)
	ON_WM_KILLFOCUS()
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_WNDLIST_MINALL, OnWndlistMinall)
	ON_BN_CLICKED(IDC_WNDLIST_CLOSE_ALL, OnWndlistCloseAll)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// WndList bZ[W nh

BOOL WndList::OnInitDialog() 
{
	CDialog::OnInitDialog();

	COLORREF crBack = GetSysColor(COLOR_BTNFACE);
	m_list.SetBkColor(crBack);
	m_list.SetTextBkColor(crBack);

	CImageList* pImg = new CImageList;
	if (pImg->Create(32, 32, TRUE, 0, 4)) {
		pImg->Add(theApp.LoadIcon(IDR_MAIN));
		pImg->Add(theApp.LoadIcon(IDR_CHAN));
		pImg->Add(theApp.LoadIcon(IDR_TALK));
		pImg->Add(theApp.LoadIcon(IDR_CHAT));
		m_list.SetImageList(pImg, LVSIL_NORMAL);
	}

	HWND hWnd = m_pCurFrame
		? m_pCurFrame->m_hWnd
		: g_pFrame->m_hWnd;

	m_list.AddIcon(g_pFrame, g_pFrame->m_hWnd == hWnd);
	CString sKey;
	CFrame* pFrame;
	POSITION pos = g_frames.GetStartPosition();
	while (pos) {
		g_frames.GetNextAssoc(pos, sKey, pFrame);
		m_list.AddIcon(pFrame, pFrame->m_hWnd == hWnd);
	}
	
	CenterWindow(GetDesktopWindow());
	ShowWindow(SW_SHOWNORMAL);

	m_fActive = TRUE;
	return TRUE;
}

void WndList::OnOK() 
{
	m_pCurFrame = m_list.GetCurFrame();
	DestroyWindow();
}

void WndList::OnCancel() 
{
	m_pCurFrame = 0;
	DestroyWindow();
}

void WndList::PostNcDestroy() 
{
	g_pWndList = 0;
	if (m_pCurFrame)
		m_pCurFrame->ActivateFrame();
	delete this;
}

void WndList::OnDestroy() 
{
	m_fActive = FALSE;
	CDialog::OnDestroy();
}

void WndList::OnKillFocus(CWnd* pNewWnd) 
{
	CDialog::OnKillFocus(pNewWnd);
	if (m_fActive)
		DestroyWindow();
}

void WndList::OnWndlistMinall() 
{
	g_frames.MinimizeAll();
	OnCancel();
}

void WndList::OnWndlistCloseAll() 
{
	g_frames.CloseAll();
	OnCancel();
}
