#include "stdafx.h"
#include "mchat.h"
#include "dcc.h"

// internals

void CSockListen::msg(LPCTSTR psz)
{
	m_parent->SendMessage(WMU_SOCK, WMU_SOCK_MSG, (LONG)psz);
}

void CSockData::msg(LPCTSTR psz)
{
	m_parent->SendMessage(WMU_SOCK, WMU_SOCK_MSG, (LONG)psz);
}

// listen operation

BOOL CSockListen::DoListen(u_short& usPort)
{
	u_short usPortEnd = usPort + 100;
	while (!Create(usPort, SOCK_STREAM, FD_ACCEPT)) {
		if (usPort != 0)
			if (++usPort < usPortEnd)
				continue;
		CString s;
		s.Format("*Create Socket %d", GetLastError());
		msg(s);
		return FALSE;
	}
	if (!Listen()) {
		CString s;
		s.Format("*Listen %d", GetLastError());
		msg(s);
		return FALSE;
	}
	SOCKADDR_IN addr_in;
	int addrlen = sizeof addr_in;
	if (!GetSockName((SOCKADDR*)&addr_in, &addrlen)) {
		Close();
		CString s;
		s.Format("*Cannot get sockname %d", GetLastError());
		msg(s);
		return FALSE;
	}
	usPort = addr_in.sin_port;

	return TRUE;
}

// listen override

void CSockListen::OnAccept(int n)
{
	if (n != 0) {
		CString s;
		s.Format("*On Accept %d", n);
		msg(s);
		return;
	}
	if (!Accept(*m_pData)) {
		CString s;
		s.Format("*Accept %d", GetLastError());
		msg(s);
		return;
	}
	Close();

	int nSndSize = 1024 * 32;
	if (!m_pData->SetSockOpt(SO_SNDBUF, &nSndSize, sizeof nSndSize))
		TRACE("SetSockOpt SO_SNDBUF %d\n", GetLastError());
	m_parent->SendMessage(WMU_SOCK, WMU_SOCK_CONNECT);
	m_pData->AsyncSelect();
}

// data operation

CSockData::CSockData(BOOL fText)
{
	m_hFile = INVALID_HANDLE_VALUE;
	m_fpRead = m_fpWrite = 0L;
	m_fText = fText;

	m_nText = 0;
	m_nBufRcv = 4096;
	m_pBufRcv = new char [m_nBufRcv + 1];

	if (!m_fText)
		return;

	// text mode only
	TCHAR dir[MAX_PATH];
	if (!GetTempPath(sizeof dir, dir))
		strcpy(dir, ".");

	LPTSTR pFile = m_sFile.GetBuffer(MAX_PATH);
	UINT r = GetTempFileName(dir, "mcha", 0, pFile);
	m_sFile.ReleaseBuffer();

	TRACE("Temp file: %s\n", m_sFile);
	if (r == 0) {
		TRACE("Can't create temp file\n");
		return;
	}

	m_hFile = CreateFile(
		m_sFile, GENERIC_READ | GENERIC_WRITE, 0, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_TEMPORARY, 0);
}

CSockData::~CSockData()
{
	if (m_hFile != INVALID_HANDLE_VALUE)
		CloseHandle(m_hFile);
	if (!m_sFile.IsEmpty())
		DeleteFile(m_sFile);
	if (m_pBufRcv)
		delete [] m_pBufRcv;
}

void CSockData::DoSend(LPCTSTR pStr, int nLen)
{
	if (m_hFile == INVALID_HANDLE_VALUE) {
		int nSend = Send(pStr, nLen);
		if (nSend < 0) {
			CString s;
			s.Format("*Send %d", GetLastError());
			msg(s);
		}
		return;
	}

	SetFilePointer(m_hFile, m_fpWrite, NULL, FILE_BEGIN);
	DWORD dwWritten;
	WriteFile(m_hFile, pStr, nLen, &dwWritten, NULL);
	m_fpWrite += dwWritten;

	if (m_fpWrite > m_fpRead)
		AsyncSelect(FD_READ|FD_WRITE|FD_OOB|FD_CLOSE);
}

// connect to receive data
BOOL CSockData::DoConnect(u_long ulHost, u_short usPort)
{
	if (!Create()) {
		CString s;
		s.Format("*Create Socket %d", GetLastError());
		msg(s);
		return FALSE;
	}

	int nBufSize = 1024 * 32;
	if (!SetSockOpt(SO_RCVBUF, &nBufSize, sizeof nBufSize))
		TRACE("SetSockOpt SO_RCVBUF %d\n", GetLastError());

	SOCKADDR_IN addr;
	memset(&addr, 0, sizeof addr);
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = ntohl(ulHost);
	addr.sin_port = ntohs(usPort);
	if (!Connect((SOCKADDR*)&addr, sizeof addr)) {
		if (GetLastError() == WSAEWOULDBLOCK)
			return TRUE;
		CString s;
		s.Format("*Connection failure %d", GetLastError());
		msg(s);
		return FALSE;
	}
	DoReceive();
	return TRUE;
}

void CSockData::DoReceive()
{
	if (m_fText) {
		char* pBuf = m_pBufRcv + m_nText;
		int nBuf = m_nBufRcv - m_nText;
		int nLen = Receive(pBuf, nBuf);
		if (nLen <= 0) {
			if (nLen == 0)
				return;
			CString s;
			s.Format("*Receive %d", GetLastError());
			msg(s);
			return;
		}
		pBuf[nLen] = 0;
		m_nText += nLen;

		char* p1 = m_pBufRcv;
		char* p2;
		while ((p2 = strchr(p1, '\n')) != NULL) {
			int n = ++p2 - p1;
			m_parent->SendMessage(WMU_SOCK_DATA, n, (LONG)p1);
			p1 = p2;
			m_nText -= n;
		}

		if (m_nText >= m_nBufRcv) {	// overflow
			m_parent->SendMessage(WMU_SOCK_DATA, m_nText, (LONG)m_pBufRcv);
			m_nText = 0;
			return;
		}

		if (p1 > m_pBufRcv)
			CopyMemory(m_pBufRcv, p1, m_nText);
		return;
	}

	int nLen = Receive(m_pBufRcv, m_nBufRcv);
	if (nLen <= 0) {
		if (nLen == 0)
			return;
		CString s;
		s.Format("*Receive %d", GetLastError());
		msg(s);
		return;
	}
	m_pBufRcv[nLen] = 0;
	m_parent->SendMessage(WMU_SOCK_DATA, nLen, (LONG)m_pBufRcv);
}

// overrides for data

void CSockData::OnSend(int n)
{
	if (n != 0) {
		CString s;
		s.Format("*On Send %d", n);
		msg(s);
		return;
	}

	//TRACE("OnSend\n");

	AsyncSelect(FD_READ|FD_OOB|FD_CLOSE);

	if (m_hFile == INVALID_HANDLE_VALUE)
		return;

	char buf[1024];
	int nRead = m_fpWrite - m_fpRead;
	if (nRead >= sizeof buf)
		nRead = sizeof buf;
	if (nRead <= 0)
		return;

	SetFilePointer(m_hFile, m_fpRead, NULL, FILE_BEGIN);
	DWORD dwRead;
	if (!ReadFile(m_hFile, buf, nRead, &dwRead, NULL))
		TRACE("read error: %s\n", m_sFile);

	if (m_fText) {
		DWORD dw = 0;
		while (dw < dwRead && buf[dw++] != '\n')
			;
		dwRead = dw;
	}

	if (dwRead > 0) {
		//TRACE("at %d/%d send: %d -> ", m_fpRead, m_fpWrite, dwRead);
		int nSend = Send(buf, dwRead);
		if (nSend < 0) {
			CString s;
			s.Format("*OnSend2 %d", GetLastError());
			msg(s);
			TRACE("%s\n", s);
			return;
		}
		//TRACE("%d\n", nSend);
		m_fpRead += nSend;
	}

	if (m_fpRead >= m_fpWrite) {
		m_fpRead = m_fpWrite = 0L;
		AsyncSelect(FD_READ|FD_OOB|FD_CLOSE);
	} else
		AsyncSelect(FD_READ|FD_WRITE|FD_OOB|FD_CLOSE);
}

void CSockData::OnReceive(int n)
{
	if (n != 0) {
		CString s;
		s.Format("*On Receive %d", n);
		msg(s);
		return;
	}
	DoReceive();
}

void CSockData::OnClose(int n)
{
	if (n != 0) {
		CString s;
		s.Format("*Closed %d", n);
		msg(s);
	}
	m_parent->SendMessage(WMU_SOCK, WMU_SOCK_CLOSE);
}

void CSockData::OnConnect(int n)
{
	if (n != 0) {
		CString s;
		s.Format("*Connection failure %d", n);
		msg(s);
		return;
	}
	m_parent->PostMessage(WMU_SOCK, WMU_SOCK_CONNECT);
}
