#include "stdafx.h"
#include "mchat.h"

Fonts g_fonts;

static char szFmtFont[] =
	"%ld,%ld,%ld,%ld,%ld,%d,%d,%d,%d,%d,%d,%d,%d,%06lX,%s";
static char szScanFont[] =
	"%ld,%ld,%ld,%ld,%ld,%d,%d,%d,%d,%d,%d,%d,%d,%lX,%32c";
#define FONT_FIELD_NUM	15

static TCHAR BASED_CODE szFont[] = _T("Font");

static BOOL LoadReg(FontColor& fc)
{
	ASSERT(!fc.m_sName.IsEmpty());
	LOGFONT lf;
	ZeroMemory(&lf, sizeof lf);
	CString s = theApp.GetProfileString(szFont, fc.m_sName);
	if (!s.IsEmpty() && sscanf(s, szScanFont,
			&lf.lfHeight, &lf.lfWidth, &lf.lfEscapement, &lf.lfOrientation,
			&lf.lfWeight, &lf.lfItalic, &lf.lfUnderline, &lf.lfStrikeOut,
			&lf.lfCharSet, &lf.lfOutPrecision, &lf.lfClipPrecision,
			&lf.lfQuality, &lf.lfPitchAndFamily, &fc.m_color, lf.lfFaceName)
			== FONT_FIELD_NUM) {
		if (fc.m_font.CreateFontIndirect(&lf))
			return TRUE;
	}
	return FALSE;
}

static void SaveReg(FontColor& fc)
{
	LOGFONT lf;
	if (!fc.m_font.GetLogFont(&lf))
		return;

	CString s;
	s.Format(szFmtFont,
		lf.lfHeight, lf.lfWidth, lf.lfEscapement, lf.lfOrientation,
		lf.lfWeight, (int)lf.lfItalic, (int)lf.lfUnderline, (int)lf.lfStrikeOut,
		(int)lf.lfCharSet, (int)lf.lfOutPrecision, (int)lf.lfClipPrecision,
		(int)lf.lfQuality, (int)lf.lfPitchAndFamily, fc.m_color, lf.lfFaceName);
	theApp.WriteProfileString(szFont, fc.m_sName, s);
}

static TCHAR BASED_CODE szText[] = _T("Message");
static TCHAR BASED_CODE szNotice[] = _T("Notice");
static TCHAR BASED_CODE szStat[] = _T("Status");
static TCHAR BASED_CODE szError[] = _T("Error");
static TCHAR BASED_CODE szNickList[] = _T("NickList");
static TCHAR BASED_CODE szKeyword[] = _T("Keyword");
static TCHAR BASED_CODE szTime[] = _T("Time");
static TCHAR BASED_CODE szNick[] = _T("Nick");
static TCHAR BASED_CODE szBack[] = _T("Back");

struct FontDef {
	char* m_pszName;	// reg and label
	int m_lfHeight;		// font height
	int m_lfWeight;		// font weight
	BOOL m_fLanguage;	// false=ansi only
	COLORREF m_cr;		// font color
	int m_idc;			// control id for setup:fonts
};

FontDef arFontDef[] = {
	{ szText, -16, FW_NORMAL, TRUE, 0, IDC_FONT_TEXT },
	{ szNotice, -14, FW_BOLD, TRUE, RGB(0, 128, 0), IDC_FONT_NOTICE },
	{ szStat, -14, FW_BOLD, TRUE, RGB(0, 0, 128), IDC_FONT_STAT },
	{ szError, -14, FW_BOLD, TRUE, RGB(255, 0, 0), IDC_FONT_ERROR },
	{ szNickList, -12, FW_NORMAL, FALSE, RGB(0, 0, 0), IDC_FONT_NICKLIST },
	{ szKeyword, -16, FW_BOLD, TRUE, RGB(0, 0, 255), IDC_FONT_KEYWORD },
	{ szTime, -12, FW_NORMAL, FALSE, RGB(0, 0, 0), IDC_FONT_TIME },
	{ szNick, -15, FW_NORMAL, FALSE, RGB(32, 32, 32), IDC_FONT_NICK },
};

void Fonts::LoadAll()
{
	m_crBack = GetSysColor(COLOR_WINDOW);
	CString s = theApp.GetProfileString(szFont, szBack);
	if (!s.IsEmpty()) {
		COLORREF cr;
		if (sscanf(s, "%lX", &cr) == 1)
			m_crBack = cr;
	}

	BYTE nCharSet = SHIFTJIS_CHARSET;
	HFONT hFont = (HFONT)GetStockObject(SYSTEM_FONT);
	if (!hFont)
		hFont = (HFONT)GetStockObject(SYSTEM_FIXED_FONT);
	LOGFONT lf;
	if (hFont) {
		GetObject(hFont, sizeof lf, &lf);
		nCharSet = lf.lfCharSet;
	}
	ZeroMemory(&lf, sizeof lf);

	arFontDef[Fonts::TypeText].m_cr = GetSysColor(COLOR_WINDOWTEXT);

	lf.lfOutPrecision = OUT_TT_PRECIS;
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	lf.lfQuality = DRAFT_QUALITY;
	lf.lfPitchAndFamily = VARIABLE_PITCH;

	int i;
	for (i = 0; i < sizeof arFontDef / sizeof arFontDef[0]; i++) {
		FontDef* p = &arFontDef[i];
		FontColor* pFC = &m_arFC[i];
		pFC->m_sName = p->m_pszName;
		if (!LoadReg(*pFC)) {
			lf.lfHeight = p->m_lfHeight;
			lf.lfWeight = p->m_lfWeight;
			lf.lfCharSet = p->m_fLanguage ? nCharSet : 0;

			BOOL f = pFC->m_font.CreateFontIndirect(&lf);
			pFC->m_color = p->m_cr;

			if (!f && i == Fonts::TypeText) {
				TRACE("Cannot create default font, use stock\n");
				HFONT hFont = (HFONT)GetStockObject(SYSTEM_FONT);
				if (!hFont) {
					TRACE("Cannot get system font, use fixed\n");
					hFont = (HFONT)GetStockObject(SYSTEM_FIXED_FONT);
				}
				ASSERT(hFont);
				GetObject(hFont, sizeof lf, &lf);
				lf.lfFaceName[0] = 0;
				pFC->m_font.CreateFontIndirect(&lf);
			}
		}

		pFC->m_idc = p->m_idc;
		m_map.SetAt((FType)i, pFC);

		if (i == Fonts::TypeText)
			ChangeTextFont();
	}
}

void Fonts::SaveAll()
{
	CString s;
	s.Format("%06lX", m_crBack);
	theApp.WriteProfileString(szFont, szBack, s);

	POSITION pos = m_map.GetStartPosition();
	while (pos) {
		FType type;
		FontColor* pFC;
		m_map.GetNextAssoc(pos, type, pFC);
		SaveReg(*pFC);
	}
}

void Fonts::ChangeTextFont()
{
	LOGFONT lf;
	FontColor* pfc;
	if (m_map.Lookup(Fonts::TypeText, pfc))
		pfc->m_font.GetLogFont(&lf);
	else {
		HFONT hFont = (HFONT)GetStockObject(SYSTEM_FONT);
		GetObject(hFont, sizeof lf, &lf);
	}

	if ((HFONT)m_fontInput)
		m_fontInput.DeleteObject();
	lf.lfWeight = FW_NORMAL;
	lf.lfItalic = FALSE;
	lf.lfUnderline = FALSE;
	m_fontInput.CreateFontIndirect(&lf);

	if ((HFONT)m_fontBold)
		m_fontBold.DeleteObject();
	lf.lfWeight = FW_BOLD;
	lf.lfUnderline = FALSE;
	m_fontBold.CreateFontIndirect(&lf);

	if ((HFONT)m_fontUL)
		m_fontUL.DeleteObject();
	lf.lfWeight = FW_NORMAL;
	lf.lfUnderline = TRUE;
	m_fontUL.CreateFontIndirect(&lf);

	if ((HFONT)m_fontBoldUL)
		m_fontBoldUL.DeleteObject();
	lf.lfWeight = FW_BOLD;
	lf.lfUnderline = TRUE;
	m_fontBoldUL.CreateFontIndirect(&lf);
}

// select text color by 'type'
// if pFont is null, select font by 'type'
// else use 'pFont'
void Fonts::Select(CDC& dc, FType type, CFont* pFont /*==NULL*/)
{
	FontColor* pfc;
	if (m_map.Lookup(type, pfc)) {
		dc.SetTextColor(pfc->m_color);
		if (pFont)
			dc.SelectObject(pFont);
		else if (!dc.SelectObject(&pfc->m_font))
			TRACE("font select err type:%d\n", type);
	} else
		TRACE("font lookup err. type:%d\n", type);
}

#if 0
CSize Fonts::GetCharSizeMax(CDC& dc)
{
	CSize sz(1, 1);
	POSITION pos = m_map.GetStartPosition();
	while (pos) {
		FType type;
		FontColor* pfc;
		m_map.GetNextAssoc(pos, type, pfc);
		FSave save(dc, type);
		TEXTMETRIC tm;
		dc.GetTextMetrics(&tm);
		int nHeight = tm.tmHeight +
			(tm.tmExternalLeading > 0 ? tm.tmExternalLeading : 1);
		if (nHeight > sz.cy)
			sz.cy = nHeight;
		if (tm.tmMaxCharWidth > sz.cx)
			sz.cx = tm.tmMaxCharWidth;
	}
	return sz;
}
#endif

// font saver

void FSave::Save(CDC& dc, Fonts::FType type)
{
	m_pDC = &dc;
	FontColor* pfc;
	if (g_fonts.m_map.Lookup(type, pfc)) {
		m_color = dc.SetTextColor(pfc->m_color);
		m_pFont = dc.SelectObject(&pfc->m_font);
		m_fSaved = TRUE;
	} else
		TRACE("fsave; lookup fail %d\n", type);
}

void FSave::Restore()
{
	if (m_fSaved) {
		m_pDC->SetTextColor(m_color);
		m_pDC->SelectObject(m_pFont);
		m_fSaved = FALSE;
	}
}
