#include "stdafx.h"
#include "mchat.h"
#include "DlgNick.h"
#include "DlgDcc.h"
#include "DlgRcv.h"
#include "DlgJoin.h"
#include "DlgInvited.h"
#include "DlgMsg.h"
#include "ident.h"
#include "dlgwhois.h"
#include "listchan.h"
#include "dlgmisc.h"

CStringArray g_aKnownNicks;	// known nicks

void OpenConversation(); //fwd

static int g_nNickCount = 0;
static BOOL g_fListChan = FALSE; // retrieving channel list

// spam

class Spam
{
public:
	enum Spam_t { SpamNo, SpamDetect, SpamCont, SpamReset };

	Spam() {
		dwLast = GetTickCount();
		nCount = 0;
		fSpam = false;
	}

	Spam_t Check()
	{
		DWORD dwNow = GetTickCount();
		if (fSpam) {
			if (dwNow - dwLast < 1000 * 60) {
				nCount++;
				dwLast = dwNow;
				return SpamCont;
			}
			dwLast = dwNow;
			nCount = 0;
			fSpam = false;
			return SpamReset;
		}
		if (dwNow - dwLast > 1000 * 2) {
			dwLast = dwNow;
			nCount = 0;
			fSpam = false;
			return SpamNo;
		}
		dwLast = dwNow;
		if (++nCount >= 5) {
			if (fSpam)
				return SpamCont;
			fSpam = true;
			return SpamDetect;
		}
		return SpamNo;
	}

	DWORD dwLast;
	int nCount;
	bool fSpam;
};

void AFXAPI DestructElements(Spam** ppSpam, int nCount )
{
	while (nCount-- > 0) {
		Spam* pSpam = *ppSpam++;
		delete pSpam;
	}
}

class MapSpam : public CMap<CString, LPCTSTR, Spam*, Spam*&>
{
public:
	bool Count(const Packet& pac)
	{
		if (pac.sHost.IsEmpty())
			return false;
		if (pac.sNick.CompareNoCase(g_con.m_sNick) == 0)
			return false;

		Spam* pSpam;
		if (Lookup((LPCTSTR)pac.sHost, pSpam)) {
			//TRACE("Checking: %s %d\n", pAddr, pSpam->nCount);
			switch (pSpam->Check()) {
			case Spam::SpamDetect:
				{
					CString s;
					s.Format("Spam from %s (%s)", pac.sHost, pac.sPrefix);
					g_pFrame->LineStat(s);
				}
				//THROUGH
			case Spam::SpamCont:
				g_pFrame->LineRaw();
				return true;
			case Spam::SpamReset:
				{
					CString s;
					s.Format("Enable %s (%s)", pac.sHost, pac.sPrefix);
					g_pFrame->LineStat(s);
				}
				//THROUGH
			}
			return false;
		}

		pSpam = new Spam;
		SetAt((LPCTSTR)pac.sHost, pSpam);

		TRACE("Add %s (%s)\n", (LPCTSTR)pac.sHost, (LPCTSTR)pac.sPrefix);
		return false;
	}
};

MapSpam g_mapSpam;

// receiving part

CFrame* GetTargetFrame(const Packet& pac, const CString& sTarget)
{
	BOOL fBeeped = FALSE;
	CFrame* pFrame;
	if (sTarget.CompareNoCase(g_con.m_sNick) == 0 && !pac.sNick.IsEmpty()) {
		pFrame = GetFrame(pac.sNick);
		if (IsMainFrame(pFrame)) {
			fBeeped = optBeep.Play(BEEP_PrivMsgNoTalk);
			if (g_opt.fPrivMsgPopup) {
				if (g_mapSpam.Count(pac))
					return g_pFrame;
				pFrame = new FrameTalk(pac.sNick);
			}
			if (g_opt.fPrivMsgWhois && !IsKnownNick(pac.sNick))
				CmdWhois(pac.sNick);
		} else if (pFrame->IsIconic())
			fBeeped = optBeep.Play(BEEP_PrivMsgIcon);
		if (!fBeeped)
			optBeep.Play(BEEP_PrivMsg);
	} else {
		pFrame = GetFrame(sTarget);
		if (pFrame->IsIconic())
			fBeeped = optBeep.Play(BEEP_ChanMsgIcon);
		if (!fBeeped)
			optBeep.Play(BEEP_ChanMsg);
	}
	return pFrame;
}

static char BASED_CODE szNotice[] = _T("NOTICE");

static void GotDCC(const Packet& pac, LPCTSTR pStr)
{
	CString s;
	LPCTSTR p = pStr;
	CString sType(Token(p));
	sType.MakeUpper();
	CString sArg(Token(p));
	CString sAddr(Token(p));
	CString sPort(Token(p));

	u_long ulHost = atol(sAddr);
	u_short usPort = atoi(sPort);

	struct in_addr in;
	in.s_addr = htonl(ulHost);

	s.Format("*%s DCC %s %s %d %s",
		pac.sNick, sType, inet_ntoa(in), usPort, (LPCTSTR)sArg);
	g_pFrame->LineStat(s);

	if (ulHost == 0 || usPort == 0)
		goto err;

	if (!isdigit(*(LPCTSTR)sAddr) || !isdigit(*(LPCTSTR)sPort))
		goto err;

	if (sType == "CHAT") {
		if (sArg.CompareNoCase("CHAT") == 0) {
			if (g_opt.nDccChat == DCC_CHAT_IGNORE)
				return;
			if (g_opt.nDccChat == DCC_CHAT_PROMPT) {
				MessageBeep(MB_ICONQUESTION);
				s.Format(IDS_MSG_DCC_CHAT_PROMPT, pac.sNick);
				int r = AfxMessageBox(s, MB_YESNO | MB_ICONQUESTION);
				if (r != IDYES)
					return;
			} else
				optBeep.Play(BEEP_DCC);
			FrameChat* pFrame = new FrameChat(pac.sNick);
			pFrame->ChatConnect(ulHost, usPort);
			return;
		}
	} else if (sType == "SEND") {
		MessageBeep(MB_ICONQUESTION);
		CString sSize(Token(p));
		u_long ulSize = atol(sSize);
		new CDlgRcv(pac.sNick, sArg, ulSize, ulHost, usPort, g_pFrame);
		return;
	}
err:
	s.Format(IDS_MSG_DCC_ERR, pac.sNick);
	g_pFrame->LineError(s);
}

static void GotCTCP(const Packet& pac, CString& sCTCP)
{
	CString s;

	LPCTSTR pCTCP = sCTCP;
	CString sCmd(Token(pCTCP));
	sCmd.MakeUpper();

	if (sCmd == "CLIENTINFO") {
		Cmd(szNotice, pac.sNick, ":\001CLIENTINFO :not supported\001");
	} else if (sCmd == "FINGER") {
		s.Format(":\001FINGER :%s@%s\001",
			g_opt.sLoginName, g_opt.sHostName);
		Cmd(szNotice, pac.sNick, s);
	} else if (sCmd == "USERINFO") {
		s.Format(":\001USERINFO :%s\001", g_opt.sUserInfo);
		Cmd(szNotice, pac.sNick, s);
	} else if (sCmd == "VERSION") {
		CString sVer;
		sVer.LoadString(IDS_VERSION);
		s.Format(":\001VERSION MuriChat %s for Win32\001", sVer);
		Cmd(szNotice, pac.sNick, s);
	} else if (sCmd == "PING") {
		bool fErr = false;
		int i;
		BYTE c;
		for (i = 0; (c = pCTCP[i] & 0xff) != 0; i++) {
			if (i > 20 || c < ' ' || c >= 0x80) {
				g_pFrame->LineRaw();
				fErr = true;
				break;
			}
		}
		if (!fErr) {
			s.Format(":\001PING %s\001", pCTCP);
			Cmd(szNotice, pac.sNick, s);
		}
	} else if (sCmd == "TIME") {
		time_t t;
		time(&t);
		CString sTime(ctime(&t));
		int i;
		while ((i = sTime.GetLength()) > 0 && sTime[i - 1] < ' ')
			sTime = sTime.Left(i - 1);
		s.Format(":\001TIME %s\001", sTime);
		Cmd(szNotice, pac.sNick, s);
	} else if (sCmd == "ACTION") {
		LPCTSTR pParm = pac.sParm;
		CString sTarget(Token(pParm));
		CFrame* pFrame = GetTargetFrame(pac, sTarget);
		pFrame->LineNotice(pac.sNick, pCTCP);
		pFrame->NotifyMessage();
		return;
	} else if (sCmd == "DCC") {
		GotDCC(pac, pCTCP);
		return;
	} else {
		//s.Format(":\001ERRORMSG %s :Unknown query\001", sCmd);
		//Cmd(szNotice, pac.sNick, s);
	}

	s.Format("query %s %s", sCmd, pCTCP);
	g_pFrame->LineNotice(pac.sNick, s, g_opt.fMsgQuery);
}

BOOL ExtractCtrlA(CString& sSrc, CString& sDst)
{
	LPCTSTR p = sSrc;
	LPCTSTR p1;
	LPCTSTR p2;
	if ((p1 = strchr(p, '\001')) != 0) {
		if ((p2 = strchr(p1 + 1, '\001')) == 0) {
			sDst = CString(p1 + 1);
			sSrc = sSrc.Left(p1 - p);
		} else {
			sDst = CString(p1 + 1, p2 - (p1 + 1));
			sSrc = sSrc.Left(p1 - p) + sSrc.Right(lstrlen(p2 + 1));
		}
		return TRUE;
	}
	return FALSE;
}

static void GotPrivMsg(const Packet& pac)
{
	LPCTSTR pParm = pac.sParm;
	CString sTarget(Token(pParm));

	if (IgnoreMatch(pac.sNick, pac.sAddr))
		return;

	CString s(pac.sTrail);
	CString sCTCP;
	if (ExtractCtrlA(s, sCTCP)) {
		if (g_mapSpam.Count(pac))
			return;
		GotCTCP(pac, sCTCP);
		return;
	}

	if (s.IsEmpty())
		return;

	GetTargetFrame(pac, sTarget)->LineText(pac.sNick, s);

	g_arPlug.OnPriv(g_con, pac, sTarget, s);
}

static BOOL GotReply(const Packet& pac, LPCTSTR pReply)
{
	CString sToken(Token(pReply));
	sToken.MakeUpper();
	if (sToken == "PING") {
		DWORD dwNow = GetTickCount();
		TCHAR* pEnd;
		DWORD dwGet = _tcstoul(pReply, &pEnd, 10);
		CString s;
		if (dwGet > 0)
			s.Format("Ping: %.3f sec.", (float)(dwNow - dwGet) / 1000);
		else
			s.Format("Ping: Unknown: %s", pReply);
		g_frames.MsgReply(pac.sNick, s);
		return TRUE;
	}
	if (sToken == "TIME") {
		CString s;
		s.Format("Time: %s", pReply);
		g_frames.MsgReply(pac.sNick, s);
		return TRUE;
	}
	return FALSE;
}

static void GotNotice(const Packet& pac)
{
	if (IgnoreMatch(pac.sNick, pac.sAddr))
		return;

	CString s(pac.sTrail);
	CString sReply;

	if (ExtractCtrlA(s, sReply)) {
		if (GotReply(pac, sReply))
			return;
		CString s1;
		s1.Format("<%s>", (LPCTSTR)sReply);
		g_frames.MsgReply(pac.sNick, s1);
	}
	if (s.IsEmpty())
		return;

	LPCTSTR pParm = pac.sParm;
	CString sTarget(Token(pParm));

	if (pac.sAddr.IsEmpty()) {
		g_pFrame->LineNotice(NULL, s);
	} else {
		CFrame* pFrame = GetTargetFrame(pac, sTarget);
		pFrame->LineNotice(pac.sNick, s);
	}
}

static void GotInvite(const Packet& pac, CString& sChan)
{
	if (IgnoreMatch(pac.sNick, pac.sAddr))
		return;

	CFrame* pFrame = GetFrame(sChan);
	if (!sChan.IsEmpty() && IsMainFrame(pFrame)) {
		switch (g_opt.nInvite) {
		case INVITE_JOIN:
			optBeep.Play(BEEP_Invited);
			CmdJoin(sChan);
			break;
		case INVITE_IGNORE:
			break;
		default:
			MessageBeep(MB_ICONQUESTION);
			new DlgInvited(pac.sNick, sChan, g_pFrame);
		}
	}
}

BOOL IsKnownNick(const CString& sNick)
{
	BOOL fFound = FALSE;
	int i;
	for (i = 0; i < g_aKnownNicks.GetSize(); i++) {
		if (g_aKnownNicks[i] == sNick) {
			fFound = TRUE;
			break;
		}
	}
	if (!fFound)
		g_aKnownNicks.Add(sNick);
	return fFound;
}

static bool IsNoFrameChannel(const CString& sChan)
{
	if (sChan[0] != '&')
		return false;

	LPCTSTR ar[] = {
		"&AUTH", "&CHANNEL", "&ERRORS", "&HASH", "&KILLS",
		"&LOCAL", "&NOTICES", "&NUMERICS", "&SERVERS", "&SERVICES", 0
	};

	LPCTSTR* pp;
	for (pp = ar; *pp; pp++) {
		if (sChan.CompareNoCase(*pp) == 0)
			return true;
	}
	return false;
}

static void GotCmdA(Packet& pac)
{
	pac.sCmd.MakeUpper();

	if (pac.sCmd == "PRIVMSG") {
		GotPrivMsg(pac);
		return;
	}

	CFrame* pFrame;

	if (pac.sCmd == "PING") {
		Cmd("PONG", pac.sTrail);
		return;
	}

	LPCTSTR pParm = pac.sParm;
	if (pac.sCmd == "JOIN") {
		if (g_mapSpam.Count(pac))
			return;
		LPCTSTR pTrail = pac.sTrail;
		CString sChan(Token(pTrail));
		if (sChan.IsEmpty())
			sChan = Token(pParm);
		int n = sChan.Find('\007'); // ^G
		CString sMode;
		if (n >= 0) {
			sMode = CString((LPCTSTR)sChan + n + 1);
			sChan = sChan.Left(n);
		}
		if (sChan.IsEmpty()) {
			g_pFrame->LineRaw();
			return;
		}
		pFrame = GetFrame(sChan);
		if (pac.sNick.CompareNoCase(g_con.m_sNick) == 0) {
			if (IsMainFrame(pFrame) && !IsNoFrameChannel(sChan))
				pFrame = new FrameChan(sChan);
		}
		optBeep.Play(BEEP_Join);
		LPCTSTR pMode = sMode.IsEmpty() ? NULL : (LPCTSTR)sMode;
		pFrame->MsgJoin(pac, sChan, pMode);
		return;
	}
	if (pac.sCmd == "PART") {
		if (g_mapSpam.Count(pac))
			return;
		CString sChan(Token(pParm));
		GetFrame(sChan)->MsgPart(pac, sChan);
		optBeep.Play(BEEP_Part);
		return;
	}
	if (pac.sCmd == "QUIT") {
		g_frames.MsgQuit(pac.sNick, pac.sTrail);
		optBeep.Play(BEEP_Quit);
		return;
	}
	if (pac.sCmd == "MODE") {
		CString sTarget(Token(pParm));
		if (sTarget.CompareNoCase(g_con.m_sNick) == 0) {
			g_pFrame->MsgUserMode(sTarget, pac.sTrail);
		} else
			GetFrame(sTarget)->MsgMode(pac, sTarget, pParm);
		return;
	}
	if (pac.sCmd == "TOPIC") {
		CString sChan(Token(pParm));
		GetFrame(sChan)->MsgTopic(pac);
		return;
	}
	if (pac.sCmd == "KICK") {
		CString sChan(Token(pParm));
		GetFrame(sChan)->MsgKick(pac.sNick, sChan, pParm, pac.sTrail);
		return;
	}
	if (pac.sCmd == "NICK") {
		if (g_mapSpam.Count(pac))
			return;
		LPCTSTR pTrail = pac.sTrail;
		CString sTo(Token(pTrail));
		g_frames.MsgNick(pac.sNick, sTo);
		if (g_con.m_sNick.CompareNoCase(pac.sNick) == 0) {
			g_con.m_sNick = sTo;
			g_frames.MakeAllTitle();
		} else
			optBeep.Play(BEEP_NickChange);
		return;
	}
	if (pac.sCmd == "INVITE") {
		if (g_mapSpam.Count(pac))
			return;
		CString sWhom(Token(pParm));
		CString sChan(Token(pParm));
		if (sChan.IsEmpty()) {
			LPCTSTR pTrail = pac.sTrail;
			sChan = Token(pTrail);
		}
		pFrame = GetFrame(sChan);
		CString s;
		s.Format("*%s Invite %s to %s", pac.sNick, sWhom, sChan);
		pFrame->LineNotice(NULL, s);
		if (sWhom.CompareNoCase(g_con.m_sNick) == 0) {
			GotInvite(pac, sChan);
		}
		return;
	}
	if (pac.sCmd == szNotice) {
		if (g_mapSpam.Count(pac))
			return;
		GotNotice(pac);
		return;
	}
	g_pFrame->LineRaw();
}

static void GotCmdN(int nCmd, Packet& pac)
{
	LPCTSTR pParm = pac.sParm;
	CString sTo(Token(pParm));

	switch (nCmd) {
	case 1:// Welcome to... nick!account@domain.name
		if (g_opt.fUseDccFakeAddr && g_opt.fUseDccFakeAuto) {
			CmdWhois(g_con.m_sNick);
			g_con.m_fDccFakeWhois = TRUE;
		}
		goto others;

	case 5:
		{
			CString s(pac.sTrail);
			int n = s.Find("server ");
			if (n >= 0) {
				s.Delete(0, n + 7);
				n = s.FindOneOf(", ");
				if (n < 0)
					n = s.GetLength();
				CString sServer(s.Left(n));
				s.Delete(0, n);
				int nPort = 6667;
				n = s.Find("port ");
				if (n >= 0) {
					s.Delete(0, n + 5);
					if ((n = atoi(s)) > 0)
						nPort = n;
				}
				g_pFrame->LineText(NULL, pac.sTrail);
				ChangeServer(sServer, nPort, "");
				break;
			}
		}
		goto others;

	case 221://RPL_UMODEIS "<user mode string>"
		g_pFrame->MsgUserMode(sTo, pParm);
		break;

	case 366://RPL_ENDOFNAMES "<channel> :End of /NAMES list"
	case 369://RPL_ENDOFWHOWAS "<nick> :End of WHOWAS"
	case 333://unknown
	case 368://RPL_ENDOFBANLIST "<channel> :End of channel ban list"
	case 349://RPL_ENDOFEXCEPTIONLIST "<channel> :End of channel exception list"
	case 347://RPL_ENDOFINVITELIST "<channel> :End of channel invite list"
		break;

	case 372://RPL_MOTD ":- <text>"
	case 375://RPL_MOTDSTART ":- <server> Message of the day - "
	case 377://?
		g_pFrame->LineText(NULL, pac.sTrail);
		break;

	case 401://ERR_NOSUCHNICK "<nickname> :No such nick/channel"
		{
			CString sNick(Token(pParm));
			CString s;
			s.Format("*%s: %s", (LPCTSTR)sNick, (LPCTSTR)pac.sTrail);
			GetFrame(sNick)->LineError(s);
		}
		break;

	case 403://ERR_NOSUCHCHANNEL "<channel name> :No such channel"
		{
			CString sChan(Token(pParm));
			CString s;
			s.Format("*%s: %s", sChan, pac.sTrail);
			g_pFrame->LineError(s);
		}
		break;

	case 421://ERR_UNKNOWNCOMMAND "<command> :Unknown command"
		{
			CString s;
			s.Format("%s: %s", pParm, (LPCTSTR)pac.sTrail);
			g_pFrame->LineError(s);
		}
		break;

	case 311://RPL_WHOISUSER "<nick> <user> <host> * :<real name>"
		{
			CString sNick(Token(pParm));
			CString sUser(Token(pParm));
			CString sHost(Token(pParm));
			if (g_con.m_fDccFakeWhois && g_con.m_sNick.CompareNoCase(sNick) == 0) {
				g_con.m_fDccFakeWhois = FALSE;
				g_pFrame->ParseMyAddress(sHost);
				break;
			}
			if (!WhoisCreate(sNick, sUser, sHost, pac.sTrail)) {
				CString s;
				s.Format("%s is %s@%s (%s)",
					sNick, sUser, sHost, pac.sTrail);
				g_pFrame->LineNotice(NULL, s);
			}
		}
		break;

	case 312://RPL_WHOISSERVER "<nick> <server> :<server info>"
		{
			CString sNick(Token(pParm));
			CString sServer(Token(pParm));
			if (!WhoisServer(sNick, sServer, pac.sTrail)) {
				CString s;
				s.Format("%s is on server %s (%s)", sNick, sServer, pac.sTrail);
				g_pFrame->LineNotice(NULL, s);
			}
		}
		break;

	case 313://RPL_WHOISOPERATOR "<nick> :is an IRC operator"
		{
			CString sNick(Token(pParm));
			if (!WhoisInfo(sNick, pac.sTrail)) {
				CString s;
				s.Format("%s %s", sNick, pac.sTrail);
				g_pFrame->LineNotice(NULL, s);
			}
		}
		break;

    case 314://RPL_WHOWASUSER "<nick> <user> <host> * :<real name>"
		{
			CString sNick(Token(pParm));
			CString sUser(Token(pParm));
			CString sHost(Token(pParm));
			if (!WhoisCreate(sNick, sUser, sHost, pac.sTrail)) {
				CString s;
				s.Format("%s was %s@%s (%s)", sNick, sUser, sHost, pac.sTrail);
				g_pFrame->LineNotice(NULL, s);
			}
		}
		break;

	case 317://RPL_WHOISIDLE "<nick> <integer> :seconds idle"
		{
			CString sNick(Token(pParm));
			CString sIdle(Token(pParm));
			if (!WhoisIdle(sNick, sIdle, pac.sTrail)) {
				CString s;
				s.Format("%s is %s %s", sNick, sIdle, pac.sTrail);
				g_pFrame->LineNotice(NULL, s);
			}
		}
		break;

	case 319://RPL_WHOISCHANNELS "<nick> :{[@|+]<channel><space>}"
		{
			CString sNick(Token(pParm));
			if (!WhoisChannels(sNick, pac.sTrail)) {
				CString s;
				s.Format("%s is on channels: %s", sNick, pac.sTrail);
				g_pFrame->LineNotice(NULL, s);
			}
		}
		break;

	case 301://RPL_AWAY "<nick> :<away message>"
		{
			CString sNick(Token(pParm));
			if (!WhoisAway(sNick, pac.sTrail)) {
				CString s;
				s.Format("%s is away: %s", sNick, pac.sTrail);
				GetFrame(sNick)->LineNotice(NULL, s);
			}
		}
		break;

	case 305://RPL_UNAWAY ":You are no longer marked as being away"
		if (g_con.m_fAway) {
			g_con.m_fAway = FALSE;
			CmdNick(g_con.m_sNickBeforeAway);
		}
		break;

	case 306://RPL_NOWAWAY ":You have been marked as being away"
		if (!g_con.m_fAway) {
			g_con.m_fAway = TRUE;
			g_con.m_sNickBeforeAway = g_con.m_sNick;
			if (!g_opt.sNickAway.IsEmpty())
				CmdNick(g_opt.sNickAway);
		}
		break;

	case 318://RPL_ENDOFWHOIS "<nick> :End of /WHOIS list"
		break;

	case 303://RPL_ISON
		{
			CString sNick;
			LPCTSTR p = pac.sTrail;
			while (sNick = Token(p), !sNick.IsEmpty()) {
				if (ison.NickOn(sNick)) {
					if (g_opt.fIsOnPopup) {
						CIsonPop::Nick(sNick);
					} else {
						CString s;
						s.Format("Found Nick: %s", sNick);
						g_pFrame->LineStat(s);
					}
				}
			}
		}
		break;

	case 324://RPL_CHANNELMODEIS "<channel> <mode> <mode params>"
		{
			CString sChan(Token(pParm));
			GetFrame(sChan)->RplChanMode(pac, (LPCTSTR)sChan, pParm);
		}
		break;

	case 331://RPL_NOTOPIC
		{
			CString sChan(Token(pParm));
			GetFrame(sChan)->RplNoTopic(sChan);
			return;
		}
		break;

	case 332://RPL_TOPIC
		{
			CString sChan(Token(pParm));
			GetFrame(sChan)->RplTopic(sChan, pac.sTrail);
		}
		break;

	case 341://RPL_INVITING "<nick> <channel>"
		{
			CString sNick(Token(pParm));
			CString sChan(Token(pParm));
			CString s;
			s.Format("$ Inviting %s to %s", (LPCTSTR)sNick, (LPCTSTR)sChan);
			GetFrame(sChan)->LineStat(s);
		}
		break;

	case 352://RPL_WHOREPLY
			//"<channel> <user> <host> <server> <nick> \
			// <H|G>[*][@|+] :<hopcount> <real name>"
		{
			CString token[6];
			GetToken(pParm, token, 6);
			// 0:chan 1:user 2:host 3:server 4:nick 5:flag
			LPCTSTR pTrail = pac.sTrail;
			int nHopCount = atoi(Token(pTrail));
			BOOL fOp = FALSE;
			LPCTSTR pFlag = token[5];
			while (*pFlag) {
				if (*pFlag++ == '@') {
					fOp = TRUE;
					break;
				}
			}

			CUser user(token[4], token[1], token[2],
				token[3], fOp, nHopCount, pTrail);

			g_frames.RplWho(token[0], user);
			if (g_opt.fServerCollection)
				AddServer(user.m_sServer);
		}
		break;

	case 315://RPL_ENDOFWHO		"<name> :End of /WHO list"
		{
			CString sChan(Token(pParm));
			GetFrame(sChan)->RplWhoEnd();
		}
		break;

	case 321://RPL_LISTSTART "Channel :Users  Name"
		// 2.9 server doesn't sends this message anymore
		break;

	case 322://RPL_LIST "<channel> <# visible> :<topic>"
		{
			if (*pParm == '*')
				return;
			if (!g_fListChan && g_fileListChan.m_hFile == CFile::hFileNull) {
				g_fListChan = TRUE;
				CString sFile(GetDataPathname(IDS_FILE_LISTCHAN));
				if (!g_fileListChan.Open(sFile,
						CFile::modeCreate | CFile::modeReadWrite |
						CFile::shareDenyWrite | CFile::typeText)) {
					CString s;
					s.Format(IDS_FMT_OPENFILE_ERR, sFile);
					g_pFrame->LineError(s);
				}
			}
			CString sChan(Token(pParm));
			CString sNum(Token(pParm));
			CString s;
			if (g_fileListChan.m_hFile != CFile::hFileNull) {
				s = sChan + ' ' + sNum + ' ' + pac.sTrail + '\n';
				g_fileListChan.WriteString(s);
			} else {
				s = sChan + ' ' + sNum + ' ' + pac.sTrail;
				g_pFrame->LineStat(s);
			}
		}
		break;

	case 323://RPL_LISTEND ":End of /LIST"
		{
			g_fListChan = FALSE;
			CString s;
			s.LoadString(IDS_MSG_CHANLIST_DONE);
			g_pFrame->LineStat(s);
			if (g_fileListChan.m_hFile != CFile::hFileNull) {
				new CWndListChan();
				g_fileListChan.Close();
			}
		}
		break;

	case 353://RPL_NAMREPLY
		{
			CString sMark(Token(pParm));
			CString sChan(Token(pParm));
			if (sChan == "=")
				sChan = Token(pParm);
			GetFrame(sChan)->RplNames(sChan, pac.sTrail);
		}
		break;

	case 364://RPL_LINKS "<mask> <server> :<hopcount> <server info>"
		{
			CString sMask(Token(pParm));
			CString sServer(Token(pParm));
			CString s;
			s.Format("%s %s :%s\n", sMask, sServer, pac.sTrail);
			g_pFrame->LineStat(s);
		}
		break;
	case 365://RPL_ENDOFLINKS "<mask> :End of /LINKS list"
		{
			CString sMask(Token(pParm));
			CString s;
			s.Format("%s :%s\n", sMask, pac.sTrail);
			g_pFrame->LineStat(s);
		}
		break;

	case 367://RPL_BANLIST "<channel> <banid>"
		{
			CString sChan(Token(pParm));
			CString sBan(Token(pParm));
			GetFrame(sChan)->RplBanList(sChan, sBan);
		}
		break;

	case 348://RPL_EXCEPTIONLIST "<channel> <exception-id>"
		{
			CString sChan(Token(pParm));
			CString sException(Token(pParm));
			GetFrame(sChan)->RplExceptionList(sChan, sException);
		}
		break;

	case 346://RPL_INVITELIST "<channel> <invite-id>"
		{
			CString sChan(Token(pParm));
			CString sInvite(Token(pParm));
			GetFrame(sChan)->RplInviteList(sChan, sInvite);
		}
		break;

	case 329://unknown ignore
		break;

	case 376://RPL_ENDOFMOTD ":End of /MOTD command"
	case 422://ERR_NOMOTD ":MOTD File is missing"
		if (!g_con.m_fInConversation)
			OpenConversation();
		break;

	case 432://ERR_ERRONEUSNICKNAME "<nick> :Erroneus nickname"
		{
			CString sNick(Token(pParm));
			if (sNick.IsEmpty())
				sNick = sTo;
			CString s;
			s.Format("Erroneus nickname: %s", (LPCTSTR)sNick);
			// use first parm is not destination,
			// cuz nick is not yet decided
			g_pFrame->LineError(s);
			if (g_con.m_sNick.CompareNoCase(sTo) != 0)
				goto prompt_nick;
		}
		break;

	case 437://Nick/channel is temporarily unavailable
		{
			CString sNick(Token(pParm));
			CString s;
			s.Format("%s, %s", sNick, (LPCTSTR)pac.sTrail);
			g_pFrame->LineError(s);
			if (g_con.m_sNick.CompareNoCase(sNick) != 0)
				goto next_nick;
		}
		break;
	case 433://ERR_NICKNAMEINUSE "<nick> :Nickname is already in use"
		{
			CString s;
			s.Format("%s, %s", pParm, (LPCTSTR)pac.sTrail);
			g_pFrame->LineError(s);
		}
	next_nick:
		if (!g_con.m_fInConversation && !g_opt.sNick2.IsEmpty()
				&& ++g_nNickCount <= 30) {
			LPCTSTR psNick2 = g_opt.sNick2;
			if (_tcsstr(psNick2, _T("%d"))) {
				CString s;
				s.Format(psNick2, g_nNickCount);
				g_con.m_sNick = s;
			} else
				g_con.m_sNick = (g_nNickCount % 2) == 0
					? g_opt.sNick : psNick2;
			CmdNick(g_con.m_sNick);
			break;
		}
	prompt_nick:
		{
			CDlgNick dlg;
			dlg.m_sOldNick = dlg.m_sNick = g_con.m_sNick;
			if (dlg.DoModal() == IDOK)
				CmdNick(dlg.m_sNick);
		}
		break;

	case 443://ERR_USERONCHANNEL "<user> <channel> :is already on channel"
		{
			CString sNick(Token(pParm));
			CString sChan(Token(pParm));
			CString s;
			s.Format("$ %s is already on channel %s", (LPCTSTR)sNick, (LPCTSTR)sChan);
			GetFrame(sChan)->LineError(s);
		}
		break;

	case 473://ERR_INVITEONLYCHAN "<channel> :Cannot join channel (+i)"
	case 474://ERR_BANNEDFROMCHAN "<channel> :Cannot join channel (+b)"
	case 475://ERR_BADCHANNELKEY "<channel> :Cannot join channel (+k)"
		{
			CString sChan(Token(pParm));
			LPCTSTR pReason = (nCmd == 473 ? "Invite-Only"
				: nCmd == 474 ? "Banned" : "Keyword");
			CString s;
			s.Format(IDS_ERR_JOIN, sChan, pReason);
			GetFrame(sChan)->LineError(s);
			new DlgMsg(s);
			// Can't use modal messagebox.
			//AfxMessageBox(s);
		}
		break;

	case 482://ERR_CHANOPRIVSNEEDED "<channel> :You're not channel operator"
		{
			CString sChan(Token(pParm));
			GetFrame(sChan)->LineError(pac.sTrail);
		}
		break;

	others:
	default:
		if (nCmd < 300) {
			if (*pParm) {
				CString s;
				s.Format("%s %s", pParm, pac.sTrail);
				g_pFrame->LineText(NULL, s);
			} else
				g_pFrame->LineText(NULL, pac.sTrail);
		} else if (pac.sAddr.IsEmpty()) {
			CString s;
			s.Format("%03d %s :%s", nCmd, pParm, pac.sTrail);
			g_pFrame->LineText(NULL, s);
		} else
			g_pFrame->LineRaw();
	}
}

// Main parser
void ParseLine(const char* p)
{
	SkipSpace(p);

	Packet pac;

	if (*p == ':') {
		p++;
		pac.sPrefix = Token(p);
	}

	int n = pac.sPrefix.Find('!');
	if (n < 0) {
		pac.sNick = pac.sPrefix;
	} else {
		pac.sNick = pac.sPrefix.Left(n);
		pac.sAddr = (LPCTSTR)pac.sPrefix + n + 1;

		int i = pac.sAddr.Find('@');
		pac.sHost = (LPCTSTR)pac.sAddr + ((i >= 0) ? (i + 1) : 0);
	}

	int nCmd = 0;

	if (*p) {
		pac.sCmd = Token(p);
		if (pac.sCmd.GetLength() >= 3) {
			LPCTSTR pCmd = pac.sCmd;
			if (isdigit(pCmd[0]) && isdigit(pCmd[1]) && isdigit(pCmd[2]))
				nCmd = atoi(pCmd);
		}
	}

	char* q;
	if (*p == ':') {
		//pac.sParm.Empty();
		pac.sTrail = p + 1;
	} else if ((q = strstr(p, " :")) != 0) {
		pac.sParm = CString(p, q - p);
		pac.sTrail = q + 2;
	} else {
		pac.sParm = p;
		//pac.sTrail.Empty();
	}

	if (nCmd)
		GotCmdN(nCmd, pac);
	else
		GotCmdA(pac);
}

// sending part

void CmdUserReg()
{
	CString s(g_sd.m_sPassword);
	if (!s.IsEmpty())
		Cmd("PASS", s);

	Cmd("NICK", g_con.m_sNick);

	CmdAdd("USER");
	CmdAdd(g_opt.sLoginName);
	CmdAdd(g_opt.sHostName);
	s = g_sd.m_sHostname;
	CmdAdd(s);
	s.Format(":%s", g_opt.sRealName);
	CmdAdd(s);
	CmdSend();
}

void CmdNick(LPCTSTR sNick)
{
	if (sock.GetStatus() & SOCK_CONNECT)
		Cmd("NICK", sNick);
	if (!g_con.m_fInConversation) {
		g_con.m_sNick = sNick;
		g_frames.MakeAllTitle();
	}
}

void CmdJoin(LPCTSTR pTarget, LPCTSTR psKey)
{
	if (!pTarget || !*pTarget)
		return;
	CmdAdd("JOIN");
	CmdAdd(pTarget);
	if (psKey)
		CmdAdd(psKey);
	CmdSend();
}

void CmdTalk(LPCTSTR pTarget)
{
	if (!pTarget || !*pTarget)
		return;
	CFrame* pFrame = GetFrame(pTarget);
	if (IsMainFrame(pFrame))
		new FrameTalk(pTarget);
	else
		pFrame->ActivateFrame(pFrame->IsIconic() ? SW_RESTORE : SW_SHOW);
}

void CmdAway(LPCTSTR pMsg /* == 0 */)
{
	if (pMsg && *pMsg) {
		g_con.m_sLastAway = pMsg;
		CmdTrail("AWAY", pMsg);
	} else
		Cmd("AWAY");
}

void CmdMe(LPCTSTR pTarget, LPCTSTR pMsg)
{
	if (!pTarget || !*pTarget)
		return;
	CString s;
	s.Format("PRIVMSG %s :\001ACTION %s\001", pTarget, pMsg);
	Cmd(s);
}

void CmdInvite(LPCTSTR pNick, LPCTSTR pChan)
{
	Cmd("INVITE", pNick, pChan);
}

void CmdWhois(LPCTSTR pTarget)
{
	if (g_opt.fWhoisDetail)
		Cmd("WHOIS", pTarget, pTarget);
	else
		Cmd("WHOIS", pTarget);
}

// CMD DCC

void CmdDccChat(CString& sTarget)
{
	FrameChat* pFrame = new FrameChat(sTarget);
	if (!pFrame->ChatListen())
		return;
}

void CmdDccSend(LPCTSTR pNick, LPCTSTR pFile)
{
	CDlgDcc* pDlg = new CDlgDcc(CDlgDcc::TypeSnd);
	if (!pDlg->OpenFile(pFile))
		return;

	u_short usPort = g_opt.nDccPort;
	if (!pDlg->DoListen(usPort))
		return;

	u_long ulHost = htonl(sock.sockaddr.sin_addr.s_addr);
	if (g_opt.fUseDccFakeAddr)
		ulHost = htonl(g_opt.addrDccFakeAddr);
	usPort = htons(usPort);

	CString sFileSend(EscapeString(ExtractFileName(pDlg->m_sFile)));
	CString s;
	s.Format("PRIVMSG %s :\001DCC SEND %s %lu %u %u\001",
		pNick, sFileSend, ulHost, usPort, pDlg->GetFileSize());
	Cmd(s);
}

void CmdDccSend(LPCTSTR pNick, HDROP hDrop)
{
	int nFiles = DragQueryFile(hDrop, (UINT)-1, NULL, 0);
	char buf[_MAX_PATH];
	int i;
	for (i = 0; i < nFiles; i++) {
		DragQueryFile(hDrop, i, buf, sizeof buf);
		CmdDccSend(pNick, buf);
	}
}

// CMD CTCP

void CmdCtcpPing(LPCTSTR pTarget)
{
	DWORD t = GetTickCount();
	CString s;
	s.Format("PRIVMSG %s :\001PING %ld\001", pTarget, t);
	Cmd(s);
}

void CmdCtcpTime(LPCTSTR pTarget)
{
	CString s;
	s.Format("PRIVMSG %s :\001TIME\001", pTarget);
	Cmd(s);
}

void CmdCtcpUserinfo(LPCTSTR pTarget)
{
	CString s;
	s.Format("PRIVMSG %s :\001USERINFO\001", pTarget);
	Cmd(s);
}

void CmdCtcpFinger(LPCTSTR pTarget)
{
	CString s;
	s.Format("PRIVMSG %s :\001FINGER\001", pTarget);
	Cmd(s);
}

void CmdCtcpVersion(LPCTSTR pTarget)
{
	CString s;
	s.Format("PRIVMSG %s :\001VERSION\001", pTarget);
	Cmd(s);
}

void CmdCtcpClientinfo(LPCTSTR pTarget)
{
	CString s;
	s.Format("PRIVMSG %s :\001CLIENTINFO\001", pTarget);
	Cmd(s);
}

// misc

void OpenConversation()
{
	if (!g_con.m_fInConversation) {
		g_con.m_fInConversation = TRUE;
		if (g_con.m_fAway)
			CmdAway(g_con.m_sLastAway);
		g_nNickCount = 0;
		g_frames.Rejoin();
		g_mapChan.AutoJoin();
		ison.Start(g_opt.nIsOnInterval);
		g_opt.usermode.Apply(g_con.m_sNick);
		g_con.m_nConnectRetry = 0;
	}
}

void CloseConversation()
{
	if (g_con.m_fInConversation) {
		g_con.m_fInConversation = FALSE;
		ison.Stop();
	}
	if (ident.m_fListening)
		ident.End();
	g_fListChan = FALSE;
	if (g_fileListChan.m_hFile != CFile::hFileNull)
		g_fileListChan.Close();
}

UINT g_tmidQuit = 0;

BOOL CmdQuit(LPCTSTR pQuitMsg, BOOL fReconnect)
{
	g_con.m_fReconnecting = fReconnect;
	g_con.m_fQuitting = TRUE;
	CloseConversation();
	WORD stat = sock.GetStatus();
	if (stat & SOCK_CONNECT) {
		CmdTrail("QUIT", pQuitMsg ? pQuitMsg : LoadQuitMsg());
		g_tmidQuit = g_pFrame->SetTimer(TMID_QUIT, 3000, NULL);
		return TRUE;	// true means quitting process is in progress
	}
	if (stat & SOCK_CONNWAIT) {
		sock.DoClose();
		SockNotice(SN_CLOSED);
	}
	return FALSE;
}

void ReConnect(LPCTSTR pQuitMsg)
{
	WORD stat = sock.GetStatus();
	if (stat & SOCK_CONNECT) {
		CmdQuit(pQuitMsg, TRUE);
		return;
	}
	if (stat & SOCK_CONNWAIT)
		sock.DoClose();
	g_con.m_fReconnecting = TRUE;
	g_pFrame->PostMessage(WM_COMMAND, ID_CONNECT);
}

void ChangeServer(LPCTSTR pServer, UINT nPort, LPCTSTR pPassword)
{
	CmdQuit("Server Change", TRUE);
	g_sd.Pack(pServer, nPort, pPassword);
	g_frames.MakeAllTitle();
}

// sock notify handler

void SockNotice(int nCode, LPCTSTR pStr)
{
	CString s;

	switch (nCode) {
	case SN_CONNECTED:
		optBeep.Play(BEEP_Connect);
		if (g_opt.fIdentd && !ident.m_fListening)
			ident.Start();
		g_pFrame->ShowStatus(IDS_MSG_CONNECTED);
		g_pFrame->StatChange(TRUE);
		CmdUserReg();
		g_nNickCount = 0;
		g_con.m_fReconnecting = g_con.m_fQuitting = FALSE;
		break;
	case SN_CANNOTRESOLVE:
		MessageBeep(MB_ICONEXCLAMATION);
		g_pFrame->LineError(IDS_MSG_CANNOTRESOLVE);
		goto connect_retry;
		//break;
	case SN_CONNFAIL:
		MessageBeep(MB_ICONEXCLAMATION);
		g_pFrame->LineError(IDS_MSG_CONNFAILED);
		goto connect_retry;
		//break;
	case SN_CLOSED:
		if (g_tmidQuit) {
			g_pFrame->KillTimer(g_tmidQuit);
			g_tmidQuit = 0;
		}
		if (g_con.m_fInConversation)
			CloseConversation();
		sock.DoClose();
		optBeep.Play(BEEP_Disconnect);
		g_pFrame->ShowStatus(IDS_MSG_CLOSED);
		g_frames.MsgDisconnect();
		g_pFrame->StatChange(FALSE);
		if (g_fExitApp) {
			g_pFrame->PostMessage(WM_CLOSE);
			break;
		}
	connect_retry:
		if (g_con.m_fReconnecting || !g_con.m_fQuitting && g_opt.fReconnect) {
			if (++g_con.m_nConnectRetry < CONNECT_RETRY_MAX) {
				g_pFrame->PostMessage(WM_COMMAND, ID_CONNECT);
				g_con.m_fReconnecting = TRUE;
			} else {
				g_con.m_nConnectRetry = 0;
				g_con.m_fReconnecting = FALSE;
			}
		}
		g_con.m_fQuitting = FALSE;
		break;
	case SN_ERROR:
		g_pFrame->LineError(pStr);
		break;
	default:
		s.Format(IDS_MSG_NOTICE, nCode);
		g_pFrame->LineStat(s);
	}
}

void TimerEventQuit()
{
	SockNotice(SN_CLOSED);
}

void TimerEventIsOn()
{
	if (!g_con.m_fInConversation)
		return;

	CString s;
	int i;
	for (i = 0; i < ison.GetSize(); i++) {
		CIsOnEntry& entry = ison.ElementAt(i);
		if (s.GetLength() + entry.m_sNick.GetLength() > 80) {
			Cmd("ISON", s);
			s.Empty();
		}
		s += entry.m_sNick + ' ';
	}
	if (!s.IsEmpty())
		Cmd("ISON", s);
	ison.Expire();
}

// channel/user mode command
void ModeCmd::Add(BOOL fPlus, TCHAR c, LPCTSTR pParm)
{
	if (!m_nCmd || m_fPlus != fPlus)
		m_sCmd += ((m_fPlus = fPlus) ? '+' : '-');
	m_sCmd += c;
	if (pParm) {
		if (!m_sParm.IsEmpty())
			m_sParm += ' ';
		m_sParm += pParm;
	}
	if (++m_nCmd >= 3)
		Flush();
}

void ModeCmd::Flush()
{
	if (!m_nCmd)
		return;
	if (!m_sParm.IsEmpty())
		Cmd("Mode", m_sTarget, m_sCmd, m_sParm);
	else
		Cmd("Mode", m_sTarget, m_sCmd);
	m_nCmd = 0;
	m_sCmd.Empty();
	m_sParm.Empty();
}

// user mode

void UserMode::Clear()
{
	m_fInvisible = FALSE;
	m_fWallops = FALSE;
}

CString UserMode::ToString()
{
	CString s( '+');
	if (m_fInvisible)
		s += 'i';
	if (m_fWallops)
		s += 'w';
	if (s == "+")
		s.Empty();
	return s;
}

void UserMode::FromString(LPCTSTR p)
{
	CString sOP;
	CString sParm;
	CString s;
	SkipSpace(p);
	while (*p) {
		s = Token(p);
		if (s[0] == '+' || s[0] == '-')
			sOP += s;
		else
			sParm += (s + " ");
	}

	if (sOP.GetLength() < 2 && sParm.IsEmpty())
		return;

	LPCTSTR pParm = sParm;
	BOOL fPlus = TRUE;
	LPCTSTR pOP;
	for (pOP = sOP; *pOP; pOP++) {
		switch (*pOP) {
		case '+':
			fPlus = TRUE;
			break;
		case '-':
			fPlus = FALSE;
			break;
		case 'i':
			m_fInvisible = fPlus;
			break;
		case 'w':
			m_fWallops = fPlus;
			break;
		}
	}
}

void UserMode::Apply(LPCTSTR sTarget)
{
	if (sock.GetStatus() & SOCK_CONNECT) {
		ModeCmd cmd(sTarget);
		cmd.Add(m_fInvisible, 'i');
		cmd.Add(m_fWallops, 'w');
		cmd.Flush();
	}
}
