// lbBeep.cpp : Cve[V t@C

#include "stdafx.h"
#include "resource.h"
#include "profile.h"
#include "lbBeep.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BeepItem::BeepItem(TagBeep& beep)
{
	m_pBeep = &beep;
	m_fBeep = beep.m_fBeep;
	m_pLabel = beep.m_sLabel;
	m_fWave = beep.m_fWave;
	m_sWave = beep.m_sWave;
}

void BeepItem::Apply()
{
	m_pBeep->m_fBeep = m_fBeep;
	m_pBeep->m_fWave = m_fWave;
	m_pBeep->m_sWave = m_sWave;

}

// ClbBeep

#define CHECKBOXSIZE	12
#define SEPSIZE			2

ClbBeep::ClbBeep()
{
}

ClbBeep::~ClbBeep()
{
}

void ClbBeep::Init() 
{
	int nLabelWidth = 10;
	int nWaveWidth = 10;
	CClientDC dc(this);

	int i;
	for (i = 0; i < optBeep.GetSize(); i++) {
		TagBeep& tag = optBeep[i];
		int n = dc.GetTextExtent(tag.m_sLabel).cx;
		if (n > nLabelWidth)
			nLabelWidth = n;
		n = dc.GetTextExtent(tag.m_sWave).cx;
		if (n > nWaveWidth)
			nWaveWidth = n;
	}

	m_cxBeepCheckFrom = SEPSIZE;
	m_cxBeepCheckTo = m_cxBeepCheckFrom + CHECKBOXSIZE;
	m_cxLabelFrom = m_cxBeepCheckTo + SEPSIZE;
	m_cxWaveCheckFrom = m_cxLabelFrom + nLabelWidth + SEPSIZE;
	m_cxWaveCheckTo = m_cxWaveCheckFrom + CHECKBOXSIZE;
	m_cxWaveFrom = m_cxWaveCheckTo + SEPSIZE;
	m_cxEnd = m_cxWaveFrom + nWaveWidth + SEPSIZE;

	for (i = 0; i < optBeep.GetSize(); i++) {
		TagBeep& tag = optBeep[i];
		BeepItem* pItem = new BeepItem(tag);
		AddString((LPCTSTR)pItem);
	}
}

void ClbBeep::Save()
{
	int i;
	for (i = 0; i < GetCount(); i++) {
		BeepItem* pItem = (BeepItem*)GetItemData(i);
		if ((DWORD)pItem != LB_ERR)
			pItem->Apply();
	}
	optBeep.Save();
}

BOOL ClbBeep::SelectWaveFile(BeepItem* pItem)
{
	LPCTSTR szFilter = "Wave Files(*.wav)|*.wav|All Files (*.*)|*.*||";
 	CFileDialog dlg(TRUE, "wav", pItem->m_sWave,
		OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_LONGNAMES, szFilter);
	dlg.m_ofn.lpstrTitle = "Select Wave File";
	if (dlg.DoModal() != IDOK)
		return FALSE;
	pItem->m_sWave = dlg.GetPathName();
	return TRUE;
}

UINT ClbBeep::ItemFromPoint(CPoint pt, BOOL &bOutside)
{
	// If we are on Win95, just call the base class
	if((BYTE)::GetVersion() >= 4)
		return CListBox::ItemFromPoint(pt, bOutside);

	if(GetCount() < 1) {
		bOutside = TRUE;
		return 0;
	}

	CRect rectClient;
	GetClientRect(&rectClient);

	bOutside = !rectClient.PtInRect(pt);

	if (pt.y < rectClient.top)
		pt.y = rectClient.top;
	if (pt.x < rectClient.left)
		pt.x = rectClient.left;

	CRect rectCheck;
	int lastY = -1;
	int nIndex;
	for (nIndex = GetTopIndex(); nIndex < GetCount(); nIndex++) {
		GetItemRect(nIndex , &rectCheck);
		rectCheck.right += 1;
		if(!rectClient.PtInRect(rectCheck.TopLeft()))
			break;
		if (rectCheck.PtInRect(pt))
			return nIndex;
		// check for case where mouse is below the last item in a column of
		// an LBS_MULTICOLUMN list box
		if ((GetStyle() & LBS_MULTICOLUMN) &&
				nIndex < GetCount() - 1 &&
				pt.x >= rectCheck.left &&
				pt.x < rectCheck.right) {
			CRect rectNext;
			GetItemRect(nIndex + 1, &rectNext);
			if (rectNext.left > rectCheck.left)
				return nIndex;
		}
		if (pt.y >= rectCheck.top)
			lastY = nIndex;
	}
	return lastY;
}
 

BEGIN_MESSAGE_MAP(ClbBeep, CListBox)
	//{{AFX_MSG_MAP(ClbBeep)
	ON_WM_LBUTTONDOWN()
	ON_WM_MEASUREITEM_REFLECT()
	ON_WM_CHAR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// ClbBeep bZ[W nh

void ClbBeep::MeasureItem(LPMEASUREITEMSTRUCT lpMIS) 
{
	lpMIS->itemWidth = m_cxEnd;
}

void ClbBeep::DrawItem(LPDRAWITEMSTRUCT lpDIS) 
{
	CDC* pDC = CDC::FromHandle(lpDIS->hDC);
	UINT itemState = lpDIS->itemState;
	CRect rcItem(lpDIS->rcItem);

	BeepItem* pItem;
	if ((int)lpDIS->itemID >= 0 && (pItem = (BeepItem*)lpDIS->itemData) != 0) {
		COLORREF crText;
		COLORREF crBack;
		if (itemState & ODS_SELECTED) {
			crText = GetSysColor(COLOR_HIGHLIGHTTEXT);
			crBack = GetSysColor(COLOR_HIGHLIGHT);
		} else {
			crText = GetSysColor(COLOR_WINDOWTEXT);
			crBack = GetSysColor(COLOR_WINDOW);
		}
		COLORREF crTextSave = pDC->SetTextColor(crText);
		COLORREF crBackSave = pDC->SetBkColor(crBack);
		pDC->FillSolidRect(&rcItem, crBack);

		int n = (rcItem.Height() - CHECKBOXSIZE) / 2;
		CRect rcCheck(
			m_cxBeepCheckFrom, rcItem.top + n,
			m_cxBeepCheckTo, rcItem.bottom - n);
		UINT nButtonState = pItem->m_fBeep ? DFCS_CHECKED : 0;
		pDC->DrawFrameControl(&rcCheck, DFC_BUTTON,
			DFCS_ADJUSTRECT|DFCS_BUTTONCHECK | nButtonState);

		CRect rcText(m_cxLabelFrom, rcItem.top, rcItem.right, rcItem.bottom);
		pDC->DrawText(pItem->m_pLabel, &rcText,
			DT_LEFT|DT_NOPREFIX|DT_VCENTER|DT_SINGLELINE);

		rcCheck.left = m_cxWaveCheckFrom;
		rcCheck.right = m_cxWaveCheckTo;
		nButtonState = pItem->m_fWave ? DFCS_CHECKED
			: pItem->m_fBeep ? 0
			: DFCS_INACTIVE;
		pDC->DrawFrameControl(&rcCheck, DFC_BUTTON,
			DFCS_ADJUSTRECT|DFCS_BUTTONCHECK | nButtonState);

		rcText.left = m_cxWaveFrom;
		rcText.right = rcItem.right;
		if (!pItem->m_fWave)
			pDC->SetTextColor(GetSysColor(COLOR_GRAYTEXT));
		pDC->DrawText(pItem->m_sWave, &rcText,
			DT_LEFT|DT_NOPREFIX|DT_VCENTER|DT_SINGLELINE);

		pDC->SetTextColor(crTextSave);
		pDC->SetTextColor(crBackSave);
	}

	if (itemState & ODS_FOCUS)
		pDC->DrawFocusRect(&rcItem);
}

void ClbBeep::DeleteItem(LPDELETEITEMSTRUCT lpDIS) 
{
	BeepItem* pItem = (BeepItem*)lpDIS->itemData;
	delete pItem;

	CListBox::DeleteItem(lpDIS);
}

void ClbBeep::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CListBox::OnLButtonDown(nFlags, point);
	if (nFlags != MK_LBUTTON)
		return;
	BOOL fOutside;
	int index = ItemFromPoint(point, fOutside);
	if (fOutside)
		return;

	BeepItem* pItem = (BeepItem*)GetItemData(index);
	if ((DWORD)pItem == LB_ERR)
		return;

	CRect rc;
	if (GetItemRect(index, &rc) == LB_ERR)
		return;
	if (!rc.PtInRect(point))
		return;

	int x = point.x;
	if (x >= m_cxBeepCheckFrom && x < m_cxBeepCheckTo) {
		pItem->m_fBeep = !pItem->m_fBeep;
		if (!pItem->m_fBeep && pItem->m_fWave)
			pItem->m_fWave = FALSE;
	} else if (x >= m_cxWaveCheckFrom && x < m_cxWaveCheckTo) {
		pItem->m_fWave = !pItem->m_fWave;
		if (pItem->m_fWave) {
			pItem->m_fBeep = TRUE;
			if (pItem->m_sWave.IsEmpty() && !SelectWaveFile(pItem))
				pItem->m_fWave = FALSE;
		}
	} else if (x >= m_cxWaveFrom) {
		if (!SelectWaveFile(pItem))
			return;
		pItem->m_fWave = TRUE;
	} else
		return;

	InvalidateRect(&rc);
}

void ClbBeep::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CListBox::OnChar(nChar, nRepCnt, nFlags);

	if (nChar != ' ')
		return;

	int nIndex = GetCurSel();
	if (nIndex == LB_ERR)
		return;

	BeepItem* pItem = (BeepItem*)GetItemData(nIndex);
	if ((DWORD)pItem == LB_ERR)
		return;

	if (pItem->m_fBeep) {
		if (pItem->m_fWave)
			pItem->m_fBeep = pItem->m_fWave = FALSE;
		else if (SelectWaveFile(pItem))
			pItem->m_fWave = TRUE;
		else
			pItem->m_fBeep = FALSE;
	} else {
		pItem->m_fBeep = TRUE;
		pItem->m_fWave = FALSE;
	}

	CRect rc;
	if (GetItemRect(nIndex, &rc) != LB_ERR)
		InvalidateRect(&rc);
}
