#include "stdafx.h"
#include "match.h"

// simple string match

Match::Match()
{
	m_nType = TypeNone;
}

void Match::Set(LPCTSTR psz)
{
	if (!psz || !*psz) {
		m_nType = TypeNone;
		m_sPtn.Empty();
		return;
	}
	if (*psz == '*') {
		if (psz[1] == 0) {
			m_nType = TypeAny;
			m_sPtn = '*';
			return;
		}
		m_nType = TypeTail;
		m_sPtn = CString(psz + 1);
		return;
	}
	int iLastChar = lstrlen(psz) - 1;
	if (psz[iLastChar] == '*') {
		m_nType = TypeHead;
		m_sPtn = CString(psz, iLastChar);
		return;
	}
	m_nType = TypeFull;
	m_sPtn = CString(psz);
}

BOOL Match::IsMatch(LPCTSTR pszStr) const
{
	//TRACE("Match.IsMatch(%s) type=%d ptn=<%s>\n",
	//	pszStr, m_nType, (LPCTSTR)m_sPtn);
	switch (m_nType) {
	case TypeAny:
		return TRUE;
	case TypeFull:
		return lstrcmpi(pszStr, m_sPtn) == 0;
	case TypeHead:
		return strnicmp(pszStr, m_sPtn, m_sPtn.GetLength()) == 0;
	case TypeTail:
		{
			int nLen = lstrlen(pszStr);
			int nPtn = m_sPtn.GetLength();
			if (nLen < nPtn)
				return FALSE;
			LPCTSTR pszTail = pszStr + (nLen - nPtn);
			return lstrcmpi(pszTail, m_sPtn) == 0;
		}
	default:
		return FALSE;
	}
}

BOOL Match::IsSame(const Match& m) const
{
	return (m_nType == m.m_nType && m_sPtn.CompareNoCase(m.m_sPtn) == 0);
}

Match::operator CString() const
{
	CString s;
	switch (m_nType) {
	case TypeTail:
		s = CString('*') + m_sPtn;
		break;
	case TypeHead:
		s = m_sPtn + '*';
		break;
	case TypeAny:
		s = '*';
		break;
	default:
		s = m_sPtn;
	}
	return s;
}

// nick & address

void NickAddr::Set(LPCTSTR pStr)
{
	TCHAR pBuf[256];
	lstrcpyn(pBuf, pStr, sizeof pBuf);

	LPTSTR pExc = strchr(pBuf, '!');
	if (pExc) {
		*pExc++ = 0;
		m_sNick = CString(pBuf);
	} else {
		pExc = pBuf;
		m_sNick = '*';
	}

	LPTSTR pAt = strchr(pExc, '@');
	if (pAt) {
		*pAt++ = 0;
		m_sAccount = CString(pExc);
	} else {
		pAt = pExc;
		m_sAccount = '*';
	}

	if (*pAt)
		m_sAddress = pAt;
	else
		m_sAddress = '*';
}

CString NickAddr::GetString()
{
	CString s(m_sNick + '!' + m_sAccount + '@' + m_sAddress);
	return s;
}

// nick & address match

void NickMatch::Set(LPCTSTR psz)
{
	NickAddr na(psz);
	m_nick.Set(na.m_sNick);
	m_account.Set(na.m_sAccount);
	m_address.Set(na.m_sAddress);
}

BOOL NickMatch::IsMatch(LPCTSTR psz) const
{
	TCHAR buf[128];
	LPCTSTR pExc = strchr(psz, '!');
	if (pExc) {
		int nLen = pExc++ - psz;
		if (nLen >= sizeof buf - 1)
			nLen = sizeof buf - 2;
		memcpy(buf, psz, nLen);
		buf[nLen] = 0;
	} else {
		buf[0] = 0;
		pExc = psz;
	}
	if (!m_nick.IsMatch(buf))
		return FALSE;

	LPCTSTR pAt = strchr(pExc, '@');
	if (pAt) {
		int nLen = pAt++ - pExc;
		if (nLen >= sizeof buf - 1)
			nLen = sizeof buf - 2;
		memcpy(buf, pExc, nLen);
		buf[nLen] = 0;
	} else {
		buf[0] = 0;
		pAt = pExc;
	}
	if (!m_account.IsMatch(buf))
		return FALSE;

	return m_address.IsMatch(pAt);
}

CString NickMatch::GetString() const
{
	CString s;
	s.Format("%s!%s@%s",
		CString(m_nick), CString(m_account), CString(m_address));
	return s;
}

BOOL NickMatch::IsSame(const NickMatch& nm) const
{
	return m_nick.IsSame(nm.m_nick) &&
		m_account.IsSame(nm.m_account) &&
		m_address.IsSame(nm.m_address);
}

// list of matches

POSITION LsNickMatch::Find(const NickMatch& nmFind)
{
	POSITION pos = GetHeadPosition();
	while (pos) {
		POSITION pos1 = pos;
		NickMatch& nm = GetNext(pos);
		if (nm.IsSame(nmFind))
			return pos1;
	}
	return 0;
}

void LsNickMatch::Copy(const LsNickMatch& ls)
{
	RemoveAll();
	POSITION pos = ls.GetHeadPosition();
	while (pos) {
		NickMatch nm = ls.GetNext(pos);
		AddTail(nm);
	}
}

BOOL LsNickMatch::IsMatch(LPCTSTR psz)
{
	POSITION pos = GetHeadPosition();
	while (pos) {
		NickMatch& nm = GetNext(pos);
		if (nm.IsMatch(psz))
			return TRUE;
	}
	return FALSE;
}
