// irc.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "mchat.h"
#include "dlglsch.h"
#include "dlgmisc.h"

#include <winreg.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*********************************************************************
* FUNCTION: GetNumScrollLines
* Purpose : An OS independant method to retrieve the number of wheel
*           scroll lines
* Params  : none
* Returns : UINT number of scroll lines
*********************************************************************/
UINT GetNumScrollLines(void)
{
	HKEY hdlKey;
	DWORD keyDataType;
	CHAR charData[128];
	DWORD  dwDataBufSize;
	UINT ucNumLines=3;  // 3 is the default
	OSVERSIONINFO osversion;

	memset(&osversion, 0, sizeof(OSVERSIONINFO));
	osversion.dwOSVersionInfoSize =sizeof(OSVERSIONINFO);
	GetVersionEx(&osversion);
	// Read registry directly for Windows 9x & WinNT3.51, if WinNT 4.0 
	// and above then use SystemParametersInfo

	if (osversion.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ||
		osversion.dwPlatformId == VER_PLATFORM_WIN32_NT && 
			osversion.dwMajorVersion < 4) {
		// Read registry directly
		if (RegOpenKeyEx(HKEY_CURRENT_USER, 
				"Control Panel\\Desktop", 
				0,
				KEY_QUERY_VALUE,
				&hdlKey) == ERROR_SUCCESS) {
			dwDataBufSize = sizeof(charData);
			if (RegQueryValueEx(hdlKey, 
					"WheelScrollLines",
					NULL,  // reserved
					&keyDataType,
					(LPBYTE) &charData,
					&dwDataBufSize) == ERROR_SUCCESS) {
				//String representation
				ucNumLines = strtoul(charData, NULL, 10);
			}
		}
	} else if (osversion.dwPlatformId == VER_PLATFORM_WIN32_NT &&
			osversion.dwMajorVersion >= 4) {
		SystemParametersInfo(SPI_GETWHEELSCROLLLINES, 0, &ucNumLines, 0);
	}
	return(ucNumLines);
}

static TCHAR BASED_CODE szVersion[] = _T("Version");
static TCHAR BASED_CODE szCurrent[] = _T("Current");

Con g_con; // current connection status

BOOL g_fLoginName;		// login name is legal
BOOL g_fExitApp;		// now closing application
BOOL g_fVersionChange;

UINT g_nScrollLines = 3;		// scroll lines per mouse wheel

// CApp

BEGIN_MESSAGE_MAP(CApp, CWinApp)
	//{{AFX_MSG_MAP(CApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_SETUP, OnSetup)
	ON_COMMAND(ID_CONNECT, OnConnect)
	ON_UPDATE_COMMAND_UI(ID_CONNECT, OnUpdateConnect)
	ON_COMMAND(ID_DISCONNECT, OnDisconnect)
	ON_UPDATE_COMMAND_UI(ID_DISCONNECT, OnUpdateDisconnect)
	ON_COMMAND(ID_WINDOWS, OnWindows)
	ON_COMMAND(ID_LISTCHAN, OnListchan)
	ON_UPDATE_COMMAND_UI(ID_JOIN, OnUpdateIsConnect)
	ON_UPDATE_COMMAND_UI(ID_TALKWITH, OnUpdateIsConnect)
	ON_UPDATE_COMMAND_UI(ID_LISTCHAN, OnUpdateIsConnect)
	//}}AFX_MSG_MAP
	// Standard file based document commands
END_MESSAGE_MAP()

// CApp construction

CApp::CApp()
{
}

// The one and only CApp object

CApp theApp;

// CApp initialization

#include "tracewin.h"

static char szHelpKey[] = "SOFTWARE\\Microsoft\\Windows\\Help";

BOOL CApp::InitInstance()
{
#ifdef _DEBUG
	CMfxTrace::Init();
	afxMemDF |= checkAlwaysMemDF;
#endif

	LPCTSTR pStr = ExtractFileName(m_pszHelpFilePath);
	CString sDir = ExtractDirName(m_pszHelpFilePath);
	HKEY hKey;
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, szHelpKey, 0,
			KEY_READ | KEY_WRITE, &hKey) == ERROR_SUCCESS) {
		RegSetValueEx(hKey, pStr, 0, REG_SZ,
			(const unsigned char*)(LPCTSTR)sDir,
			sDir.GetLength() + 1);
		RegCloseKey(hKey);
	}

	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	SetRegistryKey(IDS_REGISTRY);

	CString sVersion = GetProfileString(szVersion, szCurrent);
	CString s;
	s.LoadString(IDS_VERSION);
	if (s != sVersion)
		g_fVersionChange = TRUE;

	g_nScrollLines = GetNumScrollLines();
	if (g_nScrollLines < 1)
		g_nScrollLines = 1;

	g_fonts.LoadAll();
	LoadServers();
	LoadServerInfo();
	LoadOptions();

	g_con.m_sNick = g_opt.sNick;

	if (g_opt.sLoginName.IsEmpty()) {
		char buf[50];
		DWORD dwLen = sizeof buf;
		if (GetUserName(buf, &dwLen)) {
			g_fLoginName = TRUE;
			LPCTSTR psz = buf;
			g_opt.sLoginName = Token(psz);
		} else
			g_opt.sLoginName = g_con.m_sNick;
	}

	if (g_opt.sHostName.IsEmpty())
		g_opt.sHostName = "localhost";

	NickListInit();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	sock.Init();

	g_pFrame = m_pFrame = new FrameMain;
	if (!m_pFrame->LoadFrame(IDR_MAIN, WS_OVERLAPPEDWINDOW))
		return FALSE;
	m_pFrame->ShowWindow(m_nCmdShow);
	m_pFrame->UpdateWindow();
	m_pMainWnd = m_pFrame;
	m_pFrame->FontChange();

	if (g_opt.fDebugLog)
		OpenDebugFile();

	g_mapChan.Load();

	PlugInit();

	ison.Load();
	IgnoreInit();

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
#if 0
	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;
#endif

	if (cmdInfo.m_nShellCommand == CCommandLineInfo::FileOpen)
		ConnectTo(cmdInfo.m_strFileName);
	else if (g_opt.fConnectOnStartup)
		m_pFrame->PostMessage(WM_COMMAND, ID_CONNECT);

	return TRUE;
}

void CApp::ConnectTo(LPCTSTR pStr)
{
	CString sUrl(pStr);
	if (sUrl.Left(4).CompareNoCase("irc:") == 0)
		sUrl = (LPCTSTR)sUrl + 4;
	if (sUrl.Left(2).Compare("//") == 0)
		sUrl = (LPCTSTR)sUrl + 2;

	CString sHost;
	int nPort = 6667;
	CString sPass;
	CString sTarget;

	int n = sUrl.ReverseFind(',');
	if (n >= 0) {
		sPass = (LPCTSTR)sUrl + n + 1;
		sUrl = sUrl.Left(n);
	}
	n = sUrl.ReverseFind('/');
	if (n >= 0) {
		sTarget = (LPCTSTR)sUrl + n + 1;
		sUrl = sUrl.Left(n);
	}
	n = sUrl.ReverseFind(':');
	if (n >= 0) {
		nPort = atoi((LPCTSTR)sUrl + n + 1);
		sUrl = sUrl.Left(n);
	}
	sHost = sUrl;

	if (!sHost.IsEmpty()) {
		if (nPort <= 0)
			nPort = 6667;
		g_sd.Pack(sHost, nPort, sPass);
		g_frames.MakeAllTitle();
	}

	m_pFrame->PostMessage(WM_COMMAND, ID_CONNECT);
}

void CApp::UpdateVersion()
{
	CString s;
	s.LoadString(IDS_VERSION);
	WriteProfileString(szVersion, szCurrent, s);
	g_fVersionChange = FALSE;
	TRACE("version info updated\n");
}

int CApp::ExitInstance() 
{
	sock.End();

	SaveServerInfo();
	SaveServers();

	g_frames.ClearFrames();
	g_mapChan.Save();
	g_aKnownNicks.RemoveAll();

	if (IsDebugLogging())
		CloseDebugFile();

	return CWinApp::ExitInstance();
}

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString	m_sVersion;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_sVersion = _T("");
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_VERSION, m_sVersion);
	//}}AFX_DATA_MAP
}

BOOL CAboutDlg::OnInitDialog() 
{
	CString sVer;
	sVer.LoadString(IDS_VERSION);
	m_sVersion.Format("MuriChat Version %s", sVer);
	CDialog::OnInitDialog();
	return TRUE;
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

// CApp commands

void CApp::OnSetup() 
{
	PopupSetup();
}

void CApp::OnConnect() 
{
	CWaitCursor wait;
	if (CmdQuit(0, TRUE))
		return;		// quitting process is incomplete

	CString sMsg;
	sMsg.Format(IDS_MSG_CONNECTING, (LPCTSTR)g_sd.m_sHostname, g_sd.m_nPort);
	m_pFrame->LineStat(sMsg);
	m_pFrame->UpdateWindow();
	sock.DoConnect(g_sd.m_sHostname, g_sd.m_nPort);
}

void CApp::OnUpdateConnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((sock.GetStatus() & (SOCK_CONNECT | SOCK_CONNWAIT)) == 0
		&& !g_sd.m_sHostname.IsEmpty() && g_sd.m_nPort > 0);
}

void CApp::OnDisconnect() 
{
	CmdQuit();
	g_con.m_nConnectRetry = 0;
}

void CApp::OnUpdateDisconnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((sock.GetStatus() & (SOCK_CONNECT | SOCK_CONNWAIT)) != 0);
}

void CApp::OnUpdateIsConnect(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((sock.GetStatus() & SOCK_CONNECT) != 0);
}

void CApp::OnWindows() 
{
	g_frames.SelectFrame();
}

void CApp::OnListchan() 
{
	CDlgListChan dlg;
	if (dlg.DoModal() != IDOK)
		return;
	m_pFrame->LineStat(IDS_MSG_RETRIEVE_CHANLIST);
	Cmd("LIST", dlg.m_sParm);
}

#include <afxpriv.h>

void CApp::WinHelp(DWORD dwData, UINT nCmd)
{
	if (nCmd == HELP_CONTEXT) {
		if (dwData >= HID_BASE_COMMAND &&
			dwData < (HID_BASE_COMMAND+0x10000))
			nCmd = HELP_CONTEXTPOPUP;
		else if (dwData >= HID_BASE_RESOURCE &&
				dwData < (HID_BASE_RESOURCE+0x10000)) {
			int nID = (dwData & 0xffff);
			if (nID == IDR_MAIN)
				nCmd = HELP_FINDER;
		}
	}
	CWinApp::WinHelp(dwData, nCmd);
}

// url handling

BOOL OpenURL(LPCTSTR psURL)
{
	TRACE("OpenURL=<%s>\n", psURL);
	LPCTSTR psCmd;
	LPCTSTR psParm;
	if (g_opt.sBrowser.IsEmpty()) {
		psCmd = psURL;
		psParm = NULL;
	} else {
		psCmd = g_opt.sBrowser;
		psParm = psURL;
	}
	HINSTANCE hInst = ShellExecute(::GetDesktopWindow(),
		NULL, psCmd, psParm, NULL, SW_SHOW);
	if ((DWORD)hInst <= 32) {
		TRACE("ShellExecute: %lu\n", (DWORD)hInst);
		CString s;
		s.Format(IDS_FMT_OPENURL_ERR, psCmd, psParm);
		AfxMessageBox(s, MB_OK|MB_ICONEXCLAMATION);
		return TRUE;
	}
	return FALSE;
}

#if 0
BOOL CApp::OnIdle(LONG lCount) 
{
	TRACE("-%ld.", lCount);

	return CWinApp::OnIdle(lCount);
}
#endif
