// mchat.h : main header file for the MChat application
//

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols

#include "sock.h"
#include "misc.h"
#include "profile.h"
#include "frame.h"

// CApp:
// See mchat.cpp for the implementation of this class
//

class CApp : public CWinApp
{
public:
	CApp();

	void UpdateVersion();

	CFrame* m_pFrame;		// the mainframe

	void WinHelp(DWORD dwData, UINT nCmd);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation
protected:

	void LoadFontSetting();
	void SaveFontSetting();
	void LoadServerInfo();
	void SaveServerInfo();
	void LoadOptions();

	void ConnectTo(LPCTSTR pStr);

	//{{AFX_MSG(CApp)
	afx_msg void OnAppAbout();
	afx_msg void OnSetup();
	afx_msg void OnConnect();
	afx_msg void OnUpdateConnect(CCmdUI* pCmdUI);
	afx_msg void OnDisconnect();
	afx_msg void OnUpdateDisconnect(CCmdUI* pCmdUI);
	afx_msg void OnUserset();
	afx_msg void OnOption();
	afx_msg void OnWindows();
	afx_msg void OnListchan();
	afx_msg void OnUpdateIsConnect(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

// channel/user mode command
class ModeCmd
{
public:
	ModeCmd(LPCTSTR sTarget)
	{
		m_nCmd = 0;
		m_sTarget = sTarget;
	}
	void Add(BOOL fPlus, TCHAR c, LPCTSTR pParm = 0);
	void Flush();

protected:
	CString m_sTarget;
	int m_nCmd;
	BOOL m_fPlus;
	CString m_sCmd;
	CString m_sParm;
};

// mchat.cpp
extern CApp theApp;

extern Con g_con; // connection status

extern BOOL g_fLoginName;		// login name is legal
extern BOOL g_fExitApp;			// now closing application
extern BOOL g_fVersionChange;

extern UINT g_nScrollLines;		// number of scroll lines

BOOL OpenURL(LPCTSTR psURL);

// irccmd.cpp

extern UINT g_tmidQuit;

// misc
extern CStringArray g_aKnownNicks;	// known nicks
BOOL IsKnownNick(const CString& sNick);
void ReConnect(LPCTSTR pQuitMsg = 0);
void ChangeServer(LPCTSTR pServer, UINT nPort, LPCTSTR pPassword);
void TimerEventQuit();
void TimerEventIsOn();
void SockNotice(int nCode, LPCTSTR pStr = 0);
// receiving
void ParseLine(const char* p);
// sending
void CmdUserReg();
void CmdNick(LPCTSTR sNick);
void CmdJoin(LPCTSTR pTarget, LPCTSTR psKey = 0);
void CmdTalk(LPCTSTR pTarget);
void CmdAway(LPCTSTR pMsg = 0);
void CmdMe(LPCTSTR pTarget, LPCTSTR pMsg);
void CmdInvite(LPCTSTR pNick, LPCTSTR pChan);
void CmdWhois(LPCTSTR pTarget);
void AutoJoin();
void CmdCtcpPing(LPCTSTR pTarget);
void CmdCtcpTime(LPCTSTR pTarget);
void CmdCtcpUserinfo(LPCTSTR pTarget);
void CmdCtcpFinger(LPCTSTR pTarget);
void CmdCtcpVersion(LPCTSTR pTarget);
void CmdCtcpClientinfo(LPCTSTR pTarget);
void CmdDccChat(CString& sNick);
void CmdDccSend(LPCTSTR pNick, LPCTSTR pFile = 0);
void CmdDccSend(LPCTSTR pNick, HDROP hDrop);
BOOL CmdQuit(LPCTSTR pQuitMsg = 0, BOOL fReconnect = FALSE);

// user messages
#define WMU_SOCK				(WM_APP + 100)	// for dcc
#define WMU_SOCK_DATA			(WMU_SOCK + 1)
#define WMU_SOCK_ADDR			(WMU_SOCK + 2)	// for async-gethostbyaddr
#define WMU_SOCK_HOST			(WMU_SOCK + 3)	// for async-gethostbyname
#define WMU_SOCK_MYHOST			(WMU_SOCK + 4)	// for async-gethostbyname
#define WMU_SOCK_DNSLOOKUP		(WMU_SOCK + 5)	// for async-gethostbyname

#define WMU_NICKLIST_RECREATE	(WM_APP + 110)	// re-create nicklist
#define WMU_DEFER_CHANMODE		(WM_APP + 111)	// do auto-op
#define WMU_NOTIFYICON			(WM_APP + 112)	// for tray icon
#define WMU_KEYWORD				(WM_APP + 113)	// keyword match
#define WMU_BKCOLOR_CHANGE		(WM_APP + 114)	// background color changed

// setup.cpp
extern BOOL PopupSetup();

// dlgign.cpp
void IgnoreInit();
void IgnorePopup(CWnd* pWnd = NULL, LPCTSTR pStr = 0);
BOOL IgnoreMatch(const CString sNick, const CString sAddr);

class CUser
{
public:
	CUser() {
		m_fOp = FALSE;
		m_nHopCount = 0;
	}
	CUser(LPCTSTR pNick, LPCTSTR pAddr) {
		m_sNick = pNick;
		m_sAddr = pAddr;
		m_fOp = FALSE;
		m_nHopCount = 0;
		m_sPrefix = m_sNick + '!' + pAddr;
	}
	CUser(LPCTSTR pNick, LPCTSTR pAccount, LPCTSTR pHost,
			LPCTSTR pServer, BOOL fOp, int nHopCount, LPCTSTR pRealName) {
		m_sNick = pNick;
		m_sAddr = CString(pAccount) + '@' + pHost;
		m_sServer = pServer;
		m_sRealName = pRealName;
		m_fOp = fOp;
		m_nHopCount = nHopCount;
		m_sPrefix = m_sNick + '!' + pAccount + '@' + pHost;
	}

	CString m_sNick;
	CString m_sAddr;
	CString m_sServer;
	CString m_sRealName;
	BOOL m_fOp;
	int m_nHopCount;

	CString m_sPrefix;

	CString Hostname() {
		int n = m_sAddr.Find('@', 0);
		if (n < 0)
			return m_sAddr;
		return (LPCTSTR)m_sAddr + n + 1;
	}
};

// NickList.cpp
void NickListInit();

// plug.cpp
class ArPlug : public CArray<Plug*, Plug*>
{
public:
	void OnPriv(const Con& con, const Packet& pac,
		const CString& sTarget, const CString& sMsg);
};

extern ArPlug g_arPlug;

void PlugInit();
