#include "stdafx.h"
#include "misc.h"
#include "resource.h"
#include "MCrypt.h"

static MCrypt g_mc("MuriChat");

// server definition

CString SvrDef::ToString(bool fCrypt) const
{
	CString s;
	s.Format("%s %d", m_sHostname, m_nPort);
	if (!m_sPassword.IsEmpty()) {
		s += " ?";
		if (fCrypt) {
			s += g_mc.Encode(m_sPassword);
		}
	}
	return s;
}

SvrDef& SvrDef::FromString(LPCTSTR psz)
{
	m_sHostname = Token(psz);
	CString s(Token(psz));
	m_nPort = atoi(s);
	m_sPassword = Token(psz);

	if (!m_sPassword.IsEmpty() && m_sPassword[0] == '?') {
		CString ss((LPCTSTR)m_sPassword + 1);
		m_sPassword = g_mc.Decode(ss);
	}
	return *this;
}

// server list, load/save

BOOL fListServersModified = FALSE;
ListServers listServers;
MapServers mapServers;

static CString sFileServers;

void LoadServers()
{
	sFileServers = GetDataPathname(IDS_FILE_SERVERS);
	CStdioFile file;
	if (!file.Open(sFileServers,
			CFile::modeRead | CFile::shareDenyWrite | CFile::typeText))
		return;

	g_mc.Reset();

	TRY {
		CString s;
		while (file.ReadString(s)) {
			SvrDef sd(s);
			if (sd.m_sHostname.IsEmpty() || sd.m_nPort <= 0)
				continue;
			listServers.AddTail(sd);
		}
	} CATCH(CFileException, e) {
	} END_CATCH
	MapServersMake();
}

void SaveServers()
{
	if (!fListServersModified)
		return;
	CStdioFile file;
	if (!file.Open(sFileServers,
			CFile::modeCreate | CFile::modeWrite | CFile::typeText))
		return;

	g_mc.Reset();

	POSITION pos = listServers.GetHeadPosition();
	CString s;
	while (pos) {
		SvrDef& rSD = listServers.GetNext(pos);
		s = rSD.ToString(true) + '\n';
		file.WriteString(s);
	}
}

void MapServersMake()
{
	mapServers.RemoveAll();
	POSITION pos = listServers.GetHeadPosition();
	while (pos) {
		POSITION posCur = pos;
		SvrDef& rSD = listServers.GetNext(pos);
		mapServers.SetAt(rSD.m_sHostname, posCur);
	}
}

// token

CString Token(LPCTSTR& pSrc)
{
	LPCTSTR p = pSrc;
	SkipNotSpace(p);
	CString s(pSrc, p - pSrc);
	SkipSpace(p);
	pSrc = p;
	return s;
}

LPCTSTR GetToken(LPCTSTR pSrc, CString sDest[], int nNumToken)
{
	LPCTSTR pFrom = pSrc;
	LPCTSTR pTo;
	int nLen;
	int i;
	for (i = 0; i < nNumToken; i++) {
		pTo = pFrom;
		SkipNotSpace(pTo);
		nLen = pTo - pFrom;
		LPTSTR pDest = sDest[i].GetBufferSetLength(nLen);
		if (nLen)
			memcpy(pDest, pFrom, nLen);
		sDest[i].ReleaseBuffer(nLen);
		pFrom = pTo;
		SkipSpace(pFrom);
	}
	return pFrom;
}

CString CutToken(CString& sSrc)
{
	LPCTSTR p;
	LPCTSTR pSrc;
	p = pSrc = sSrc;
	SkipNotSpace(p);
	CString s(pSrc, p - pSrc);
	SkipSpace(p);
	sSrc = p;
	return s;
}

void CutToken(CString& sSrc, CString& sDst)
{
	LPCTSTR p;
	LPCTSTR pSrc;
	p = pSrc = sSrc;
	SkipNotSpace(p);
	sDst = CString(pSrc, p - pSrc);
	SkipSpace(p);
	sSrc = p;
}

void ExtractPureText(CString& s, LPCTSTR psz)
{
	LPTSTR pStr = s.GetBuffer(lstrlen(psz));
	TCHAR c;
	while ((c = *psz++) != 0) {
		if (c == CHAR_BOLD || c == CHAR_UNDERLINE)
			continue;
		*pStr++ = c;
	}
	*pStr = 0;
	s.ReleaseBuffer();
}

BOOL IsNiceNick(LPCTSTR pNick)
{
	if (!isascii(*pNick))
		return FALSE;
	if (!isalpha(*pNick) && !strchr("[]\\`^{}_|", *pNick))
		return FALSE;
	while (*++pNick) {
		if (isascii(*pNick) && isalnum(*pNick))
			continue;
		if (!strchr("-[]\\`^{}_|", *pNick))
			return FALSE;
	}
	return TRUE;
}

// file access & manipulate

void GetDataDir(CString& sDir)
{
	char* p = sDir.GetBuffer(_MAX_PATH);
	GetModuleFileName(AfxGetApp()->m_hInstance, p, _MAX_PATH - 1);
	char* q = strrchr(p, '\\');
	if (q == 0)
		q = p;
	else
		q++;
	*q = 0;
	sDir.ReleaseBuffer();
}

CString GetDataPathname(LPCTSTR pFilename)
{
	CString sPath;
	GetDataDir(sPath);
	return sPath + pFilename;
}

CString GetDataPathname(int nIDStr)
{
	CString sFilename;
	sFilename.LoadString(nIDStr);
	return GetDataPathname(sFilename);
}

void DirExtend(CString& sPath, LPCTSTR pStr)
{
	int n = sPath.GetLength();
	if (n) {
		char c = sPath[n - 1];
		if (c != '\\' && c != '/' && c != ':')
			sPath += "\\";
	}
	sPath += pStr;
}

BOOL IsFileReadable(LPCTSTR pFilename)
{
	CFile file;
	if (!file.Open(pFilename, CFile::modeRead | CFile::shareDenyNone))
		return FALSE;
	file.Close();
	return TRUE;
}

CString ExtractDirName(LPCTSTR pPath)
{
	LPCTSTR pEnd = 0;
	LPCTSTR p = pPath;
	char c;
	while ((c = *p++) != 0) {
		if (c == ':') {
			if (*p == '/' || *p == '\\')
				p++;
			pEnd = p;
		} else if (c == '/' || c == '\\')
			pEnd = p - 1;
	}
	return pEnd ? CString(pPath, pEnd - pPath) : "";
}

LPCTSTR ExtractFileName(LPCTSTR pPath)
{
	LPCTSTR pFile = pPath;
	LPCTSTR p = pPath;
	char c;
	while ((c = *p++) != 0) {
		if (c == '/' || c == '\\' || c == ':')
			pFile = p;
	}
	return pFile;
}

void MoveIntoDesktop(int& x, int& y, int& cx, int& cy)
{
	CRect rc;
	::GetClientRect(::GetDesktopWindow(), &rc);

	if (cx > rc.Width() || cx < 10)
		cx = rc.Width();
	if (cy > rc.Height() || cy < 10)
		cy = rc.Height();

	if (x + cx > rc.right)
		x = rc.right - cx;
	if (y + cy > rc.bottom)
		y = rc.bottom - cy;

	if (x < 0)
		x = 0;
	if (y < 0)
		y = 0;
}

#if 0
LPCTSTR SkipUsernamePrefix(LPCTSTR p)
{
	TCHAR c;
	while ((c = *p) != 0) {
		if (c == '~' || c == '^' || c == '+'
				|| c == '=' || c == '-') {
			p++;
			continue;
		}
		break;
	}
	return p;
}
#endif

CString EscapeString(LPCTSTR p)
{
	CString s;
	while (*p) {
		unsigned char c = *(unsigned char*)p++;
		if (c == ' ') {
			s += '+';
		//} else if (_ismbblead(c) && _ismbbtrail(*p)) {
		//	s += c;
		//	s += *p++;
		} else if (!isascii(c) || !isalnum(c) && strchr("_.-", c) == 0) {
			char buf[10];
			sprintf(buf, "%%%02X", (unsigned int)c);
			s += buf;
		} else
			s += c;
	}
	return s;
}

bool HexToChar(unsigned char *p, unsigned char& cOut)
{
	if (!isxdigit(p[0]) || !isxdigit(p[1]))
		return false;
	cOut = (p[0] >= 'A') ? ((p[0] & 0xDF) - 'A' + 10) : (p[0] - '0');
	cOut <<= 4;
	cOut |= (p[1] >= 'A') ? ((p[1] & 0xDF) - 'A' + 10) : (p[1] - '0');
	return true;
}

CString UnescapeString(LPCTSTR p)
{
	CString s;
	unsigned char cOut;
	while (*p) {
		unsigned char c = *(unsigned char*)p++;
		if (c == '+')
			s += ' ';
		else if (c == '%' && HexToChar((unsigned char*)p, cOut)) {
			s += cOut;
			p += 2;
		} else
			s += c;
	}
	return s;
}

// debugging

HANDLE hndDebug = INVALID_HANDLE_VALUE;

void OpenDebugFile()
{
	CString sFile(GetDataPathname(IDS_FILE_DEBUG));
	hndDebug = CreateFile(sFile, GENERIC_WRITE,
		FILE_SHARE_READ, 0, CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL, 0);
}

void WriteDebugFile(char* pszMark, LPCTSTR pszStr, int nLen)
{
	if (hndDebug == INVALID_HANDLE_VALUE)
		return;
	DWORD dwWritten;
	WriteFile(hndDebug, pszMark, lstrlen(pszMark), &dwWritten, NULL);
	WriteFile(hndDebug, pszStr, nLen, &dwWritten, NULL);
	WriteFile(hndDebug, "\r\n", 2, &dwWritten, NULL);
}

void CloseDebugFile()
{
	CloseHandle(hndDebug);
	hndDebug = INVALID_HANDLE_VALUE;
}

void TempForeground(CWnd* pWnd)
{
	if (pWnd == NULL)
		return;
	pWnd->SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0,
		SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
	pWnd->SetWindowPos(&CWnd::wndNoTopMost, 0, 0, 0, 0,
		SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
}

#if 0 // no use
void StrRecSplit(LPCTSTR pStr, StrRec* pRec, LPCTSTR pDels)
{
	while (pRec->flag != SR_END) {
		LPCSTR p;
		for (p = pStr; *p && strchr(pDels, *p) == 0; p++)
			;
		CString s(pStr, p - pStr);
		if (pRec->flag & SR_INT)
			*(int*)pRec->obj = atoi(s);
		else if (pRec->flag & SR_CSTRING)
			*(CString*)pRec->obj = s;
		else if (pRec->flag & SR_BOOL)
			*(BOOL*)pRec->obj = atoi(s) ? 1 : 0;
		else
			ASSERT(FALSE);
		pStr = *p ? p + 1 : p;
		pRec++;
	}
}
#endif
