#ifndef _MISC_H_
#define _MISC_H_

// timer ID
#define TMID_ISON		1	// ison
#define TMID_FLASH		2	// flash window
#define TMID_DCC		3	// dcc
#define TMID_DCCEND		4	// dcc finish
#define TMID_QUIT		5	// quit and wait for socket close
#define TMID_DEFERMSG	6	// defer post msg
#define TMID_SOCK		7	// socket timer
#define TMID_DCCRCV		8	// dcc auto receive

class SvrDef {
public:
	SvrDef() {};
	SvrDef(LPCTSTR psz) {
		FromString(psz);
	};
	SvrDef(LPCTSTR pHostname, UINT nPort, LPCTSTR pPassword) {
		Pack(pHostname, nPort, pPassword);
	};

	CString m_sHostname;
	UINT m_nPort;
	CString m_sPassword;

	SvrDef& SvrDef::Pack(LPCTSTR pHostname,
			UINT nPort, LPCTSTR pPassword) {
		m_sHostname = pHostname;
		m_nPort = nPort;
		m_sPassword = pPassword;
		return *this;
	};
	void SvrDef::UnPack(CString& sHostname,
			UINT& nPort, CString& sPassword) {
		sHostname = m_sHostname;
		nPort = m_nPort;
		sPassword = m_sPassword;
	};

	CString ToString(bool fCrypt = false) const;
	SvrDef& FromString(LPCTSTR str);

	BOOL operator==(SvrDef& sd) {
		return m_sHostname == sd.m_sHostname &&
			m_nPort == sd.m_nPort &&
			m_sPassword == sd.m_sPassword;
	};
};

// server list

typedef CList<SvrDef, SvrDef&> ListServers;
typedef CMap<CString, LPCTSTR, POSITION, POSITION> MapServers;

extern ListServers listServers;
extern MapServers mapServers;
extern BOOL fListServersModified;

void LoadServers();
void SaveServers();
void MapServersMake();

inline POSITION MapServersLookup(LPCTSTR pHostname)
{
	POSITION pos;
	return mapServers.Lookup(pHostname, pos) ? pos : 0;
}

inline void AddServer(LPCTSTR pHostname, UINT nPort = 6667)
{
	if (*pHostname == '*')
		return;
	if (MapServersLookup(pHostname))
		return;
	SvrDef sd(pHostname, nPort, "");
	POSITION pos = listServers.AddTail(sd);
	if (!pos)
		return;
	mapServers.SetAt(pHostname, pos);
	fListServersModified = TRUE;
}

// misc

inline void SkipSpace(const char*& p)
{
	while (*p == ' ')
		p++;
}

inline void SkipNotSpace(const char*& p)
{
	while (*p && *p != ' ')
		p++;
}

inline BOOL HaveSpace(const char* p)
{
	BYTE c;
	while ((c = (*p++ & 0xff)) != 0)
		if (c <= ' ')
			return TRUE;
	return FALSE;
}

CString Token(LPCTSTR& pSrc);
LPCTSTR GetToken(LPCTSTR pSrc, CString sDest[], int nNumToken);
CString CutToken(CString& sSrc);
void CutToken(CString& sSrc, CString& sDst);

void GetDataDir(CString& sDir);
CString GetDataPathname(LPCTSTR pFilename);
CString GetDataPathname(int nIDStr);
void DirExtend(CString& sPath, LPCTSTR pStr);
BOOL IsFileReadable(LPCTSTR pFilename);
CString ExtractDirName(LPCTSTR pPath);
LPCTSTR ExtractFileName(LPCTSTR pPath);

//LPCTSTR SkipUsernamePrefix(LPCTSTR p);

BOOL IsNiceNick(LPCTSTR pNick);

// special chars
#define CTRL(c)			(c - '@')
#define CHAR_BOLD		CTRL('B')
#define CHAR_UNDERLINE	CTRL('_')
#define CHAR_REVERSE	CTRL('V')

// font types
#define F_BOLD			0x01
#define F_UNDERLINE		0x02
#define F_REVERSE		0x04
#define F_KEYWORD		0x80
#define F_TIME			0x40
#define F_NICK			0x20

void ExtractPureText(CString& s, LPCTSTR psz);

void TempForeground(CWnd* pWnd);

void MoveIntoDesktop(int& x, int& y, int& cx, int& cy);

CString EscapeString(LPCTSTR p);
CString UnescapeString(LPCTSTR p);

bool HexToChar(unsigned char *p, unsigned char& cOut);

// debug logg
void OpenDebugFile();
void WriteDebugFile(char* pszMark, LPCTSTR pszStr, int nLen);
void CloseDebugFile();

extern HANDLE hndDebug;

#define IsDebugLogging()	(hndDebug != INVALID_HANDLE_VALUE)

#if 0 // no use
typedef struct {
	int flag;
	void *obj;
} StrRec;

#define SR_END		0
#define SR_INT		0x01
#define SR_CSTRING	0x02
#define SR_BOOL		0x04

void StrRecSplit(LPCTSTR pStr, StrRec* pRec, LPCTSTR pDels);
#endif

#endif // _MISC_H_
