
#include "stdafx.h"
#include "mchat.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Plug::Plug(LPCTSTR pszFile) : m_sFile(pszFile)
{
	m_hModule = 0;
	Property = 0;
	OnPriv = 0;
};

Plug::~Plug()
{
	if (m_hModule) {
		TRACE("Free '%s'\n", m_sFile);
		FreeLibrary(m_hModule);
		m_hModule = NULL;
	}
}

BOOL Plug::Load()
{
	TRACE("Loding %s\n", m_sFile);
	m_hModule = LoadLibrary(m_sFile);
	if (m_hModule == NULL) {
		TRACE("Load error '%s'\n", m_sFile);
		return FALSE;
	}
	PLUG_INIT_PROT funcInit;
	funcInit = (PLUG_INIT_PROT)GetProcAddress(m_hModule, PLUG_INIT_NAME);
	if (!funcInit) {
		TRACE("plug '%s' no init\n", m_sFile);
		return FALSE;
	}
	BOOL f = (*funcInit)(this);
	TRACE("Plug file='%s' name='%s' init=%d\n", m_sFile, m_sName, f);
	if (!f)
		return FALSE;
	return TRUE;
}

void AFXAPI DestructElements(Plug** pPlug, int nCount)
{
	while (nCount-- > 0) {
		delete *pPlug++;
	}
}

ArPlug g_arPlug;

static BOOL Load(LPCTSTR szFile)
{
	Plug* p = new Plug(szFile);
	if (!p->Load()) {
		delete p;
		return FALSE;
	}
	g_arPlug.Add(p);
	return TRUE;
}

void PlugInit()
{
	CString sPath;
	TCHAR* pPath = sPath.GetBufferSetLength(_MAX_PATH);
	if (!GetModuleFileName(NULL, pPath, _MAX_PATH - 1)) {
		TRACE("GetModuleFileName error\n");
		return;
	}
	char* p = strrchr(pPath, '\\');
	if (p)
		*p = '\0';
	sPath.ReleaseBuffer();

	CString sFind(sPath + "\\plugins\\*.*");
	TRACE("Searching '%s'\n", sFind);

	CFileFind find;
	if (!find.FindFile(sFind)) {
		TRACE("FindFile err\n");
		return;
	}
	while (find.FindNextFile()) {
		if (find.IsDirectory())
			continue;
		CString sFile(find.GetFilePath());
		if (sFile.Right(4) == ".dll")
			Load(sFile);
	}
}

void ArPlug::OnPriv(const Con& con, const Packet& pac,
					   const CString& sTarget, const CString& sMsg)
{
	int i;
	for (i = 0; i < GetSize(); i++) {
		Plug* plug = GetAt(i);
		if (plug->OnPriv)
			plug->OnPriv(con, pac, sTarget, sMsg);
	}
}
