#include "misc.h"

// Quit message
CString LoadQuitMsg();
void SaveQuitMsg(LPCTSTR pMsg);

// nicklist

#define NL_NONE		-1
#define NL_NICK		0
#define NL_ADDR		1
#define NL_NAME		2
#define NL_SERV		3
#define NL_END		4

LPCTSTR GetNLString(int nNL, int* pnWidth = 0);

typedef CArray<int, int> ArNLItem;

// beep

struct BeepInit {
	LPCTSTR szLabel;
	LPCTSTR szRegBeep;
	LPCTSTR szRegWave;
	BOOL fBeepDefault;
	UINT nBeep;
};

class TagBeep {
public:
	TagBeep()
	{
		m_fBeep = FALSE;
		m_fWave = FALSE;
	}
	TagBeep(BeepInit& init);

	void Save();

	CString m_sLabel;	// label string for setup-dialog
	CString m_sRegBeep;	// beep reg key, bool
	CString m_sRegWave; // wave file reg key, string

	BOOL m_fBeep;		// flag for enable/disable
	UINT m_nBeep;		// beep style MB_...
	CString m_sWave;	// wave file path
	BOOL m_fWave;		// work, true for m_sWave is not empty
};

#define BEEP_Connect		0
#define BEEP_Disconnect		1
#define BEEP_Kicked			2
#define BEEP_Invited		3
#define BEEP_PrivMsg		4
#define BEEP_PrivMsgNoTalk	5
#define BEEP_PrivMsgIcon	6
#define BEEP_ChanMsg		7
#define BEEP_ChanMsgIcon	8
#define BEEP_DCC			9
#define BEEP_FoundNick		10
#define BEEP_Join			11
#define BEEP_Part			12
#define BEEP_Quit			13
#define BEEP_Keyword		14
#define BEEP_NickChange		15

class OptBeep : public CArray<TagBeep, TagBeep&>
{
public:
	void Load();
	void Save();
	BOOL Play(int nPlay);
};

extern OptBeep optBeep;

// user mode

class UserMode
{
public:
	UserMode() { Clear(); };

	void Clear();
	CString ToString();
	void FromString(LPCTSTR p);
	void Apply(LPCTSTR sTarget);

	BOOL m_fInvisible;	//i - marks a users as invisible;
	BOOL m_fWallops;	//w - user receives wallops;
	//s - marks a user for receipt of server notices;
	//o - operator flag.
};

// other options

struct OptionTag {
	// user
	CString sNick;
	CString sNick2;
	CString sNickAway;
	CString sRealName;
	CString sUserInfo;
	CString sLoginName;
	CString sHostName;
	UserMode usermode;

	// logging
	BOOL fLogMain;
	BOOL fLogChan;
	BOOL fLogTalk;

	CString sFileMain;
	CString sDirChan;
	CString sDirTalk;
	CString sLogExt;

	BOOL fAppend;
	BOOL fLogDate;
	CString sLogDateFormat;

	BOOL fLogNewDate;

	// title
	CString sMain;
	CString sChan;
	CString sTalk;
	CString sChat;

	// format
	BOOL fMsgJoin;
	BOOL fMsgKick;
	BOOL fMsgMode;
	BOOL fMsgNick;
#define DEF_MSGNICK		"[%s] "
	BOOL fMsgPart;
	BOOL fMsgQuit;
	BOOL fMsgTopic;
	BOOL fMsgQuery;
	BOOL fMsgReplyToMain;	// ctcp reply to only main frame

	BOOL fMsgTime;
	CString sMsgTime;	// timestamp format
	CString sMsgNick;	// nick print format

	// events
	BOOL fPrivMsgPopup;
	BOOL fPrivMsgWhois;

	UINT nInvite;
#define INVITE_PROMPT	0
#define INVITE_JOIN		1
#define INVITE_IGNORE	2

	UINT nDccChat;
#define DCC_CHAT_POPUP	0
#define DCC_CHAT_PROMPT	1
#define DCC_CHAT_IGNORE	2

	BOOL fReconnect;
	BOOL fRejoin;

	BOOL fFlashInactive;
	BOOL fFlashMinimized;
	BOOL fRestoreMinimized;
	//BOOL fContinuousFlash;

	// ison
	BOOL fIsOnQuit;		// check nicks also leaving from IRC
	BOOL fIsOnPopup;	// report by popup
	UINT nIsOnInterval;	// check interval
#define ISON_MIN_INTERVAL	10
#define ISON_DEF_INTERVAL	30

	// NickList
	ArNLItem arNLItem;
	UINT nNLPos;
#define NLPOS_TOP		0
#define NLPOS_LEFT		1
#define NLPOS_RIGHT		2
#define NLPOS_BOTTOM	3

	// Keyword
	BOOL fKeywordWholeLine;
	BOOL fRestoreByKeyword;

	// dcc
	UINT nDccSendPackets;
	UINT nChatEOL;
#define CHATEOL_CRLF	0
#define CHATEOL_LF		1
	UINT nChatCode;
#define CHATCODE_JIS	0
#define CHATCODE_SJIS	1
	BOOL fUseDccFakeAddr;
	BOOL fUseDccFakeAuto;
	u_long addrDccFakeAddr;
	UINT nDccPort;

	// others
	BOOL fMinimizeByReturnKey;
	BOOL fDestNick;
	BOOL fIdentd;
	BOOL fConnectOnStartup;
	BOOL fServerCollection;
	BOOL fMinimizeMain;
	BOOL fHideMain;
	BOOL fAutoSimple;
	BOOL fWhoisDetail;
	BOOL fConfirmPaste;
	BOOL fImeOn;
	CString sBrowser;
	TCHAR cCtrlChar;

	BOOL fDebugLog;
};

extern OptionTag g_opt;	// options

void OptSaveUser();
void OptSaveLog();
void OptSaveTitle();
void OptSaveFormat();
void OptSaveEvents();
void OptSaveIsOn();
void OptSaveNickList();
void OptSaveKeyword();
void OptSaveDcc();
void OptSaveOthers();

extern SvrDef g_sd; // server definition

// fonts

struct FontColor {
	CFont m_font;
	COLORREF m_color;
	CString m_sName;	// label for setup and reg name
	int m_idc;			// control id for setup:fonts
};

class Fonts {
public:
	enum FType {
		TypeText = 0,
		TypeNotice,
		TypeStat,
		TypeError,
		TypeNickList,
		TypeKeyword,
		TypeTime,
		TypeNick,
		NumberOfTypes,
	};

	void LoadAll();
	void SaveAll();
	void ChangeTextFont();
	void Select(CDC& dc, FType type, CFont* pFont = NULL);

	FontColor& GetFontColor(FType type)
	{
		if (type < 0 || type >= NumberOfTypes) {
			ASSERT(FALSE);
			type = TypeText;
		}
		return m_arFC[type];
	}

	FontColor m_arFC[NumberOfTypes];

	CFont m_fontBold;
	CFont m_fontUL;
	CFont m_fontBoldUL;
	CFont m_fontInput;

	COLORREF m_crBack;	// background color

protected:
	CMap<FType, FType, FontColor*, FontColor*> m_map;

	friend class FSave;
};

class FSave {
public:
	FSave()	{
		m_fSaved = FALSE;
	}
	FSave(CDC& dc, Fonts::FType type) {
		Save(dc, type);
	}
	~FSave() {
		Restore();
	}

	void Save(CDC& dc, Fonts::FType type);
	void Restore();

protected:	
	BOOL m_fSaved;
	CDC* m_pDC;
	CFont* m_pFont;
	COLORREF m_color;
};

extern Fonts g_fonts;

class ArKeyword : public CArray<CString, CString&>
{
public:
	void Load();
	void Save();
	int Search(LPCTSTR pStr, int nCol, int& nStart);
};

extern ArKeyword g_arKeyword;
