// s_server.cpp : implementation file

#include "stdafx.h"
#include "mchat.h"
#include "setup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// SetupServer property page

IMPLEMENT_DYNCREATE(SetupServer, CPropertyPage)

SetupServer::SetupServer() : CPropertyPage(SetupServer::IDD)
{
	//{{AFX_DATA_INIT(SetupServer)
	m_sHostname = _T("");
	m_sPassword = _T("");
	m_nPort = 0;
	//}}AFX_DATA_INIT
	m_fModify = FALSE;
	g_sd.UnPack(m_sHostname, m_nPort, m_sPassword);
}

SetupServer::~SetupServer()
{
}

void SetupServer::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupServer)
	DDX_Control(pDX, IDC_SL_DELETE, m_btDelete);
	DDX_Control(pDX, IDC_SL_CONNECT, m_btConnect);
	DDX_Control(pDX, IDC_SL_APPEND, m_btAppend);
	DDX_Control(pDX, IDC_MOVE_UP, m_btUp);
	DDX_Control(pDX, IDC_MOVE_TOP, m_btTop);
	DDX_Control(pDX, IDC_MOVE_DOWN, m_btDown);
	DDX_Control(pDX, IDC_MOVE_BOTTOM, m_btBottom);
	DDX_Control(pDX, IDC_SVRLIST, m_lbSvr);
	DDX_Text(pDX, IDC_HOSTNAME, m_sHostname);
	DDX_Text(pDX, IDC_PASSWORD, m_sPassword);
	DDX_Text(pDX, IDC_PORT, m_nPort);
	DDV_MinMaxUInt(pDX, m_nPort, 1, 65535);
	//}}AFX_DATA_MAP
}

void SetupServer::UpDown()
{
	int n = m_lbSvr.GetCurSel();
	m_btTop.EnableWindow(n > 0);
	m_btUp.EnableWindow(n > 0);

	BOOL f = (n >= 0 && n < m_lbSvr.GetCount() - 1);
	m_btDown.EnableWindow(f);
	m_btBottom.EnableWindow(f);

	m_btDelete.EnableWindow(n >= 0);
	m_btConnect.EnableWindow(n >= 0);
}

void SetupServer::Goto(int id)
{
	GotoDlgCtrl(GetDlgItem(id));
}

int SetupServer::LookupSvr(SvrDef& sd)
{
	int i;
	for (i = 0; i < m_lbSvr.GetCount(); i++) {
		POSITION pos = (POSITION)m_lbSvr.GetItemDataPtr(i);
		if (!pos || pos == (POSITION)LB_ERR)
			continue;
		SvrDef& rSD = m_list.GetAt(pos);
		if (rSD == sd)
			return i;
	}
	return -1;
}

BOOL SetupServer::Check()
{
	if (m_sHostname.IsEmpty()) {
		AfxMessageBox(IDS_MSG_HOSTNAME,
			MB_OK|MB_APPLMODAL|MB_ICONINFORMATION);
		Goto(IDC_HOSTNAME);
		return FALSE;
	}
	return TRUE;
}


BEGIN_MESSAGE_MAP(SetupServer, CPropertyPage)
	//{{AFX_MSG_MAP(SetupServer)
	ON_LBN_SELCHANGE(IDC_SVRLIST, OnSelchangeSvrlist)
	ON_BN_CLICKED(IDC_SL_APPEND, OnSlAppend)
	ON_BN_CLICKED(IDC_SL_DELETE, OnSlDelete)
	ON_BN_CLICKED(IDC_SL_CONNECT, OnSlConnect)
	ON_BN_CLICKED(IDC_MOVE_TOP, OnMoveTop)
	ON_BN_CLICKED(IDC_MOVE_UP, OnMoveUp)
	ON_BN_CLICKED(IDC_MOVE_DOWN, OnMoveDown)
	ON_BN_CLICKED(IDC_MOVE_BOTTOM, OnMoveBottom)
	ON_LBN_DBLCLK(IDC_SVRLIST, OnDblclkSvrlist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// SetupServer message handlers

BOOL SetupServer::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	m_lbSvr.SetRedraw(FALSE);
	m_list.AddTail(&listServers);
	POSITION pos = m_list.GetHeadPosition();
	CString str;
	while (pos) {
		POSITION posCur = pos;
		SvrDef& rSD = m_list.GetNext(pos);
		str = rSD.ToString();
		int n = m_lbSvr.AddString(str);
		if (n < 0)
			continue;
		if (m_lbSvr.SetItemDataPtr(n, posCur) == LB_ERR)
			ASSERT(FALSE);
	}
	int i = LookupSvr(g_sd);
	if (i >= 0)
		m_lbSvr.SetCurSel(i);
	UpDown();
	m_lbSvr.SetRedraw(TRUE);

	return TRUE;
}

void SetupServer::OnSelchangeSvrlist() 
{
	int n = m_lbSvr.GetCurSel();
	if (n >= 0) {
		POSITION pos = (POSITION)m_lbSvr.GetItemDataPtr(n);
		if (!pos || pos == (POSITION)LB_ERR)
			return;
		SvrDef& rSD = m_list.GetAt(pos);
		rSD.UnPack(m_sHostname, m_nPort, m_sPassword);
		UpdateData(FALSE);
		((CSetup*)GetParent())->UpdateTitle();
	}
	UpDown();
}

void SetupServer::OnOK() 
{
	int i = m_lbSvr.GetCurSel();
	if (i >= 0) {
		POSITION pos = (POSITION)m_lbSvr.GetItemDataPtr(i);
		if (pos && pos != (POSITION)LB_ERR)
			g_sd = m_list.GetAt(pos);
		((CSetup*)GetParent())->UpdateTitle();
	}
	if (m_fModify) {
		listServers.RemoveAll();
		int i;
		for (i = 0; i < m_lbSvr.GetCount(); i++) {
			POSITION pos = (POSITION)m_lbSvr.GetItemDataPtr(i);
			if (!pos || pos == (POSITION)LB_ERR)
				continue;
			SvrDef& rSD = m_list.GetAt(pos);
			listServers.AddTail(rSD);
		}
		MapServersMake();
		fListServersModified = TRUE;
	}
}

BOOL SetupServer::OnKillActive() 
{
	if (!CPropertyPage::OnKillActive())
		return FALSE;
	return Check();
}

void SetupServer::OnSlAppend() 
{
	if (!UpdateData(TRUE))
		return;
	if (!Check())
		return;

	SvrDef sd(m_sHostname, m_nPort, m_sPassword);
	int i = LookupSvr(sd);
	if (i == -1) {
		POSITION pos = m_list.AddHead(sd);
		SvrDef& rSD = m_list.GetAt(pos);
		CString str(sd.ToString());
		i = m_lbSvr.InsertString(0, str);
		if (i < 0)
			return;
		m_lbSvr.SetItemDataPtr(i, pos);
		m_fModify = TRUE;
	}
	m_lbSvr.SetCurSel(i);
	UpDown();
}

void SetupServer::OnSlDelete() 
{
	int n = m_lbSvr.GetCurSel();
	if (n < 0)
		return;
	m_lbSvr.DeleteString(n);
	if (n >= m_lbSvr.GetCount())
		n--;
	if (n >= 0)
		m_lbSvr.SetCurSel(n);
	Goto(IDC_SVRLIST);
	UpDown();
	m_fModify = TRUE;
}

void SetupServer::OnSlConnect() 
{

	CPropertySheet* pSheet = (CPropertySheet*)GetParent();
	if (!pSheet)
		return;
	pSheet->PostMessage(WM_COMMAND, IDOK);
	ReConnect("server change");
}

void SetupServer::OnDblclkSvrlist() 
{
	OnSlConnect();
}

void SetupServer::Move(int nFrom, int nTo)
{
	CString s;
	m_lbSvr.GetText(nFrom, s);
	POSITION pos = (POSITION)m_lbSvr.GetItemDataPtr(nFrom);
	m_lbSvr.DeleteString(nFrom);
	int n = m_lbSvr.InsertString(nTo, s);
	if (n < 0) {
		ASSERT(FALSE);
		return;
	}
	m_lbSvr.SetItemDataPtr(n, pos);
	m_lbSvr.SetCurSel(nTo != -1 ? n : nFrom);
	UpDown();
	m_fModify = TRUE;
	Goto(IDC_SVRLIST);
}

void SetupServer::OnMoveTop() 
{
	int n = m_lbSvr.GetCurSel();
	if (n < 1) // 0 or ERR
		return;
	Move(n, 0);
}

void SetupServer::OnMoveUp() 
{
	int n = m_lbSvr.GetCurSel();
	if (n < 1) // 0 or ERR
		return;
	Move(n, n - 1);
}

void SetupServer::OnMoveDown() 
{
	int n = m_lbSvr.GetCurSel();
	if (n < 0 || n >= m_lbSvr.GetCount() - 1)
		return;
	Move(n, n + 1);
}

void SetupServer::OnMoveBottom() 
{
	int n = m_lbSvr.GetCurSel();
	if (n < 0 || n >= m_lbSvr.GetCount() - 1)
		return;
	Move(n, -1);
}
