// Setup.cpp : implementation file
//

#include "stdafx.h"
#include "mchat.h"
#include "dlgmisc.h"
#include "ident.h"
#include "Setup.h"
#include "conv.h"

#include <afxpriv.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// --------------------------------------------------------------------
// Sheet: Setup

static CSetup* g_pSetup = 0;

BOOL PopupSetup()
{
	if (g_pSetup) {
		if (IsWindow(g_pSetup->m_hWnd))
			g_pSetup->SetForegroundWindow();
		return FALSE;
	}
	CSetup dlg;
	dlg.DoModal();
	return TRUE;
}

UINT iSetupSelectPage = 0;

IMPLEMENT_DYNAMIC(CSetup, CPropertySheet)

CSetup::CSetup() : CPropertySheet(IDS_SETUP, 0, iSetupSelectPage)
{
	g_pSetup = this;
	m_fUpdateTitle = FALSE;

	m_psh.dwFlags |= PSH_NOAPPLYNOW;

	AddPage(&m_server);
	AddPage(&m_user);
	AddPage(&m_log);
	AddPage(&m_title);
	AddPage(&m_format);
	AddPage(&m_event);
	AddPage(&m_chan);
	AddPage(&m_nicks);
	AddPage(&m_beep);
	AddPage(&m_fonts);
	AddPage(&m_nicklist);
	AddPage(&m_keyword);
	AddPage(&m_dcc);
	AddPage(&m_plug);
	AddPage(&m_others);
}

CSetup::~CSetup()
{
	g_pSetup = 0;
}

BEGIN_MESSAGE_MAP(CSetup, CPropertySheet)
	//{{AFX_MSG_MAP(CSetup)
	ON_WM_DESTROY()
	ON_WM_CONTEXTMENU()
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CSetup message handlers

BOOL CSetup::OnInitDialog() 
{
	BOOL r = CPropertySheet::OnInitDialog();
	ModifyStyleEx(0, WS_EX_CONTEXTHELP);
	CButton* pBtn = (CButton*)GetDlgItem(ID_APPLY_NOW);
	if (pBtn) {
		pBtn->EnableWindow(FALSE);
		pBtn->SetWindowText("");
	}
	CenterWindow(CWnd::GetDesktopWindow());
	return r;
}

void CSetup::OnDestroy() 
{
	iSetupSelectPage = GetActiveIndex();
	if (m_fUpdateTitle)
		g_frames.MakeAllTitle();

	CPropertySheet::OnDestroy();
}

BOOL CSetup::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if (pHelpInfo->iContextType == HELPINFO_WINDOW) {
		RECT rc;
		if (::GetWindowRect((HWND)pHelpInfo->hItemHandle, &rc))
			::SetCursorPos(rc.left, rc.top);
		AfxGetApp()->WinHelp(
			HID_BASE_COMMAND + pHelpInfo->iCtrlId,
			HELP_CONTEXTPOPUP);
	}
	return TRUE;
}

void CSetup::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (this == pWnd)
		return;
	CWnd* pControl = pWnd->WindowFromPoint(point);
	if (!pControl || pControl == pWnd) {
		CPoint pt(point);
		pWnd->ScreenToClient(&pt);
		pControl = pWnd->ChildWindowFromPoint(pt);
		if (pControl && (pControl->GetStyle() & BS_GROUPBOX)) {
			while ((pControl = pControl->GetNextWindow()) != NULL) {
				CRect rc;
				pControl->GetWindowRect(&rc);
				if (rc.PtInRect(point))
					break;
			}
		}
	}
	if (pControl) {
		int nID = pControl->GetDlgCtrlID();
		if (nID)
			::WinHelp(m_hWnd,
				AfxGetApp()->m_pszHelpFilePath,
				HELP_CONTEXTPOPUP, HID_BASE_COMMAND + nID);
	}
}

// Page: User

IMPLEMENT_DYNCREATE(SetupUser, CPropertyPage)

SetupUser::SetupUser() : CPropertyPage(SetupUser::IDD)
{
	//{{AFX_DATA_INIT(SetupUser)
	m_sNick = g_opt.sNick;
	m_sNick2 = g_opt.sNick2;
	m_sNickAway = g_opt.sNickAway;
	m_sRealName = g_opt.sRealName;
	m_sUserInfo = g_opt.sUserInfo;
	m_sLoginName = g_opt.sLoginName;
	m_sHostName = g_opt.sHostName;
	//}}AFX_DATA_INIT
}

SetupUser::~SetupUser()
{
}

void SetupUser::DecideNick2()
{
	CString sNick;
	GetDlgItemText(IDC_NICK, sNick);
	CString s;
	GetDlgItemText(IDC_NICK2, s);
	if (s.IsEmpty()) {
		if (!sNick.IsEmpty()) {
			SetDlgItemText(IDC_NICK2, sNick + _T("%d"));
		}
	}
	GetDlgItemText(IDC_NICKAWAY, s);
	if (s.IsEmpty()) {
		if (!sNick.IsEmpty()) {
			CString sAway(_T("Away"));
			CString sNickAway(sNick + sAway);
			if (sNickAway.GetLength() > 9)
				sNickAway = sNick.Left(9 - sAway.GetLength()) + sAway;
			SetDlgItemText(IDC_NICKAWAY, sNickAway);
		}
	}
}

void SetupUser::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupUser)
	DDX_Text(pDX, IDC_NICK, m_sNick);
	DDV_MaxChars(pDX, m_sNick, 9);
	DDX_Text(pDX, IDC_REALNAME, m_sRealName);
	DDX_Text(pDX, IDC_USERINFO, m_sUserInfo);
	DDX_Text(pDX, IDC_LOGINNAME, m_sLoginName);
	DDX_Text(pDX, IDC_USER_HOSTNAME, m_sHostName);
	DDX_Text(pDX, IDC_NICK2, m_sNick2);
	DDX_Text(pDX, IDC_NICKAWAY, m_sNickAway);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupUser, CDialog)
	//{{AFX_MSG_MAP(SetupUser)
	ON_EN_KILLFOCUS(IDC_NICK, OnKillfocusNick)
	ON_BN_CLICKED(IDC_USERMODE, OnUsermode)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void SetupUser::OnOK() 
{
	if (m_sNick != g_opt.sNick) {
		CmdNick(g_opt.sNick = m_sNick);
		((CSetup*)GetParent())->UpdateTitle();
	}

	DecideNick2();
	g_opt.sNick2 = m_sNick2;
	g_opt.sNickAway = m_sNickAway;
	g_opt.sRealName = m_sRealName;
	g_opt.sUserInfo = m_sUserInfo;
	g_opt.sLoginName = m_sLoginName;
	if (!m_sHostName.IsEmpty())
		g_opt.sHostName = m_sHostName;
	OptSaveUser();
}

BOOL SetupUser::OnInitDialog() 
{
	CDialog::OnInitDialog();
	GotoDlgCtrl(GetDlgItem(IDC_NICK));
	return FALSE;
}

void SetupUser::OnKillfocusNick() 
{
	DecideNick2();
}

BOOL SetupUser::NoOmit(UINT idc)
{
	CString s;
	GetDlgItemText(idc, s);
	if (s.IsEmpty()) {
		AfxMessageBox(IDS_MSG_NO_OMIT,
			MB_OK | MB_APPLMODAL | MB_ICONINFORMATION);
		GotoDlgCtrl(GetDlgItem(idc));
		return FALSE;
	}
	return TRUE;
}

BOOL SetupUser::NoSpace(UINT idc)
{
	CString s;
	GetDlgItemText(idc, s);
	if (strchr(s, ' ') != 0) {
		AfxMessageBox(IDS_MSG_NO_SPACE,
			MB_OK | MB_APPLMODAL | MB_ICONINFORMATION);
		GotoDlgCtrl(GetDlgItem(idc));
		return FALSE;
	}
	return TRUE;
}

BOOL SetupUser::No8Bit(UINT idc)
{
	CString s;
	GetDlgItemText(idc, s);
	LPCTSTR p;
	for (p = s; *p; p++) {
		if ((*p & 0x80) != 0) {
			AfxMessageBox(IDS_MSG_NO8BIT,
				MB_OK | MB_APPLMODAL | MB_ICONINFORMATION);
			GotoDlgCtrl(GetDlgItem(idc));
			return FALSE;
		}
	}
	return TRUE;
}

BOOL SetupUser::OnKillActive() 
{
	if (!CPropertyPage::OnKillActive())
		return FALSE;

	if (!NoOmit(IDC_NICK) || !NoSpace(IDC_NICK) || !No8Bit(IDC_NICK))
		return FALSE;

	CString s;
	GetDlgItemText(IDC_NICK, s);
	if (!IsNiceNick(s)) {
		AfxMessageBox(IDS_ERR_NICK,
			MB_OK | MB_APPLMODAL | MB_ICONINFORMATION);
		GotoDlgCtrl(GetDlgItem(IDC_NICK));
		return FALSE;
	}

	return NoOmit(IDC_REALNAME) && NoOmit(IDC_LOGINNAME)
		&& NoSpace(IDC_LOGINNAME) && No8Bit(IDC_LOGINNAME);
}

void SetupUser::OnUsermode() 
{
	DlgUserMode dlg(this);
	if (dlg.DoModal() == IDOK)
		g_opt.usermode.Apply(g_con.m_sNick);
}

// DlgUserMode _CAO

DlgUserMode::DlgUserMode(CWnd* pParent /*=NULL*/)
	: CDialog(DlgUserMode::IDD, pParent)
{
	//{{AFX_DATA_INIT(DlgUserMode)
	m_fInvisible = g_opt.usermode.m_fInvisible;
	m_fWallops = g_opt.usermode.m_fWallops;
	//}}AFX_DATA_INIT
}


void DlgUserMode::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DlgUserMode)
	DDX_Check(pDX, IDC_UMODE_INVISIBLE, m_fInvisible);
	DDX_Check(pDX, IDC_UMODE_WALLOPS, m_fWallops);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(DlgUserMode, CDialog)
	//{{AFX_MSG_MAP(DlgUserMode)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// DlgUserMode bZ[W nh

void DlgUserMode::OnOK() 
{
	if (!UpdateData(TRUE))
		return;

	g_opt.usermode.m_fInvisible = m_fInvisible;
	g_opt.usermode.m_fWallops = m_fWallops;

	CDialog::OnOK();
}

// --------------------------------------------------------------------
// Page: Log

IMPLEMENT_DYNCREATE(SetupLog, CPropertyPage)

SetupLog::SetupLog() : CPropertyPage(SetupLog::IDD)
{
	//{{AFX_DATA_INIT(SetupLog)
	m_fLogMain = g_opt.fLogMain;
	m_fLogTalk = g_opt.fLogTalk;
	m_sFileMain = g_opt.sFileMain;
	m_sDirTalk = g_opt.sDirTalk;
	m_fLogChan = g_opt.fLogChan;
	m_sDirChan = g_opt.sDirChan;
	m_fAppend = g_opt.fAppend;
	m_sLogExt = g_opt.sLogExt;
	m_fLogDate = g_opt.fLogDate;
	m_sDateFormat = g_opt.sLogDateFormat;
	m_fLogNewDate = g_opt.fLogNewDate;
	m_fDebugLog = g_opt.fDebugLog;
	//}}AFX_DATA_INIT
}

SetupLog::~SetupLog()
{
}

BOOL SetupLog::Check(int nID, LPCTSTR pPath, BOOL fDir)
{
	CString sDir = fDir ? pPath : ExtractDirName(pPath);
	CFileStatus stat;
	if (sDir.IsEmpty()) {
		AfxMessageBox(IDP_SPECIFY_FOLDER, MB_OK|MB_APPLMODAL|MB_ICONINFORMATION);
		goto err;
	}
	if (!CFile::GetStatus(sDir, stat)) {
		CString s;
		s.Format(IDS_FMT_NO_FOLDER, sDir);
		if (AfxMessageBox(s, MB_YESNO|MB_APPLMODAL|MB_ICONQUESTION) != IDYES)
			goto err;
		SECURITY_ATTRIBUTES sa = { sizeof SECURITY_ATTRIBUTES };
		if (!CreateDirectory(sDir, &sa)) {
			CString s;
			s.Format(IDS_CANNOT_CREATE_FOLDER, sDir);
			AfxMessageBox(s, MB_OK|MB_APPLMODAL|MB_ICONEXCLAMATION);
			goto err;
		}
	}
	return TRUE;
err:
	GotoDlgCtrl(GetDlgItem(nID));
	return FALSE;
}

void SetupLog::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupLog)
	DDX_Check(pDX, IDC_LOG_MAIN, m_fLogMain);
	DDX_Check(pDX, IDC_LOG_TALK, m_fLogTalk);
	DDX_Text(pDX, IDC_LOGDIR_MAIN, m_sFileMain);
	DDX_Text(pDX, IDC_LOGDIR_TALK, m_sDirTalk);
	DDX_Check(pDX, IDC_LOG_CHAN, m_fLogChan);
	DDX_Text(pDX, IDC_LOGDIR_CHAN, m_sDirChan);
	DDX_Check(pDX, IDC_LOG_APPEND, m_fAppend);
	DDX_Text(pDX, IDC_LOG_EXT, m_sLogExt);
	DDX_Check(pDX, IDC_LOG_DATE, m_fLogDate);
	DDX_Text(pDX, IDC_LOG_DATE_FORMAT, m_sDateFormat);
	DDX_Check(pDX, IDC_LOG_NEW_DATE, m_fLogNewDate);
	DDX_Check(pDX, IDC_DEBUGLOG, m_fDebugLog);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupLog, CPropertyPage)
	//{{AFX_MSG_MAP(SetupLog)
	ON_BN_CLICKED(IDC_LOG_MAIN, OnLogMain)
	ON_BN_CLICKED(IDC_LOG_CHAN, OnLogChan)
	ON_BN_CLICKED(IDC_LOG_TALK, OnLogTalk)
	ON_BN_CLICKED(IDC_LOG_DATE, OnLogDate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL SetupLog::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	GetDlgItem(IDC_LOGDIR_MAIN)->EnableWindow(m_fLogMain);
	GetDlgItem(IDC_LOGDIR_CHAN)->EnableWindow(m_fLogChan);
	GetDlgItem(IDC_LOGDIR_TALK)->EnableWindow(m_fLogTalk);
	GetDlgItem(IDC_LOG_DATE_FORMAT)->EnableWindow(m_fLogDate);

	return TRUE;
}

void SetupLog::OnOK()
{
	if (!UpdateData(TRUE))
		return;

	g_opt.fLogMain = m_fLogMain;
	g_opt.fLogChan = m_fLogChan;
	g_opt.fLogTalk = m_fLogTalk;
	g_opt.sFileMain = m_sFileMain;
	g_opt.sDirChan = m_sDirChan;
	g_opt.sDirTalk = m_sDirTalk;
	g_opt.fAppend = m_fAppend;
	g_opt.sLogExt = m_sLogExt;
	g_opt.fLogDate = m_fLogDate;
	g_opt.sLogDateFormat = m_sDateFormat;
	g_opt.fLogNewDate = m_fLogNewDate;
	g_opt.fDebugLog = m_fDebugLog;

	OptSaveLog();
}

void SetupLog::DoCheckBox(int idCheck, int idEdit)
{
	BOOL f = ((CButton*)GetDlgItem(idCheck))->GetCheck();
	CWnd* pWnd = GetDlgItem(idEdit);
	pWnd->EnableWindow(f);
	if (f)
		GotoDlgCtrl(pWnd);
}

void SetupLog::OnLogMain() 
{
	DoCheckBox(IDC_LOG_MAIN, IDC_LOGDIR_MAIN);
}

void SetupLog::OnLogChan() 
{
	DoCheckBox(IDC_LOG_CHAN, IDC_LOGDIR_CHAN);
}

void SetupLog::OnLogTalk() 
{
	DoCheckBox(IDC_LOG_TALK, IDC_LOGDIR_TALK);
}

void SetupLog::OnLogDate() 
{
	DoCheckBox(IDC_LOG_DATE, IDC_LOG_DATE_FORMAT);
}

BOOL SetupLog::OnKillActive() 
{
	if (!UpdateData(TRUE))
		return FALSE;

	if (m_fLogMain && !Check(IDC_LOGDIR_MAIN, m_sFileMain, FALSE) ||
			m_fLogChan && !Check(IDC_LOGDIR_CHAN, m_sDirChan, TRUE) ||
			m_fLogTalk && !Check(IDC_LOGDIR_TALK, m_sDirTalk, TRUE))
		return FALSE;

	return CPropertyPage::OnKillActive();
}

// --------------------------------------------------------------------
// Page: Title

IMPLEMENT_DYNCREATE(SetupTitle, CPropertyPage)

SetupTitle::SetupTitle() : CPropertyPage(SetupTitle::IDD)
{
	//{{AFX_DATA_INIT(SetupTitle)
	m_sMain = g_opt.sMain;
	m_sChan = g_opt.sChan;
	m_sTalk = g_opt.sTalk;
	m_sChat = g_opt.sChat;
	//}}AFX_DATA_INIT
}

SetupTitle::~SetupTitle()
{
}

void SetupTitle::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupTitle)
	DDX_Text(pDX, IDC_TITLE_CHAN, m_sChan);
	DDX_Text(pDX, IDC_TITLE_MAIN, m_sMain);
	DDX_Text(pDX, IDC_TITLE_TALK, m_sTalk);
	DDX_Text(pDX, IDC_TITLE_CHAT, m_sChat);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupTitle, CPropertyPage)
	//{{AFX_MSG_MAP(SetupTitle)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void SetupTitle::OnOK() 
{
	if (!UpdateData(TRUE))
		return;

	g_opt.sMain = m_sMain;
	g_opt.sChan = m_sChan;
	g_opt.sTalk = m_sTalk;
	g_opt.sChat = m_sChat;

	((CSetup*)GetParent())->UpdateTitle();

	OptSaveTitle();
}

// --------------------------------------------------------------------
// Page: Format

IMPLEMENT_DYNCREATE(SetupFormat, CPropertyPage)

SetupFormat::SetupFormat() : CPropertyPage(SetupFormat::IDD)
{
	//{{AFX_DATA_INIT(SetupFormat)
	m_fMsgJoin = g_opt.fMsgJoin;
	m_fMsgKick = g_opt.fMsgKick;
	m_fMsgMode = g_opt.fMsgMode;
	m_fMsgNick = g_opt.fMsgNick;
	m_fMsgPart = g_opt.fMsgPart;
	m_fMsgQuit = g_opt.fMsgQuit;
	m_fMsgTopic = g_opt.fMsgTopic;
	m_fMsgTime = g_opt.fMsgTime;
	m_sMsgTime = g_opt.sMsgTime;
	m_fMsgQuery = g_opt.fMsgQuery;
	m_sMsgNick = g_opt.sMsgNick;
	m_fMsgReplyToMain = g_opt.fMsgReplyToMain;
	//}}AFX_DATA_INIT
}

SetupFormat::~SetupFormat()
{
}

void SetupFormat::UpdateState()
{
	BOOL f;
	f = ((CButton*)GetDlgItem(IDC_MSG_TIME))->GetCheck();
	GetDlgItem(IDC_MSG_TIME_FMT)->EnableWindow(f);
}

void SetupFormat::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupFormat)
	DDX_Check(pDX, IDC_MSG_JOIN, m_fMsgJoin);
	DDX_Check(pDX, IDC_MSG_KICK, m_fMsgKick);
	DDX_Check(pDX, IDC_MSG_MODE, m_fMsgMode);
	DDX_Check(pDX, IDC_MSG_NICK, m_fMsgNick);
	DDX_Check(pDX, IDC_MSG_PART, m_fMsgPart);
	DDX_Check(pDX, IDC_MSG_QUIT, m_fMsgQuit);
	DDX_Check(pDX, IDC_MSG_TOPIC, m_fMsgTopic);
	DDX_Check(pDX, IDC_MSG_TIME, m_fMsgTime);
	DDX_Text(pDX, IDC_MSG_TIME_FMT, m_sMsgTime);
	DDX_Check(pDX, IDC_MSG_CTCP_QUERY, m_fMsgQuery);
	DDX_Text(pDX, IDC_NICK_FMT, m_sMsgNick);
	DDX_Check(pDX, IDC_MSG_REPLY_MAIN, m_fMsgReplyToMain);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupFormat, CPropertyPage)
	//{{AFX_MSG_MAP(SetupFormat)
	ON_BN_CLICKED(IDC_MSG_TIME, OnMsgTime)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static CString CheckNickFmt(LPCTSTR pStr)
{
	LPCTSTR p = pStr;
	int nForm = 0;
	TCHAR c;
	while ((c = *p++) != 0) {
		if (c != '%' || !*p)
			continue;
		c = *p++;
		if (c == '%')
			continue;
		if (c == '-' && *p)
			c = *p++;
		while (isdigit(c) && *p)
			c = *p++;
		if (c != 's')
			return DEF_MSGNICK;
		nForm++;
	}
	if (nForm != 1) {
		TRACE("#of %%s = %d\n", nForm);
		return DEF_MSGNICK;
	}
	return pStr;
}

void SetupFormat::OnOK() 
{
	if (!UpdateData(TRUE))
		return;
	g_opt.fMsgJoin = m_fMsgJoin;
	g_opt.fMsgKick = m_fMsgKick;
	g_opt.fMsgMode = m_fMsgMode;
	g_opt.fMsgNick = m_fMsgNick;
	g_opt.fMsgPart = m_fMsgPart;
	g_opt.fMsgQuit = m_fMsgQuit;
	g_opt.fMsgQuery = m_fMsgQuery;
	g_opt.fMsgTopic = m_fMsgTopic;
	g_opt.fMsgTime = m_fMsgTime;
	g_opt.sMsgTime = m_sMsgTime.IsEmpty() ? "%H:%M" : m_sMsgTime;
	g_opt.sMsgNick = CheckNickFmt(m_sMsgNick);
	g_opt.fMsgReplyToMain = m_fMsgReplyToMain;

	OptSaveFormat();
}

BOOL SetupFormat::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	UpdateState();
	return TRUE;
}

void SetupFormat::OnMsgTime() 
{
	UpdateState();
}

// --------------------------------------------------------------------
// Page: Event

IMPLEMENT_DYNCREATE(SetupEvent, CPropertyPage)

SetupEvent::SetupEvent() : CPropertyPage(SetupEvent::IDD)
{
	//{{AFX_DATA_INIT(SetupEvent)
	m_fPrivMsgPopup = g_opt.fPrivMsgPopup;
	m_fPrivMsgWhois = g_opt.fPrivMsgWhois;
	m_fReconnect = g_opt.fReconnect;
	m_fRejoin = g_opt.fRejoin;
	m_fFlashInactive = g_opt.fFlashInactive;
	m_fFlashMinimized = g_opt.fFlashMinimized;
	m_fRestoreMinimized = g_opt.fRestoreMinimized;
	//}}AFX_DATA_INIT
	m_nInvite = g_opt.nInvite;
	m_nDccChat = g_opt.nDccChat;
}

SetupEvent::~SetupEvent()
{
}

void SetupEvent::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupEvent)
	DDX_Check(pDX, IDC_PRIVMSG_POPUP, m_fPrivMsgPopup);
	DDX_Check(pDX, IDC_PRIVMSG_WHOIS, m_fPrivMsgWhois);
	DDX_Check(pDX, IDC_RECONNECT, m_fReconnect);
	DDX_Check(pDX, IDC_REJOIN, m_fRejoin);
	DDX_Check(pDX, IDC_FLASH_INACTIVE, m_fFlashInactive);
	DDX_Check(pDX, IDC_FLASH_MINIMIZE, m_fFlashMinimized);
	DDX_Check(pDX, IDC_RESTORE_MINIMIZED, m_fRestoreMinimized);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupEvent, CPropertyPage)
	//{{AFX_MSG_MAP(SetupEvent)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void SetupEvent::OnOK() 
{
	if (!UpdateData(TRUE))
		return;
	g_opt.fPrivMsgPopup = m_fPrivMsgPopup;
	g_opt.fPrivMsgWhois = m_fPrivMsgWhois;

	int n;
	switch (GetCheckedRadioButton(IDC_INVITE_JOIN, IDC_INVITE_IGNORE)) {
	case IDC_INVITE_JOIN:
		n = INVITE_JOIN;
		break;
	case IDC_INVITE_IGNORE:
		n = INVITE_IGNORE;
		break;
	default:
		n = INVITE_PROMPT;
	}
	g_opt.nInvite = n;
	switch (GetCheckedRadioButton(IDC_DCC_CHAT_POPUP, IDC_DCC_CHAT_IGNORE)) {
	case IDC_DCC_CHAT_POPUP:
		n = DCC_CHAT_POPUP;
		break;
	case IDC_DCC_CHAT_IGNORE:
		n = DCC_CHAT_IGNORE;
		break;
	default:
		n = DCC_CHAT_PROMPT;
	}
	g_opt.nDccChat = n;
	g_opt.fReconnect = m_fReconnect;
	g_opt.fRejoin = m_fRejoin;

	g_opt.fFlashInactive = m_fFlashInactive;
	g_opt.fFlashMinimized = m_fFlashMinimized;
	g_opt.fRestoreMinimized = m_fRestoreMinimized;
	//g_opt.fContinuousFlash = m_fContinuousFlash;

	OptSaveEvents();
}

BOOL SetupEvent::OnInitDialog() 
{
	CDialog::OnInitDialog();

	int n;
	switch (m_nInvite) {
	case INVITE_JOIN:
		n = IDC_INVITE_JOIN;
		break;
	case INVITE_IGNORE:
		n = IDC_INVITE_IGNORE;
		break;
	default:
		n = IDC_INVITE_PROMPT;
	}
	CheckRadioButton(IDC_INVITE_JOIN, IDC_INVITE_IGNORE, n);

	switch (m_nDccChat) {
	case DCC_CHAT_POPUP:
		n = IDC_DCC_CHAT_POPUP;
		break;
	case DCC_CHAT_PROMPT:
		n = IDC_DCC_CHAT_PROMPT;
		break;
	default:
		n = IDC_DCC_CHAT_IGNORE;
	}
	CheckRadioButton(IDC_DCC_CHAT_POPUP, IDC_DCC_CHAT_IGNORE, n);

	return TRUE;
}

// --------------------------------------------------------------------
// Page: Channels

IMPLEMENT_DYNCREATE(SetupChan, CPropertyPage)

SetupChan::SetupChan() : CPropertyPage(SetupChan::IDD)
{
	//{{AFX_DATA_INIT(SetupChan)
	//}}AFX_DATA_INIT
}

SetupChan::~SetupChan()
{
}

void SetupChan::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupChan)
	DDX_Control(pDX, IDC_CHANNEL, m_edChan);
	DDX_Control(pDX, IDC_CHANNELS_DELETE, m_btDelete);
	DDX_Control(pDX, IDC_CHANNELS_APPEND, m_btAppend);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(SetupChan, CPropertyPage)
	//{{AFX_MSG_MAP(SetupChan)
	ON_BN_CLICKED(IDC_CHANNELS_APPEND, OnAppend)
	ON_BN_CLICKED(IDC_CHANNELS_DELETE, OnDelete)
	ON_EN_CHANGE(IDC_CHANNEL, OnChangeChannel)
	ON_LBN_SELCHANGE(IDC_CHANNELS, OnSelChangeChannels)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL SetupChan::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	m_sFile = GetDataPathname(IDS_FILE_CHANNELS);
	if (m_lbChan.SubclassDlgItem(IDC_CHANNELS, this)) {
		m_lbChan.SetCheckStyle(BS_AUTOCHECKBOX);

		POSITION pos = g_mapChan.GetStartPosition();
		while (pos) {
			CString sChan;
			Chan* pChan;
			g_mapChan.GetNextAssoc(pos, sChan, pChan);
			int nIdx = m_lbChan.AddString(sChan);
			if (nIdx >= 0)
				m_lbChan.SetCheck(nIdx, pChan->m_fAutoJoin);
		}
	}

	m_fModify = FALSE;
	m_btAppend.EnableWindow(FALSE);
	m_btDelete.EnableWindow(FALSE);

	return TRUE;
}

void SetupChan::OnOK() 
{
	if (m_fModify) {
		int i;
		CString sChan;
		for (i = 0; i < m_lbChan.GetCount(); i++) {
			m_lbChan.GetText(i, sChan);
			if (sChan.IsEmpty())
				continue;
			BOOL fAuto = m_lbChan.GetCheck(i);
			g_mapChan.SetAutoJoin(sChan, fAuto);
		}
		m_fModify = FALSE;
	}
}

void SetupChan::OnAppend() 
{
	CString s;
	m_edChan.GetWindowText(s);
	if (s.IsEmpty())
		return;
	m_edChan.SetWindowText("");
	m_edChan.SetFocus();
	int n = m_lbChan.AddString(s);
	if (n >= 0)
		m_lbChan.SetCurSel(n);
	g_mapChan.Append(s);
	m_fModify = TRUE;
}

void SetupChan::OnDelete() 
{
	int n = m_lbChan.GetCurSel();
	if (n < 0)
		return;
	m_lbChan.SetFocus();
	CString s;
	m_lbChan.GetText(n, s);
	m_lbChan.SetRedraw(FALSE);
	int nCount = m_lbChan.DeleteString(n);
	if (n >= nCount && n > 0)
		n--;
	if (n >= 0 && n < nCount)
		m_lbChan.SetCurSel(n);
	m_btDelete.EnableWindow(n >= 0);
	m_lbChan.SetRedraw(TRUE);
	m_edChan.SetWindowText(s);
	g_mapChan.Delete(s);
	m_fModify = TRUE;
}

void SetupChan::OnChangeChannel() 
{
	BOOL f = (m_edChan.LineLength() > 0);
	m_btAppend.EnableWindow(f);
}

void SetupChan::OnSelChangeChannels()
{
	int n = m_lbChan.GetCurSel();
	if (n < 0) {
		m_btDelete.EnableWindow(FALSE);
		return;
	}
	m_btDelete.EnableWindow(TRUE);
	m_fModify = TRUE; // for set checkbox
}

// --------------------------------------------------------------------
// Page: Nicks

int FindString(const CStringArray& as, LPCTSTR pszStr)
{
	int i;
	for (i = 0; i < as.GetSize(); i++) {
		if (as[i] == pszStr)
			return i;
	}
	return -1;
}

IMPLEMENT_DYNCREATE(SetupNicks, CPropertyPage)

SetupNicks::SetupNicks() : CPropertyPage(SetupNicks::IDD)
{
	//{{AFX_DATA_INIT(SetupNicks)
	m_nInterval = 0;
	m_fPopup = FALSE;
	//}}AFX_DATA_INIT
	m_fPopup = g_opt.fIsOnPopup;
	m_nInterval = g_opt.nIsOnInterval;

	m_fChangeInterval = FALSE;
}

SetupNicks::~SetupNicks()
{
}

void SetupNicks::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupNicks)
	DDX_Control(pDX, IDC_ISON_DELETE, m_btDelete);
	DDX_Control(pDX, IDC_ISON_APPEND, m_btAppend);
	DDX_Control(pDX, IDC_ISON_NICKLIST, m_lbNick);
	DDX_Control(pDX, IDC_ISON_NICK, m_edNick);
	DDX_Text(pDX, IDC_ISON_INTERVAL, m_nInterval);
	DDV_MinMaxUInt(pDX, m_nInterval, 10, 300);
	DDX_Check(pDX, IDC_ISON_POPUP, m_fPopup);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupNicks, CPropertyPage)
	//{{AFX_MSG_MAP(SetupNicks)
	ON_EN_CHANGE(IDC_ISON_NICK, OnChangeNick)
	ON_LBN_SELCHANGE(IDC_ISON_NICKLIST, OnSelchangeNicklist)
	ON_BN_CLICKED(IDC_ISON_APPEND, OnAppend)
	ON_BN_CLICKED(IDC_ISON_DELETE, OnDelete)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL SetupNicks::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	int i;
	for (i = 0; i < ison.GetSize(); i++) {
		CIsOnEntry& entry = ison.ElementAt(i);
		m_lbNick.AddString(entry.m_sNick);
	}
	m_btAppend.EnableWindow(FALSE);
	m_btDelete.EnableWindow(FALSE);

	return TRUE;
}

void SetupNicks::OnChangeNick() 
{
	BOOL f = (m_edNick.LineLength() > 0);
	m_btAppend.EnableWindow(f);
}

void SetupNicks::OnSelchangeNicklist() 
{
	int n = m_lbNick.GetSelCount();
	BOOL f = (n > 0);
	m_btDelete.EnableWindow(f);
}

void SetupNicks::OnAppend() 
{
	CString s;
	m_edNick.GetWindowText(s);
	if (s.IsEmpty())
		return;
	m_edNick.SetWindowText("");
	int n = m_lbNick.FindStringExact(-1, s);
	if (n < 0) {
		int niDel = FindString(m_asDel, s);
		if (niDel >= 0)
			m_asDel.RemoveAt(niDel);
		else
			m_asAdd.Add(s);
		n = m_lbNick.AddString(s);
	}
	if (n >= 0) {
		m_lbNick.SetSel(-1, FALSE);
		m_lbNick.SetSel(n, TRUE);
	}
	m_btAppend.EnableWindow(FALSE);
	m_btDelete.EnableWindow(n >= 0);
	GotoDlgCtrl(&m_edNick);
}

void SetupNicks::OnDelete() 
{
	m_btDelete.EnableWindow(FALSE);
	int nSel = m_lbNick.GetSelCount();
	if (nSel < 1)
		return;
	int *panIndex = new int[nSel];
	nSel = m_lbNick.GetSelItems(nSel, panIndex);
	int i;
	for (i = nSel - 1; i >= 0; i--) {
		int n = panIndex[i];
		CString s;
		m_lbNick.GetText(n, s);
		int niAdd = FindString(m_asAdd, s);
		if (niAdd >= 0)
			m_asAdd.RemoveAt(niAdd);
		else
			m_asDel.Add(s);
		m_lbNick.DeleteString(n);
	}
	delete [] panIndex;
	GotoDlgCtrl(&m_lbNick);
}

void SetupNicks::OnOK() 
{
	BOOL fModify = m_asDel.GetSize() > 0 || m_asAdd.GetSize() > 0;
	if (fModify) {
		int i;
		for (i = 0; i < m_asDel.GetSize(); i++)
			ison.DelNick(m_asDel[i]);
		for (i = 0; i < m_asAdd.GetSize(); i++)
			ison.AddNick(m_asAdd[i]);
		ison.Save();
	}
	g_opt.fIsOnPopup = m_fPopup;
	if (g_opt.nIsOnInterval != m_nInterval) {
		g_opt.nIsOnInterval = m_nInterval;
		m_fChangeInterval = TRUE;
		ison.Start(g_opt.nIsOnInterval);
	}
	OptSaveIsOn();
}

// --------------------------------------------------------------------
// Page: Beep

IMPLEMENT_DYNCREATE(SetupBeep, CPropertyPage)

SetupBeep::SetupBeep() : CPropertyPage(SetupBeep::IDD)
{
	//{{AFX_DATA_INIT(SetupBeep)
	//}}AFX_DATA_INIT
}

SetupBeep::~SetupBeep()
{
}

void SetupBeep::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupBeep)
	DDX_Control(pDX, IDC_STATIC_SOUNDFILE, m_stSoundFile);
	DDX_Control(pDX, IDC_BEEP, m_lbBeep);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupBeep, CPropertyPage)
	//{{AFX_MSG_MAP(SetupBeep)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL SetupBeep::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	m_lbBeep.Init();

	CRect rc;
	m_lbBeep.GetWindowRect(&rc);
	m_lbBeep.GetParent()->ScreenToClient(&rc);
	int x = rc.left + m_lbBeep.GetHeaderCol2XPos();

	m_stSoundFile.GetWindowRect(&rc);
	m_stSoundFile.GetParent()->ScreenToClient(&rc);
	rc.OffsetRect(x - rc.left, 0);
	m_stSoundFile.MoveWindow(rc);

	return TRUE;
}

void SetupBeep::OnOK() 
{
	if (!UpdateData(TRUE))
		return;
	m_lbBeep.Save();
}

// Page: Fonts

IMPLEMENT_DYNCREATE(SetupFonts, CPropertyPage)

SetupFonts::SetupFonts() : CPropertyPage(SetupFonts::IDD)
{
	//{{AFX_DATA_INIT(SetupFonts)
		//  - ClassWizard ͂̈ʒuɃȍǉ܂B
	//}}AFX_DATA_INIT
}

SetupFonts::~SetupFonts()
{
}

void SetupFonts::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupFonts)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂͍폜܂B
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupFonts, CPropertyPage)
	//{{AFX_MSG_MAP(SetupFonts)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WMU_BKCOLOR_CHANGE, OnBkColorChange)
END_MESSAGE_MAP()

BOOL SetupFonts::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	int i;
	for (i = 0; i < Fonts::NumberOfTypes; i++) {
		FontColor* pFC = &g_fonts.m_arFC[i];
		FontBtn* pFB = &m_arFontBtn[i];
		pFB->Init(pFC->m_idc, this,
			&pFC->m_font, &pFC->m_color, g_fonts.m_crBack);
	}

	m_colorBtn.Init(IDC_BACKGROUND, this, &g_fonts.m_crBack);

	return TRUE;
}

void SetupFonts::OnOK() 
{
	BOOL fChange = FALSE;
	int i;
	for (i = 0; i < Fonts::NumberOfTypes; i++) {
		FontColor* pFC = &g_fonts.m_arFC[i];
		FontBtn* pFB = &m_arFontBtn[i];
		if (pFB->IsChanged()) {
			pFB->Apply();
			if (i == Fonts::TypeText)
				g_fonts.ChangeTextFont();
			fChange = TRUE;
		}
	}

	if (m_colorBtn.IsChanged()) {
		m_colorBtn.Apply();
		fChange = TRUE;
	}

	if (fChange) {
		g_frames.FontChange();
		g_fonts.SaveAll();
	}
	CPropertyPage::OnOK();
}

LONG SetupFonts::OnBkColorChange(UINT wParam, LONG lParam)
{
	COLORREF cr = (COLORREF)wParam;
	int i;
	for (i = 0; i < Fonts::NumberOfTypes; i++) {
		FontBtn* pFB = &m_arFontBtn[i];
		pFB->SetBkColor(cr);
	}
	return 0;
}

// --------------------------------------------------------------------
// Page: NickList

IMPLEMENT_DYNCREATE(SetupNicklist, CPropertyPage)

SetupNicklist::SetupNicklist() : CPropertyPage(SetupNicklist::IDD)
{
	//{{AFX_DATA_INIT(SetupNicklist)
		//  - ClassWizard ͂̈ʒuɃȍǉ܂B
	//}}AFX_DATA_INIT
	m_fModify = FALSE;
}

SetupNicklist::~SetupNicklist()
{
}

void SetupNicklist::UpDown()
{
	int n = m_lbItem.GetCurSel();
	m_btUp.EnableWindow(n > 0);

	BOOL f = (n >= 0 && n < m_lbItem.GetCount() - 1);
	m_btDown.EnableWindow(f);
}

void SetupNicklist::Move(int nFrom, int nTo)
{
	int nNL = m_lbItem.GetItemData(nFrom);
	BOOL fCheck = m_lbItem.GetCheck(nFrom);
	CString s;
	m_lbItem.GetText(nFrom, s);
	m_lbItem.DeleteString(nFrom);
	int n = m_lbItem.InsertString(nTo, s);
	if (n >= 0) {
		m_lbItem.SetItemData(n, nNL);
		m_lbItem.SetCheck(n, fCheck);
		m_lbItem.SetCurSel(n);
	}
	UpDown();
	GotoDlgCtrl(GetDlgItem(IDC_NICKLIST_ITEM));
	m_fModify = TRUE;
}

void SetupNicklist::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupNicklist)
	DDX_Control(pDX, IDC_NICKLIST_DOWN, m_btDown);
	DDX_Control(pDX, IDC_NICKLIST_UP, m_btUp);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupNicklist, CPropertyPage)
	//{{AFX_MSG_MAP(SetupNicklist)
	ON_BN_CLICKED(IDC_NICKLIST_UP, OnNicklistUp)
	ON_BN_CLICKED(IDC_NICKLIST_DOWN, OnNicklistDown)
	ON_LBN_SELCHANGE(IDC_NICKLIST_ITEM, OnSelchangeNicklistItem)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// SetupNicklist bZ[W nh

BOOL SetupNicklist::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	CMap<int, int, BOOL, BOOL> mapNL;

	if (m_lbItem.SubclassDlgItem(IDC_NICKLIST_ITEM, this)) {
		m_lbItem.SetCheckStyle(BS_AUTOCHECKBOX);
		ArNLItem& ar = g_opt.arNLItem;
		int i;
		for (i = 0; i < ar.GetSize(); i++) {
			int nNL = ar[i];
			if (nNL < 0 || nNL >= NL_END) {
				ASSERT(FALSE);
				continue;
			}
			mapNL.SetAt(nNL, TRUE);
			int nIdx = m_lbItem.AddString(GetNLString(nNL));
			if (nIdx >= 0) {
				m_lbItem.SetItemData(nIdx, nNL);
				m_lbItem.SetCheck(nIdx, TRUE);
			}
		}
		for (i = 0; i < NL_END; i++) {
			BOOL f;
			if (!mapNL.Lookup(i, f)) {
				int nIdx = m_lbItem.AddString(GetNLString(i));
				if (nIdx >= 0)
					m_lbItem.SetItemData(nIdx, i);
			}
		}
	}
	UpDown();

	m_nPosition = g_opt.nNLPos;
	int n;
	switch (m_nPosition) {
	case NLPOS_TOP:
		n = IDC_NLPOS_TOP;
		break;
	case NLPOS_LEFT:
		n = IDC_NLPOS_LEFT;
		break;
	case NLPOS_BOTTOM:
		n = IDC_NLPOS_BOTTOM;
		break;
	default:
		n = IDC_NLPOS_RIGHT;
	}
	CheckRadioButton(IDC_NLPOS_TOP, IDC_NLPOS_BOTTOM, n);

	return TRUE;
}

void SetupNicklist::OnOK() 
{
	if (m_fModify) {
		ArNLItem& ar = g_opt.arNLItem;
		ar.RemoveAll();
		int i;
		for (i = 0; i < m_lbItem.GetCount(); i++) {
			BOOL f = m_lbItem.GetCheck(i);
			if (!f)
				continue;
			int nNL = m_lbItem.GetItemData(i);
			ar.Add(nNL);
		}
		g_frames.PostMsgChan(WMU_NICKLIST_RECREATE);
	}

	UINT n;
	switch (GetCheckedRadioButton(IDC_NLPOS_TOP, IDC_NLPOS_BOTTOM)) {
	case IDC_NLPOS_TOP:
		n = NLPOS_TOP;
		break;
	case IDC_NLPOS_LEFT:
		n = NLPOS_LEFT;
		break;
	case IDC_NLPOS_BOTTOM:
		n = NLPOS_BOTTOM;
		break;
	default:
		n = NLPOS_RIGHT;
	}
	if (n != g_opt.nNLPos) {
		g_opt.nNLPos = n;
		m_fModify = TRUE;
	}

	if (m_fModify)
		OptSaveNickList();

	CPropertyPage::OnOK();
}

void SetupNicklist::OnNicklistUp() 
{
	int n = m_lbItem.GetCurSel();
	if (n < 0)
		return;
	Move(n, n - 1);
}

void SetupNicklist::OnNicklistDown() 
{
	int n = m_lbItem.GetCurSel();
	if (n < 0 || n >= m_lbItem.GetCount() - 1)
		return;
	Move(n, n + 1);
}

void SetupNicklist::OnSelchangeNicklistItem() 
{
	m_fModify = TRUE;
	UpDown();
}

// Page: Others

IMPLEMENT_DYNCREATE(SetupOthers, CPropertyPage)

SetupOthers::SetupOthers() : CPropertyPage(SetupOthers::IDD)
{
	//{{AFX_DATA_INIT(SetupOthers)
	m_fDestNick = g_opt.fDestNick;
	m_fIdentd = g_opt.fIdentd;
	m_fConnectOnStartup = g_opt.fConnectOnStartup;
	m_fServerCollection = g_opt.fServerCollection;
	m_sCtrlChar = g_opt.cCtrlChar;
	m_fMinimizeByReturnKey = g_opt.fMinimizeByReturnKey;
	m_fHideMain = g_opt.fHideMain;
	m_fMinimizeMain = g_opt.fMinimizeMain;
	m_fWhoisDetail = g_opt.fWhoisDetail;
	m_fConfirmPaste = g_opt.fConfirmPaste;
	m_fImeOn = g_opt.fImeOn;
	//}}AFX_DATA_INIT

	m_sBrowser = g_opt.sBrowser;
}

SetupOthers::~SetupOthers()
{
}

void SetupOthers::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupOthers)
	DDX_Control(pDX, IDC_BROWSER, m_btBrowser);
	DDX_Check(pDX, IDC_OPT_DESTNICK, m_fDestNick);
	DDX_Check(pDX, IDC_OPT_IDENTD, m_fIdentd);
	DDX_Check(pDX, IDC_CONNECTONSTART, m_fConnectOnStartup);
	DDX_Check(pDX, IDC_SERVERCOLLECTION, m_fServerCollection);
	DDX_Text(pDX, IDC_OPT_CTRLCHAR, m_sCtrlChar);
	DDV_MaxChars(pDX, m_sCtrlChar, 1);
	DDX_Check(pDX, IDC_MINIMIZEBYRETURNKEY, m_fMinimizeByReturnKey);
	DDX_Check(pDX, IDC_HIDE_MAIN, m_fHideMain);
	DDX_Check(pDX, IDC_MINIMIZE_MAIN, m_fMinimizeMain);
	DDX_Check(pDX, IDC_WHOIS_DETAIL, m_fWhoisDetail);
	DDX_Check(pDX, IDC_CONFIRM_PASTE, m_fConfirmPaste);
	DDX_Check(pDX, IDC_IME_ON, m_fImeOn);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(SetupOthers, CDialog)
	//{{AFX_MSG_MAP(SetupOthers)
	ON_BN_CLICKED(IDC_BROWSER, OnBrowser)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL SetupOthers::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_btBrowser.SetCheck(m_sBrowser.IsEmpty() ? FALSE : TRUE);

	return TRUE;
}

void SetupOthers::OnOK() 
{
	if (!UpdateData(TRUE))
		return;

	g_opt.fMinimizeByReturnKey = m_fMinimizeByReturnKey;
	g_opt.fDestNick = m_fDestNick;
	g_opt.fIdentd = m_fIdentd;
	g_opt.fConnectOnStartup = m_fConnectOnStartup;
	g_opt.fServerCollection= m_fServerCollection;
	g_opt.fMinimizeMain = m_fMinimizeMain;
	g_opt.fHideMain = m_fHideMain;
	g_opt.fWhoisDetail = m_fWhoisDetail;
	g_opt.fConfirmPaste = m_fConfirmPaste;
	g_opt.sBrowser = m_sBrowser;
	g_opt.fImeOn = m_fImeOn;

	if (m_sCtrlChar.GetLength() > 0)
		g_opt.cCtrlChar = m_sCtrlChar[0];

	OptSaveOthers();

	if (sock.GetStatus() & SOCK_CONNECT) {
		if (g_opt.fIdentd) {
			if (!ident.m_fListening)
				ident.Start();
		} else if (ident.m_fListening)
			ident.End();
	}
}

void SetupOthers::OnBrowser() 
{
	if (m_btBrowser.GetCheck() == 0) {
		m_sBrowser.Empty();
		return;
	}
	LPCTSTR szFilter = "Exe Files(*.exe)|*.exe|All Files (*.*)|*.*||";
 	CFileDialog dlg(TRUE, "exe", m_sBrowser,
		OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_LONGNAMES, szFilter);
	dlg.m_ofn.lpstrTitle = "Select browser program";
	if (dlg.DoModal() == IDOK)
		m_sBrowser = dlg.GetPathName();
}

// SetupDcc vpeB y[W

IMPLEMENT_DYNCREATE(SetupDcc, CPropertyPage)

SetupDcc::SetupDcc() : CPropertyPage(SetupDcc::IDD)
{
	//{{AFX_DATA_INIT(SetupDcc)
	m_nDccSendPackets = g_opt.nDccSendPackets;
	m_nDccPort = g_opt.nDccPort;
	//}}AFX_DATA_INIT
}

SetupDcc::~SetupDcc()
{
}

void SetupDcc::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupDcc)
	DDX_Control(pDX, IDC_USE_DCC_FAKE_AUTO, m_btnUseDccFakeAuto);
	DDX_Control(pDX, IDC_DCC_FAKE_ADDR, m_edDccFakeAddr);
	DDX_Control(pDX, IDC_USE_DCC_FAKE_ADDR, m_btnUseDccFakeAddr);
	DDX_Text(pDX, IDC_DCC_PACKET_PRECEDING, m_nDccSendPackets);
	DDV_MinMaxUInt(pDX, m_nDccSendPackets, 1, 32);
	DDX_Text(pDX, IDC_DCC_PORT, m_nDccPort);
	DDV_MinMaxUInt(pDX, m_nDccPort, 0, 65535);
	//}}AFX_DATA_MAP
}

BOOL SetupDcc::GetFakeAddr(BOOL fCheckValid)
{
	m_fUse = m_btnUseDccFakeAddr.GetCheck();
	m_fAuto = m_btnUseDccFakeAuto.GetCheck();

	m_btnUseDccFakeAuto.EnableWindow(m_fUse);
	m_edDccFakeAddr.EnableWindow(m_fUse && !m_fAuto);

	CString sAddr;
	m_edDccFakeAddr.GetWindowText(sAddr);
	m_nAddr = inet_addr(sAddr);
	if ((m_fUse && !m_fAuto) && (m_nAddr == 0 || m_nAddr == INADDR_NONE)) {
		GotoDlgCtrl(GetDlgItem(IDC_DCC_FAKE_ADDR));
		if (fCheckValid)
			AfxMessageBox(IDS_ERR_IPADDR, MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}
	return TRUE;
}

BEGIN_MESSAGE_MAP(SetupDcc, CPropertyPage)
	//{{AFX_MSG_MAP(SetupDcc)
	ON_BN_CLICKED(IDC_USE_DCC_FAKE_ADDR, OnUseDccFakeAddr)
	ON_BN_CLICKED(IDC_USE_DCC_FAKE_AUTO, OnUseDccFakeAuto)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// SetupDcc bZ[W nh

BOOL SetupDcc::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	CheckRadioButton(IDC_CHATEOL_CRLF, IDC_CHATEOL_LF,
		(g_opt.nChatEOL == CHATEOL_LF)
			? IDC_CHATEOL_LF : IDC_CHATEOL_CRLF);
	CheckRadioButton(IDC_CHATCODE_JIS, IDC_CHATCODE_SJIS,
		(g_opt.nChatCode == CHATCODE_SJIS)
			? IDC_CHATCODE_SJIS : IDC_CHATCODE_JIS);

	m_btnUseDccFakeAddr.SetCheck(g_opt.fUseDccFakeAddr);
	m_btnUseDccFakeAuto.SetCheck(g_opt.fUseDccFakeAuto);

	in_addr in;
	in.s_addr = g_opt.addrDccFakeAddr;
	m_edDccFakeAddr.SetWindowText(inet_ntoa(in));

	GetFakeAddr();

	return TRUE;
}

void SetupDcc::OnOK() 
{
	if (!UpdateData(TRUE))
		return;

	if (!GetFakeAddr(TRUE))
		return;
	g_opt.fUseDccFakeAddr = m_fUse;
	g_opt.fUseDccFakeAuto = m_fAuto;
	if (m_fUse && !m_fAuto)
		g_opt.addrDccFakeAddr = m_nAddr;
	g_opt.nDccPort = m_nDccPort;

	g_opt.nDccSendPackets = m_nDccSendPackets;

	g_opt.nChatEOL =
		(GetCheckedRadioButton(IDC_CHATEOL_CRLF,
			IDC_CHATEOL_LF) == IDC_CHATEOL_LF)
			? CHATEOL_LF : CHATEOL_CRLF;

	g_opt.nChatCode =
		(GetCheckedRadioButton(IDC_CHATCODE_JIS,
			IDC_CHATCODE_SJIS) == IDC_CHATCODE_SJIS)
			? CHATCODE_SJIS : CHATCODE_JIS;

	OptSaveDcc();
}

void SetupDcc::OnUseDccFakeAddr() 
{
	GetFakeAddr();
}

void SetupDcc::OnUseDccFakeAuto() 
{
	GetFakeAddr();
}

BOOL SetupDcc::OnKillActive() 
{
	if (!CPropertyPage::OnKillActive())
		return FALSE;

	if (!GetFakeAddr(TRUE))
		return FALSE;

	return TRUE;
}

// SetupPlug vpeB y[W

IMPLEMENT_DYNCREATE(SetupPlug, CPropertyPage)

SetupPlug::SetupPlug() : CPropertyPage(SetupPlug::IDD)
{
	//{{AFX_DATA_INIT(SetupPlug)
		//  - ClassWizard ͂̈ʒuɃȍǉ܂B
	//}}AFX_DATA_INIT
}

SetupPlug::~SetupPlug()
{
}

void SetupPlug::EnableProperty()
{
	int i = m_lbPlug.GetCurSel();
	if (i < 0 || !m_lbPlug.GetCheck(i)) {
		m_btProperty.EnableWindow(FALSE);
		return;
	}
	Plug* pPlug = (Plug*)m_lbPlug.GetItemData(i);
	m_btProperty.EnableWindow(pPlug->Property != NULL);
}

void SetupPlug::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupPlug)
	DDX_Control(pDX, IDC_PROPERTY, m_btProperty);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(SetupPlug, CPropertyPage)
	//{{AFX_MSG_MAP(SetupPlug)
	ON_LBN_SELCHANGE(IDC_PLUGLIST, OnSelchangePluglist)
	ON_BN_CLICKED(IDC_PROPERTY, OnProperty)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// SetupPlug bZ[W nh

BOOL SetupPlug::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	if (m_lbPlug.SubclassDlgItem(IDC_PLUGLIST, this)) {
		m_lbPlug.SetCheckStyle(BS_AUTOCHECKBOX);

		int i;
		for (i = 0; i < g_arPlug.GetSize(); i++) {
			Plug* pPlug = g_arPlug[i];
			int nIdx = m_lbPlug.AddString(pPlug->m_sName);
			if (nIdx < 0)
				continue;
			m_lbPlug.SetItemData(nIdx, (DWORD)pPlug);
		}
	}
	if (m_lbPlug.GetCount() > 0)
		m_lbPlug.SetCurSel(0);
	EnableProperty();

	return TRUE;
}

void SetupPlug::OnSelchangePluglist() 
{
	EnableProperty();
}

void SetupPlug::OnProperty() 
{
	int i = m_lbPlug.GetCurSel();
	if (i < 0)
		return;
	Plug* pPlug = (Plug*)m_lbPlug.GetItemData(i);
	if(pPlug->Property != NULL)
		pPlug->Property();
}

// SetupKeyword vpeB y[W

IMPLEMENT_DYNCREATE(SetupKeyword, CPropertyPage)

SetupKeyword::SetupKeyword() : CPropertyPage(SetupKeyword::IDD)
{
	//{{AFX_DATA_INIT(SetupKeyword)
	m_fKeywordWholeLine = g_opt.fKeywordWholeLine;
	m_fKeywordRestore = g_opt.fRestoreByKeyword;
	//}}AFX_DATA_INIT
}

SetupKeyword::~SetupKeyword()
{
}

void SetupKeyword::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(SetupKeyword)
	DDX_Control(pDX, IDC_LIST_KEYWORD, m_lsKeyword);
	DDX_Control(pDX, IDC_KEYWORD_DELETE, m_btDelete);
	DDX_Control(pDX, IDC_KEYWORD_APPEND, m_btAppend);
	DDX_Control(pDX, IDC_KEYWORD, m_edKeyword);
	DDX_Check(pDX, IDC_KEYWORD_WHOLE_LINE, m_fKeywordWholeLine);
	DDX_Check(pDX, IDC_KEYWORD_RESTORE, m_fKeywordRestore);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(SetupKeyword, CPropertyPage)
	//{{AFX_MSG_MAP(SetupKeyword)
	ON_EN_CHANGE(IDC_KEYWORD, OnChangeKeyword)
	ON_BN_CLICKED(IDC_KEYWORD_APPEND, OnKeywordAppend)
	ON_BN_CLICKED(IDC_KEYWORD_DELETE, OnKeywordDelete)
	ON_LBN_SELCHANGE(IDC_LIST_KEYWORD, OnSelchangeListKeyword)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// SetupKeyword bZ[W nh

BOOL SetupKeyword::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	int i;
	for (i = 0; i < g_arKeyword.GetSize(); i++)
		m_lsKeyword.AddString(g_arKeyword[i]);
	
	m_btAppend.EnableWindow(FALSE);
	m_btDelete.EnableWindow(FALSE);

	return TRUE;
}

void SetupKeyword::OnChangeKeyword() 
{
	BOOL f = (m_edKeyword.LineLength() > 0);
	m_btAppend.EnableWindow(f);
}

void SetupKeyword::OnKeywordAppend() 
{
	CString s;
	m_edKeyword.GetWindowText(s);
	if (s.IsEmpty())
		return;
	m_edKeyword.SetWindowText("");
	m_edKeyword.SetFocus();
	int n = m_lsKeyword.AddString(s);
	if (n >= 0)
		m_lsKeyword.SetCurSel(n);
}

void SetupKeyword::OnKeywordDelete() 
{
	int n = m_lsKeyword.GetCurSel();
	if (n < 0)
		return;
	m_lsKeyword.SetFocus();
	CString s;
	m_lsKeyword.GetText(n, s);
	m_lsKeyword.SetRedraw(FALSE);
	int nCount = m_lsKeyword.DeleteString(n);
	if (n >= nCount && n > 0)
		n--;
	if (n >= 0 && n < nCount)
		m_lsKeyword.SetCurSel(n);
	m_btDelete.EnableWindow(n >= 0);
	m_lsKeyword.SetRedraw(TRUE);
	m_edKeyword.SetWindowText(s);
}

void SetupKeyword::OnSelchangeListKeyword() 
{
	int n = m_lsKeyword.GetCurSel();
	if (n < 0) {
		m_btDelete.EnableWindow(FALSE);
		return;
	}
	m_btDelete.EnableWindow(TRUE);
}

void SetupKeyword::OnOK() 
{
	if (!UpdateData(TRUE))
		return;

	g_opt.fKeywordWholeLine = m_fKeywordWholeLine;
	g_opt.fRestoreByKeyword = m_fKeywordRestore;

	g_arKeyword.RemoveAll();
	int i;
	CString s;
	for (i = 0; i < m_lsKeyword.GetCount(); i++) {
		m_lsKeyword.GetText(i, s);
		if (s.IsEmpty())
			continue;
		g_arKeyword.Add(s);
	}
	OptSaveKeyword();
}
