#include "stdafx.h"

#include "mchat.h"
#include "conv.h"

//#define SHOWALLFUNC

struct Erros {
	int no;
	char *msg;
} errors[] = {
	{ WSANOTINITIALISED, "NOTINITIALISED" },
	{ WSAENETDOWN, "ENETDOWN" },
	{ WSAEACCES, "EACCES" },
	{ WSAEAFNOSUPPORT, "EAFNOSUPPORT" },
	{ WSAEADDRINUSE, "EADDRINUSE" },
	{ WSAEADDRNOTAVAIL, "EADDRNOTAVAIL" },
	{ WSAECONNABORTED, "connection aborted" },
	{ WSAECONNREFUSED, "connection refused" },
	{ WSAECONNRESET, "connection reset" },
	{ WSAEDESTADDRREQ, "EDESTADDRREQ" },
	{ WSAEFAULT, "EFAULT" },
	{ WSAEINPROGRESS, "EINPROGRESS" },
	{ WSAEINVAL, "EINVAL" },
	{ WSAEISCONN, "EISCONN" },
	{ WSAEMFILE, "EMFILE" },
	{ WSAEMSGSIZE, "EMSGSIZE" },
	{ WSAENETUNREACH, "network is unreachable" },
	{ WSAENOBUFS, "ENOBUFS" },
	{ WSAENOTCONN, "ENOTCONN" },
	{ WSAENOTSOCK, "ENOTSOCK" },
	{ WSAEOPNOTSUPP, "EOPNOTSUPP" },
	{ WSAEPROTONOSUPPORT, "EPROTONOSUPPORT" },
	{ WSAEPROTOTYPE, "EPROTOTYPE" },
	{ WSAENETRESET, "ENETRESET" },
	{ WSAESHUTDOWN, "ESHUTDOWN" },
	{ WSAESOCKTNOSUPPORT, "ESOCKTNOSUPPORT" },
	{ WSAETIMEDOUT, "timed out" },
	{ WSAEWOULDBLOCK, "EWOULDBLOCK" },
	{ WSAENOPROTOOPT, "ENOPROTOOPT" },
	{ WSAHOST_NOT_FOUND, "host not found" },
	{ WSAEHOSTUNREACH, "host is unreachable" },
	{ WSATRY_AGAIN, "TRY_AGAIN" },
	{ WSANO_RECOVERY, "NO_RECOVERY" },
	{ WSANO_DATA, "NO_DATA" },
	{ 0, 0 }
};

void CSock::ShowSockErr(char* s, int nCode)
{
	CString str;
	Erros* p = errors;
	while (p->no) {
		if (nCode == p->no) {
			str.Format(IDS_SOCKMSG, p->msg);
			SockNotice(SN_ERROR, str);
			TRACE("%s:%s\n", s, p->msg);
			return;
		}
		p++;
	}
	str.Format(IDS_SOCKERR, nCode);
	SockNotice(SN_ERROR, str);
	TRACE("%s:%s\n", s, (LPCTSTR)str);

#if 0
	LPTSTR lpBuf = 0;
	if (::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
			FORMAT_MESSAGE_FROM_SYSTEM,
			0, nCode,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
			(LPTSTR)&lpBuf, 256, NULL) == 0) {
		CString str;
		str.Format(IDS_SOCKERR, nCode);
		SockNotice(SN_ERROR, str);
		TRACE("%s:%s\n", (LPCTSTR)s, (LPCTSTR)str);
		return;
	}
	SockNotice(SN_ERROR, lpBuf);
	LocalFree(lpBuf);
#endif
}

CSock sock; // main socket

#ifdef SHOWALLFUNC
#define Func(_s) \
	static char _name[] = _s; \
	TRACE("-- %s\n", _name)
#else
#define Func(_s) \
	static char _name[] = _s
#endif
#define Msg() ShowSockErr(_name, WSAGetLastError())
#define Msg1(_nCode) ShowSockErr(_name, _nCode)

CCriticalSection CSock::m_critTimer;
CList<CSock*, CSock*> CSock::m_listSock;
UINT CSock::m_idTimer = 0;

VOID CALLBACK CSock::TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	//TRACE("*");

	m_critTimer.Lock();

	bool bAny = false;
	POSITION pos = m_listSock.GetHeadPosition();
	while (pos) {
		CSock* p = m_listSock.GetNext(pos);
		if (p->m_bSleepSend) {
			//TRACE("(AsyncON)");
			if (p->CanSend())
				p->AsyncSelect(FD_READ|FD_WRITE|FD_CLOSE);
			bAny = true;
		}
	}
	if (!bAny) {
		::KillTimer(NULL, m_idTimer);
		m_idTimer = 0;
		//TRACE(">\n");
	}

	m_critTimer.Unlock();
}

CSock::CSock()
{
	m_nRcv = 0;
	m_wStat = SOCK_NONE;
	m_fRcvSuspend = FALSE;
	m_hAsyncReq = 0;
	m_hHostEnt = 0;
	m_dwSendMS = 0;
	m_bSleepSend = false;
}

void CSock::Init()
{
	m_critTimer.Lock();
	m_listSock.AddHead(this);
	m_critTimer.Unlock();
}

void CSock::End()
{
	m_critTimer.Lock();
	POSITION pos = m_listSock.GetHeadPosition();
	while (pos) {
		POSITION posThis = pos;
		CSock* p = m_listSock.GetNext(pos);
		if (p == this) {
			m_listSock.RemoveAt(posThis);
			break;
		}
	}
	m_critTimer.Unlock();
}

CSock::~CSock()
{
	Close();
	Clear();
}

void CSock::Clear()
{
	while (!m_listSend.IsEmpty())
		delete [] (BYTE*)m_listSend.RemoveHead();
	while (!m_listSendPre.IsEmpty())
		delete [] (BYTE*)m_listSendPre.RemoveHead();
}

void CSock::SetSleepSend(bool b)
{
	//TRACE("%c", b ? '+' : '-');

	m_bSleepSend = b;

	m_critTimer.Lock();

	if (b) {
		if (!m_idTimer) {
			m_idTimer = ::SetTimer(NULL, 0, 200, TimerProc);
			//TRACE("<");
		}
	}

	m_critTimer.Unlock();
}

void CSock::SendData(BYTE* pSnd, int nSnd, bool bUrgent)
{
	if (bUrgent) {
		if (m_listSendPre.IsEmpty() && CanSend() && Send(pSnd, nSnd) != SOCKET_ERROR) {
			MarkSent();
			return;
		}
		m_listSendPre.Put(pSnd, nSnd);
	} else if (m_listSend.IsEmpty() && CanSend() && Send(pSnd, nSnd) != SOCKET_ERROR) {
		MarkSent();
		return;
	} else
		m_listSend.Put(pSnd, nSnd);

	SetSleepSend(true);
}

void CSock::DoClose()
{
	if (m_hAsyncReq) {
		WSACancelAsyncRequest(m_hAsyncReq);
		m_hAsyncReq = 0;
	}
	if (m_hHostEnt) {
		WSACancelAsyncRequest(m_hHostEnt);
		m_hHostEnt = 0;
	}
	Close();
	m_wStat = SOCK_NONE;
	m_bSleepSend = false;
}

void CSock::OnClose(int n)
{
	Func("Sock, on close");

	if (n)
		Msg1(n);
	m_wStat = SOCK_NONE;
	SockNotice(SN_CLOSED);
}

BOOL CSock::DoConnect(LPCTSTR psHost, int nPort)
{
	Func("Sock, do connect");

	m_nRcv = 0;
	m_fRcvSuspend = FALSE;

	if (m_hSocket == INVALID_SOCKET && !Create()) {
		Msg();
		return FALSE;
	}

	static LINGER linger = { 1, 2 };
	if (!SetSockOpt(SO_LINGER, &linger, sizeof linger))
		Msg();

	int nRcvSize = 1024 * 32;
	if (!SetSockOpt(SO_RCVBUF, &nRcvSize, sizeof nRcvSize))
		Msg();
	//if (!SetSockOpt(SO_SNDBUF, &nBufSize, sizeof nBufSize))
	//	Msg();

	BOOL fCon;
	memset(&sockPeer, 0, sizeof sockPeer);
	sockPeer.sin_family = AF_INET;
	sockPeer.sin_port = htons(nPort);
	sockPeer.sin_addr.s_addr = inet_addr(psHost);
	if (sockPeer.sin_addr.s_addr == INADDR_NONE) {
		m_hAsyncReq = WSAAsyncGetHostByName(
			AfxGetMainWnd()->m_hWnd,
			WMU_SOCK_HOST, psHost,
			m_bufHostEnt, sizeof (m_bufHostEnt));
		if (m_hAsyncReq) {
			m_wStat = SOCK_CONNWAIT;
			return TRUE;
		}
		Msg();
		fCon = Connect(psHost, nPort);
	} else
		fCon = Connect((SOCKADDR*)&sockPeer, sizeof sockPeer);

	if (!fCon) {
		int er = WSAGetLastError();
		if (er != WSAEWOULDBLOCK) {
			Msg1(er);
			SockNotice(SN_CONNFAIL);
			return FALSE;
		}
		m_wStat = SOCK_CONNWAIT;
	} else
		m_wStat = SOCK_CONNECT;
	return TRUE;
}

void CSock::ContinueConnect(WORD hHandle, LPARAM lParam)
{
	Func("Sock, continue connect");

	if ((HANDLE)hHandle != m_hAsyncReq) {
		TRACE("asyncreq handle mismatch %d, %d\n", hHandle, m_hAsyncReq);
	}
	m_hAsyncReq = 0;

	if (WSAGETASYNCERROR(lParam)) {
		Msg1(WSAGETASYNCERROR(lParam));
		DoClose();
		SockNotice(SN_CANNOTRESOLVE);
		return;
	}

	HOSTENT* pEnt = (HOSTENT*)m_bufHostEnt;
	sockPeer.sin_addr.s_addr = *(unsigned long*)pEnt->h_addr_list[0];

	if (!Connect((SOCKADDR*)&sockPeer, sizeof sockPeer)) {
		int er = WSAGetLastError();
		if (er != WSAEWOULDBLOCK) {
			Msg1(er);
			SockNotice(SN_CONNFAIL);
			return;
		}
		m_wStat = SOCK_CONNWAIT;
	} else
		m_wStat = SOCK_CONNECT;
}

void CSock::OnConnect(int n)
{
	Func("Sock, on connect");

	if (n) {
		Msg1(n);
		DoClose();
		SockNotice(SN_CONNFAIL);
		return;
	}
	m_wStat = (m_wStat | SOCK_CONNECT) & ~SOCK_CONNWAIT;
	Clear();
	int nLen = sizeof sockaddr;
	if (!GetSockName((SOCKADDR*)&sockaddr, &nLen))
		Msg();
	else
		m_hHostEnt = WSAAsyncGetHostByAddr(
			AfxGetMainWnd()->m_hWnd,
			WMU_SOCK_ADDR, (const char*)&sockaddr.sin_addr.s_addr,
			4, PF_INET, m_bufHostEnt, sizeof m_bufHostEnt);
	SockNotice(SN_CONNECTED);
}

BOOL CSock::GotMyHostname(WORD hHandle, LPARAM lParam)
{
	Func("Sock, got my hostname");

	if (!m_hHostEnt)
		return FALSE;
	if ((HANDLE)hHandle != m_hHostEnt) {
		TRACE("asyncreq handle mismatch %d, %d\n", hHandle, m_hHostEnt);
		return FALSE;
	}
	m_hHostEnt = 0;

	if (WSAGETASYNCERROR(lParam)) {
		Msg1(WSAGETASYNCERROR(lParam));
		return FALSE;
	}
	hostent* pEnt = (hostent*)&sock.m_bufHostEnt;
	m_sHostname = pEnt->h_name;
	return TRUE;
}

void CSock::OnSend(int n)
{
	Func("Sock, on send");

	if (n != 0) {
		Msg1(n);
		return;
	}

	//TRACE("(OnSend)\n");

	if (!m_listSendPre.IsEmpty()) {
		if (CanSend()) {
			DataSend* p = m_listSendPre.RemoveHead();
			if (Send(p->aData, p->nLen) == SOCKET_ERROR)
				Msg();
			else
				MarkSent();
			delete [] (BYTE*)p;
		}
	} else if (!m_listSend.IsEmpty()) {
		if (CanSend()) {
			DataSend* p = m_listSend.RemoveHead();
			if (Send(p->aData, p->nLen) == SOCKET_ERROR)
				Msg();
			else
				MarkSent();
			delete [] (BYTE*)p;
		}
	} else {
		SetSleepSend(false);
		return; // no data to send
	}

	SetSleepSend(!m_listSendPre.IsEmpty() || !m_listSend.IsEmpty());
}

void CSock::OnReceive(int n)
{
	Func("Sock, on receive");

	static int nNest = 0;
	if (nNest) {
		TRACE("nesting\n");
		return;
	}
	nNest++;

	int nRcv;
	if (n)
		Msg1(n);
	else if ((nRcv = sizeof m_aRcv - m_nRcv) <= 0) {
		TRACE("%s: receive buffer full\n", _name);
		m_fRcvSuspend = TRUE;
		PassData();
	} else if ((nRcv = Receive(m_aRcv + m_nRcv, nRcv)) > 0) {
		m_nRcv += nRcv;
		PassData();
	} else if (nRcv == 0)
		TRACE("receive 0\n");
	else
		Msg();
	nNest--;
}

void CSock::PassData()
{
	Func("Sock, pass data");

	BOOL fGotLine = FALSE;

	BYTE* pEnd = m_aRcv + m_nRcv;
	BYTE* pFrom;
	BYTE* pTo;

	for (pFrom = pTo = m_aRcv; pTo < pEnd; pTo++) {
		if (*pTo != '\r' && *pTo != '\n')
			continue;
		fGotLine = TRUE;
		if (pTo > pFrom)
			GotLine(pFrom, pTo - pFrom);
		while (++pTo < pEnd) {
			if (*pTo != '\r' && *pTo != '\n')
				break;
		}
		pFrom = pTo;
	}

	if (!fGotLine && m_nRcv >= sizeof m_aRcv) {
		// too big line, force process it
		GotLine(m_aRcv, m_nRcv);
		m_nRcv = 0;
	} else if (pFrom < pEnd) {
		int n = pEnd - pFrom;
		memcpy(m_aRcv, pFrom, n);
		m_nRcv = n;
	} else
		m_nRcv = 0;

	if (m_fRcvSuspend) {
		m_fRcvSuspend = FALSE;
		if (!AsyncSelect(FD_READ|FD_WRITE|FD_CLOSE))
			Msg();
	}
}

void CSock::GotLine(BYTE* pLine, int nLine)
{
	if (IsDebugLogging())
		WriteDebugFile("R ", (char*)pLine, nLine);

	int n = sizeof m_aLine - 1;
	jis2sjis(pLine, nLine, (BYTE*)m_aLine, n);
	m_aLine[n] = 0;
	ParseLine(m_aLine);
}

// send irc commands

static char bufSnd[512];	// send buffer
static int nSnd;			// size of data in send buffer

void CmdAdd(LPCTSTR pStr)
{
	int n = lstrlen(pStr);
	if (nSnd + n > sizeof bufSnd - 3)
		n = sizeof bufSnd - 3 - nSnd;
	if (n < 1)
		return;
	if (nSnd)
		bufSnd[nSnd++] = ' ';
	memcpy(&bufSnd[nSnd], pStr, n);
	nSnd += n;
}

void CmdAddTrail(LPCTSTR pStr)
{
	int n = lstrlen(pStr);
	if (nSnd + n > sizeof bufSnd - 4)
		n = sizeof bufSnd - 4 - nSnd;
	if (n < 1)
		return;
	if (nSnd) {
		bufSnd[nSnd++] = ' ';
		bufSnd[nSnd++] = ':';
	}
	memcpy(&bufSnd[nSnd], pStr, n);
	nSnd += n;
}

void CmdSend(bool bUrgent)
{
	bufSnd[nSnd++] = '\r';
	bufSnd[nSnd++] = '\n';

	static BYTE aBuf[1024];
	int nBuf = sizeof aBuf - 2;
	sjis2jis((BYTE*)bufSnd, nSnd, aBuf, nBuf);
	if (sock.GetStatus() & SOCK_CONNECT) {
		if (IsDebugLogging())
			WriteDebugFile("S ", (char*)aBuf, nBuf - 2);
		sock.SendData(aBuf, nBuf, bUrgent);
	}
	nSnd = 0;
}
