
#define SOCKBUF		1050

struct DataSend {
	int nLen;
	BYTE aData[1];
};

class ListSend : public CList<DataSend*, DataSend*>
{
public:
	void Put(BYTE* pSnd, int nSnd)
	{
		DataSend* p = (DataSend*)new BYTE[sizeof DataSend + nSnd];
		if (!p)
			return;
		CopyMemory(p->aData, pSnd, p->nLen = nSnd);
		AddTail(p);
	}
};

class CSock : public CAsyncSocket
{
public:
	CSock();
	~CSock();

	void Init();
	void End();

	void Clear();
	BOOL DoConnect(LPCTSTR psHost, int nPort);
	void ContinueConnect(WORD hHandle, LPARAM lParam);
	BOOL GotMyHostname(WORD hHandle, LPARAM lParam);
	void DoClose();
	void SendData(BYTE* buf, int n, bool bUrgent);

#define SOCK_NONE		0
#define SOCK_CONNWAIT	1
#define SOCK_CONNECT	2

	WORD GetStatus() {
		return m_wStat;
	};

	SOCKADDR_IN sockPeer;	// peer socket address
	HANDLE m_hAsyncReq;		// async task handle

	SOCKADDR_IN sockaddr;	// my sock addr
	HANDLE m_hHostEnt;		// async task handle
	CString m_sHostname;	// my hostname

protected:
	char m_bufHostEnt[MAXGETHOSTSTRUCT];	// work for async req

	WORD m_wStat;
	BOOL m_fRcvSuspend;

	ListSend m_listSend;
	ListSend m_listSendPre;

	bool m_bSleepSend;
	void SetSleepSend(bool b);

	DWORD m_dwSendMS;

	bool CanSend() { return (DWORD)(GetTickCount() - m_dwSendMS) > 1000; }
	void MarkSent() { m_dwSendMS = GetTickCount(); }

	static CCriticalSection m_critTimer;
	static CList<CSock*, CSock*> m_listSock;
	static UINT m_idTimer;
	static VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);

	BYTE m_aRcv[SOCKBUF]; // receive raw data
	int m_nRcv;

public:
	char m_aLine[SOCKBUF]; // received one line, after converted

protected:
	void PassData();
	void GotLine(BYTE* pLine, int nLine);
	void ShowSockErr(char* s, int nCode);

	// on
	void OnConnect(int n);
	void OnClose(int n);
	void OnSend(int n);
	void OnReceive(int n);
};

#define SN_CONNECTED		1
#define SN_CLOSED			2
#define SN_ERROR			3
#define SN_CONNFAIL			4
#define SN_CANNOTRESOLVE	5

extern CSock sock; // main socket

// send irc commands

void CmdAdd(LPCTSTR pStr);
void CmdAddTrail(LPCTSTR pStr);
void CmdSend(bool bUrgent = false);

inline void Cmd(LPCTSTR cmd) { CmdAdd(cmd), CmdSend(); }
inline void Cmd(LPCTSTR cmd, LPCTSTR parm) { CmdAdd(cmd); CmdAdd(parm); CmdSend(); }
inline void Cmd(LPCTSTR cmd, LPCTSTR p1, LPCTSTR p2) {
	CmdAdd(cmd); CmdAdd(p1); CmdAdd(p2); CmdSend(); }
inline void Cmd(LPCTSTR cmd, LPCTSTR p1, LPCTSTR p2, LPCTSTR p3) {
	CmdAdd(cmd); CmdAdd(p1); CmdAdd(p2); CmdAdd(p3); CmdSend(); }
inline void CmdTrail(LPCTSTR cmd, LPCTSTR pTrail) {
	CmdAdd(cmd); CmdAddTrail(pTrail); CmdSend(); }
inline void CmdTrail(LPCTSTR cmd, LPCTSTR p1, LPCTSTR pTrail) {
	CmdAdd(cmd); CmdAdd(p1); CmdAddTrail(pTrail); CmdSend(); }
inline void CmdTrail(LPCTSTR cmd, LPCTSTR p1, LPCTSTR p2, LPCTSTR pTrail) {
	CmdAdd(cmd); CmdAdd(p1); CmdAdd(p2); CmdAddTrail(pTrail); CmdSend(); }

inline void CmdPart(LPCTSTR pszChannel) { Cmd("PART", pszChannel); }
inline void CmdWho(LPCTSTR pTarget) { Cmd("WHO", pTarget); }

// commands urgent
inline void CmdU(LPCTSTR p) { CmdAdd(p), CmdSend(true); }
inline void CmdU(LPCTSTR p, LPCTSTR p1) { CmdAdd(p), CmdAdd(p1), CmdSend(true); }
inline void CmdU(LPCTSTR p, LPCTSTR p1, LPCTSTR p2) {
	CmdAdd(p), CmdAdd(p1), CmdAdd(p2), CmdSend(true); }
inline void CmdU(LPCTSTR p, LPCTSTR p1, LPCTSTR p2, LPCTSTR p3) {
	CmdAdd(p), CmdAdd(p1), CmdAdd(p2), CmdAdd(p3), CmdSend(true); }

inline void CmdKick(LPCTSTR pChan, LPCTSTR pNick) { CmdU("KICK", pChan, pNick); }
inline void CmdBan(LPCTSTR pChan, LPCTSTR pStr) { CmdU("MODE", pChan, "+b", pStr); }
